/**@name LIBRARY: cietCRYOGTW.ctl

@author: Enrique BLANCO (EN/ICE)

Creation Date: 27/02/2009

Modification History:

version 1.0

External Function : 
  . CRYOGTW_ObjectListGetValueTime
  . CRYOGTW_MenuConfiguration
  . CRYOGTW_HandleMenu
  . CRYOGTW_WidgetRegisterCB
  . CRYOGTW_WidgetDisconnection

Internal Functions :

Purpose: This library contains CRYOGTW device functions.

Usage: Public

PVSS manager usage: NG, NV

Constraints:
  . Global variables and $parameters defined in GW_FGC faceplate and widget
  . PVSS version: 3.6 
  . operating system: WXP and Linux.
  . distributed system: yes.
*/

//@{

//---------------------------------------------------------------------------------------------------------------------------------------

// CRYOGTW_ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
  - sDeviceName, string, input, device name
  - sDeviceType, string, input, unicos object
  - dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
  . PVSS version: 3.6 
  . operating system: WXP and Linux.
  . distributed system: yes.
*/

CRYOGTW_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
  // DebugN("FUNCTION::CRYOGTW_MenuConfiguration");
  // DebugN(sDeviceName,sDeviceType);
  CMW_ObjectListGetValueTime(sDeviceName, sDeviceType, dsReturnData);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CRYOGTW_MenuConfiguration
/**
Purpose: configuration of popup menu

Parameters:
  - sDpName, string, input, device name
  - sDpType, string, input, unicos object
  - dsAccessOk, dyn_string, input, authorized action
  - menuList, dyn_string, output, menu to pop-up
  
Usage: Public

PVSS manager usage: NG, NV

Constraints:
  . Constants UN_POPUPMENU_****
  . PVSS version: 3.6 
  . operating system: WXP and Linux.
  . distributed system: yes.
*/
CRYOGTW_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
     // DebugN("FUNCTION::CRYOGTW_MenuConfiguration");
     // DebugN(sDpName, sDpType, dsAccessOk);
  _UnPlc_MenuConfiguration(sDpName, sDpType, dsAccessOk, menuList);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CRYOGTW_HandleMenu
/**
Purpose: handle the answer of the popup menu

Parameters:
  - deviceName, string, input, the device dp name
  - sDpType, string, input, the device dp type
  - menuList, dyn_string, input, list of requested action
  - menuAnswer, int, input, selected action from the menuList
  
Usage: Public

PVSS manager usage: NG, NV

Constraints:
  . Constants UN_POPUPMENU_****
  . PVSS version: 3.6 
  . operating system: WXP and Linux.
  . distributed system: yes.
*/
CRYOGTW_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
{
   // DebugN("FUNCTION::CRYOGTW_HandleMenu");
  _UnPlc_HandleMenu(deviceName, sDpType, menuList, menuAnswer);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CRYOGTW_WidgetRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
  . Global variables and $parameters defined in GW_FGC faceplate and widget
  . PVSS version: 3.6 
  . operating system: WXP and Linux.
  . distributed system: yes.
*/

CRYOGTW_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
  // DebugN("FUNCTION::CRYOGTW_WidgetRegisterCB");
  CMW_WidgetRegisterCB(sDp, bSystemConnected);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CRYOGTW_WidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
  - sWidgetType, string, input, widget type. 
Usage: External

PVSS manager usage: NG, NV

Constraints:
  . Global variables and $parameters defined in GW_FGC faceplate and widget
  . PVSS version: 3.6 
  . operating system: WXP and Linux.
  . distributed system: yes.
*/

CRYOGTW_WidgetDisconnection(string sWidgetType)
{
  // DebugN("FUNCTION::CRYOGTW_WidgetDisconnection");
  CMW_WidgetDisconnection(sWidgetType);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CRYOGTW_FaceplateRegisterCB
/**
Purpose: faceplate register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
  . Global variables and $parameters defined in GW_FGC faceplate and widget
  . PVSS version: 3.6 
  . operating system: WXP and Linux.
  . distributed system: yes.
*/

CRYOGTW_FaceplateRegisterCB(string sDp, bool bSystemConnected)
{
  // DebugN("FUNCTION::CRYOGTW_FaceplateRegisterCB");
  CMW_FaceplateRegisterCB(sDp, bSystemConnected);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// CRYOGTW_FaceplateDisconnection
/**
Purpose: Animate Faceplate disconnection

Parameters:
   
Usage: External

PVSS manager usage: NG, NV

Constraints:
  . Global variables and $parameters defined in GW_FGC faceplate and widget
  . PVSS version: 3.6 
  . operating system: WXP and Linux.
  . distributed system: yes.
*/

CRYOGTW_FaceplateDisconnection()
{
  // DebugN("FUNCTION::CRYOGTW_FaceplateDisconnection");
  CMW_FaceplateDisconnection();
}

//---------------------------------------------------------------------------------------------------------------------------------------
//@}

