/**@name LIBRARY: cietCalibrationFunctions.ctl

@author: Enrique Blanco (AB/CO)

Creation Date: 26-09-2006

Modification History: 

version 1.0

External Function : 
	. cietCalibration_TableStatusConnect
	. cietCalibration_TableStatusDisconnect
	. cietCalibration_TableStatusDisconnection : animate faceplate disconnection

Internal Functions :
	. cietCalibration_TableStatusRegisterCB : register callback function
		
Purpose: This library contains the functions for managing the panels dealing with the calibration curves treatment (load/change curves)

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in getCalibration faceplate 
	. PVSS version: 3.0
	. operating system: Win XP
	. distributed system: yes.
*/

// constant declaration

//---------------------------------------------------------------------------------------------------------------------------------------
//@{

// cietCalibration_fillCalCurveTable
/**
Purpose: fill in the calibration points

Parameters:
	. shapeTable:		(String)	Table name to fill in
	. shapeNameCurve:	(String)	Curve name 	
	. idCurve:			(int)		identify if the table is the first or the second (only TT case)
	. sDevType:			(String)	Identify the type of the device (TT,PT,EH)

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in calibration table 
	. PVSS version: 3.0
	. operating system: Win XP
	. distributed system: yes.
*/

cietCalibration_fillCalCurveTable()
{
	dyn_float pointsCalc;
	string sDescrip,sDpe,sDpeId;
	shape table=getShape(g_tableName);
	shape title=getShape(g_tableTitleName);
	string sDevType;
	
	//DebugN("FUNCTION: fillCalCurveTable(g_tableName="+g_tableName+" g_tableTitleName= "+g_tableTitleName+" )");
	
	// g_sCalCurve    must contain the right refererence: CalibrationCurve1 (TT), CalibrationCurve2 (TT), CalibrationCurve (PT,EH)
 	sDpe = $sDpName + ".ProcessInput.Curve."+g_sCalCurve;
 	sDpeId = $sDpName + ".ProcessInput.Curve."+g_sCalCurveId;
	//DebugN("---> sDpe = "+sDpe);
	
	dpGet(sDpeId,sDescrip);
	//DebugN("---> sDescrip = "+sDescrip);
	
	if (sDescrip==title.text)
 	 	{
 	 		dpGet(sDpe,pointsCalc);
 	 		//DebugN("---> pointsCalc= "+ pointsCalc);
 	 		//DebugN("---> dynlen(pointsCalc)="+dynlen(pointsCalc));
 			
 	 		if (dynlen(pointsCalc)>0)
 	 			{
 	 				table.deleteAllLines();
 	 				for (int i=1;i<=65;i++)
	 					table.appendLine("Resistance",pointsCalc[i],"Value",pointsCalc[i+65]);
	 			}
		 	else
		 		DebugN("---> <ERROR>: Problem getting the points of the calibration curve <"+sDescrip+">");
 		}
 	
}

//---------------------------------------------------------------------------------------------------------------------------------------
//@{

// cietCalibration_TableStatusRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in calibration table 
	. PVSS version: 3.0
	. operating system: Win XP
	. distributed system: yes.
*/

cietCalibration_TableStatusRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, sSystemName, sPlcName;
	int iAction, iRes, iAlertType1;
	dyn_string exceptionInfo;
	bool bRemote;
	
//	DebugN("FUNCTION: cietCalibration_TableStatusRegisterCB(sDp= "+sDp+" , bSystemConnected= "+bSystemConnected+")");
	
	deviceName = unGenericDpFunctions_getDpName($sDpName);
	// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	
	// Just debugging...
	//DebugN("----> systemAlarm==="+sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName);
	
	if(sPlcName != "") {
		if (dpExists(sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;
		unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;																			 
	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	
//	DebugN("----> TableStatusRegisterCB","deviceName= "+deviceName,"iAction= "+iAction);
//	DebugN("----> g_sCalCurve= "+g_sCalCurve);
//	DebugN("----> g_tableName= "+g_tableName);
//	DebugN("----> g_tableTitleName= "+g_tableTitleName);
//	DebugN("UN_ACTION_DISCONNECT="+UN_ACTION_DISCONNECT+ " UN_ACTION_DPCONNECT=" +UN_ACTION_DPCONNECT+" UN_ACTION_DPDISCONNECT_DISCONNECT="+UN_ACTION_DPDISCONNECT_DISCONNECT + " UN_ACTION_NOTHING="+UN_ACTION_NOTHING);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			cietCalibration_TableStatusDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
		
			g_bDisconnectCB = false;
			if(g_bUnSystemAlarmPlc) {
				iRes = dpConnect("cietCalibration_TableStatusLockCallBack", 
								deviceName + ".ProcessInput.Curve."+g_sCalCurve+":_lock._alert_hdl._locked");

				g_bCallbackConnected = (iRes >= 0);
			}
			else
				{
					//DebugN("case UN_ACTION_DPCONNECT");
					cietCalibration_TableStatusDisconnection();
					
				}
				
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
		
			if(g_bUnSystemAlarmPlc) 
				iRes = dpDisconnect("cietCalibration_TableStatusLockCallBack", 
									deviceName + ".ProcessInput.Curve."+g_sCalCurve+":_lock._alert_hdl._locked");

			if(g_bDisconnectCB)
				cietCalibration_TableStatusDisconnect(deviceName, iRes);

			g_bCallbackConnected = !(iRes >= 0);
			g_bDisconnectCB = false;
				cietCalibration_TableStatusDisconnection();
			
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------
//@{

// cietCalibration_TableStatusLockCallBack
/**
Purpose: set up lock callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in calibration table 
	. PVSS version: 3.0
	. operating system: Win XP
	. distributed system: yes.
*/

cietCalibration_TableStatusLockCallBack(string sdpe, bool bLockState)
{
	int iRes;
	string deviceName;

//	DebugN("FUNCTION: cietCalibration_TableStatusLockCallBack(sdpe= "+sdpe,"bLockState= "+bLockState);
//	DebugN("----> g_sCalCurve= "+g_sCalCurve);
//	DebugN("----> g_tableName= "+g_tableName);
//	DebugN("----> g_tableTitleName= "+g_tableTitleName);
	
	deviceName = unGenericDpFunctions_getDpName(sdpe);
	//DebugN(" ----> deviceName= "+deviceName);
	if(bLockState) { 
		if(g_bDisconnectCB) {
			cietCalibration_TableStatusDisconnect(deviceName, iRes);
			g_bDisconnectCB = false;
		}
		cietCalibration_TableStatusDisconnection();
	}
	else { 
		g_bDisconnectCB = true;
		cietCalibration_TableStatusConnect(deviceName);
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietCalibration_TableStatusConnect  
/**
Purpose: connect function for calibration table

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in calibration table
	. PVSS version: 3.0 
	. operating system: Win XP
	. distributed system: yes.
*/
cietCalibration_TableStatusConnect(string deviceName)
{
int iAction, iRes, iAlertType1, iAlertType2, iRangeType;
	dyn_string exceptionInfo, dsDeviceNameSplit, dsTemp,dsFIPInfos;
	string sPlcName, sSystemName, sAlias, sAgentName, sNature,sDpAgentName,	sDescrip ;

//	DebugN("FUNCTION: cietCalibration_TableStatusConnect(deviceName= "+deviceName+")");
//	DebugN("----> g_sCalCurve= "+g_sCalCurve);
//	DebugN("----> g_tableName= "+g_tableName);
//	DebugN("----> g_tableTitleName= "+g_tableTitleName);
	// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	
	if(sPlcName != "") 
		{
			if (dpExists(sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName))
				g_bUnSystemAlarmPlc = true;
			else 
				g_bUnSystemAlarmPlc = false;
		}
	else
		g_bUnSystemAlarmPlc = false;			
	
	if(g_bSystemConnected)		
		cietCalibration_fillCalCurveTable();	
	
	if(g_bUnSystemAlarmPlc) {
	
	/*	sAlias = unGenericDpFunctions_getAlias(deviceName);
		dsDeviceNameSplit=strsplit(sAlias, ".");*/
		
		iRes = dpConnect("cietCalibration_TableStatusAnimationCB",
						 					deviceName + ".ProcessInput.Curve."+g_sCalCurve, 					
						 					sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".enabled",
/*											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".enabled",	*/
											deviceName + ".ProcessInput.Curve."+g_sCalCurve+":_online.._invalid");
	}
	else
		cietCalibration_TableStatusDisconnection();
	g_bCallbackConnected = (iRes >= 0);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietCalibration_TableStatusDisconnect  
/**
Purpose: disconnect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in cietTT faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietCalibration_TableStatusDisconnect(string deviceName)
{
	int iRes;
	string sPlcName, sSystemName, sAlias,sAgentName;
	dyn_string dsDeviceNameSplit;
	
	//DebugN("FUNCTION: cietCalibration_TableStatusDisconnect(deviceName= "+deviceName+")");
	if(g_bUnSystemAlarmPlc) {
// get the PLC name 
		sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
		sSystemName = unGenericDpFunctions_getSystemName(deviceName);
		
	/*	sAlias = unGenericDpFunctions_getAlias(deviceName);*/
		sAgentName = cietGenericFunctions_getAgentName(deviceName);
		sAlias = unGenericDpFunctions_getAlias(sAgentName);
		dsDeviceNameSplit=strsplit(sAlias, ".");
	
		iRes = dpDisconnect("cietCalibration_TableStatusAnimationCB",
						 					deviceName + ".ProcessInput.Curve."+g_sCalCurve, 					
						 					sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".enabled",
/*											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".enabled",	*/
											deviceName + ".ProcessInput.Curve."+g_sCalCurve+":_online.._invalid");
	}
	g_bCallbackConnected = !(iRes >= 0);
}




//---------------------------------------------------------------------------------------------------------------------------------------

// cietCalibration_TableStatusAnimationCB
/**
Purpose: Animate Calibration Table 

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in cietTT faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietCalibration_TableStatusAnimationCB(string sDp_calCurve, dyn_float fCalCurve,																																																																															
										string sDpSystemIntegrityAlarmValue, int iSystemIntegrityAlarmValue, 
										string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
									/* 	string sDpSystemAlarmValue, int iSystemAlarmValue,
										string sDpSystemAlarmEnabled, bool bSystemAlarmEnabled,*/
										string sDpInvalid, bool bCodeInvalid)
{

int iAlarmValue,iStart;
string sDpPLCName;

//	DebugN("FUNCTION: cietCalibration_TableStatusAnimationCB");
//	DebugN("----> g_sCalCurve= "+g_sCalCurve);
//	DebugN("----> g_tableName= "+g_tableName);
//	DebugN("----> g_tableTitleName= "+g_tableTitleName);
	//DebugN("---> fCalCurve="+fCalCurve);
	
	if ((iSystemIntegrityAlarmValue >= CRYO_SYSTEM_INGETRITY_ALARM_LIMIT)
		/*	||
			(iSystemAlarmValue < CRYO_SYSTEM_INGETRITY_ALARM_LIMIT)*/)
			iAlarmValue=c_unSystemIntegrity_alarm_value_level1;
	else
		iAlarmValue=c_unSystemIntegrity_no_alarm_value;
		
	iStart=strpos(sDpSystemIntegrityAlarmValue, c_cietSystemIntegrity_DS) + strlen(c_cietSystemIntegrity_DS);
	sDpPLCName=substr(sDpSystemIntegrityAlarmValue,  iStart, strpos(sDpSystemIntegrityAlarmValue, ".alarm") - iStart);
	
	cietGenericFunctions_Header(sDpPLCName, iAlarmValue, bSystemIntegrityAlarmEnabled/* && bSystemAlarmEnabled*/, bCodeInvalid);
	
	// Animate calibration curve
	if(g_bSystemConnected)		
		cietCalibration_fillCalCurveTable();
	
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietCalibration_TableStatusDisconnection
/**
Purpose: Animate calibration table  disconnection

Parameters: None

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: Win XP
	. distributed system: yes.
*/

cietCalibration_TableStatusDisconnection()
{

	//DebugN("FUNCTION: cietCalibrationFaceplateStatusDisconnection");
	//	unGenericObject_ColorBoxDisconnect(makeDynString("IO_ERROR","RESET", "AG_ERROR" ));

unGenericObject_DisplayValueDisconnect(makeDynString(""));
	
}



//---------------------------------------------------------------------------------------------------------------------------------------
