/**@name LIBRARY: cietConfig_EHBS.ctl

@author: Alexey Merezhin (EN/ICE)

Creation Date: 16/10/2013

Modification History: 
 16/10/13  amerezhi: init the object


version 1.0

External Function : 
	. cietConfig_EHBS_setConfig :  set  configuration of given DI datapoint
	. cietConfig_EHBS_checkConfig : check data before configuration
	
Internal Functions :
	. CRYOGTW_EHBS_checkConfig : 
	. CRYOGTW_EHBS_setConfig :
	
Purpose: 
This library contains the function used to set ciet EHBS configuration.

Usage: Public

PVSS manager usage: Ctrl, NV, NG

Constraints:
	. JCOP framework
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

//Constants

//@{
//------------------------------------------------------------------------------------------------------------------------

// cietConfig_EHBS_checkConfig
/**
Purpose: check data before configuration

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietConfig_EHBS_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH)
	{
	// 1. Common
//	DebugN("unConfigGenericFunctions_checkParameters");
		unConfigGenericFunctions_checkParameters(configLine, "EHBS", exceptionInfo);
	// 2. call the device check function
//		DebugN("unConfigGenericFunctions_checkDeviceConfig");
		unConfigGenericFunctions_checkDeviceConfig(configLine, "EHBS", exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","unConfigEHBS_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// cietConfig_EHBS_setConfig
/**
Purpose: set configuration of given EHBS datapoint

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietConfig_EHBS_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	
// 1. Initialization : checking + dpCreate
	unConfigGenericFunctions_checkAll(dsConfigs, "EHBS", exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)		// Good inputs
	{
		sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
		unConfigGenericFunctions_createDp(sDpName, "EHBS", exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)	// Creation ok or dp already exists
		{
// 2. Generate device
// 2.1. Parameters
			unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
	// 2.2 call the device check function
			unConfigGenericFunctions_setDeviceConfig(dsConfigs, "EHBS", exceptionInfo);
		}		
	}
}

//------------------------------------------------------------------------------------------------------------------------

// CRYOGTW_EHBS_checkConfig
/**
Purpose: check data before configuration for CRYOGTW front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CRYOGTW_EHBS_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	bool bOk=false;

	if (dynlen(configLine) == (ciet_CONFIG_EHBS_LENGTH + UN_CONFIG_COMMON_LENGTH))
		bOk = true;
	else if (dynlen(configLine) == (ciet_CONFIG_EHBS_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		bOk = true;
//				DebugN("unConfigGenericFunctions_checkArchive");
		unConfigGenericFunctions_checkArchive(configLine[ciet_CONFIG_EHBS_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																					configLine[ciet_CONFIG_EHBS_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																					configLine[ciet_CONFIG_EHBS_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																					exceptionInfo);
	}
	else
		bOk = false;
	
	if(bOk) {
//	DebugN("unConfigGenericFunctions_checkDeadBand");
	/*	unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EHBS_DEADBAND_heatVolMeas], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EHBS_DEADBANDTYPE], exceptionInfo);
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EHBS_DEADBAND_heatZeroVoltRefVolMeas], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EHBS_DEADBANDTYPE], exceptionInfo);
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EHBS_DEADBAND_heatZeroVoltRefVolMeasInVolts], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EHBS_DEADBANDTYPE], exceptionInfo);
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EHBS_DEADBAND_heatZeroHuitVoltRefVolMeas], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EHBS_DEADBANDTYPE], exceptionInfo);
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EHBS_DEADBAND_heatZeroHuitVoltRefVolMeasInVolts], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EHBS_DEADBANDTYPE], exceptionInfo);
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EHBS_DEADBAND_heatRelativeRefVol], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EHBS_DEADBANDTYPE], exceptionInfo);		
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EHBS_DEADBAND_HYSTERESIS], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EHBS_DEADBANDTYPE], exceptionInfo);
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EHBS_DEADBAND_CYCLEPERIOD], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EHBS_DEADBANDTYPE], exceptionInfo);	
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EHBS_DEADBAND_suppliedVoltageCalc], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EHBS_DEADBANDTYPE], exceptionInfo);
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EHBS_DEADBAND_suppliedCurrentCalc], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EHBS_DEADBANDTYPE], exceptionInfo);		
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EHBS_DEADBAND_thermocoupleVoltageOrPtResistanceCalc], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EHBS_DEADBANDTYPE], exceptionInfo);
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EHBS_DEADBAND_thermocoupleOrPtTemperatureCalc], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EHBS_DEADBANDTYPE], exceptionInfo);																
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EHBS_DEADBAND_powerCalc], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EHBS_DEADBANDTYPE], exceptionInfo);					*/																																																																																																										
//		DebugN("unConfigGenericFunctions_checkArchiveParameters");
		unConfigGenericFunctions_checkArchiveParameters(0, configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_EHBS_ARCHIVEACTIVE], exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","CRYOGTW_EHBS_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// CRYOGTW_EHBS_setConfig
/**
Purpose:  set configuration of given EHBS datapoint for CRYOGTW front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: win XP.
	. distributed system: yes.
*/

CRYOGTW_EHBS_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName, sCMWDeviceName;
	dyn_string dsDpe, dsDescription;
	int i;
	bool bArchive;
	int iArchiveTimeFilter, iArchiveType, iSmoothProcedure;
	dyn_float dfMax = makeDynFloat(ciet_CONFIG_EHBS_MAXVALUE_heatVolMeas, ciet_CONFIG_EHBS_MAXVALUE_heatZeroVoltRefVolMeas,
																	ciet_CONFIG_EHBS_MAXVALUE_heatZeroVoltRefVolMeasInVolts, ciet_CONFIG_EHBS_MAXVALUE_heatZeroHuitVoltRefVolMeas,
																	ciet_CONFIG_EHBS_MAXVALUE_heatZeroHuitVoltRefVolMeasInVolts, ciet_CONFIG_EHBS_MAXVALUE_heatRelativeRefVol,
																	ciet_CONFIG_EHBS_MAXVALUE_HYSTERESIS, ciet_CONFIG_EHBS_MAXVALUE_CYCLEPERIOD,
																	ciet_CONFIG_EHBS_MAXVALUE_suppliedVoltageCalc, ciet_CONFIG_EHBS_MAXVALUE_suppliedCurrentCalc,
																 	ciet_CONFIG_EHBS_MAXVALUE_thermocoupleVoltageOrPtResistanceCalc, ciet_CONFIG_EHBS_MAXVALUE_thermocoupleOrPtTemperatureCalc, 
																 	ciet_CONFIG_EHBS_MAXVALUE_thermocoupleVoltageOrPtResistanceCalc, ciet_CONFIG_EHBS_MAXVALUE_thermocoupleOrPtTemperatureCalc, 
																 	dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_EHBS_RANGEMAX],dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_EHBS_RANGEMAX],dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_EHBS_RANGEMAX]);
	dyn_float dfMin = makeDynFloat(ciet_CONFIG_EHBS_MINVALUE_heatVolMeas, ciet_CONFIG_EHBS_MINVALUE_heatZeroVoltRefVolMeas,
																	ciet_CONFIG_EHBS_MINVALUE_heatZeroVoltRefVolMeasInVolts, ciet_CONFIG_EHBS_MINVALUE_heatZeroHuitVoltRefVolMeas,
																	ciet_CONFIG_EHBS_MINVALUE_heatZeroHuitVoltRefVolMeasInVolts, ciet_CONFIG_EHBS_MINVALUE_heatRelativeRefVol,
																	ciet_CONFIG_EHBS_MINVALUE_HYSTERESIS, ciet_CONFIG_EHBS_MINVALUE_CYCLEPERIOD,
																	ciet_CONFIG_EHBS_MINVALUE_suppliedVoltageCalc, ciet_CONFIG_EHBS_MINVALUE_suppliedCurrentCalc,
																	ciet_CONFIG_EHBS_MINVALUE_thermocoupleVoltageOrPtResistanceCalc, ciet_CONFIG_EHBS_MINVALUE_thermocoupleOrPtTemperatureCalc, 
																	ciet_CONFIG_EHBS_MINVALUE_thermocoupleVoltageOrPtResistanceCalc, ciet_CONFIG_EHBS_MINVALUE_thermocoupleOrPtTemperatureCalc, 
																	dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_EHBS_RANGEMIN],dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_EHBS_RANGEMIN],dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_EHBS_RANGEMIN]);
																	

	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
	
// if archive name are in the config->overwrite panel setting.	
	if(dynlen(dsConfigs) == (ciet_CONFIG_EHBS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		if(dsConfigs[ciet_CONFIG_EHBS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[ciet_CONFIG_EHBS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];

		if(dsConfigs[ciet_CONFIG_EHBS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[ciet_CONFIG_EHBS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];

		if(dsConfigs[ciet_CONFIG_EHBS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[ciet_CONFIG_EHBS_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
	}
	
// set aliases
	// set alias on only all DPEs below ProcessInput
//			cietConfigGenericDpFunctions_setAlias(sDpName, dsDpe, dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_ALIAS],exceptionInfo);
// set descriptions
	unGenericDpFunctions_getDeviceDefinitionDPEDescription("EHBS", dsDpe, dsDescription);
	cietConfigGenericDpFunctions_setDescription(sDpName, dsDpe, dsDescription, exceptionInfo);

// set unit
	cietConfigGenericDpFunctions_setUnit(sDpName, makeDynString(".ProcessInput.Data.suppliedVoltageCalc", ".ProcessInput.Data.thermistorVoltageCalc",
																".ProcessInput.Data.heatZeroVoltRefVolMeasInVolts", ".ProcessInput.Data.heatZeroHuitVoltRefVolMeasInVolts",
																".ProcessInput.Data.acSupplyVoltage"),
												ciet_CONFIG_EHBS_FORMAT_VOLT, exceptionInfo);
																								
	cietConfigGenericDpFunctions_setUnit(sDpName, 
																								makeDynString(".ProcessInput.Data.setPoint",".ProcessInput.Data.forcedSetPoint",
																															".ProcessInput.Data.powerCalc"), 
																								dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_EHBS_UNIT], exceptionInfo);
																								
	cietConfigGenericDpFunctions_setUnit(sDpName, 
																								makeDynString(".ProcessInput.Data.cyclePeriod"), 
																								ciet_CONFIG_EHBS_FORMAT_TIME, exceptionInfo);	

	cietConfigGenericDpFunctions_setUnit(sDpName, 
																								makeDynString(".ProcessInput.Data.threshold"), 
																								ciet_CONFIG_EHBS_FORMAT_THCOUPLE, exceptionInfo);	
	cietConfigGenericDpFunctions_setUnit(sDpName, 
																								makeDynString(".ProcessInput.Data.suppliedCurrentCalc"), 
																								ciet_CONFIG_EHBS_FORMAT_A, exceptionInfo);	
	cietConfigGenericDpFunctions_setUnit(sDpName, makeDynString(".ProcessInput.Data.thermocoupleTemperatureCalc", ".ProcessInput.Data.ptTemperatureCalc", ".ProcessInput.Data.thermistorTemperatureCalc"),
											ciet_CONFIG_EHBS_FORMAT_THCOUPLE, exceptionInfo);
	cietConfigGenericDpFunctions_setUnit(sDpName, 
																								makeDynString(".ProcessInput.Data.heaterResistance", ".ProcessInput.Data.ptSensorRange", ".ProcessInput.Data.ptResistanceCalc"), 
																								ciet_CONFIG_EHBS_FORMAT_OHM, exceptionInfo);	
	cietConfigGenericDpFunctions_setUnit(sDpName, makeDynString(".ProcessInput.Data.thermocoupleVoltageCalc",
																".ProcessInput.Data.protectionVolMeasInVolts", ".ProcessInput.Data.protectionRefVolMeasInVolts"), 
																								ciet_CONFIG_EHBS_FORMAT_MILIVOLT, exceptionInfo);
																								
	// set format
	cietConfigGenericDpFunctions_setFormat(sDpName,makeDynString(".ProcessInput.Data.heatVolMeas", 
																															".ProcessInput.Data.heatZeroVoltRefVolMeas",
																															".ProcessInput.Data.heatZeroHuitVoltRefVolMeas",
																															".ProcessInput.Data.heatRelativeRefVol",
																															".ProcessInput.Data.heatCurrentMeas",
																															".ProcessInput.Data.protectionVolMeas",
																															".ProcessInput.Data.protectionRefVolMeas")
																															,"#####", exceptionInfo);
	cietConfigGenericDpFunctions_setFormat(sDpName,makeDynString(".ProcessInput.Data.setPoint",
																									".ProcessInput.Data.forcedSetPoint",
																									".ProcessInput.Data.suppliedVoltageCalc",
																									".ProcessInput.Data.suppliedCurrentCalc",
																									".ProcessInput.Data.thermocoupleVoltageCalc",
																									".ProcessInput.Data.ptResistanceCalc",
																									".ProcessInput.Data.thermistorVoltageCalc",
																									".ProcessInput.Data.protectionVolMeasInVolts",
																									".ProcessInput.Data.protectionRefVolMeasInVolts",
																									".ProcessInput.Data.heatZeroVoltRefVolMeasInVolts",
																									".ProcessInput.Data.heatZeroHuitVoltRefVolMeasInVolts"),
																									"#####.##", exceptionInfo);

  	cietConfigGenericDpFunctions_setFormat(sDpName, makeDynString(".ProcessInput.Data.acSupplyVoltage", ".ProcessInput.Data.heaterResistance", ".ProcessInput.Data.threshold", ".ProcessInput.Data.heatCurrentACScale",
  		".ProcessInput.Data.heatCurrentACOffset", ".ProcessInput.Data.heatVoltageACScale", ".ProcessInput.Data.heatCurrentDCScale", ".ProcessInput.Data.heatVoltageDCScale", ".ProcessInput.Data.heatVoltageDCSetpointScale",
  		".ProcessInput.Data.cyclePeriod", ".ProcessInput.Data.protectionSensorOffset", ".ProcessInput.Data.ptSensorRange"), "#####.###", exceptionInfo);	

	cietConfigGenericDpFunctions_setFormat(sDpName, makeDynString(".ProcessInput.Data.powerCalc",".ProcessInput.Data.thermocoupleTemperatureCalc",".ProcessInput.Data.ptTemperatureCalc"), "#####.###", exceptionInfo);																									

	cietConfigGenericDpFunctions_setFormat(sDpName, makeDynString(".ProcessInput.DeadBand.thermocoupleOrPtTemperatureCalcDeadBand", ".ProcessInput.DeadBand.suppliedVoltageDeadBand", 
																  ".ProcessInput.DeadBand.suppliedCurrentDeadBand", ".ProcessInput.DeadBand.thermistorTemperatureCalcDeadBand", 
																  ".ProcessInput.DeadBand.setPointDeadBand"), "#####.###", exceptionInfo);
																									
	// save the calibration curve name
	dpSet(sDpName+ ".ProcessInput.Curve.curveId",dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_EHBS_CALIBRATION] );
		
	// set pv_range on all floats
	cietConfigGenericDpFunctions_setPvRange(sDpName, 
																					makeDynString(".ProcessInput.Data.heatVolMeas", 
																												".ProcessInput.Data.heatZeroVoltRefVolMeas",
																												".ProcessInput.Data.heatZeroVoltRefVolMeasInVolts",
																												".ProcessInput.Data.heatZeroHuitVoltRefVolMeas",
																												".ProcessInput.Data.heatZeroHuitVoltRefVolMeasInVolts",
																												".ProcessInput.Data.heatRelativeRefVol",
																												".ProcessInput.Data.diagByte1",
																												".ProcessInput.Data.cyclePeriod", 
																												".ProcessInput.Data.suppliedVoltageCalc",
																												".ProcessInput.Data.suppliedCurrentCalc",
																												".ProcessInput.Data.thermocoupleVoltageCalc",
																												".ProcessInput.Data.ptResistanceCalc",
																												".ProcessInput.Data.thermocoupleTemperatureCalc",																												
																												".ProcessInput.Data.ptTemperatureCalc",																												
																												".ProcessInput.Data.powerCalc",
																												".ProcessInput.Data.setPoint",
																												".ProcessInput.Data.forcedSetPoint"),
																					dfMin, dfMax, exceptionInfo);

															
	// set archiving 
	unConfigGenericFunctions_getArchiving(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_EHBS_ARCHIVEACTIVE], 
																				bArchive, dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_EHBS_ARCHIVETIMEFILTER],	
																				iArchiveType, iSmoothProcedure);
									
	cietConfigGenericFunctions_setArchiving("EHBS", sDpName, bArchive, dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL], 
																											dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG],
																											dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT], 
																											iArchiveType,
																											iSmoothProcedure,
																											dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_EHBS_ARCHIVETIMEFILTER],
																											exceptionInfo);

	sCMWDeviceName = dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_EHBS_CMWDEVICE];
	fwDevice_getAddressElements("EHBS", dsDpe, exceptionInfo, "");
	
	cietConfigGenericFunctions_setCMWAdress(sDpName, sCMWDeviceName, dsConfigs[UN_CONFIG_PLC_ADDITIONAL_DRIVER_NUM], dsDpe, exceptionInfo);	

}
//@}
	

mapping cietConfig_EHBS_getPtSensorRanges() {
	mapping m;
	m[0] = "N/A - 0uA";
	m[10000] = "0-20000 ohm - 1 uA";
	m[1000] = "0-2000 ohm - 10 uA";
	m[100] = "0-200 ohm - 100 uA";
	return m;
}