/**@name LIBRARY: cietConfigGenericFunctions.ctl

@author: Celine Vuillermoz (AB-CO-IS)

Creation Date: 26/09/2005
Modification History: 

  01/12/2008: Herve
    - old/new smoothing config
    modified function: cietConfigGenericDpFunctions_setSmoothing
      
	17/12/07	Pablo:	cietConfigGenericFunctions_setArchive: sensorStatus, agentStatus and segmentStatus info to Event Archive.
	20/07/07	Pablo:	cietConfigGenericFunctions_getAddressConfig removed.
	22/03/07	Pablo:	New function getPvRange from one dpe.
	21/11/06	Quique:	Change ST_GTWDEV addressing
	02/10/06	Quique: fipDiag* dpe being integer type, transformation must be of type char!
	29/09/06	Quique:	New additional tag for the dyn_float output CMW
										New ciet_CMW_ADDRESS_ST_GTWDEV_PROPERTY_SUFIX usage for ST_GTWDEV dpe
	20/09/06	Quique: Modify function: cietConfigGenericFunctions_getArchiveParameters
										fArchiveTimeFilter = -1; --> fArchiveTimeFilter = "";

	18/09/06	Quique: CMW new version imposes new transformation value when setting CMW addresses
	
	21/08/06	Quique: special case in calibration curve "CMW address transformation". 
						calibrationCurve is allways float but the transformation in this case must be DOUBLE!
						
	17/08/06	Quique:	Update formats and set CMW address (bug inversed lowlevel and active bits)
	

version 1.0

External Function : 
	. cietConfigGenericDpFunctions_setAlias: set alias
	. cietConfigGenericDpFunctions_setDescription: set description
	. cietConfigGenericDpFunctions_setSmoothing: set smoothing config
	. cietConfigGenericDpFunctions_setPvRange: set the pv_range config
	. cietConfigGenericDpFunctions_getPvRange: get the pv_range config from one dpe.
	. cietConfigGenericDpFunctions_setUnit: set unit
	. cietConfigGenericDpFunctions_setFormat: set format
	
Internal Functions :
	
Purpose: 
This library contains generic functions used to during the import of the ciet devices.

Usage: Public

PVSS manager usage: Ctrl, NV, NG

Constraints:
	. JCOP framework
	. PVSS version: 3.0
	. operating system: Linux and WXP but tested only on WXP.
	. distributed system: yes.
*/

//Constants


//@{

//------------------------------------------------------------------------------------------------------------------------
/*
// cietConfigGenericDpFunctions_setAlias
/**
Purpose: Set the alias on all the DPE of the device DP, remove first the DPE pattern

Parameters:
		sDeviceTypeName: string, input, the device type
		sDeviceDpName: string, input, the device DP name
		bSetAllDpe: bool, input, true=set the alias on all DPE, false=set the alias on LEAF DPE
		sDeviceTypeName: string, input, the device type
		sAliasPrefix: string, input, the alias prefix
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietConfigGenericDpFunctions_setAlias(string sDeviceDpName, dyn_string dsDpe, string Alias, dyn_string &exceptionInfo)
{
	int i, len;
	
	len = dynlen(dsDpe);
	
	for(i=1;i<=len;i++) {
		unConfigGenericFunctions_setAlias(sDeviceDpName+dsDpe[i], Alias+":"+dsDpes[i], exceptionInfo);
		////DebugN(sDeviceDpName+dsDpe[i], Alias+":"+dsDpes[i], exceptionInfo);
		}
}
//------------------------------------------------------------------------------------------------------------------------

// cryoTnnelConfigGenericDpFunctions_setDescription
/**
Purpose: Set the alias on all the DPE of the device DP, remove first the DPE pattern

Parameters:
		sDeviceDpName: string, input, the device DP name
		dsDpe: dyn_string, input, list of DPE
		dsDescription: dyn_string, input, list of description
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietConfigGenericDpFunctions_setDescription(string sDeviceDpName, dyn_string dsDpe, dyn_string dsDescription, dyn_string &exceptionInfo)
{
	int i, len;
	
	len = dynlen(dsDpe);
	
	if(len == dynlen(dsDescription)) {
		for(i=1;i<=len;i++) {
			unConfigGenericFunctions_setDescription(sDeviceDpName+dsDpe[i], dsDescription[i], exceptionInfo);
		}
	}
	else
		fwException_raise(exceptionInfo, "ERROR", "cietConfigGenericDpFunctions_setDescription: wrong parameter size", "");
}

//------------------------------------------------------------------------------------------------------------------------

// cietConfigGenericDpFunctions_setSmoothing
/**
Purpose: Set the the smooting config

Parameters:
		sDeviceDpName: string, input, the device DP name
		dsConfigs: dyn_string, input, config line from the import file
		dsDpe: dyn_string, input, list of DPE
		diOffsetType: dyn_int, input, list of offset for type
		diOffsetValue: dyn_int, input, list of offset for values
		dfMin: dyn_float, input, min value
		dfMax: dyn_float, input, max value
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietConfigGenericDpFunctions_setSmoothing(string sDeviceDpName, dyn_string dsConfigs, dyn_string dsDpe, int iOffsetType, 
																				dyn_int diOffsetValue, dyn_float dfMin, dyn_float dfMax, dyn_string &exceptionInfo)
{
	int i, len;
	int iDeadBandType, iType;
	float fDeadBand;
	
	len = dynlen(dsDpe);
	if(len == dynlen(diOffsetValue)
				&& (len == dynlen(dfMin)) && (len == dynlen(dfMax))) {
		iDeadBandType =	dsConfigs[iOffsetType];
		for(i=1;i<=len;i++) {
//DEADBAND SETTING
	 		unConfigGenericFunctions_GetDeadBandValue(dsConfigs[diOffsetValue[i]], fDeadBand,  
	 																							iDeadBandType, iType, dfMax[i], dfMin[i]);
	 		if (fDeadBand>0 && (iDeadBandType=="1" || iDeadBandType=="2")){	
	 			fwSmoothing_set(sDeviceDpName + dsDpe[i], iType, fDeadBand, UN_CONFIG_DEFAULT_SMOOTHING_TIME_FILTER, exceptionInfo);
			}
                        else if(iDeadBandType == UN_CONFIG_DEADBAND_OLD_NEW)
                                fwSmoothing_set(sDeviceDpName + dsDpe[i], iType, fDeadBand, UN_CONFIG_DEFAULT_SMOOTHING_TIME_FILTER, exceptionInfo);
                        else{
				fwSmoothing_delete(sDeviceDpName + dsDpe[i], exceptionInfo);
			}
		}
	}
	else
		fwException_raise(exceptionInfo, "ERROR", "cietConfigGenericDpFunctions_setSmoothing: wrong parameter size", "");
}

//------------------------------------------------------------------------------------------------------------------------

// cietConfigGenericDpFunctions_setPvRange
/**
Purpose: Set the the pv_range config

Parameters:
		sDeviceDpName: string, input, the device DP name
		dsConfigs: dyn_string, input, config line from the import file
		dsDpe: dyn_string, input, list of DPE
		dfMin: dyn_float, input, min value
		dfMax: dyn_float, input, max value
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietConfigGenericDpFunctions_setPvRange(string sDeviceDpName, dyn_string dsDpe, dyn_float dfMin, dyn_float dfMax, 
																				dyn_string &exceptionInfo)
{
	int i, len;
	int iDeadBandType, iType;
	float fDeadBand;
	
	len = dynlen(dsDpe);
	if((len == dynlen(dfMin)) && (len == dynlen(dfMax))) {
		for(i=1;i<=len;i++) {

			fwPvRange_set(sDeviceDpName + dsDpe[i], dfMin[i], dfMax[i],
			  						UN_CONFIG_DEFAULT_RANGE_NEGATE_RANGE, UN_CONFIG_DEFAULT_RANGE_IGNORE_OUTSIDE,
			  						UN_CONFIG_DEFAULT_RANGE_INCLUSIVE_MIN, UN_CONFIG_DEFAULT_RANGE_INCLUSIVE_MAX,
			  						exceptionInfo);
			  								/*//DebugN(sDeviceDpName + dsDpe[i], dfMin[i], dfMax[i],
			  						UN_CONFIG_DEFAULT_RANGE_NEGATE_RANGE, UN_CONFIG_DEFAULT_RANGE_IGNORE_OUTSIDE,
			  						UN_CONFIG_DEFAULT_RANGE_INCLUSIVE_MIN, UN_CONFIG_DEFAULT_RANGE_INCLUSIVE_MAX,
			  						exceptionInfo);*/
		}
	}
	else
		fwException_raise(exceptionInfo, "ERROR", "cietConfigGenericDpFunctions_setPvRange: wrong parameter size", "");
}

//------------------------------------------------------------------------------------------------------------------------

// cietConfigGenericDpFunctions_getPvRange
/**
Purpose: Get pv_range config from a dpe

Parameters:
		sDpe: string, input, dpElement
		fMin: float, input, min value
		fMax: float, input, max value
		bConfigExist: if config exist
		exceptionInfo: string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietConfigGenericDpFunctions_getPvRange(string sDpe, float &fMin, float &fMax, bool &bConfigExists,
																				dyn_string &exceptionInfo)
{

bool bNegateRange, bIgnoreOutside, bInclusiveMin, bInclusiveMax;

fwPvRange_get(sDpe, bConfigExists, fMin, fMax, bNegateRange, bIgnoreOutside, bInclusiveMin, bInclusiveMax, exceptionInfo);	
}

//------------------------------------------------------------------------------------------------------------------------

// cietConfigGenericDpFunctions_setUnit
/**
Purpose: Set the unit

Parameters:
		sDeviceDpName: string, input, the device DP name
		dsConfigs: dyn_string, input, config line from the import file
		dsDpe: dyn_string, input, list of DPE
		sUnit: string, input, unit
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietConfigGenericDpFunctions_setUnit(string sDeviceDpName, dyn_string dsDpe, string sUnit, dyn_string &exceptionInfo)
{
	int i, len;
	
	len = dynlen(dsDpe);
	for(i=1;i<=len;i++) {
		unConfigGenericFunctions_setUnit(sDeviceDpName+dsDpe[i], sUnit, exceptionInfo);
	}
}

//------------------------------------------------------------------------------------------------------------------------

// cietConfigGenericDpFunctions_setFormat
/**
Purpose: Set the the pv_range config

Parameters:
		sDeviceDpName: string, input, the device DP name
		dsConfigs: dyn_string, input, config line from the import file
		dsDpe: dyn_string, input, list of DPE
		sFormat: string, input, unit
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietConfigGenericDpFunctions_setFormat(string sDeviceDpName, dyn_string dsDpe, string sFormat, dyn_string &exceptionInfo)
{
	int i, len;
	
	len = dynlen(dsDpe);
	for(i=1;i<=len;i++) {
		unConfigGenericFunctions_setFormat(sDeviceDpName+dsDpe[i], sFormat, exceptionInfo);
	}
}

//------------------------------------------------------------------------------------------------------------------------

// cietConfigGenericFunctions_getArchiveParameters
/**
Purpose:  Get archive parameters of a dp

Parameters:

Usage: Internal function

PVSS manager usage: NG, NV
*/

void cietConfigGenericFunctions_getArchiveParameters(string sDpe, string &sArchiveActive,  float &fArchiveTimeFilter)
{
	bool bArchiveActive;
	
	dpGet(sDpe + ":_archive.._archive" , bArchiveActive);
		
	if (bArchiveActive)
		{
		unExportDevice_getArchiveTemp(sDpe, bArchiveActive, sArchiveActive, fArchiveTimeFilter);
//		//DebugN("cietConfigGenericFunctions_getArchiveParameters", bArchiveActive, sArchiveActive, fArchiveTimeFilter);
		}
	else
		{	
		sArchiveActive = "N";
		// fArchiveTimeFilter = -1;
		fArchiveTimeFilter = "";
		}		
}

//------------------------------------------------------------------------------------------------------------------------

// cietConfigGenericFunctions_getArchiveNames
/**
Purpose: Get Archives names (Bool, Analog and Event values)

Parameters:

Usage: Internal function

PVSS manager usage: NG, NV
*/
void cietConfigGenericFunctions_getArchiveNames(string sDp, string sDeviceType, dyn_string &dsNames)
{
int i, j, iLen, iLen_j;
string sArchiveClass, sName;
dyn_string dsDpe, dsTemp, dsCheck;       
    
  switch(sDeviceType)
    {
    case "DI":
			dynAppend(dsDpe, ".ProcessInput.Data.digitalInput" + DPE_ARCHIVE_DELIMITER);
			dynAppend(dsDpe, ".ProcessInput.Data.sensorStatus" + DPE_ARCHIVE_DELIMITER);
			dynAppend(dsDpe, UN_CONFIG_EXPORT_UNDEF);			
			break;
    case "EH":
			dynAppend(dsDpe, ".ProcessInput.Data.prodVariable1[1]" + DPE_ARCHIVE_DELIMITER);
			dynAppend(dsDpe, ".ProcessInput.Data.heatVolMeas" + DPE_ARCHIVE_DELIMITER);
			dynAppend(dsDpe, UN_CONFIG_EXPORT_UNDEF);			
			break;
    case "EHBS":
			dynAppend(dsDpe, ".ProcessInput.Data.prodVariable1[1]" + DPE_ARCHIVE_DELIMITER);
			dynAppend(dsDpe, ".ProcessInput.Data.heatVolMeas" + DPE_ARCHIVE_DELIMITER);
			dynAppend(dsDpe, UN_CONFIG_EXPORT_UNDEF);			
			break;
    case "TT":
    case "LT":
    case "PT":
			dynAppend(dsDpe, ".ProcessInput.Data.sensorByte[1]" + DPE_ARCHIVE_DELIMITER);
			dynAppend(dsDpe, ".ProcessInput.Data.senVolMeas" + DPE_ARCHIVE_DELIMITER);
			dynAppend(dsDpe, UN_CONFIG_EXPORT_UNDEF);			
			break;
    default:
			dynAppend(dsDpe, UN_CONFIG_EXPORT_UNDEF);
			dynAppend(dsDpe, UN_CONFIG_EXPORT_UNDEF);			
			dynAppend(dsDpe, UN_CONFIG_EXPORT_UNDEF);			
			break;
		}
		
		iLen=dynlen(dsDpe);
		for (i=1; i<=iLen; i++)
			{
			if (dsDpe[i]!=UN_CONFIG_EXPORT_UNDEF)
				{
				dynClear(dsTemp);
				dynClear(dsCheck);
				
				dsTemp=strsplit(dsDpe[i], DPE_ARCHIVE_DELIMITER);
				iLen_j=dynlen(dsTemp);
				
				for (j=1; j<=iLen_j; j++)
					{
					sName="";
					sArchiveClass="";
					
					dpGet(sDp + dsTemp[j] + ":_archive.1._class", sArchiveClass);
		
					if(dpExists(sArchiveClass + ".general.arName"))
				 		dpGet(sArchiveClass + ".general.arName:_online.._value", sName);
				 	else
				 		{
				 		if (i==1)
							sName=UN_CONFIG_EXPORT_ARCHIVE_1;
						else if (i==2)
							sName=UN_CONFIG_EXPORT_ARCHIVE_2;
						else if (i==3)
							sName=UN_CONFIG_EXPORT_ARCHIVE_3;
						else
							sName=UN_CONFIG_EXPORT_UNDEF;	
				 		}
				 	
				 	dynAppend(dsCheck, sName);	
					}
				
				dynUnique(dsCheck);
				
				if (dynlen(dsCheck)==1)		//check if same archive for same type
					sName=dsCheck[1];	
				else
					sName=UN_CONFIG_EXPORT_UNDEF;	
					
				}
			else		//default
				{
				if (i==1)
					sName=UN_CONFIG_EXPORT_ARCHIVE_1;
				else if (i==2)
					sName=UN_CONFIG_EXPORT_ARCHIVE_2;
				else if (i==3)
					sName=UN_CONFIG_EXPORT_ARCHIVE_3;
				else
					sName=UN_CONFIG_EXPORT_UNDEF;	
				}
							
			dynAppend(dsNames, sName);	
			}
		
}

//------------------------------------------------------------------------------------------------------------------------
// cietConfigGenericFunctions_getAddressConfig --- NOT USED! 
/**
Purpose:  Get address config parameters of a dp

Parameters:

Usage: Internal function

PVSS manager usage: NG, NV
*/
/*
cietConfigGenericFunctions_getAddressConfig(string sDpe, dyn_string &dsAddressParameters)
{
	dynAppend(dsAddressParameters, makeDynString(UN_CONFIG_EXPORT_UNDEF+UN_PARAMETER_DELIMITER+
														UN_CONFIG_EXPORT_UNDEF+UN_PARAMETER_DELIMITER+
														UN_CONFIG_EXPORT_UNDEF));
}
*/
//------------------------------------------------------------------------------------------------------------------------
// cietConfigGenericFunctions_getSmoothing
/**
Purpose:  Get the deadband type and deadband value

Parameters:

Usage: Internal function

PVSS manager usage: NG, NV
*/

cietConfigGenericFunctions_getSmoothing(string sDpName, dyn_string dsDpe, int &iSmoothingType, dyn_float &dfDeadBand)
{
	int i, len, iSmoothProcedure, bIsSmoothDefined, iDeadBandType;
	bool bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax, bDoesExist;
	float fMinValue , fMaxValue, fDeadbandTemp, fTimeInterval, fDeadband;
	dyn_string exceptionInfo;
	
	len = dynlen(dsDpe);
	for(i=1;i<=len;i++) {
		//Get RangeMax and RangeMin
		fwPvRange_get(sDpName + dsDpe[i], bDoesExist , fMinValue , fMaxValue , bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax , exceptionInfo);

		//Get smoothProcedure, fDeadband & timeInterval
		unExportDevice_fwSmoothing_getTemp(sDpName + dsDpe[i], bIsSmoothDefined, iSmoothProcedure, fDeadbandTemp, fTimeInterval);
		
		//Get unicos DeadBand
		unExportDevice_getDeadBand(fDeadbandTemp, bIsSmoothDefined, fMaxValue, fMinValue, iSmoothProcedure, fDeadband, iDeadBandType);	
		dfDeadBand[i] = fDeadband;
		iSmoothingType = iDeadBandType;
	}
	
}

//------------------------------------------------------------------------------------------------------------------------

// cietConfigGenericDpFunctions_checkString
/**
Purpose: Check that the string is with correct characters

Parameters:
		sStringToCheck: string, input, the string to check
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietConfigGenericDpFunctions_checkString(string sStringToCheck, dyn_string &exceptionInfo)
{
	if(!unConfigGenericFunctions_nameCheck(sStringToCheck))
		fwException_raise(exceptionInfo, "ERROR", "cietConfigGenericDpFunctions_checkString: wrong string \n(must be 0..9, a..z, A..Z, _ and < 64 char): "+sStringToCheck, "");
}

//------------------------------------------------------------------------------------------------------------------------

// cietConfigGenericFunctions_setArchiving
/**
Purpose: get the archive config from fwDefinition and set it to the DP.
WARNING: from the time being the smoothing is set to old/new --> it should be read from the fwDeviceDefinition

Parameters:
		sDeviceType, 			string, 		input, 	device type
		sDeviceDpName, 		string,			input, 	device DP name
		bArchive,					bool,				input,	true=set archive config, false=delete archive config
		sBoolArchive,			string,			input, 	bool archive name
		sAnalogArchive,		string,			input, 	Analog archive name
		sEventArchive,		string,			input, 	event archive name
		exceptionInfo, 		dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: Win XP.
	. distributed system: yes.
*/

cietConfigGenericFunctions_setArchiving(string sDeviceType, string sDeviceDpName, bool bArchive, string sBoolArchive, string sAnalogArchive, string sEventArchive, int iArchiveType, int iSmoothProcedure, int iArchiveTimeFilter, dyn_string &exceptionInfo)
{
	dyn_string dsDPE;
	int i, len, len2;
	string sArchive;
	dyn_string dsSplit;
	
// get archive DPE from fwDeviceDefinition
	fwDevice_getArchiveElements(sDeviceType, dsDPE, exceptionInfo);
////DebugN(exceptionInfo, dsDPE, bArchive);
	if(dynlen(exceptionInfo) <= 0) {
		len = dynlen(dsDPE);
		if(bArchive) {
// set the archive config with old/new smoothing
			for(i=1;i<=len;i++) {
// if dpe=evStsReg01 or evStsReg02 --> sEventArchive
// if dpElementType = bool --> sBoolArchive
// else --> sAnalogArchive
				sArchive = "";
				dsSplit = strsplit(dsDPE[i], ".");
				len2 = dynlen(dsSplit);
				if(len2 > 0) {
					if((dsSplit[len2] == "evStsReg01") || (dsSplit[len2] == "evStsReg02") || (dsSplit[len2] == "sensorStatus") || (dsSplit[len2] == "agentStatus") || (dsSplit[len2] == "segmentStatus")) {
						sArchive = sEventArchive;
					}
					else {
						switch(dpElementType(sDeviceDpName+dsDPE[i])) {
							case DPEL_BOOL:
								sArchive = sBoolArchive;
								break;
							default:
								sArchive = sAnalogArchive;
								break;
						}
					}
				}
////DebugN(i, dsDPE[i], sArchive);
				if(sArchive != "") {
					fwArchive_set(sDeviceDpName + dsDPE[i],
												sArchive,
												iArchiveType,
												iSmoothProcedure,
												0,
									  		iArchiveTimeFilter, 
							 		 			//0,
							  				exceptionInfo);
				}
			}
		}
		else {
// delete the archive config
			for(i=1;i<=len;i++) {
				fwArchive_delete(sDeviceDpName + dsDPE[i], exceptionInfo);
			}
		}
	}
}
//------------------------------------------------------------------------------------------------------------------------

// cietConfigGenericFunctions_setCMWAdress
/**
Purpose: get the archive config from fwDefinition and set it to the DP.
WARNING: from the time being the smoothing is set to old/new --> it should be read from the fwDeviceDefinition

Parameters:
		sDpName, 			string, 		input, 	device type
		sDeviceName, 		string,			input, 	device DP name
		iNum,					int,				input,	true=set archive config, false=delete archive config
		dsDpe,			dyn_string,			input, 	bool archive name
		exceptionInfo,		dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: Win XP.
	. distributed system: yes.
*/

cietConfigGenericFunctions_setCMWAdress(string sDpName, string sDeviceName, int iDriverNum, dyn_string dsDpe, dyn_string &exceptionInfo)	
{
	string sProperty, sTag, sDirection, sTransformation,sRefCMW;
	dyn_string dsTemp;
	int iMode, iTransformation;
	const string CMW_DELIMITER = "$";
	bool bLowLevel,bActive;
	int iSubIndex;
	dyn_string sSplit;

	//DebugN("FUNCTION: cietConfigGenericFunctions_setCMWAdress(dsDpe="+sDpName+" sDeviceName="+sDeviceName+" dsDpe= "+dsDpe);
	for(int i=1; i<=dynlen(dsDpe);i++)
	{
		// Get CMW property and tag from dpe name		
		dsTemp = strsplit(dsDpe[i], ".");		
		
		bActive=true;
		if ((dynlen(dsTemp)<4) && (dsTemp[2]=="ST_GTWDEV"))
			{
			 	//DebugN("*************************** dsTemp ",dsTemp);
				sProperty = "Data"+ciet_CMW_ADDRESS_ST_GTWDEV_PROPERTY_SUFIX;
				sTag = ciet_CMW_ADDRESS_ST_GTWDEV_TAG;		
			}
		else
			{	 
				sProperty = dsTemp[3];
				sTag = dsTemp[4];
			}
		
		// Special case: SensorByte subindex usage
		iSubIndex=0;
		if (patternMatch("*sensorByte*", dsDpe[i]) || (patternMatch("*prodVariable*", dsDpe[i]) && !patternMatch("*RawData.prodVariable*", dsDpe[i])) || patternMatch("*consVariable*", dsDpe[i])) 
			{
				int iLow,iHigh;
				// read index x  (.ProcessInput.Data.sensorByte[1])
				iLow = strpos(dsDpe[i],"[");
				iHigh =strpos(dsDpe[i],"]");
				iSubIndex=(int)substr(dsDpe[i], iLow+1,iHigh-iLow-1)-1;
				// eliminate "[x]"
				sSplit=strsplit(sTag, "[");
				sTag=sSplit[1];								//	(sensorByte)
				
				////DebugN("-------------------------------------");
				////DebugN("case SensorByte CMW addressing");
				////DebugN("Position of index= "+strpos(dsDpe[i],"["));
				////DebugN("Index extracted="+substr(dsDpe[i], strpos(dsDpe[i],"[")+1,1));
				////DebugN("iSubIndex= "+iSubIndex+" sTag= "+sTag); 
				////DebugN("-------------------------------------");
				
				// Correct property to reflect new "RawData" property extracted from propery "Data"
				sProperty="RawData";
			}
		// Case calibrationCurve. The sufix in property must be added.	
		if ( 	(patternMatch("*Output*", dsDpe[i])) && 
					(patternMatch("*calibrationCurve*", dsDpe[i])))
			sProperty=sProperty+ciet_CMW_ADDRESS_OUTPUT_PROPERTY_SUFIX_DIM;
				
		// Case ST_GTWDEV (Alex pending confirmation ...)
		sRefCMW=sDeviceName+CMW_DELIMITER+sProperty+CMW_DELIMITER+sTag;
		
		// iTransformation will also include the fact of not willing to have monitorON
		iTransformation=0;

		if(patternMatch("*Output*", dsDpe[i])) //Output DPE
		{
			// USe CMW_NO_MONITORON to avoid monitorON with properties declared as [W] only in FESA.
			sDirection = DPATTR_ADDR_MODE_OUTPUT_SINGLE;	
			iTransformation=CMW_NO_MONITORON;
			if (patternMatch("*Config*", dsDpe[i]) || patternMatch("*DeadBand*", dsDpe[i]) || patternMatch("*Curve*", dsDpe[i]) || patternMatch("*RawData*", dsDpe[i]))
				{
					sRefCMW=sRefCMW+ciet_CMW_ADDRESS_OUTPUT_SUFIX;			// To avoid double CMW address defintion in case I/O properties
					if (patternMatch("*calibrationCurve*", dsDpe[i]))		// Dimension of the array [2x65]
						sRefCMW=sRefCMW+ciet_CMW_ADDRESS_OUTPUT_TAG_SUFIX_DIM;
         if (patternMatch("*prodVariable1*", dsDpe[i]))	
						sRefCMW=sRefCMW+ciet_CMW_ADDRESS_OUTPUT_TAG_SUFIX_DIM;
         if (patternMatch("*prodVariable3*", dsDpe[i]))		
						sRefCMW=sRefCMW+ciet_CMW_ADDRESS_OUTPUT_TAG_SUFIX_DIM;
				}
			// //DebugN("PatterMatch OUTPUT found  : sDirection="+sDirection);		
			iMode = 0;
			bLowLevel = false;
			//DebugN("case OUTPUT");
		}	
		else //Input DPE
		{
			////DebugN("case INPUT");
			sDirection = DPATTR_ADDR_MODE_INPUT_SPONT;
			if ( (patternMatch("*Data*", dsDpe[i])) 					&& 
					 (!patternMatch("*sensorByte*", dsDpe[i])) 		&&
					 (!patternMatch("*prodVariable*", dsDpe[i])) 	&&
					 (!patternMatch("*consVariable*", dsDpe[i])) )
				{
					////DebugN("-- case INPUT (Data)");
					sRefCMW=sRefCMW+ciet_CMW_ADDRESS_INPUT_SUFIX;			// add time from FESA: timeStamp tag	
				}
			else if (	(patternMatch("*sensorByte*", dsDpe[i])) 		||
					  		(patternMatch("*prodVariable*", dsDpe[i])) 	||
					 		  (patternMatch("*consVariable*", dsDpe[i])) ||
					 		  (patternMatch("*Diag*", dsDpe[i])) ) 
				{
					////DebugN("-- case INPUT (RawData)");
					sDirection = DPATTR_ADDR_MODE_INPUT_SQUERY;
					iTransformation=CMW_NO_MONITORON;
				}
				
			// //DebugN("PatterMatch INPUT found  : sDirection="+sDirection);
			iMode = 66;
			bLowLevel = true;
		}	
		
		// Special case of calibration curves (format float but transformation must be double)
		if (patternMatch("*calibrationCurve*", dsDpe[i]))
			{
				iTransformation += 1005; //double
				// //DebugN(" cietConfigGenericFunctions_setCMWAdress():: dpe= "+dsDpe[i]+" iTransformation="+iTransformation);
			}	
		else if (patternMatch("*fipDiag*", dsDpe[i]))
			{
				iTransformation += 1009; //char
			}
		else
			{
				// Get CMW transformation from DPE Type
				switch(dpElementType(sDpName+dsDpe[i]))
				{
					case DPEL_BIT32:
					case DPEL_BIT32_STRUCT: 			
						iTransformation += 1002; //longlong
					break;
					case DPEL_UINT:
					case DPEL_UINT_STRUCT:
						iTransformation += 1003; //long
					break;
					case DPEL_INT:
					case DPEL_INT_STRUCT:
						iTransformation += 1004; //short
					break;		
					case DPEL_TIME:
					case DPEL_TIME_STRUCT:
						iTransformation += 1005; //double
					break;		
					case DPEL_FLOAT:
					case DPEL_FLOAT_STRUCT:
					case DPEL_DYN_FLOAT:
					case DPEL_DYN_FLOAT_STRUCT:	
						iTransformation += 1006; //float
					break;		
					case DPEL_BOOL:
					case DPEL_BOOL_STRUCT:
					case DPEL_DYN_BOOL:
					case DPEL_DYN_BOOL_STRUCT:	
						iTransformation += 1007; //bool
					break;		
					case DPEL_STRING:
					case DPEL_STRING_STRUCT:
					case DPEL_DYN_STRING:
					case DPEL_DYN_STRING_STRUCT:	
						iTransformation += 1008; //string
					break;		
					case DPEL_CHAR:
					case DPEL_CHAR_STRUCT:
					case DPEL_DYN_CHAR:
					case DPEL_DYN_CHAR_STRUCT:			
						iTransformation += 1009; //char
					break;		
					default:
						iTransformation += 1000; //unknown
					break;		
				}
			}
			
		// Set Adress
		// //DebugN("fwPeriphAddress_set()");
		// //DebugN("sRefCMW="+sRefCMW);
		// //DebugN("sDirection="+sDirection);
		// //DebugN("sDpName =",sDpName + dsDpe[i]);
		// //DebugN("----------");
		
		
		//DebugN("sRefCMW	="+sRefCMW,"sDirection= "+sDirection,"iTransformation="+iTransformation);	
		
		fwPeriphAddress_set(sDpName + dsDpe[i],
									makeDynString(fwPeriphAddress_TYPE_CMW,
									iDriverNum, 
									sRefCMW, 
									sDirection,
									iTransformation, //1002=longlong 1003=long 1004=short 1005=double 1006=float	1007=bool 1008=string 1009=char
									bActive,	// active
									"",
									"",
									"",
									"",
									bLowLevel,
									iSubIndex,
									0,
									0,
									"",
									//iMode), 
									sDirection), 
									exceptionInfo);		
	}								

}
//@}
