/**@name LIBRARY: cietConfig_LT.ctl

@author: Celine VUILLERMOZ (AB-CO-IS)

Creation Date: 20 07 2005

Modification History: 
        .      16/02/09       Quique            rMax changes the units (now is used to hold DL in "mm")
        .      12/02/09       Quique            SetCalibrationCurve
	.	22/06/07	Pablo		Format for Inclination, MaxLength and MaxResistance to 3 decimals.
	.	19/02/07	Pablo		Max and Min Range for Engineering values, taken from import file.
											Added format and unit for offset.
	.	16/02/07	Pablo		Added Range Trend configuration.
	.	18/12/06	Pablo		Changed format resistanceCalc (#.####)
	. 21/08/06	Quique	Include UNIT read from Specs

version 1.0

External Function : 
	. cietConfig_LT_setConfig :  set configuration of given DI datapoint
	. cietConfig_LT_checkConfig : check data before configuration
	
Internal Functions :
	
Purpose: 
This library contains the function used to set Unicos DI configuration.

Usage: Public

PVSS manager usage: Ctrl, NV, NG

Constraints:/**@name LIBRARY: cietConfig_LT.ctl

*/

//Constants



//@{


// cietConfigLT_checkConfig
/**
Purpose: check data before configuration

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietConfig_LT_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH)
	{
	// 1. Common
		unConfigGenericFunctions_checkParameters(configLine, "LT", exceptionInfo);
	// 2. call the device check function
		unConfigGenericFunctions_checkDeviceConfig(configLine, "LT", exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","unConfigLT_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// cietConfig_LT_setConfig
/**
Purpose: set configuration of given LT datapoint

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietConfig_LT_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	
// 1. Initialization : checking + dpCreate
	unConfigGenericFunctions_checkAll(dsConfigs, "LT", exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)		// Good inputs
	{
		sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
		unConfigGenericFunctions_createDp(sDpName, "LT", exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)	// Creation ok or dp already exists
		{
// 2. Generate device
// 2.1. Parameters
			unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
	// 2.2 call the device check function
			unConfigGenericFunctions_setDeviceConfig(dsConfigs, "LT", exceptionInfo);
		}		
	}
}


//------------------------------------------------------------------------------------------------------------------------

// CRYOGTW_LT_checkConfig
/**
Purpose: check data before configuration for CRYOGTW front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CRYOGTW_LT_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	bool bOk=false;
	
	if (dynlen(configLine) == (ciet_CONFIG_LT_LENGTH + UN_CONFIG_COMMON_LENGTH))
		bOk = true;
	else if (dynlen(configLine) == (ciet_CONFIG_LT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		bOk = true;
		unConfigGenericFunctions_checkArchive(configLine[ciet_CONFIG_LT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																					configLine[ciet_CONFIG_LT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																					configLine[ciet_CONFIG_LT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																					exceptionInfo);
	}
	else
		bOk = false;
	
	if(bOk) {/*
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_LT_DEADBAND_senVolMeas], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_LT_DEADBANDTYPE], exceptionInfo);
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_LT_DEADBAND_refVolMeas], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_LT_DEADBANDTYPE], exceptionInfo);																
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_LT_DEADBAND_gainValue], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_LT_DEADBANDTYPE], exceptionInfo);
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_LT_DEADBAND_resistanceCalc], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_LT_DEADBANDTYPE], exceptionInfo);																
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_LT_DEADBAND_levelCalc], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_LT_DEADBANDTYPE], exceptionInfo);
		unConfigGenericFunctions_checkDeadBand(configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_LT_DEADBAND_levelmmCalc], 
																configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_LT_DEADBANDTYPE], exceptionInfo);	*/															
		unConfigGenericFunctions_checkArchiveParameters(0, configLine[UN_CONFIG_COMMON_LENGTH + ciet_CONFIG_LT_ARCHIVEACTIVE], exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","CRYOGTW_LT_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// CRYOGTW_LT_setConfig
/**
Purpose:  set configuration of given LT datapoint for CRYOGTW front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: win XP.
	. distributed system: yes.
*/

CRYOGTW_LT_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName, sCMWDeviceName;
	dyn_string dsDpe, dsDescription;
	bool bArchive;
	int iArchiveTimeFilter, iArchiveType, iSmoothProcedure, i;
	dyn_float dfMax = makeDynFloat(ciet_CONFIG_LT_MAXVALUE_senVolMeas, ciet_CONFIG_LT_MAXVALUE_refVolMeas,
																		ciet_CONFIG_LT_MAXVALUE_gainValue,ciet_CONFIG_LT_MAXVALUE_resistanceCalc,
		  															dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_LT_RANGEMAX],dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_LT_RANGEMAX]);
	dyn_float dfMin = makeDynFloat(ciet_CONFIG_LT_MINVALUE_senVolMeas, ciet_CONFIG_LT_MINVALUE_refVolMeas,
																		ciet_CONFIG_LT_MINVALUE_gainValue,ciet_CONFIG_LT_MINVALUE_resistanceCalc,
		  															dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_LT_RANGEMIN],dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_LT_RANGEMIN]);

	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
	
// if archive name are in the config->overwrite panel setting.	
	if(dynlen(dsConfigs) == (ciet_CONFIG_LT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		if(dsConfigs[ciet_CONFIG_LT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[ciet_CONFIG_LT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];

		if(dsConfigs[ciet_CONFIG_LT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[ciet_CONFIG_LT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];

		if(dsConfigs[ciet_CONFIG_LT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[ciet_CONFIG_LT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
	}
	
// set aliases
	// set alias on only all DPEs below ProcessInput
	//cietConfigGenericDpFunctions_setAlias(sDpName, dsDpe, dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_ALIAS],exceptionInfo);

// set descriptions
	unGenericDpFunctions_getDeviceDefinitionDPEDescription("LT", dsDpe, dsDescription);
	cietConfigGenericDpFunctions_setDescription(sDpName, dsDpe, dsDescription, exceptionInfo);
		
		// set unit
	cietConfigGenericDpFunctions_setUnit(sDpName, makeDynString(".ProcessInput.Data.senVolMeas", ".ProcessInput.Data.refVolMeas"), 
								    "", exceptionInfo);
	cietConfigGenericDpFunctions_setUnit(sDpName, makeDynString(".ProcessInput.Data.gainValue"), 
								    ciet_CONFIG_LT_UNIT_DB, exceptionInfo);
	cietConfigGenericDpFunctions_setUnit(sDpName, makeDynString(".ProcessInput.Data.resistanceCalc"), 
								    ciet_CONFIG_LT_UNIT_OHMS, exceptionInfo);
        // rMax has changed and now the name is DL (dead length) which unit is mm
        cietConfigGenericDpFunctions_setUnit(sDpName, makeDynString(".ProcessInput.Data.rMax"),
                                                                    ciet_CONFIG_LT_UNIT_MM, exceptionInfo); 
	cietConfigGenericDpFunctions_setUnit(sDpName, makeDynString(".ProcessInput.Data.offset",".ProcessInput.Data.offset"), 
								    ciet_CONFIG_LT_UNIT_OHMS, exceptionInfo);	
        // lMax has changed and now the name is AL (active length), but still the units is mm																			
	cietConfigGenericDpFunctions_setUnit(sDpName, makeDynString(".ProcessInput.Data.levelmmCalc",".ProcessInput.Data.lMax"), 
								    ciet_CONFIG_LT_UNIT_MM, exceptionInfo);
        cietConfigGenericDpFunctions_setUnit(sDpName, makeDynString(".ProcessInput.Data.levelOffset",".ProcessInput.Data.levelOffset"), 
								    ciet_CONFIG_LT_UNIT_MM, exceptionInfo);
	// cietConfigGenericDpFunctions_setUnit(sDpName, makeDynString(".ProcessInput.Data.levelCalc"), 
	//							    ciet_CONFIG_LT_UNIT_POURCENT, exceptionInfo);
  cietConfigGenericDpFunctions_setUnit(sDpName, makeDynString(".ProcessInput.Data.levelCalc"), 
  						              dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_LT_UNIT], exceptionInfo);
																				
	cietConfigGenericDpFunctions_setUnit(sDpName, makeDynString(".ProcessInput.Data.inclination"), 
						 ciet_CONFIG_LT_UNIT_ANGLE, exceptionInfo);
	
// save the calibration curve name
	dpSet(sDpName+ ".ProcessInput.Curve.curveId",
	      dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_LT_CALIBRATION] );

               
        
        
// set format
	cietConfigGenericDpFunctions_setFormat(sDpName, 
																							makeDynString(".ProcessInput.Data.senVolMeas", ".ProcessInput.Data.refVolMeas",
																							".ProcessInput.Data.gainValue", ".ProcessInput.Data.levelCalc", 
																							".ProcessInput.Data.levelmmCalc"), 
																							"#####.##", exceptionInfo);

	cietConfigGenericDpFunctions_setFormat(sDpName, 
																							makeDynString(".ProcessInput.Data.senVolMeas", ".ProcessInput.Data.refVolMeas"), 
																							"######", exceptionInfo);

	cietConfigGenericDpFunctions_setFormat(sDpName,".ProcessInput.Data.resistanceCalc", "#####.###", exceptionInfo);		
	cietConfigGenericDpFunctions_setFormat(sDpName,".ProcessInput.Data.rMax", "#####.###", exceptionInfo);		
	cietConfigGenericDpFunctions_setFormat(sDpName,".ProcessInput.Data.lMax", "#####.###", exceptionInfo);		 
	cietConfigGenericDpFunctions_setFormat(sDpName,".ProcessInput.Data.inclination", "#####.###", exceptionInfo);
        cietConfigGenericDpFunctions_setFormat(sDpName,".ProcessInput.Data.gainCorrection", "#####.###", exceptionInfo);
        cietConfigGenericDpFunctions_setFormat(sDpName,".ProcessInput.Data.levelOffset", "#####.###", exceptionInfo);																							
	cietConfigGenericDpFunctions_setFormat(sDpName,".ProcessInput.Data.offset", "#####.###", exceptionInfo);	
	cietConfigGenericDpFunctions_setFormat(sDpName,".ProcessInput.DeadBand.resistanceCalcDeadBand", "#####.###", exceptionInfo);	
	cietConfigGenericDpFunctions_setFormat(sDpName,".ProcessInput.Data.filterSamples", "#####", exceptionInfo);			


// set smoothing on all floats
/*
	cietConfigGenericDpFunctions_setSmoothing(sDpName, dsConfigs, 
																					makeDynString(".ProcessInput.Data.senVolMeas", 
																												".ProcessInput.Data.refVolMeas",
																												".ProcessInput.Data.gainValue",
																												".ProcessInput.Data.resistanceCalc",
																												".ProcessInput.Data.levelCalc",
																												".ProcessInput.Data.levelmmCalc"),
																					UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_LT_DEADBANDTYPE,
																					makeDynInt(UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_LT_DEADBAND_senVolMeas,
																											UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_LT_DEADBAND_refVolMeas,
																											UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_LT_DEADBAND_gainValue,
																											UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_LT_DEADBAND_resistanceCalc,
																											UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_LT_DEADBAND_levelCalc,
																											UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_LT_DEADBAND_levelmmCalc),
																					dfMin, dfMax, exceptionInfo);
	*/

// set pv_range on all floats
	cietConfigGenericDpFunctions_setPvRange(sDpName, 
																					makeDynString(".ProcessInput.Data.senVolMeas", 
																												".ProcessInput.Data.refVolMeas",
																												".ProcessInput.Data.gainValue",
																												".ProcessInput.Data.resistanceCalc",
																												 ".ProcessInput.Data.levelCalc",
																												".ProcessInput.Data.levelmmCalc"),
																					dfMin, dfMax, exceptionInfo);


// set Range to visualize on plot
/*	unGenericDpFunctions_setFaceplateRange(sDpName, makeDynString(".ProcessInput.Data.levelCalc"),
																			 		dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_LT_RANGEMAX], dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_LT_RANGEMIN], exceptionInfo);
																		 		
	unGenericDpFunctions_setFaceplateRange(sDpName, makeDynString(".ProcessInput.Data.levelmmCalc"),
																			 		dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_LT_RANGEMAX], dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_LT_RANGEMIN], exceptionInfo);
																			 		
	unGenericDpFunctions_setFaceplateRange(sDpName, makeDynString(".ProcessInput.Data.senVolMeas"),
																			 		ciet_CONFIG_LT_MAXVALUE_senVolMeas, ciet_CONFIG_LT_MINVALUE_senVolMeas, exceptionInfo);
																			 		
	unGenericDpFunctions_setFaceplateRange(sDpName, makeDynString(".ProcessInput.Data.refVolMeas"),
																			 		ciet_CONFIG_LT_MAXVALUE_refVolMeas, ciet_CONFIG_LT_MINVALUE_refVolMeas, exceptionInfo);
*/
// set archiving 
/*	unConfigGenericFunctions_getArchiving(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_LT_ARCHIVEACTIVE], 
																				bArchive, iArchiveTimeFilter,	iArchiveType, iSmoothProcedure);
	unConfigGenericFunctions_setArchiving("LT", sDpName, bArchive, dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL], 
																											dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG],
																											dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT], exceptionInfo);*/
		// set archiving 
	unConfigGenericFunctions_getArchiving(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_LT_ARCHIVEACTIVE], 
																				bArchive, dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_LT_ARCHIVETIMEFILTER],	
																				iArchiveType, iSmoothProcedure);
																				
/*	DebugN															("LT", sDpName, bArchive, dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL], 
																											dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG],
																											dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT], 
																											iArchiveType,
																											iSmoothProcedure,
																											dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_EH_ARCHIVETIMEFILTER],
																											exceptionInfo);*/
									
	cietConfigGenericFunctions_setArchiving("LT", sDpName, bArchive, dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL], 
																											dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG],
																											dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT], 
																											iArchiveType,
																											iSmoothProcedure,
																											dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_LT_ARCHIVETIMEFILTER],
																											exceptionInfo);																									

	sCMWDeviceName = dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_LT_CMWDEVICE];
	fwDevice_getAddressElements("LT", dsDpe, exceptionInfo, "");
	
	cietConfigGenericFunctions_setCMWAdress(sDpName, sCMWDeviceName, dsConfigs[UN_CONFIG_PLC_ADDITIONAL_DRIVER_NUM], dsDpe, exceptionInfo);	
													
}

//------------------------------------------------------------------------------------------------------------------------

//@}
	

