/**@name LIBRARY: cietConfig_TT.ctl

@author: Celine VUILLERMOZ (AB-CO-IS)

Creation Date: 11 08 2005

Modification History: 

 	26/03/2007	Pablo: Added crate Level parameter for reset Agent.
	21/03/2007 	Pablo:	Set Config for Reset	Agent

version 1.0
 
External Function : 
	. cietConfig_WFIPOddAgent_setConfig :  set configuration of given DI datapoint
	. cietConfig_WFIPOddAgent_checkConfig : check data before configuration
	
Internal Functions :
	
Purpose: 
This library contains the function used to set Unicos DI configuration.

Usage: Public

PVSS manager usage: Ctrl, NV, NG

Constraints:
	. JCOP framework
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

//Constants



//@{

//------------------------------------------------------------------------------------------------------------------------

// cietConfigWFIPOddAgent_checkConfig
/**
Purpose: check data before configuration

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietConfig_WFIPOddAgent_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	string sType = "WFIPOddAgent";
	
	if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH)
	{
	// 1. Common
		unConfigGenericFunctions_checkParameters(configLine, sType, exceptionInfo);
	// 2. call the device check function
		unConfigGenericFunctions_checkDeviceConfig(configLine, sType, exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","unConfig"+sType+"_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// cietConfigWFIPOddAgent_setConfig
/**
Purpose: set configuration of given WFIPOddAgent datapoint

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietConfig_WFIPOddAgent_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	
// 1. Initialization : checking + dpCreate
	unConfigGenericFunctions_checkAll(dsConfigs, "WFIPOddAgent", exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)		// Good inputs
	{
		sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
		unConfigGenericFunctions_createDp(sDpName, "WFIPOddAgent", exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)	// Creation ok or dp already exists
		{
// 2. Generate device
// 2.1. Parameters
			unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
	// 2.2 call the device check function
			unConfigGenericFunctions_setDeviceConfig(dsConfigs, "WFIPOddAgent", exceptionInfo);
		}		
	}
}

//------------------------------------------------------------------------------------------------------------------------

// CRYOGTW_WFIPOddAgent_checkConfig
/**
Purpose: check data before configuration for DQGTW front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CRYOGTW_WFIPOddAgent_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	string sType = "WFIPOddAgent";

	bool bOk=false;
	if (dynlen(configLine) == (ciet_CONFIG_COMMON_AGENT_LENGTH + UN_CONFIG_COMMON_LENGTH))
		bOk = true;
	else if (dynlen(configLine) == (ciet_CONFIG_COMMON_AGENT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		bOk = true;
		unConfigGenericFunctions_checkArchive(configLine[ciet_CONFIG_COMMON_AGENT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																					configLine[ciet_CONFIG_COMMON_AGENT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																					configLine[ciet_CONFIG_COMMON_AGENT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																					exceptionInfo);
	}
	else
		fwException_raise(exceptionInfo,"ERROR","CRYOGTW_"+sType+"_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");	
		
}

//------------------------------------------------------------------------------------------------------------------------

// CRYOGTW_WFIPOddAgent_setConfig
/**
Purpose:  set configuration of given WFIPOddAgent datapoint for DQGTW front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: win XP.
	. distributed system: yes.
*/

CRYOGTW_WFIPOddAgent_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
		string sDpName, sCMWDeviceName;
	dyn_string dsDpe, dsDescription, dsTemp;
	bool bArchive;
	int iArchiveTimeFilter, iArchiveType, iSmoothProcedure;

	
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

// if archive name are in the config->overwrite panel setting.	
	if(dynlen(dsConfigs) == (ciet_CONFIG_COMMON_AGENT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		if(dsConfigs[ciet_CONFIG_COMMON_AGENT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[ciet_CONFIG_COMMON_AGENT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];

		if(dsConfigs[ciet_CONFIG_COMMON_AGENT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[ciet_CONFIG_COMMON_AGENT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];

		if(dsConfigs[ciet_CONFIG_COMMON_AGENT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[ciet_CONFIG_COMMON_AGENT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
	}
	
		// Set Config for Reset	Agent
		
		if(dsConfigs[ UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_COMMON_AGENT_CRATE_LEVEL] != "")	// Crate Level
		dpSet(sDpName+".Config.crateLevel",dsConfigs[ UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_COMMON_AGENT_CRATE_LEVEL]);		
	
		if(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_COMMON_AGENT_PARTNER] != "")	// Partner
		dpSet(sDpName+".Config.partner",dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_COMMON_AGENT_PARTNER]);
		
		if(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_COMMON_AGENT_OPPOSITE_ODD] != "")	// Opposite Odd
		dpSet(sDpName+".Config.oppositeOdd",dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_COMMON_AGENT_OPPOSITE_ODD]);
		
		if(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_COMMON_AGENT_OPPOSITE_EVEN] != "")	// Opposite Even
		dpSet(sDpName+".Config.oppositeEven",dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_COMMON_AGENT_OPPOSITE_EVEN]);
	
		if(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_COMMON_AGENT_OPPOSITE_NET] != "")	// Opposite Even
		dpSet(sDpName+".Config.oppositeNet",dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_COMMON_AGENT_OPPOSITE_NET]);	

		// set archiving 
	unConfigGenericFunctions_getArchiving(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_COMMON_AGENT_ARCHIVEACTIVE], 
																				bArchive, dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_COMMON_AGENT_ARCHIVETIMEFILTER],	
																				iArchiveType, iSmoothProcedure);
																				
	cietConfigGenericFunctions_setArchiving("WFIPOddAgent", sDpName, bArchive, dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL], 
																											dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG],
																											dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT], 
																											iArchiveType,
																											iSmoothProcedure,
																											dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH +ciet_CONFIG_COMMON_AGENT_ARCHIVETIMEFILTER],
																											exceptionInfo);
// set aliases
	// set alias on only all DPEs below ProcessInput
	//cietConfigGenericDpFunctions_setAlias(sDpName, dsDpe,	dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_ALIAS],
		//																	exceptionInfo);
// set descriptions
/*
	unGenericDpFunctions_getDeviceDefinitionDPEDescription("WFIPEvenAgent", dsDpe, dsDescription);
	cietConfigGenericDpFunctions_setDescription(sDpName, dsDpe, dsDescription, exceptionInfo);
																							
	dsTemp[ciet_PARAMETER_BUS] = dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_WFIPEVENAGENT_BUS];
	dsTemp[ciet_PARAMETER_GTWNAME] = dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_WFIPEVENAGENT_GTWNAME];
	dsTemp[ciet_PARAMETER_AGENTNUMBER]=dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_WFIPEVENAGENT_AGENTNUMBER];																						
	dsTemp[ciet_PARAMETER_AGENT_SAMELEVEL]=unGenericDpFunctions_getAlias(sDpName+".");
	dsTemp[ciet_PARAMETER_AGENT_OPPOSITELEVEL]=dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_WFIPEVENAGENT_OPPOSITELEVEL];
	
// stock agent name and channel number
	dpSetDescription(sDpName+".ProcessOutput",dsTemp[ciet_PARAMETER_BUS]+";"+dsTemp[ciet_PARAMETER_GTWNAME]+";"
	+dsTemp[ciet_PARAMETER_AGENTNUMBER]+";"+dsTemp[ciet_PARAMETER_AGENT_SAMELEVEL]+";"
	+dsTemp[ciet_PARAMETER_AGENT_OPPOSITELEVEL]+";");		
*/


	sCMWDeviceName = dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + ciet_CONFIG_COMMON_AGENT_CMWDEVICE];
	fwDevice_getAddressElements("WFIPOddAgent", dsDpe, exceptionInfo, "");
	
	cietConfigGenericFunctions_setCMWAdress(sDpName, sCMWDeviceName, dsConfigs[UN_CONFIG_PLC_ADDITIONAL_DRIVER_NUM], dsDpe, exceptionInfo);	
																
}

//------------------------------------------------------------------------------------------------------------------------

//@}
	
