/**@name LIBRARY: cietDI.ctl

@author: Celine VUILLERMOZ (AB/CO)

Creation Date: 08/08/2005

Modification History: 
 04/11/11  achiron: UN_FACEPLATE_BUTTON_CONFIG do not exist -> replaced by ciet_FACEPLATE_BUTTON_CONFIG
 04/11/11  achiron: call only cietGenericFunctions_HeaderDisconnect(); in the ciet-DPT-_FaceplateStatusDisconnection()
 02/11/11  achiron: Corrected the call to the function ciet-DTD-_WidgetDisconnect iRes was argument instead of a PlcName, sCompleteAgentName has been change in global variable g_sCompleteAgentName in order tu use it in disconnect
 	09/05/08 	Pablo: 	Button UserAccess adapted to QT.
 	05/11/07 	Pablo: 	Open wfipagent, wfipsegment and crate from popup menu.
 										debug application included.
 	23/08/07 	Pablo: 	Help on handle menu.
 										Disable some option on handle menu when widget is not selected.
 	20/06/07 	Pablo: 	Disable widget animation.
	18/06/07	Pablo:	Adapted to distributed system.
	21/11/06	Pablo:	Reordered widget warnings animations.
	21/11/06	Quique:	Include IOERROR of the agent and open its faceplate included
	17/11/06	Pablo:	ConfigFlag showed on Faceplate ok.
	17/11/06	Quique: New function: cietDI_ObjectListGetValueTime
	13/11/06	Pablo:	Modified IdCard on faceplate to transparent background.
	24/10/06	Pablo:	Modified ButtonSetState & ButtonAnimationCB to allow click on UpdateRawData Button only when there is no updating.
	10/10/06	Pablo:	Modified widgetDisconnection function to visualize right behaviour.
	26/09/06  Pablo:  Change I/O Error letter color on widget from Cyan to Orange.
	25/09/06  Pablo:  Included STATUS GATEWAY DEVICE for Widget Warning Letter Animation
	14/09/06  Pablo:  Position Status (DigitalInput)Animation on DI Faceplate.
	12/09/06  Pablo:  Included I/O Error for Widget Warning Letter Animation
	09/09/06	Quique:	Changed reset action in popup menu (right click)
	06/09/06	Quique:	Added in the faceplate the name of the segment
	01/09/06  Pablo:  Identification Card Configuration code changed.
  31/08/06  Pablo: 	Configuration Flag  Table added.
  31/08/06  Pablo: 	Identification Card Configuration added.
	28/08/06  Pablo:  Added links for popup Menu. Arranging it.
	15/08/06	Quique: Allow button change config
  10/08/06 	Pablo:  Added animation for warning boxes: I/O Error, Reset, BadConfig.
										Changed WFIP Infos display to be done by cietGenericObject_DisplayInt(), instead of cietGenericObject_DisplayInt()
	28/09/05 	Celine: Change the way to get FIP infos fo reset


version 1.0

External Function : 
	. cietDI_FaceplateStatusDisconnection : animate faceplate disconnection
	. cietDI_ButtonUserAccess : User access for Analog buttons
	. cietDI_ButtonSetState : set button state
	. cietDI_WidgetDisconnection : animate widget disconnection
	. cietDI_MenuConfiguration : configuration of popup menu

Internal Functions :
	. cietDI_FaceplateStatusRegisterCB : register callback function
	. cietDI_ButtonRegisterCB : register callback function
	. cietDI_ButtonAnimationCB : animate buttons
	. cietDI_ButtonDisconnection : animate buttons disconnection
	. cietDI_WidgetRegisterCB : register callback function
	. cietDI_WidgetAnimationCB : animate widget
		
Purpose: This library contains Analog object functions.

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. cietDI object (DPType etc.)
	. Global variables and $parameters defined in Analog faceplate and widget
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

// constant declaration
string 	g_sGTWName,	g_sAgentNumber,	g_sAgentBus, g_sAgentChannel, g_sFipPosition;

//---------------------------------------------------------------------------------------------------------------------------------------

// cietDI_ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 3.1
	. operating system: WinXP
	. distributed system: yes.
*/

cietDI_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
	float value;
	string sTime;
	  
	dpGet(sDeviceName+".ProcessInput.Data.digitalInput", value, sDeviceName+".ProcessInput.Data.digitalInput:_online.._stime", sTime);
	dsReturnData[1] = sTime;
	dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(sDeviceName+".ProcessInput.Data.digitalInput"), value) + " " 
							+ dpGetUnit(sDeviceName+".ProcessInput.Data.digitalInput");
}
//---------------------------------------------------------------------------------------------------------------------------------------
//@{

// cietDI_FaceplateStatusRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in cietDI faceplate
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietDI_FaceplateStatusRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, sSystemName, sPlcName;
	int iAction, iRes, iAlertType1;
	dyn_string exceptionInfo;
	bool bRemote;
	
	deviceName = unGenericDpFunctions_getDpName($sDpName);
// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
		unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;
		
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	if(sPlcName != "") {
		if (dpExists(sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;
																				 
	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	
//	DebugN("FaceplateStatusRegisterCB", deviceName,iAction);
	
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
//DebugN("cietDI_FaceplateStatusDisconnection();");
			cietDI_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
		
			g_bDisconnectCB = false;
			if(g_bUnSystemAlarmPlc) {
				iRes = dpConnect("cietDI_FaceplateLockCallBack", deviceName + ".ProcessInput.Data.digitalInput:_lock._alert_hdl._locked");

				g_bCallbackConnected = (iRes >= 0);
			}
			else
				cietDI_FaceplateStatusDisconnection();
			
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
		
			if(g_bUnSystemAlarmPlc) 
				iRes = dpDisconnect("cietDI_FaceplateLockCallBack", deviceName + ".ProcessInput.Data.digitalInput:_lock._alert_hdl._locked");

			if(g_bDisconnectCB)
				cietDI_FaceplateDisconnect(deviceName, iRes);

			g_bCallbackConnected = !(iRes >= 0);
			g_bDisconnectCB = false;
				cietDI_FaceplateStatusDisconnection();
			
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
	}
//---------------------------------------------------------------------------------------------------------------------------------------
// cietDI_FaceplateLockCallBack
/**
Purpose: set up the lock callback function for the faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in PT widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietDI_FaceplateLockCallBack(string sdpe, bool bLockState)
{
	int iRes;
	string deviceName;

	deviceName = unGenericDpFunctions_getDpName(sdpe);
	if(bLockState) { 
		if(g_bDisconnectCB) {
			cietDI_FaceplateDisconnect(deviceName, iRes);
			g_bDisconnectCB = false;
		}
		cietDI_FaceplateStatusDisconnection();
	}
	else { 
		g_bDisconnectCB = true;
		cietDI_FaceplateConnect(deviceName);
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietDI_FaceplateConnect
/**
Purpose: connect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in cietDI faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietDI_FaceplateConnect(string deviceName)
{

	int iAction, iRes, iAlertType1, iAlertType2, iRangeType;
	dyn_string exceptionInfo, dsDeviceNameSplit,dsTemp,dsFIPInfos;
	string sPlcName, sSystemName, sAlias,sDescrip,sAgentName;
//DebugN("cietDI_FaceplateConnect");

// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	
	if(sPlcName != "") {
		if (dpExists(sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;
		
	cietGenericFunctions_getFIPAgentInfos(deviceName, dsFIPInfos, exceptionInfo);	 //********************************************	
	
	if (dynlen(exceptionInfo) <= 0)	
	{	
		g_sGTWName = dsFIPInfos[ciet_PARAMETER_GTWNAME];
		g_sAgentBus = dsFIPInfos[ciet_PARAMETER_BUS]; 
		g_sAgentNumber = dsFIPInfos[ciet_PARAMETER_AGENTNUMBER];
		sAgentName =dsFIPInfos[ciet_PARAMETER_AGENTNAME];
		g_sAgentChannel = dsFIPInfos[ciet_PARAMETER_CHANNELNUMBER];
		
		// Get Agent name to display also ioerror info
		
		g_sCompleteAgentName=_unGenericDpFunctions_dpAliasToName(sSystemName + sAgentName);
		DebugN(g_sCompleteAgentName + " --- " + sSystemName + " --- " + sAgentName + "---" + dsFIPInfos );


	}
		
		rawDatas.appendLines(1);
		rawDatas.sortOnClick(FALSE);
		
		flags.appendLines(1); ////////////////////////////////////////      TABLE DEFINITION      ////////////////////////////
		flags.sortOnClick(FALSE);
		
	if(g_bUnSystemAlarmPlc) {
	
		sAlias = unGenericDpFunctions_getAlias(sAgentName);
		dsDeviceNameSplit=strsplit(sAlias, ".");		

		iRes = dpConnect("cietDI_FaceplateStatusAnimationCB",
										 	deviceName + ".ProcessInput.Data.configFlag", 			
										 	deviceName + ".ProcessInput.Config.sgNumber", 		
						 					deviceName + ".ProcessInput.Config.agNumber", 		
						 					deviceName + ".ProcessInput.Config.chNumber", 	
						 					deviceName + ".ProcessInput.Config.diNumber", 	
							 				deviceName + ".ProcessInput.Data.digitalInput", 
							 				deviceName + ".ProcessInput.Data.sensorStatus",
							 				g_sCompleteAgentName + "ProcessInput.Data.agentStatus",
							 				deviceName + ".ProcessInput.Data.command", 
							 				deviceName + ".ProcessInput.Data.sensorByte[1]",							 				
							 				deviceName + ".ProcessInput.Data.sensorByte[2]",							 				
							 				deviceName + ".ProcessInput.Data.sensorByte[3]",							 				
							 				deviceName + ".ProcessInput.Data.sensorByte[4]",							 				
							 				deviceName + ".ProcessInput.Data.sensorByte[5]",							 				
							 				deviceName + ".ProcessInput.Data.sensorByte[6]",							 				
							 				deviceName + ".ProcessInput.Data.sensorByte[7]",							 				
							 				deviceName + ".ProcessInput.Data.sensorByte[8]",							 											 											 											 											 											 											 											 				
						 					sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".enabled",
					/*						sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".enabled",*/
											deviceName + ".ProcessInput.Data.digitalInput:_online.._invalid");
	}
	else
		cietDI_FaceplateStatusDisconnection();
	g_bCallbackConnected = (iRes >= 0);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietDI_FaceplateDisconnect
/**
Purpose: disconnect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in cietDI faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietDI_FaceplateDisconnect(string deviceName)
{
	int iRes;
	string sPlcName, sSystemName, sAlias, sAgentName;
	dyn_string dsDeviceNameSplit;
	
//	DebugN("FaceplateDisconnect");
	if(g_bUnSystemAlarmPlc) {
// get the PLC name 
		sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
		sSystemName = unGenericDpFunctions_getSystemName(deviceName);

		sAgentName = cietGenericFunctions_getAgentName(deviceName);
		sAlias = unGenericDpFunctions_getAlias(sAgentName);
		dsDeviceNameSplit=strsplit(sAlias, ".");
	
		iRes = dpDisconnect("cietDI_FaceplateStatusAnimationCB",
										 	deviceName + ".ProcessInput.Data.configFlag", 			
										 	deviceName + ".ProcessInput.Config.sgNumber", 		
						 					deviceName + ".ProcessInput.Config.agNumber", 		
						 					deviceName + ".ProcessInput.Config.chNumber", 	
						 					deviceName + ".ProcessInput.Config.diNumber", 	
							 				deviceName + ".ProcessInput.Data.digitalInput", 
							 				deviceName + ".ProcessInput.Data.sensorStatus",
							 				g_sCompleteAgentName + "ProcessInput.Data.agentStatus",
							 				deviceName + ".ProcessInput.Data.command", 
							 				deviceName + ".ProcessInput.Data.sensorByte[1]",							 				
							 				deviceName + ".ProcessInput.Data.sensorByte[2]",							 				
							 				deviceName + ".ProcessInput.Data.sensorByte[3]",							 				
							 				deviceName + ".ProcessInput.Data.sensorByte[4]",							 				
							 				deviceName + ".ProcessInput.Data.sensorByte[5]",							 				
							 				deviceName + ".ProcessInput.Data.sensorByte[6]",							 				
							 				deviceName + ".ProcessInput.Data.sensorByte[7]",							 				
							 				deviceName + ".ProcessInput.Data.sensorByte[8]",							 											 											 											 											 											 											 											 				
						 					sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".enabled",
						/*					sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".enabled",*/
											deviceName + ".ProcessInput.Data.digitalInput:_online.._invalid");
		}									
	g_bCallbackConnected = !(iRes >= 0);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietDI_FaceplateStatusAnimationCB
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in cietDI faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietDI_FaceplateStatusAnimationCB(string sDp_configFlag, int iconfigFlag,
																				string sDp_sgNumber, int isgNumber,									 	
																				string sDp_agNumber, int iagNumber,
																				string sDp_chNumber, int ichNumber,
																				string sDp_diNumber, int idiNumber,																				
																				string sDp_digitalInput, bool bDigitalInput,
																				string sDp_sensorStatus, unsigned uSensorStatus,
																				string sDp_AgentStatus, unsigned uAgentStatus,
																				string sDp_commandIn, unsigned uCommandIn,
																				string sDp_sensorByte1, bit32 bsensorByte1,
																				string sDp_sensorByte2, bit32 bsensorByte2,
																				string sDp_sensorByte3, bit32 bsensorByte3,
																				string sDp_sensorByte4, bit32 bsensorByte4,
																				string sDp_sensorByte5, bit32 bsensorByte5,
																				string sDp_sensorByte6, bit32 bsensorByte6,
																				string sDp_sensorByte7, bit32 bsensorByte7,
																				string sDp_sensorByte8, bit32 bsensorByte8,																																																																																																																																												
																				string sDpSystemIntegrityAlarmValue, int iSystemIntegrityAlarmValue, 
																			 	string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
																			/* 	string sDpSystemAlarmValue, int iSystemAlarmValue,
																			 	string sDpSystemAlarmEnabled, bool bSystemAlarmEnabled,*/
																			 	string sDpInvalid, bool bCodeInvalid)
{

int iAlarmValue,i,iStart;
bool bFaceplateInvalid, bDisable, bIoError, bReset=0, bBadConfig=0,bAgIoError;
string sTemp,temp1,temp2,sDpPLCName,sBusName, sCrate;
dyn_bit32 db32sensorByte;
dyn_string dsFlags = makeDynString((bit32)iconfigFlag);	

	if ((iSystemIntegrityAlarmValue >= CRYO_SYSTEM_INGETRITY_ALARM_LIMIT)/*	||
			(iSystemAlarmValue < CRYO_SYSTEM_INGETRITY_ALARM_LIMIT)*/)
			iAlarmValue=c_unSystemIntegrity_alarm_value_level1;
	else
		iAlarmValue=c_unSystemIntegrity_no_alarm_value;
		
	iStart=strpos(sDpSystemIntegrityAlarmValue, c_cietSystemIntegrity_DS) + strlen(c_cietSystemIntegrity_DS);
	sDpPLCName=substr(sDpSystemIntegrityAlarmValue,  iStart, strpos(sDpSystemIntegrityAlarmValue, ".alarm") - iStart);
	
	cietGenericFunctions_Header(sDpPLCName, iAlarmValue, bSystemIntegrityAlarmEnabled /*&& bSystemAlarmEnabled*/, bCodeInvalid);
	
	bIoError = getBit (uSensorStatus, 0); 
	bReset = getBit (uSensorStatus, 1); 		//1 = reset, 0 = running
	bBadConfig = getBit (uSensorStatus, 2);
	bDisable = getBit (uSensorStatus, 12);
  	
	g_sGTWName = cietGenericFunctions_getGTWName(sDp_sgNumber);
	
	bFaceplateInvalid = false;
	
	sCrate= cietGenericObject_getCrate ($sDpName);
	if(g_bSystemConnected)
		cietGenericObject_DisplayString(sCrate, "CRATE", "unDisplayValue_Parameter", bFaceplateInvalid);	
	
	if(g_bSystemConnected)	
		{
			// Agent io_error
			bAgIoError=getBit(uAgentStatus,0);
			unGenericObject_ColorBoxAnimate ("AG_IOERROR", !bAgIoError, 1, "", bFaceplateInvalid, "cietFaceplate_Warning");
		}
		
//	bIoError = bReset || bAgError;

	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate ("DISABLE", !bDisable, 1, "", bFaceplateInvalid, "cietFaceplate_Warning");
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate ("IO_ERROR", !bIoError, 1, "", bFaceplateInvalid, "cietFaceplate_Warning");
	if(g_bSystemConnected)	
		unGenericObject_ColorBoxAnimate ("RESET", !bReset, 1, "", bFaceplateInvalid, "cietFaceplate_Warning");
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate ("BAD_CONFIG", !bBadConfig, 1, "", bFaceplateInvalid, "cietFaceplate_Warning");
		
	// DebugN("bDigitalInput", +bDigitalInput );		
  if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate ("POSSTATUS", bDigitalInput, 1, "unFaceplate_StatusActive", bFaceplateInvalid, "unFaceplate_Unactive");
		
	if(g_bSystemConnected)			
		cietGenericObject_DisplayString(g_sGTWName, "GTWNAME", "unDisplayValue_Parameter", bFaceplateInvalid);
	if(g_bSystemConnected)			
		cietGenericObject_DisplayInt(iagNumber, "AGENTNUMBER", "unDisplayValue_Parameter", bFaceplateInvalid);	
	if(g_bSystemConnected)				
		cietGenericObject_DisplayInt(ichNumber, "AGENTCHANNEL", "unDisplayValue_Parameter", bFaceplateInvalid);	
	if(g_bSystemConnected)		
		{
			sBusName=cietGenericFuncitons_getSegmentName(sDp_sgNumber);
			// DebugN("sDp_sgNumber="+sDp_sgNumber);
			// DebugN("sBusName="+sBusName);
			cietGenericObject_DisplayString(sBusName, "BUSNAME", "unDisplayValue_Parameter", bFaceplateInvalid);		
			cietGenericObject_DisplayInt(isgNumber, "AGENTBUS", "unDisplayValue_Parameter", bFaceplateInvalid);	
		}
	if(g_bSystemConnected)				
		cietGenericObject_DisplayInt(idiNumber, "FIPPOSITION", "unDisplayValue_Parameter", bFaceplateInvalid);	


// FLAGS

		int b=0;
		bit32 B32Temp;

		B32Temp=iconfigFlag;
    sTemp = B32Temp;  

    int k=1;	
		int j=7;
	  for(i=24;i<=31;i++)
			{
				string sTemp1;
				sTemp1	= substr(sTemp,i,1);	
				if(g_bSystemConnected)		
					cietGenericObject_DisplayToCell(sTemp1, "flags", k-1, "b"+j, "unDisplayValue_Parameter", bFaceplateInvalid);
				j=j-1;	
			}



// RAW DATA	

	db32sensorByte=makeDynChar(bsensorByte1,bsensorByte2,bsensorByte3,bsensorByte4,bsensorByte5,bsensorByte6,
															bsensorByte7,bsensorByte8);

	for(i=1;i<=8;i++)
		{
		string sTemp;
		int j=i-1;
		sTemp = cietGenericObject_toHexa(db32sensorByte[i]);
		if(g_bSystemConnected)		
			rawDatas.cellValueRC(0,"B"+j,  sTemp);
		}
		
// Identify Card Configuration
	
		string sShowIdCard;
		
		sShowIdCard = cietGenericFunctions_getDeviceId ($sDpName);
		
		cietGenericObject_DisplayString(sShowIdCard, "idCard", "unDisplayValue_Parameter", bFaceplateInvalid);
		
	
		
	}

 

//---------------------------------------------------------------------------------------------------------------------------------------

// cietDI_FaceplateStatusDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietDI_FaceplateStatusDisconnection()
{

//DebugN("FaceplateStatusDisconnection");

//	unGenericObject_ColorBoxDisconnect(makeDynString("IO_ERROR","RESET", "POSSTATUS"));

	/*unGenericObject_DisplayValueDisconnect(makeDynString("GTWNAME","AGENTNUMBER","AGENTCHANNEL","AGENTBUS","B8_sensorByte",
	"B7_sensorByte", "B6_sensorByte", "B5_sensorByte", "B4_sensorByte", "B3_sensorByte", "B2_sensorByte", "B1_sensorByte"));*/
	
// 2. animate PLC Connection
//	unLHCInterlockQPS_GenericFunctions_HeaderDisconnect();
  //put in the top textbox the DATA not connected in purple
  cietGenericFunctions_HeaderDisconnect();
}



//---------------------------------------------------------------------------------------------------------------------------------------

// cietDI_ButtonUserAccess
/**
Purpose: User access for DQAMCMB buttons

Parameters: 
	- sDpName, string, input, device name
	- sDpType, string, input, dptype
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietDI_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
	bool operator, expert, admin;
	int i, length;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
        string sOperAction, sExpAction, sAdminAction;
        bool bActionDefined = false;

        unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo,CIET_ACCESS_CONTROL_DOMAIN_NAME);
        if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) ||
           (sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME))
          bActionDefined=true;
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString();
	if (operator)
	{
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
            dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_SELECT, ciet_POPUPMENU_HELP,
                                              ciet_POPUPMENU_WIDGET_SIMULATOR, ciet_POPUPMENU_OPEN_WFIPAGENT, 
                                              ciet_POPUPMENU_OPEN_WFIPSEGMENT, ciet_POPUPMENU_OPEN_CRATE));
	}
	if (expert)
	{
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
	    			dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_SELECT,ciet_FACEPLATE_BUTTON_CONFIG, 
                                             	ciet_FACEPLATE_BUTTON_RESET,ciet_FACEPLATE_BUTTON_RAWDATA, 
                                              ciet_FACEPLATE_BUTTON_DEBUG, ciet_FACEPLATE_BUTTON_DISABLE, 
                                              ciet_POPUPMENU_HELP,ciet_POPUPMENU_WIDGET_SIMULATOR, 
                                              ciet_POPUPMENU_OPEN_WFIPAGENT, ciet_POPUPMENU_OPEN_WFIPSEGMENT, 
                                              ciet_POPUPMENU_OPEN_CRATE));	
	}
        if(admin){
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
            dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_SELECT,ciet_FACEPLATE_BUTTON_CONFIG,
                                               ciet_FACEPLATE_BUTTON_RESET,ciet_FACEPLATE_BUTTON_RAWDATA,
                                                ciet_FACEPLATE_BUTTON_DEBUG, ciet_FACEPLATE_BUTTON_DISABLE,
                                                 ciet_POPUPMENU_HELP,ciet_POPUPMENU_WIDGET_SIMULATOR,
                                                  ciet_POPUPMENU_OPEN_WFIPAGENT, ciet_POPUPMENU_OPEN_WFIPSEGMENT,
                                                   ciet_POPUPMENU_OPEN_CRATE));	
        }  
  
  
  //...................................................
  /*
	bool operator, expert, admin;
	int i, length;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
	
	unGenericButtonFunctionsHMI_getUnicosPriviledges(dbPermissions, exceptionInfo);
	if (dynlen(exceptionInfo) > 0)
		{
		dynClear(dbPermissions);
		}
	while (dynlen(dbPermissions) < 4)
		{
		dynAppend(dbPermissions, false);
		}
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString();
	if (operator)
		{
		dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_SELECT, ciet_POPUPMENU_HELP,ciet_POPUPMENU_WIDGET_SIMULATOR, ciet_POPUPMENU_OPEN_WFIPAGENT, ciet_POPUPMENU_OPEN_WFIPSEGMENT, ciet_POPUPMENU_OPEN_CRATE));
		
		}
	if (expert)
		{
			dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_SELECT,ciet_FACEPLATE_BUTTON_CONFIG, ciet_FACEPLATE_BUTTON_RESET,ciet_FACEPLATE_BUTTON_RAWDATA, ciet_FACEPLATE_BUTTON_DEBUG, ciet_FACEPLATE_BUTTON_DISABLE, ciet_POPUPMENU_HELP,ciet_POPUPMENU_WIDGET_SIMULATOR, ciet_POPUPMENU_OPEN_WFIPAGENT, ciet_POPUPMENU_OPEN_WFIPSEGMENT, ciet_POPUPMENU_OPEN_CRATE));	
		}
	if (admin)
		{
			dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_SELECT,ciet_FACEPLATE_BUTTON_CONFIG, ciet_FACEPLATE_BUTTON_RESET,ciet_FACEPLATE_BUTTON_RAWDATA, ciet_FACEPLATE_BUTTON_DEBUG, ciet_FACEPLATE_BUTTON_DISABLE, ciet_POPUPMENU_HELP,ciet_POPUPMENU_WIDGET_SIMULATOR, ciet_POPUPMENU_OPEN_WFIPAGENT, ciet_POPUPMENU_OPEN_WFIPSEGMENT, ciet_POPUPMENU_OPEN_CRATE));	
		}	*/
}
 

//---------------------------------------------------------------------------------------------------------------------------------------

// cietDI_ButtonRegisterCB
/**
Purpose: register callback function

Parameters: 

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in DQAMCMB faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietDI_ButtonRegisterCB(string sDpSystemConnected, bool bSystemConnected)
{
	string deviceName, sSystemName, sDpType, sFunction;
	int iAction, iRes, iManReg01;
	dyn_string exceptionInfo, dsFunctions, dsUserAccess;
//	DebugN("ButtonRegisterCB");
	
	deviceName = unGenericDpFunctions_getDpName($sDpName);
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	
	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
//		DebugN("ButtonRegisterCB UN_ACTION_DISCONNECT");
			cietDI_ButtonDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
//		DebugN("ButtonRegisterCB UN_ACTION_DPCONNECT");
			sDpType="DI";
			unGenericObject_GetFunctions(sDpType, dsFunctions, exceptionInfo);
			sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];
						 
			if (sFunction != "")
				{
				evalScript(dsUserAccess, "dyn_string main(string deviceNameTemp, string sDpTypeTemp) {" + 
							 "dyn_string dsUserAccessReturn;" +
							 "if (isFunctionDefined(\"" + sFunction  + "\"))" +
							 "    {" +
					 			sFunction  + "(deviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
							 "    }" +
							 "return dsUserAccessReturn; }", makeDynString(), deviceName, sDpType);
						
				if (dynlen(dsUserAccess) == 0)
					fwException_raise(exceptionInfo,"ERROR", "cietDI_ButtonRegisterCB: " + getCatStr("unGeneration","UNKNOWNFUNCTION"), sFunction);
				else
					g_dsUserAccess=dsUserAccess;	
				}
			else
				fwException_raise(exceptionInfo,"ERROR", "cietDI_ButtonRegisterCB: " + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");

			iRes = dpConnect("cietDI_ButtonAnimationCB",
							 				 deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							 				 deviceName + ".statusInformation.selectedManager",
							 				 deviceName + ".ProcessInput.Data.sensorStatus");
			g_bCallbackConnected = (iRes >= 0);
			
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
//			DebugN("ButtonRegisterCB UN_ACTION_DPDISCONNECT_DISCONNECT");
			iRes = dpDisconnect("cietDI_ButtonAnimationCB",
							 						deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							 						deviceName + ".statusInformation.selectedManager",
							 						deviceName + ".ProcessInput.Data.sensorStatus");
			g_bCallbackConnected = !(iRes >= 0);
			cietDI_ButtonDisconnection();
			break;
		case UN_ACTION_NOTHING:
//				DebugN("ButtonRegisterCB UN_ACTION_NOTHING");
		default:
			break;
		}
}


//---------------------------------------------------------------------------------------------------------------------------------------

// cietDI_ButtonAnimationCB
/**
Purpose: animate buttons

Parameters: 
	- dbPermissions, dyn_bool, input, list of permissions, length = 4, order : 1=monitor, 2=operator, 3=expert, 4=admin
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietDI_ButtonAnimationCB(string sDpLock, bool bLocked, 
														 string sDpSelect, string sSelectedManager, 
														 string sDpSensorStatus, unsigned uSensorStatus)
{

bool bStatusRawData, bDisable;
string sDisableText;

	bDisable = getBit (uSensorStatus, 12);
	
	if (bDisable ==1)
			sDisableText=ciet_FACEPLATE_BUTTON_ENABLE;
	else if (bDisable ==0)
			sDisableText=ciet_FACEPLATE_BUTTON_DISABLE;				
			
	setMultiValue("DISABLE_BUTTON.buttonDisable", "text",sDisableText);		
	
	if(StatusRawDataText.text=="true")
				bStatusRawData = 1;
	else if(StatusRawDataText.text=="false")				
				bStatusRawData = 0;
	// DebugN("bStatusRawData=" + bStatusRawData);
//	DebugN("ButtonAnimationCB");
	cietDI_ButtonSetState(unGenericDpFunctions_getDpName(sDpLock), "DI", g_dsUserAccess, makeDynString(bLocked, sSelectedManager, bStatusRawData));
}



//---------------------------------------------------------------------------------------------------------------------------------------

// cietDI_ButtonSetState
/**
Purpose: set button state

Parameters: 
	- deviceName, string, input, device
	- sDpType, string, input, dptype
	- dsUserAccess, dyn_string, input, user access
	- dsData, dyn_string, parameter
		
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietDI_ButtonSetState(string deviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
	bool bSelected, buttonEnabled, bit1, bit2;
	dyn_string dsButtons;
	int i, length, alertType;
	string localManager;
	bool bLocked, bInvalid, ListEnabled, bStatusRawData;
	string sSelectedManager;
//	DebugN("ButtonSetState");

	bLocked = dsData[1];
	sSelectedManager = dsData[2];
  bStatusRawData = dsData[3];
	
// 1. Selection state
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = (localManager == "S");
	
	dsButtons = makeDynString(ciet_FACEPLATE_BUTTON_RESET,ciet_FACEPLATE_BUTTON_RAWDATA,ciet_FACEPLATE_BUTTON_CONFIG, ciet_FACEPLATE_BUTTON_DEBUG, ciet_FACEPLATE_BUTTON_DISABLE); // Each button except "Select"
	length = dynlen(dsButtons);
// 1. Selection state
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = (localManager == "S");
// 2. Button state

	for(i=1;i<=length;i++)
		{
		buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0);	// User access
		switch (dsButtons[i])
			{
			case ciet_FACEPLATE_BUTTON_CONFIG:
				buttonEnabled = buttonEnabled && bSelected;				
				break;
			case ciet_FACEPLATE_BUTTON_RESET:
				buttonEnabled = buttonEnabled && bSelected;				
				break;
			case ciet_FACEPLATE_BUTTON_DEBUG:
				buttonEnabled = buttonEnabled && bSelected;				
				break;
			case ciet_FACEPLATE_BUTTON_DISABLE:
				buttonEnabled = buttonEnabled && bSelected;				
				break;								
			case ciet_FACEPLATE_BUTTON_RAWDATA:
				buttonEnabled = buttonEnabled && bSelected && !bStatusRawData;	
				// DebugN ("Button State = " + bStatusRawData );			
				break;
			default:
				break;
			}
			setValue(ciet_FACEPLATE_BUTTON_PREFIX + dsButtons[i], "enabled", buttonEnabled);
		}
// 3. Button Select state
	unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
}


//---------------------------------------------------------------------------------------------------------------------------------------

// cietDI_UserLoginGetButtonState
/**
Purpose: function called before the button set state when a user logs in. 

Parameters:
	deviceName: string, input, the device name
	dsReturnData: dyn_string, output, the data are returned here

Usage: Internal function

PVSS manager usage:  Ctrl, NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
		-> $sDpName, string, the device DP name
		-> g_sFaceplateButtonType, string, faceplate type
	. PVSS version: 3.0 
	. operating system: NT, W2000 and Linux, but tested only under W2000.
	. distributed system: yes but not tested.
*/

cietDI_UserLoginGetButtonState(string deviceName, string sType, dyn_string &dsReturnData)
{
	
		int iRes;
	string sDpLock, sDpSelectedManager, sSelectedManager,sSystemName;
	bool bLocked;
//	DebugN("UserLoginGetButtonState");
// 2. Get parameters to call the set button function
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);

	sDpLock = deviceName + ".statusInformation.selectedManager:_lock._original._locked";
	sDpSelectedManager = deviceName + ".statusInformation.selectedManager";
	
	if (dpExists(deviceName))
	{
		iRes = dpGet(sDpLock, bLocked, sDpSelectedManager, sSelectedManager);
	}
	else
	{
		iRes = -1;
	}
	if (iRes < 0) // Errors during dpget
	{
		sDpLock = "";
		bLocked = false;
		sDpSelectedManager = "";
		sSelectedManager = "";
	}
	dsReturnData=makeDynString(bLocked, sSelectedManager, "", "", "");
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietDI_ButtonDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietDI_ButtonDisconnection()
{
			   
//	DebugN("ButtonDisconnection");
		unGenericObject_ButtonDisconnect(makeDynString(ciet_FACEPLATE_BUTTON_RESET,ciet_FACEPLATE_BUTTON_RAWDATA,ciet_FACEPLATE_BUTTON_CONFIG,UN_FACEPLATE_BUTTON_SELECT, ciet_FACEPLATE_BUTTON_DEBUG, ciet_FACEPLATE_BUTTON_DISABLE));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietDI_WidgetRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in PT widget
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietDI_WidgetRegisterCB(string sDp, bool bSystemConnected)
{

	string deviceName, deviceType;
	int iAction,iRes;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName;
	bool bRemote;
	
//		DebugN("WidgetRegisterCB");
	deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);

	if (deviceName == "")	// In case of disconnection
		{
		deviceName = g_sDpName;
		}
		// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	
	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;

	if(sPlcName != "") {
		if (dpExists(sSystemName+c_CRYO_FRONTEND_UNSYSTEMALARM+c_cietSystemIntegrity_DS+sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;

	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			cietDI_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:
			g_sDpName = deviceName;
			g_bDisconnectCB = false;
	
		if(g_bUnSystemAlarmPlc) {
					deviceType = dpTypeName(deviceName);
				if(deviceType == "DI")
					iRes = dpConnect("cietDI_WidgetLockCallBack", deviceName + ".ProcessInput.Data.digitalInput:_lock._alert_hdl._locked");
				else{
					g_bUnSystemAlarmPlc = false;
					cietDI_WidgetDisconnection(g_sWidgetType);
				}
	
				g_bCallbackConnected = (iRes >= 0);
			}
			else
					cietDI_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:

			if(g_bUnSystemAlarmPlc) 
				iRes = dpDisconnect("cietDI_WidgetLockCallBack", deviceName + ".ProcessInput.Data.digitalInput:_lock._alert_hdl._locked");

			if(g_bDisconnectCB) {
						cietDI_WidgetDisconnect(g_sDpName, sPlcName);
				}
			g_bCallbackConnected = !(iRes >= 0);
			g_bDisconnectCB = false;

			cietDI_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}

}
//---------------------------------------------------------------------------------------------------------------------------------------
// cietDI_WidgetLockCallBack
/**
Purpose: set up the lock callback function for the widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in PT widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietDI_WidgetLockCallBack(string sdpe, bool bLockState)
{
	int iRes;
	string sPlcName;

	if(bLockState) 
		{ // lock set so disconnect if not already disconnected
		if(g_bDisconnectCB) 
			{
				cietDI_WidgetDisconnect(g_sDpName, sPlcName);
			}
		g_bDisconnectCB = false;
		cietDI_WidgetDisconnection(g_sWidgetType);
		}
	else 
		{ // lock not set so connect if not already connected
		g_bDisconnectCB = true;
		sPlcName = unGenericObject_GetPLCNameFromDpName(sdpe);
		cietDI_WidgetConnect(g_sDpName,sPlcName);
		}
}
//---------------------------------------------------------------------------------------------------------------------------------------

// cietDI_WidgetConnect
/**
Purpose: Connect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Analog faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietDI_WidgetConnect(string deviceName, string sPlcName)
{
	int iRes;
	string sSystemName, sAlias,sAgentName;
	dyn_string dsDeviceNameSplit;
//			DebugN("WidgetConnect");
//  check if the device name is of type "PT" if not like of no PLC. -> not available.
	if(dpTypeName(deviceName) != "DI")
		g_bUnSystemAlarmPlc = false;
	
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	
	if(g_bUnSystemAlarmPlc)
	{
			sAgentName = cietGenericFunctions_getAgentName(deviceName);
		iRes = dpConnect("cietDI_WidgetAnimationCB", 
						 					deviceName + ".ProcessInput.Data.digitalInput",
				 							sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".enabled",
						 					deviceName + ".ProcessInput.Data.sensorStatus",	
											deviceName + ".ST_GTWDEV",						 															
									/*		sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".enabled",*/
											deviceName + ".ProcessInput.Data.digitalInput:_online.._invalid", 
									 		deviceName + ".statusInformation.selectedManager:_lock._original._locked",
						 					deviceName + ".statusInformation.selectedManager");
		g_bCallbackConnected = (iRes >= 0);		
	}else{
		cietDI_WidgetDisconnection(g_sWidgetType);
	}
}
//---------------------------------------------------------------------------------------------------------------------------------------

// cietDI_WidgetDisconnect
/**
Purpose: Disconnect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Analog faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietDI_WidgetDisconnect(string deviceName, string sPlcName)
{
	int iRes;
	string sSystemName, sAlias,sAgentName;
	dyn_string dsDeviceNameSplit;
//				DebugN("WidgetDisconnect");
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);

	sAlias = unGenericDpFunctions_getAlias(deviceName);
	dsDeviceNameSplit=strsplit(sAlias, ".");
		
	if(g_bUnSystemAlarmPlc)
	{
			sAgentName = cietGenericFunctions_getAgentName(deviceName);
		sAlias = unGenericDpFunctions_getAlias(sAgentName);
		dsDeviceNameSplit=strsplit(sAlias, ".");
		
		iRes = dpDisconnect("cietDI_WidgetAnimationCB", 
						 					deviceName + ".ProcessInput.Data.digitalInput",
			 								sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".enabled",
						 					deviceName + ".ProcessInput.Data.sensorStatus",			
											deviceName + ".ST_GTWDEV",						 													
										/*	sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".enabled",*/
											deviceName + ".ProcessInput.Data.digitalInput:_online.._invalid", 
									 		deviceName + ".statusInformation.selectedManager:_lock._original._locked",
						 					deviceName + ".statusInformation.selectedManager");
		
		g_bCallbackConnected = (iRes >= 0);
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietDI_WidgetAnimationCB
/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Analog widget
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietDI_WidgetAnimationCB(string sDp_digitalInput, bool bDigitalInput,
											 string sDpSystemIntegrityAlarmValue, int iSystemIntegrityAlarmValue, 
											 string sDpSystemIntegrityAlarmEnabled, bool bSystemIntegrityAlarmEnabled,
											 string sDp_sensorStatus, unsigned usensorStatus,	
											 string sDp_ST_GTWDEV, bool bST_GTWDEV,
										/*	 string sDpSystemAlarmValue, int iSystemAlarmValue,
											 string sDpSystemAlarmEnabled, bool bSystemAlarmEnabled,*/
											 string sDpCodeInvalid, bool bCodeInvalid,
											 string sDpLocked, bool bLocked,
											 string sDpSelectedManager, string sSelectedManager)
{
  string warningLetter, warningColor, alarmLetter, alarmColor, controlLetter, controlColor;
	string body1Color, body1BackColor, selectColor;
	bool lockVisible, warningVisible;
	
	
// WARNINGS
		
	if (getBit(usensorStatus, 12)==1)			// Disable
		{
			body1Color = "cietWidget_Disabled";		
			body1BackColor = "unWidget_Background";	
				
			warningLetter = ciet_WIDGET_TEXT_WARNING_DISABLE;
			warningColor = "cietWidget_Warning";	
			warningVisible=true;
										
			if (getBit(usensorStatus, 0)==1)			// I/O Error
				{
					warningLetter = UN_WIDGET_TEXT_WARNING_ERROR + ciet_WIDGET_TEXT_WARNING_DISABLE;
					warningColor = "cietWidget_Warning";	
					warningVisible=true;
				}
		}
	else
		{
				if (bCodeInvalid)
					{
					warningLetter = UN_WIDGET_TEXT_INVALID;
					warningVisible=true;
					warningColor = "unDataNotValid";
					body1Color = "unDataNotValid";
					if(!bDigitalInput)
						body1BackColor = "unWidget_Background";
					else
						body1BackColor = "unDataNotValid";		
					}
				else
					{
					if(!bDigitalInput)
						{
							body1Color = "unWidget_ControlStateAuto";				// Auto mode
							body1BackColor = "unWidget_Background";
				
							warningLetter = UN_WIDGET_TEXT_OLD_DATA;
							warningColor = "unDataNotValid";
							warningVisible=true;
				
						}
					else
						{
							body1Color = "unWidget_ControlStateAuto";				// Auto mode
							body1BackColor = "unWidget_ControlStateAuto";
						}
					if (getBit(usensorStatus, 0)==1)			// I/O Error
						{
							warningLetter = UN_WIDGET_TEXT_WARNING_ERROR;
							warningColor = "cietWidget_Warning";	
							warningVisible=true;
						}
					else																	// OK
						{			
		  	 			warningVisible=false;
							body1Color = "unDisplayValue_Status";	    	 			
						}  	 			
					if ( (!bSystemIntegrityAlarmEnabled) || (iSystemIntegrityAlarmValue >= CRYO_SYSTEM_INGETRITY_ALARM_LIMIT)) //Old Data
						{
							warningLetter = UN_WIDGET_TEXT_OLD_DATA;
							warningColor = "unDataNotValid";
		  				warningVisible=true;			
						}
						
					if (bST_GTWDEV==0)			// Status Gateway Device 
			   		{
				 			warningLetter= ciet_WIDGET_TEXT_STATUS_GATEWAY_DEVICE;  	   	 
		  	 			warningColor="cietWidget_Warning";
		  	 			warningVisible=true; 
			   		}
				}
		}		
				
	unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, selectColor, lockVisible);
//DebugN( body1Color, body1BackColor, warningLetter, warningColor, alarmLetter, alarmColor, controlLetter, selectColor, lockVisible);		
	if(g_bSystemConnected)
		setMultiValue("Body1", "foreCol", body1Color, 
									"Body1", "fill", "[solid]",
									"Body1", "backCol", body1BackColor,
		      				"WarningText", "visible", warningVisible,									
				  				"WarningText", "text", warningLetter,
				  				"WarningText", "foreCol", warningColor,
				  				"AlarmText", "text", alarmLetter, 
				  				"AlarmText", "foreCol", alarmColor,
				  				"ControlStateText", "text", controlLetter, 
				  				"ControlStateText", "foreCol", controlColor,
				  				"SelectArea", "foreCol", selectColor, 
				  				"LockBmp", "visible", lockVisible,
				  				"WidgetArea", "visible", true);		
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietDI_WidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
	- sWidgetType
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietDI_WidgetDisconnection(string sWidgetType)
{
	
// DebugN("WidgetDisconnection DI");
							
	setMultiValue(	"Body1", "foreCol", "unDataNoAccess",
			"Body1", "backCol", "unWidget_Background",
			"WarningText", "text", "",
			"AlarmText", "text", "",
			"ControlStateText", "text", "",						
			"LockBmp", "visible", false,
			"WidgetArea", "visible", false);																
								
}								


//@}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietDI_PanelWidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
	- sWidgetType
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietDI_PanelWidgetDisconnection(string sWidgetType)
{
	string warningText;
//DebugN("PanelWidgetDisconnection");
	warningText = UN_WIDGET_TEXT_INVALID;

	setMultiValue("Body1", "text", "",
								"WarningText", "text", warningText,
								"LockBmp", "visible", false);	
}								

//------------------------------------------------------------------------------------------------------------------------
//@}

// cietDI_MenuConfiguration
/**
Purpose: configuration of popup menu

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsAccessOk, dyn_string, input, authorized action
	- menuList, dyn_string, output, menu to pop-up
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 3.0 
	. distributed system: yes.
*/

cietDI_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{

dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, ciet_POPUPMENU_RESET,ciet_POPUPMENU_CONFIG,
 UN_POPUPMENU_FACEPLATE_TEXT, UN_POPUPMENU_TREND_TEXT, ciet_POPUPMENU_DEBUG, ciet_POPUPMENU_DISABLE, ciet_POPUPMENU_HELP, 
 ciet_POPUPMENU_WIDGET_SIMULATOR, ciet_POPUPMENU_OPEN_WFIPAGENT, ciet_POPUPMENU_OPEN_WFIPSEGMENT, ciet_POPUPMENU_OPEN_CRATE);
int granted;
								
string localManager, sSelectedManager;
bool bSelected, bLocked;

int iUnicosPopupMenuMin = UN_POPUPMENU_MIN;

// Selection state

	dpGet(sDpName + ".statusInformation.selectedManager:_lock._original._locked", bLocked);
	dpGet(sDpName + ".statusInformation.selectedManager",sSelectedManager);
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = (localManager == "S");			
	
					 
// Select
	unGenericObject_addSelectToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	
	dynAppend(menuList, "SEPARATOR");
	
// 1. Reset
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_RESET) > 0)
	{
		granted = 0;
		if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_RESET) > 0) && bSelected)
		{
			granted = 1;
		}
		dynAppend(menuList,"PUSH_BUTTON, Reset, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_RESET)+", " + granted);
	}
	
// 2. Config
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_CONFIG) > 0)
		{
			granted = 0;
			if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_CONFIG) > 0) && bSelected)
				{
					granted = 1;
				}

			dynAppend(menuList,"PUSH_BUTTON, WFip Config, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_CONFIG)+", " + granted);
		}

// 3. Disable
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_DISABLE) > 0)
		{
			granted = 0;
			if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_DISABLE) > 0) && bSelected)
				{
					granted = 1;
				}
			dynAppend(menuList,"PUSH_BUTTON, Enable/Disable, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_DISABLE)+", " + granted);
		}
		
// 4. Debug
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_DEBUG) > 0)
		{
			granted = 0;
			if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_DEBUG) > 0) && bSelected)
				{
					granted = 1;
				}
			dynAppend(menuList,"PUSH_BUTTON, Debug, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_DEBUG)+", " + granted);
		}			
	
	if (dynlen(menuList) > 0)
		{
			if (menuList[dynlen(menuList)] != "SEPARATOR")
			{
				dynAppend(menuList, "SEPARATOR");
			}
		}

// Open Agent
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_OPEN_WFIPAGENT) > 0)
		{
			granted = 0;
			if (dynContains(dsAccessOk, ciet_POPUPMENU_OPEN_WFIPAGENT) > 0)
				{
					granted = 1;
				}
			dynAppend(menuList,"PUSH_BUTTON, WFIPAgent, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_OPEN_WFIPAGENT)+", " + granted);
		}
		
// Open Segment	
if (dynContains(dsMenuConfig, ciet_POPUPMENU_OPEN_WFIPSEGMENT) > 0)
{
	granted = 0;
	if (dynContains(dsAccessOk, ciet_POPUPMENU_OPEN_WFIPSEGMENT) > 0)
		{
			granted = 1;
		}
	dynAppend(menuList,"PUSH_BUTTON, WFIPSegment, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_OPEN_WFIPSEGMENT)+", " + granted);
}
	
// Open Crate	
if (dynContains(dsMenuConfig, ciet_POPUPMENU_OPEN_CRATE) > 0)
{
	granted = 0;
	if (dynContains(dsAccessOk, ciet_POPUPMENU_OPEN_CRATE) > 0)
		{
			granted = 1;
		}
	dynAppend(menuList,"PUSH_BUTTON, Crate, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_OPEN_CRATE)+", " + granted);
}
	
// Default Unicos action
	unGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	if (dynlen(menuList) > 0)
	{
		if (menuList[dynlen(menuList)] != "SEPARATOR")
		{
			dynAppend(menuList, "SEPARATOR");
		}
	}

// 13. Help

	if (dynlen(menuList) > 0)
		{
			if (menuList[dynlen(menuList)] != "SEPARATOR")
			{
				dynAppend(menuList, "SEPARATOR");
			}
		}
		
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_HELP) > 0)
		{
			granted = 0;
			if (dynContains(dsAccessOk, ciet_POPUPMENU_HELP) > 0)
				{
					granted = 1;
				}
			dynAppend(menuList,"PUSH_BUTTON, Help, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_HELP)+", " + granted);
		}

	if (dynContains(dsMenuConfig, ciet_POPUPMENU_WIDGET_SIMULATOR) > 0)
		{
			granted = 0;
			if (dynContains(dsAccessOk, ciet_POPUPMENU_WIDGET_SIMULATOR) > 0)
				{
					granted = 1;
				}
			dynAppend(menuList,"PUSH_BUTTON, Widget Simulator, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_WIDGET_SIMULATOR)+", " + granted);
		}	
	
// Trend Action
	unGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);


		
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietDI_HandleMenu
/**
Purpose: handle the answer of the popup menu

Parameters:
	- deviceName, string, input, the device dp name
	- sDpType, string, input, the device dp type
	- menuList, dyn_string, input, list of requested action
	- menuAnswer, int, input, selected action from the menuList
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 3.0 
	. distributed system: yes.
*/
cietDI_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
{

dyn_string exceptionInfo,dsTemp, dsFIPInfos;
string descripName,sDpAgentName,sDpGTWName;
string sAgentName,sBusName;
bool bRes;
int iAgentNumber, iRes;
int iUnicosPopupMenuMin = UN_POPUPMENU_MIN;

//DebugN("HandleMenu", deviceName, sDpType, menuList, menuAnswer);
//	unGenericObject_HandleUnicosMenu(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);

// handle the select
	unGenericObject_handleSelect(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);
// handle the UNICOS default (open faceplate, diagnostic and info)
	unGenericObject_handleDefaultUnicos(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);
// handle the ciet device actions

	switch (menuAnswer-iUnicosPopupMenuMin)
	{
		case ciet_POPUPMENU_ORDER_RESET:					// Reset
			cietGenericObject_resetDevice(deviceName, exceptionInfo);
			break;
		
		case ciet_POPUPMENU_ORDER_CONFIG:					// set confi`g
			//DebugN("Right - Click Set Config");
			

   		 /// open Set Config Panel			
			// DebugN("Right click: WFip COnfig");
			ChildPanelOnModal("vision/cietDevices/General/cietSetConfigDI.pnl",
						  "WFip Conf: " + unGenericDpFunctions_getAlias(deviceName),
						  makeDynString("$sDpName:"+deviceName),0, 0);			
		break;

		case ciet_POPUPMENU_ORDER_DISABLE:					// Disable
			//DebugN("Right - Click Set Config");
			

   		 /// open Set Config Panel			
			// DebugN("Right click: WFip COnfig");
			ChildPanelOnModal("vision/cietDevices/General/cietDisable.pnl",
						  "Disable: " + unGenericDpFunctions_getAlias(deviceName),
						  makeDynString("$sDpName:"+deviceName),0, 0);			
			
		break;
		
		case ciet_POPUPMENU_ORDER_DEBUG:					// Debug
				 cietGenericObject_OpenDebug(deviceName);
		break;		

		case ciet_POPUPMENU_ORDER_OPEN_WFIPAGENT:							// Open Agent
					sAgentName=unGenericDpFunctions_getSystemName(deviceName) + cietGenericFunctions_getAgentName(deviceName);
				 unGenericObject_OpenFaceplate(unGenericDpFunctions_getWidgetDpName(sAgentName), exceptionInfo);
		break;	
		
		case ciet_POPUPMENU_ORDER_OPEN_WFIPSEGMENT:					// Open Segment
					sBusName=unGenericDpFunctions_getSystemName(deviceName) + cietGenericFuncitons_getSegmentName(deviceName);
					unGenericObject_OpenFaceplate(unGenericDpFunctions_getWidgetDpName(sBusName), exceptionInfo);
		break;	

		case ciet_POPUPMENU_ORDER_OPEN_CRATE:										// Open Crate
					cietGenericObject_openCratePanel (deviceName);
		break;	
			
		case ciet_POPUPMENU_ORDER_HELP:												// Help
					cietGenericObject_OpenHtmlFile("Help.html");
			break;			
		
		case ciet_POPUPMENU_ORDER_WIDGET_SIMULATOR:					// Widget Simulator
			ChildPanelOnRelativ("vision/cietDevices/General/cietWidgetSimulator.pnl",
			"Widget Simulator","",0, 0);			  
		break;		
		
		default:
		break;
	}
// handle the trend action
	unGenericObject_handleTrendAction(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);

	if (dynlen(exceptionInfo) > 0)
	{
		unSendMessage_toExpertException("Right Click",exceptionInfo);
	}
}
