/**@name LIBRARY: cietEH.ctl

@author: Celine VUILLERMOZ (AB/CO)

Creation Date: 08/08/2005

Modification History: 
 06/02/13  achiron: add animation for the Forced Production Raw Data. CIET-107
 04/11/11  achiron: remove the argument "Body1" in the function call setMultieValue() in the function cietEH_WidgetDisconnection 
 04/11/11  achiron: call only cietGenericFunctions_HeaderDisconnect(); in the ciet-DPT-_FaceplateStatusDisconnection()
 02/11/11  achiron: Corrected the call to the function ciet-DTD-_WidgetDisconnect iRes was argument instead of a PlcName, sCompleteAgentName has been change in global variable g_sCompleteAgentName in order tu use it in disconnect 
 07/01/09  Quique: New calculation for the Typical values + visualization of engineering values (mV) of Shunt, sensor and TH.
 	09/05/08 	Pablo: 	Button UserAccess adapted to QT.
 	09/11/07 	Pablo: 	Measurements animation on connection and disconnection.
 	05/11/07 	Pablo: 	Open wfipagent, wfipsegment and crate from popup menu.
 										debug application included.
 	23/08/07 	Pablo: 	Help on handle menu.
 										Disable some option on handle menu when widget is not selected.
 	20/06/07 Pablo: 	Disable widget animation.
	18/06/07	Pablo:	Adapted to distributed system.
	16/04/06  Pablo:	Added OverTemperature animation on Widget.
	21/02/06  Pablo:	Added curveId dp animation on faceplate.
	16/02/06  Pablo:	Added OverTemperature box in Faceplate.
	15/02/06  Pablo:	Changed typical value from box to text.
	14/02/06  Pablo:	Added typical value on references values.
	05/02/06  Pablo:	Added full scale value on references values.
	18/12/06	Pablo:	Changed warnings priority order on widget animation.
	07/12/06	Quique	Modify AG_ERROR with new AG_IOERROR
	27/11/06	Quique:	Agent IOerror info and acess to the faceplate included
	17/11/06	Quique: Include new function cietEH_ObjectListGetValueTime to visualize the object list
	13/11/06	Pablo:	Modified CalibrationCurveId on faceplate to transparent background.
	01/11/06	Pablo:	Added Calibration Curve ID on Faceplate.
	24/10/06	Pablo:	Modified ButtonSetState & ButtonAnimationCB to allow click on UpdateRawData Button only when there is no updating.
	10/10/06	Pablo:	Modified widgetDisconnection function to visualize right behaviour.
	09/10/06	Quique: Included constants for new types
	04/10/06	Quique: Corrected reading of raw data variable 1
	02/10/06	Quique: Corrected Get calibration call ($param name changed)
	26/09/06  Pablo:  Change I/O Error letter color on widget from Cyan to Orange.
	25/09/06	Pablo:  Added Indicator for Status Gateway Device on Widget.
	25/09/06	Quique: Visualize right format of the widget powerCalc/Setpoint
										RegulationLetter deleted
	15/09/06	Quique:	Correct raw data visualization
	14/09/06  Pablo:  Added Card Protection Animation on Widget and arranging on faceplate.
  12/09/06 	Pablo: 	Added Widget Animation.
  09/09/06	Quique	Change reset call in popup menu
	06/09/06	Quique	Added in the faceplate the name of the segment
  04/09/06  Pablo:	Added ForcedMode, AutoMode and SetPoint.
	28/08/06  Pablo: 	Added links for popup Menu. Arranging it.
  23/08/06	Quique:	New function included (cietEH_displayValueCyclePeriod) to visualize in the faceplate.
	22/08/06  Pablo:	Added control for DeadBands button.
  10/08/06  Pablo:  Added animation for warning boxes: I/O Error, Reset, BadConfig, Forced Mode and Card Protection.
  									Changed WFIP Infos display to be done by cietGenericObject_DisplayInt(), instead of cietGenericObject_DisplayInt()
  29/09/05  Celine: Change the way to get FIP infos fo reset

version 1.0

External Function : 
	. cietEH_FaceplateStatusDisconnection : animate faceplate disconnection
	. cietEH_ButtonUserAccess : User access for EH buttons
	. cietEH_ButtonSetState : set button state
	. cietEH_WidgetDisconnection : animate widget disconnection
	. cietEH_MenuConfiguration : configuration of popup menu

Internal Functions :
	. cietEH_FaceplateStatusRegisterCB : register callback function
	. cietEH_ButtonRegisterCB : register callback function
	. cietEH_ButtonAnimationCB : animate buttons
	. cietEH_ButtonDisconnection : animate buttons disconnection
	. cietEH_WidgetRegisterCB : register callback function
	. cietEH_WidgetAnimationCB : animate widget
	
	. cietEH_displayValueCyclePeriod : visualize right string in cyclePeriod
		
Purpose: This library contains EH object functions.

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. cietPT object (DPType etc.)
	. Global variables and $parameters defined in EH faceplate and widget
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

// constant declaration


//---------------------------------------------------------------------------------------------------------------------------------------

// cietEH_ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 3.1
	. operating system: WinXP
	. distributed system: yes.
*/
cietEH_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
	float value;
	string sTime;
	  
	dpGet(sDeviceName+".ProcessInput.Data.powerCalc", value, sDeviceName+".ProcessInput.Data.powerCalc:_online.._stime", sTime);
	dsReturnData[1] = sTime;
	dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(sDeviceName+".ProcessInput.Data.powerCalc"), value) + " " 
							+ dpGetUnit(sDeviceName+".ProcessInput.Data.powerCalc");
}

//---------------------------------------------------------------------------------------------------------------------------------------
//@{
// 
/**
Purpose: Displays the right string value using predified cyclePeriod values

Parameters:
	input:
		- iValue: value to visualize
		- sShape: place to visualize 
		- sForeCol: color animation
		- bInvalid: validity
Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: XP.
	. distributed system: yes.
*/
cietEH_displayValueCyclePeriod (int iValue, string sShape, string sForeCol, bool bInvalid)
{
	string formattedValue, sColor;
	
	 if (iValue==ciet_SETCONSTANTS_CYCLEPERIOD_1_25s) formattedValue="1.25";
	 else if (iValue==ciet_SETCONSTANTS_CYCLEPERIOD_2_5s) formattedValue="2.5";
	 else if (iValue==ciet_SETCONSTANTS_CYCLEPERIOD_5s) formattedValue="5";
	 else if (iValue==ciet_SETCONSTANTS_CYCLEPERIOD_10s) formattedValue="10";	 
	   
	sColor = sForeCol;
	if (bInvalid)
		{
		sColor = "unDataNotValid";
		}
	setMultiValue(sShape + ".display", "foreCol", sColor, sShape + ".display", "text", formattedValue);
}
//---------------------------------------------------------------------------------------------------------------------------------------
//@{

// cietEH_FaceplateStatusRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in cietPT faceplate
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietEH_FaceplateStatusRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, sSystemName, sPlcName;
	int iAction, iRes, iAlertType1;
	dyn_string exceptionInfo;
	bool bRemote;
	
	deviceName = unGenericDpFunctions_getDpName($sDpName);
// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	if(sPlcName != "") {
		if (dpExists(sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;
		
		unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;
																					 
	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	
//	DebugN("FaceplateStatusRegisterCB", deviceName,iAction);
	
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
//DebugN("cietEH_FaceplateStatusDisconnection();");
			cietEH_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
		
			g_bDisconnectCB = false;
			if(g_bUnSystemAlarmPlc) {
				iRes = dpConnect("cietEH_FaceplateLockCallBack", deviceName + ".ProcessInput.Data.powerCalc:_lock._alert_hdl._locked");

				g_bCallbackConnected = (iRes >= 0);
			}
			else
				cietEH_FaceplateStatusDisconnection();
			
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
		
			if(g_bUnSystemAlarmPlc) 
				iRes = dpDisconnect("cietEH_FaceplateLockCallBack", deviceName + ".ProcessInput.Data.powerCalc:_lock._alert_hdl._locked");

			if(g_bDisconnectCB)
				cietEH_FaceplateDisconnect(deviceName, sPlcName);

			g_bCallbackConnected = !(iRes >= 0);
			g_bDisconnectCB = false;
				cietEH_FaceplateStatusDisconnection();
			
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
	}
//---------------------------------------------------------------------------------------------------------------------------------------
// cietEH_FaceplateLockCallBack
/**
Purpose: set up the lock callback function for the faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in PT widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietEH_FaceplateLockCallBack(string sdpe, bool bLockState)
{
	int iRes;
	string deviceName;
  string sPlcName;

	deviceName = unGenericDpFunctions_getDpName(sdpe);
	if(bLockState) { 
		if(g_bDisconnectCB) {
			cietEH_FaceplateDisconnect(deviceName, sPlcName);
			g_bDisconnectCB = false;
		}
		cietEH_FaceplateStatusDisconnection();
	}
	else { 
		g_bDisconnectCB = true;
		cietEH_FaceplateConnect(deviceName);
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietEH_FaceplateConnect
/**
Purpose: connect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in cietPT faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietEH_FaceplateConnect(string deviceName)
{

	int iRes;
	dyn_string exceptionInfo, dsFIPInfos, dsDeviceNameSplit;
	string sPlcName, sSystemName, sAliasAgent, sAgentName,sDpAgentName;

// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	
	if(sPlcName != "") {
		if (dpExists(sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;

		g_sHeatVolMeasFormat 		= dpGetFormat(deviceName + ".ProcessInput.Data.heatVolMeas");
		g_sHeatVolMeasUnit 			= dpGetUnit(deviceName + ".ProcessInput.Data.heatVolMeas");
		g_sHeatRefVolMeasFormat 		= dpGetFormat(deviceName + ".ProcessInput.Data.heatRefVolMeas");
		g_sHeatRefVolMeasUnit				= dpGetUnit(deviceName + ".ProcessInput.Data.heatRefVolMeas");							
		g_sShuntVolMeasFormat 		= dpGetFormat(deviceName + ".ProcessInput.Data.shuntVolMeas");
		g_sShuntVolMeasUnit 			= dpGetUnit(deviceName + ".ProcessInput.Data.shuntVolMeas");
		g_sShuntRefVolMeasFormat 		= dpGetFormat(deviceName + ".ProcessInput.Data.shuntRefVolMeas");
		g_sShuntRefVolMeasUnit				= dpGetUnit(deviceName + ".ProcessInput.Data.shuntRefVolMeas");							
		g_sThVolMeasFormat 		= dpGetFormat(deviceName + ".ProcessInput.Data.thVolMeas");
		g_sThVolMeasUnit 			= dpGetUnit(deviceName + ".ProcessInput.Data.thVolMeas");
		g_sThRefVolMeasFormat 		= dpGetFormat(deviceName + ".ProcessInput.Data.thRefVolMeas");
		g_sThRefVolMeasUnit				= dpGetUnit(deviceName + ".ProcessInput.Data.thRefVolMeas");											
		g_sSuppliedVoltageCalcFormat 		= dpGetFormat(deviceName + ".ProcessInput.Data.suppliedVoltageCalc");
		g_sSuppliedVoltageCalcUnit  		= dpGetUnit(deviceName + ".ProcessInput.Data.suppliedVoltageCalc");		
		g_sSuppliedCurrentCalcFormat 		= dpGetFormat(deviceName + ".ProcessInput.Data.suppliedCurrentCalc");
		g_sSuppliedCurrentCalcUnit  		= dpGetUnit(deviceName + ".ProcessInput.Data.suppliedCurrentCalc");		
		g_sThCoupleVoltageCalcFormat 		= dpGetFormat(deviceName + ".ProcessInput.Data.thCoupleVoltageCalc");
		g_sThCoupleVoltageCalcUnit  		= dpGetUnit(deviceName + ".ProcessInput.Data.thCoupleVoltageCalc");		
		g_sThCoupleTemperatureCalcFormat 		= dpGetFormat(deviceName + ".ProcessInput.Data.thCoupleTemperatureCalc");
		g_sThCoupleTemperatureCalcUnit  		= dpGetUnit(deviceName + ".ProcessInput.Data.thCoupleTemperatureCalc");	
		g_sThCoupleOffsetFormat=dpGetFormat(deviceName + ".ProcessInput.Data.thCoupleOffset");
		g_sThCoupleOffsetUnit=dpGetUnit(deviceName + ".ProcessInput.Data.thCoupleOffset");	
		g_sPowerCalcFormat 		= dpGetFormat(deviceName + ".ProcessInput.Data.powerCalc");
		g_sPowerCalcUnit  		= dpGetUnit(deviceName + ".ProcessInput.Data.powerCalc");		
		g_sheaterResistanceFormat = dpGetFormat(deviceName + ".ProcessInput.Data.heaterResistance");
		g_sheaterResistanceUnit = dpGetUnit(deviceName + ".ProcessInput.Data.heaterResistance");		
		
		g_sSetPointFormat 	=dpGetFormat(deviceName + ".ProcessInput.Data.setPoint");
		g_sSetPointUnit			=dpGetUnit(deviceName + ".ProcessInput.Data.setPoint");
		g_sForcedSetPointFormat	=dpGetFormat(deviceName + ".ProcessInput.Data.forcedSetPoint");
		g_sForcedSetPointUnit		=dpGetUnit(deviceName + ".ProcessInput.Data.forcedSetPoint");
		
		g_sThresholdFormat 	=dpGetFormat(deviceName + ".ProcessInput.Data.threshold");
		g_sThresholdUnit 	=dpGetUnit(deviceName + ".ProcessInput.Data.threshold");

//GTWName
	cietGenericFunctions_getFIPAgentInfos(deviceName, dsFIPInfos, exceptionInfo);		
	
	if (dynlen(exceptionInfo) <= 0)	
	{	
		g_sGTWName = dsFIPInfos[ciet_PARAMETER_GTWNAME];
		g_sAgentBus = dsFIPInfos[ciet_PARAMETER_BUS]; 
		g_sAgentNumber = dsFIPInfos[ciet_PARAMETER_AGENTNUMBER];
		sAgentName =dsFIPInfos[ciet_PARAMETER_AGENTNAME];
		g_sAgentChannel = dsFIPInfos[ciet_PARAMETER_CHANNELNUMBER];
		
		// Get Agent name to display also ioerror info
		g_sCompleteAgentName=unGenericDpFunctions_getWidgetDpName(sSystemName +sAgentName);

	}
		rawDatas.appendLines(3);
		rawDatas.sortOnClick(FALSE);
		 
	if(g_bUnSystemAlarmPlc) {
	/*	sAliasAgent = unGenericDpFunctions_getAlias(sAgentName);
		dsDeviceNameSplit=strsplit(sAliasAgent, ".");*/
		
		iRes = dpConnect("cietEH_FaceplateStatusAnimationCB",
								 					deviceName + ".ProcessInput.Config.sgNumber", 		
						 					deviceName + ".ProcessInput.Config.agNumber", 		
						 					deviceName + ".ProcessInput.Config.chNumber", 	
							 				deviceName + ".ProcessInput.Data.heatVolMeas", 
							 				deviceName + ".ProcessInput.Data.heatRefVolMeas",
							 				deviceName + ".ProcessInput.Data.shuntVolMeas", 
							 				deviceName + ".ProcessInput.Data.shuntRefVolMeas",							 				
							 				deviceName + ".ProcessInput.Data.thVolMeas", 
							 				deviceName + ".ProcessInput.Data.thRefVolMeas",							 				
						 					deviceName + ".ProcessInput.Data.hysteresis",
						 					deviceName + ".ProcessInput.Data.cyclePeriod",
						 					deviceName + ".ProcessInput.Data.heaterType",
						 					deviceName + ".ProcessInput.Data.threshold",
						 					deviceName + ".ProcessInput.Data.setPoint",
						 					deviceName + ".ProcessInput.Data.forcedSetPoint",
						 					deviceName + ".ProcessInput.Data.sensorStatus",
						 					g_sCompleteAgentName + ".ProcessInput.Data.agentStatus",
						 					deviceName + ".ProcessInput.Data.command",
						 					deviceName + ".ProcessInput.Data.suppliedVoltageCalc",
						 					deviceName + ".ProcessInput.Data.suppliedCurrentCalc",
						 					deviceName + ".ProcessInput.Data.thCoupleVoltageCalc",									 					
						 					deviceName + ".ProcessInput.Data.thCoupleTemperatureCalc",									 					
						 					deviceName + ".ProcessInput.Data.powerCalc",						 					
						 					deviceName + ".ProcessInput.Data.thCoupleOffset",
						 					deviceName + ".ProcessInput.Data.heaterResistance",
				 							deviceName + ".ProcessInput.Curve.curveId",						 											 					
				 							deviceName + ".ProcessInput.Data.prodVariable1[1]",						 					
				 							deviceName + ".ProcessInput.Data.prodVariable1[2]",						 									 							
						 					deviceName + ".ProcessInput.Data.consVariable2[1]",
						 					deviceName + ".ProcessInput.Data.consVariable2[2]",
						 					deviceName + ".ProcessInput.Data.consVariable2[3]",
						 					deviceName + ".ProcessInput.Data.consVariable2[4]",
						 					deviceName + ".ProcessInput.Data.consVariable2[5]",
						 					deviceName + ".ProcessInput.Data.consVariable2[6]",
						 					deviceName + ".ProcessInput.Data.consVariable2[7]",
						 					deviceName + ".ProcessInput.Data.consVariable2[8]",						 											 											 											 											 											 					
						 					deviceName + ".ProcessInput.Data.prodVariable3[1]",						 											 											 											 					
						 					deviceName + ".ProcessInput.Data.prodVariable3[2]",		
						 					deviceName + ".ProcessInput.Data.prodVariable3[3]",	
						 					deviceName + ".ProcessInput.Data.prodVariable3[4]",	
						 					deviceName + ".ProcessInput.Data.prodVariable3[5]",
						 					deviceName + ".ProcessInput.Data.prodVariable3[6]",		
						 					deviceName + ".ProcessInput.Data.prodVariable3[7]",
						 					deviceName + ".ProcessInput.Data.prodVariable3[8]",				
			 				//				sDpAgentName + ".ProcessInput.Data.agentStatus",		
						 					sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".enabled",
								/*			sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+sAliasAgent+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+sAliasAgent+".enabled",*/
											deviceName + ".ProcessInput.Data.powerCalc:_online.._invalid");

	}
	else cietEH_FaceplateStatusDisconnection();
	g_bCallbackConnected = (iRes >= 0);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietEH_FaceplateDisconnect
/**
Purpose: disconnect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in cietPT faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietEH_FaceplateDisconnect(string deviceName)
{
	int iRes;
	string sPlcName, sSystemName, sAlias, sAgentName, sDpAgentName;
	dyn_string dsDeviceNameSplit;
	
//	DebugN("FaceplateDisconnect");
	if(g_bUnSystemAlarmPlc) {
// get the PLC name 
		sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
		sSystemName = unGenericDpFunctions_getSystemName(deviceName);

			sAgentName = cietGenericFunctions_getAgentName(deviceName);
	//		sDpAgentName = unGenericDpFunctions_dpAliasToName(sAgentName);
		sAlias = unGenericDpFunctions_getAlias(sAgentName);
		dsDeviceNameSplit=strsplit(sAlias, ".");
	
		iRes = dpDisconnect("cietEH_FaceplateStatusAnimationCB",
						 					deviceName + ".ProcessInput.Config.sgNumber", 		
						 					deviceName + ".ProcessInput.Config.agNumber", 		
						 					deviceName + ".ProcessInput.Config.chNumber", 								 											 					
						 					deviceName + ".ProcessInput.Data.heatVolMeas", 
							 				deviceName + ".ProcessInput.Data.heatRefVolMeas",
							 				deviceName + ".ProcessInput.Data.shuntVolMeas", 
							 				deviceName + ".ProcessInput.Data.shuntRefVolMeas",							 				
							 				deviceName + ".ProcessInput.Data.thVolMeas", 
							 				deviceName + ".ProcessInput.Data.thRefVolMeas",							 				
						 					deviceName + ".ProcessInput.Data.hysteresis",
						 					deviceName + ".ProcessInput.Data.cyclePeriod",
						 					deviceName + ".ProcessInput.Data.heaterType",
						 					deviceName + ".ProcessInput.Data.threshold",
						 					deviceName + ".ProcessInput.Data.setPoint",
						 					deviceName + ".ProcessInput.Data.forcedSetPoint",
						 					deviceName + ".ProcessInput.Data.sensorStatus",
						 					g_sCompleteAgentName + ".ProcessInput.Data.agentStatus",
						 					deviceName + ".ProcessInput.Data.command",
						 					deviceName + ".ProcessInput.Data.suppliedVoltageCalc",
						 					deviceName + ".ProcessInput.Data.suppliedCurrentCalc",		
						 					deviceName + ".ProcessInput.Data.thCoupleVoltageCalc",					 					
						 					deviceName + ".ProcessInput.Data.thCoupleTemperatureCalc",								 					
						 					deviceName + ".ProcessInput.Data.powerCalc",						 		
						 					deviceName + ".ProcessInput.Data.thCoupleOffset",
						 					deviceName + ".ProcessInput.Data.heaterResistance",	
				 							deviceName + ".ProcessInput.Curve.curveId",							 												 								
				 							deviceName + ".ProcessInput.Data.prodVariable1[1]",						 					
				 							deviceName + ".ProcessInput.Data.prodVariable1[2]",						 									 							
						 					deviceName + ".ProcessInput.Data.consVariable2[1]",
						 					deviceName + ".ProcessInput.Data.consVariable2[2]",
						 					deviceName + ".ProcessInput.Data.consVariable2[3]",
						 					deviceName + ".ProcessInput.Data.consVariable2[4]",
						 					deviceName + ".ProcessInput.Data.consVariable2[5]",
						 					deviceName + ".ProcessInput.Data.consVariable2[6]",
						 					deviceName + ".ProcessInput.Data.consVariable2[7]",
						 					deviceName + ".ProcessInput.Data.consVariable2[8]",						 											 											 											 											 											 					
						 					deviceName + ".ProcessInput.Data.prodVariable3[1]",						 											 											 											 					
						 					deviceName + ".ProcessInput.Data.prodVariable3[2]",	
						 					deviceName + ".ProcessInput.Data.prodVariable3[3]",	
						 					deviceName + ".ProcessInput.Data.prodVariable3[4]",	
						 					deviceName + ".ProcessInput.Data.prodVariable3[5]",
						 					deviceName + ".ProcessInput.Data.prodVariable3[6]",		
						 					deviceName + ".ProcessInput.Data.prodVariable3[7]",	
						 					deviceName + ".ProcessInput.Data.prodVariable3[8]",	
						 			//		sDpAgentName + ".ProcessInput.Data.agentStatus",					 											 											 											 					
						 					sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".enabled",
									/*		sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+sAlias+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+sAlias+".enabled",*/
											deviceName + ".ProcessInput.Data.powerCalc:_online.._invalid");
		}									
	g_bCallbackConnected = !(iRes >= 0);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietEH_FaceplateStatusAnimationCB
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in cietPT faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietEH_FaceplateStatusAnimationCB(string sDp_sgNumber, int isgNumber,
																				string sDp_agNumber, int iagNumber,
																				string sDp_chNumber, int ichNumber,
																				string sDp_heatVolMeas, float fheatVolMeas,
																				string sDp_heatRefVolMeas, float fheatRefVolMeas,
																				string sDp_shuntVolMeas, float fshuntVolMeas,
																				string sDp_shuntRefVolMeas, float fshuntRefVolMeas,
																				string sDp_thVolMeas, float fthVolMeas,
																				string sDp_thRefVolMeas, float fthRefVolMeas,
																				string sDp_hysteresis, int iHysteresis,
																				string sDp_cyclePeriod, int icyclePeriod,
																				string sDp_heaterType, int iHeaterType,
																				string sDp_threshold, float fThreshold,
																				string sDp_setPoint, float fSetPoint,
																				string sDp_forcedSetPoint, float fforcedSetPoint,
																				string sDp_sensorStatus, unsigned uSensorStatus,
																				string sDp_AgentStatus, unsigned uAgentStatus,
																				string sDp_commandIn, unsigned uCommandIn,
																				string sDp_suppliedVoltageCalc, float fsuppliedVoltageCalc,
																				string sDp_suppliedCurrentCalc, float fsuppliedCurrentCalc,
																				string sDp_thCoupleVoltageCalc, float fthCoupleVoltageCalc,																				
																				string sDp_thCoupleTemperatureCalc, float fthCoupleTemperatureCalc,																						
																				string sDp_powerCalc, float fpowerCalc,																		
																				string sDp_thCoupleOffset, float fthCoupleOffset,																																												
																				string sDp_thheaterResistance, float fheaterResistance,	
																				string sDp_curveId, string sCurveId,																																																															
																				string sDp_prodVariable1_1, bit32 bprodVariable1_1,						
																				string sDp_prodVariable1_2, bit32 bprodVariable1_2,																										
																				string sDp_consVariable2_1, bit32 bconsVariable2_1,						
																				string sDp_consVariable2_2, bit32 bconsVariable2_2,						
																				string sDp_consVariable2_3, bit32 bconsVariable2_3,						
																				string sDp_consVariable2_4, bit32 bconsVariable2_4,						
																				string sDp_consVariable2_5, bit32 bconsVariable2_5,						
																				string sDp_consVariable2_6, bit32 bconsVariable2_6,						
																				string sDp_consVariable2_7, bit32 bconsVariable2_7,						
																				string sDp_consVariable2_8, bit32 bconsVariable2_8,																																																																																																																																																		
																				string sDp_prodVariable3_1, bit32 bprodVariable3_1,						
																				string sDp_prodVariable3_2, bit32 bprodVariable3_2,		
																				string sDp_prodVariable3_3, bit32 bprodVariable3_3,
																				string sDp_prodVariable3_4, bit32 bprodVariable3_4,
																				string sDp_prodVariable3_5, bit32 bprodVariable3_5,
																				string sDp_prodVariable3_6, bit32 bprodVariable3_6,
																				string sDp_prodVariable3_7, bit32 bprodVariable3_7,	
																				string sDp_prodVariable3_8, bit32 bprodVariable3_8,
																			//	string sDp_Agent_status, int iAgent_status,			
																																							
																				string sDpSystemIntegrityAlarmValue, int iSystemIntegrityAlarmValue, 
																			 	string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
																		/*	 	string sDpSystemAlarmValue, int iSystemAlarmValue,
																			 	string sDpSystemAlarmEnabled, bool bSystemAlarmEnabled,*/
																			 	string sDpInvalid, bool bCodeInvalid)
{

int iAlarmValue,i,j, iStart;
bool bFaceplateInvalid, bDisable, bIoError, bReset, bBadConfig, bACMode,bForcedMode,bForcedProductionRawData, bCardProtection,bAgIoError, bOverTemperature;
string sTemp,sDpPLCName, sHeaterType, sOperationMode, sBusName, sCrate;
dyn_bit32 db32sensorByte;

dyn_bit32 db32sensorByte_1= makeDynChar(bprodVariable1_1,bprodVariable1_2);
dyn_bit32 db32sensorByte_2= makeDynChar(bconsVariable2_1,bconsVariable2_2,bconsVariable2_3,bconsVariable2_4,
																			bconsVariable2_5,bconsVariable2_6,bconsVariable2_7,bconsVariable2_8);
dyn_bit32 db32sensorByte_3= makeDynChar(bprodVariable3_1,bprodVariable3_2,bprodVariable3_3,bprodVariable3_4,bprodVariable3_5,bprodVariable3_6,bprodVariable3_7,bprodVariable3_8);	

dyn_dyn_anytype dsVariable = 	makeDynAnytype(db32sensorByte_1, db32sensorByte_2, db32sensorByte_3);

float fheatRefVolValue,fheatVolValue, fshuntRefVolValue,fshuntVolValue, fthRefVolValue, fthVolValue;
	
///////////////////////////////////

	bIoError = getBit (uSensorStatus, 0); 
	bReset = getBit (uSensorStatus, 1); 		//1 = reset, 0 = running
	bBadConfig = getBit (uSensorStatus, 2);
  bForcedMode = getBit (uSensorStatus, 3);
  bForcedProductionRawData = getBit (uSensorStatus, 13);
	bCardProtection = getBit (uSensorStatus, 4); 	//1 = protection on, 0 = protection off
	bACMode = getBit (uSensorStatus, 5);													//0 = DC Mode, 1 = AC Mode
	bOverTemperature = getBit (uSensorStatus, 6); 	//1 = Active
	bDisable = getBit (uSensorStatus, 12);	

///////////////////////////////////

	sCrate= cietGenericObject_getCrate ($sDpName);
	if(g_bSystemConnected)
		cietGenericObject_DisplayString(sCrate, "CRATE", "unDisplayValue_Parameter", bFaceplateInvalid);

	if ((iSystemIntegrityAlarmValue >= CRYO_SYSTEM_INGETRITY_ALARM_LIMIT)/*	||
			(iSystemAlarmValue < CRYO_SYSTEM_INGETRITY_ALARM_LIMIT)*/)
			iAlarmValue=c_unSystemIntegrity_alarm_value_level1;
	else
		iAlarmValue=c_unSystemIntegrity_no_alarm_value;

/*HEADER*/		
	iStart=strpos(sDpSystemIntegrityAlarmValue, c_cietSystemIntegrity_DS) + strlen(c_cietSystemIntegrity_DS);
	sDpPLCName=substr(sDpSystemIntegrityAlarmValue,  iStart, strpos(sDpSystemIntegrityAlarmValue, ".alarm") - iStart);
	cietGenericFunctions_Header(sDpPLCName, iAlarmValue, bSystemIntegrityAlarmEnabled /*&& bSystemAlarmEnabled*/, bCodeInvalid);
	
	bFaceplateInvalid = (bCodeInvalid || !bSystemIntegrityAlarmEnabled ||  (iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value));
	
//  --- Configuration  --- 

	switch(iHeaterType)
	{
		case 1:
			sHeaterType = ciet_EH_TYPE_25W;
		break;
		case 2:
			sHeaterType = ciet_EH_TYPE_100W;
		break;
		case 3:
			sHeaterType = ciet_EH_TYPE_500W_25W;
		break;
		case 4:
			sHeaterType = ciet_EH_TYPE_500W;
		break;
		default:
			sHeaterType = "";
		break;
	}

	if(bACMode) sOperationMode = "AC";
		else sOperationMode = "DC";
  
  g_sGTWName = cietGenericFunctions_getGTWName(sDp_sgNumber);

//  --- Warnings   --- 

	if(g_bSystemConnected)	
		{
			// Agent io_error
			bAgIoError=getBit(uAgentStatus,0);
			unGenericObject_ColorBoxAnimate ("AG_IOERROR", !bAgIoError, 1, "", FALSE, "cietFaceplate_Warning");
		}

	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate ("DISABLE", !bDisable, 1, "",FALSE, "cietFaceplate_Warning");
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("IO_ERROR", !bIoError, 1, "", FALSE, "cietFaceplate_Warning");
	if(g_bSystemConnected)		
		unGenericObject_ColorBoxAnimate("RESET", !bReset, 1, "", FALSE, "cietFaceplate_Warning");
	if(g_bSystemConnected)				
		unGenericObject_ColorBoxAnimate("BADCONFIG", !bBadConfig, 1, "", FALSE, "cietFaceplate_Warning");
	if(g_bSystemConnected)				
		unGenericObject_ColorBoxAnimate("CARD_PROTECTION", !bCardProtection, 1, "", FALSE, "cietFaceplate_ModeActive");

	if(g_bSystemConnected)				
		unGenericObject_ColorBoxAnimate("FORCED_MODE", !bForcedMode, 1, "", FALSE, "cietFaceplate_ModeActive");
  	if(g_bSystemConnected)				
		unGenericObject_ColorBoxAnimate("FORCED_MODE_PRODUCTION_RAWDATA", !bForcedProductionRawData, 1, "", FALSE, "cietFaceplate_ModeActive");
	if(g_bSystemConnected)				
		unGenericObject_ColorBoxAnimate("OVER_TEMP", !bOverTemperature, 1, "", FALSE, "cietFaceplate_Warning");		
		
		
//  --- Measurements  --- 
	
	if(g_bSystemConnected)			
		unGenericObject_DisplayValue(g_sHeatVolMeasFormat, g_sHeatVolMeasUnit, fheatVolMeas, "HEATVOLMEAS", "unDisplayValue_Status", bFaceplateInvalid);
	if(g_bSystemConnected)
			{
		fheatVolValue=(ciet_CONFIG_EH_FULLSCALE_heatVolMeas*fheatVolMeas)/ciet_CONFIG_EH_FULLSCALE_MAXDATA_heatVolMeas;	
		unGenericObject_DisplayValue("", ciet_CONFIG_EH_FULLSCALE_UNIT_heatVolMeas, fheatVolValue, "HEATVOLVALUE", "unDisplayValue_Status", bFaceplateInvalid);			
			}

	if(g_bSystemConnected)			
		unGenericObject_DisplayValue(g_sHeatRefVolMeasFormat, g_sHeatRefVolMeasUnit, fheatRefVolMeas, "HEATREFVOLMEAS", "unDisplayValue_Status", bFaceplateInvalid);
	if(g_bSystemConnected)
			{
		fheatRefVolValue=(ciet_CONFIG_EH_FULLSCALE_heatRefVolMeas*fheatRefVolMeas)/ciet_CONFIG_EH_FULLSCALE_MAXDATA_heatRefVolMeas;	
		unGenericObject_DisplayValue("", ciet_CONFIG_EH_FULLSCALE_UNIT_heatRefVolMeas, fheatRefVolValue, "HEATREFVOLVALUE", "unDisplayValue_Status", bFaceplateInvalid);			
			}
			
	if(g_bSystemConnected)
		setValue("HEATREFVOLTYPICAL","text",ciet_CONFIG_EH_TYPICAL_VALUE_heatRefVolMeas + " " + ciet_CONFIG_EH_TYPICAL_VALUE_UNIT_heatRefVolMeas);
		
	if(g_bSystemConnected)				
		unGenericObject_DisplayValue(g_sShuntVolMeasFormat, g_sShuntVolMeasUnit, fshuntVolMeas, "SHUNTVOLMEAS", "unDisplayValue_Status", bFaceplateInvalid);
	if(g_bSystemConnected)				
		unGenericObject_DisplayValue(g_sShuntRefVolMeasFormat, g_sShuntRefVolMeasUnit, fshuntRefVolMeas, "SHUNTREFVOLMEAS", "unDisplayValue_Status", bFaceplateInvalid);
	if(g_bSystemConnected)
			{ 
                                // New and only value for the typical value of the shunt voltage reference (Quique)
				setValue("SHUNTREFVOLTYPICAL","text",ciet_CONFIG_EH_100_TYPICAL_VALUE_shuntRefVolMeas + " " + ciet_CONFIG_EH_100_TYPICAL_VALUE_UNIT_shuntRefVolMeas);
                                
                                // New and only value for the  value of the shunt voltage  (Quique)
                                fshuntVolValue=(ciet_CONFIG_EH_COMMON_FULLSCALE_shuntVolMeas*fshuntVolMeas)/ciet_CONFIG_EH_COMMON_FULLSCALE_MAXDATA_shuntVolMeas;	
				unGenericObject_DisplayValue("", ciet_CONFIG_EH_COMMON_FULLSCALE_UNIT_shuntVolMeas, fshuntVolValue, "SHUNTVOLVALUE", "unDisplayValue_Status", bFaceplateInvalid);			

                                // New and only value for the value of the shunt voltage reference (Quique)  
				fshuntRefVolValue=(ciet_CONFIG_EH_COMMON_FULLSCALE_shuntRefVolMeas*fshuntRefVolMeas)/ciet_CONFIG_EH_COMMON_FULLSCALE_MAXDATA_shuntRefVolMeas;	
				unGenericObject_DisplayValue("", ciet_CONFIG_EH_COMMON_FULLSCALE_UNIT_shuntRefVolMeas, fshuntRefVolValue, "SHUNTREFVOLVALUE", "unDisplayValue_Status", bFaceplateInvalid);
                                
                                /*
                                if(sHeaterType==ciet_EH_TYPE_100W)
					{
						fshuntVolValue=(ciet_CONFIG_EH_100_FULLSCALE_shuntVolMeas*fshuntVolMeas)/ciet_CONFIG_EH_100_FULLSCALE_MAXDATA_shuntVolMeas;	
						unGenericObject_DisplayValue("", ciet_CONFIG_EH_100_FULLSCALE_UNIT_shuntVolMeas, fshuntVolValue, "SHUNTVOLVALUE", "unDisplayValue_Status", bFaceplateInvalid);			

						fshuntRefVolValue=(ciet_CONFIG_EH_100_FULLSCALE_shuntRefVolMeas*fshuntRefVolMeas)/ciet_CONFIG_EH_100_FULLSCALE_MAXDATA_shuntRefVolMeas;	
						unGenericObject_DisplayValue("", ciet_CONFIG_EH_100_FULLSCALE_UNIT_shuntRefVolMeas, fshuntRefVolValue, "SHUNTREFVOLVALUE", "unDisplayValue_Status", bFaceplateInvalid);			

						setValue("SHUNTREFVOLTYPICAL","text",ciet_CONFIG_EH_100_TYPICAL_VALUE_shuntRefVolMeas + " " + ciet_CONFIG_EH_100_TYPICAL_VALUE_UNIT_shuntRefVolMeas);
					}
				else if	(sHeaterType==ciet_EH_TYPE_500W_25W||sHeaterType==ciet_EH_TYPE_25W)
					{
						fshuntVolValue=(ciet_CONFIG_EH_25_500_FULLSCALE_shuntVolMeas*fshuntVolMeas)/ciet_CONFIG_EH_25_500_FULLSCALE_MAXDATA_shuntVolMeas;	
						unGenericObject_DisplayValue("", ciet_CONFIG_EH_25_500_FULLSCALE_UNIT_shuntVolMeas, fshuntVolValue, "SHUNTVOLVALUE", "unDisplayValue_Status", bFaceplateInvalid);			

						fshuntRefVolValue=(ciet_CONFIG_EH_25_500_FULLSCALE_shuntRefVolMeas*fshuntRefVolMeas)/ciet_CONFIG_EH_25_500_FULLSCALE_MAXDATA_shuntRefVolMeas;	
						unGenericObject_DisplayValue("", ciet_CONFIG_EH_25_500_FULLSCALE_UNIT_shuntRefVolMeas, fshuntRefVolValue, "SHUNTREFVOLVALUE", "unDisplayValue_Status", bFaceplateInvalid);			

						setValue("SHUNTREFVOLTYPICAL","text",ciet_CONFIG_EH_25_500_TYPICAL_VALUE_shuntRefVolMeas + " " + ciet_CONFIG_EH_25_500_TYPICAL_VALUE_UNIT_shuntRefVolMeas);
					}				
				else if	(sHeaterType==ciet_EH_TYPE_500W)
                                */
                                if	(sHeaterType==ciet_EH_TYPE_500W)  	
					{
						unGenericObject_DisplayValue("", "", "", "HEATVOLVALUE", "unDisplayValue_Status", bFaceplateInvalid);			
						unGenericObject_DisplayValue("", "", "", "HEATVOLMEAS", "unDisplayValue_Status", bFaceplateInvalid);								
						unGenericObject_DisplayValue("", "", "", "HEATREFVOLVALUE", "unDisplayValue_Status", bFaceplateInvalid);			
						unGenericObject_DisplayValue("", "", "", "HEATREFVOLMEAS", "unDisplayValue_Status", bFaceplateInvalid);								
						setValue("HEATREFVOLTYPICAL","text","");
                                                Text_HeaterRefTypical.visible=false;
                                                
						unGenericObject_DisplayValue("", "", "", "SHUNTVOLVALUE", "unDisplayValue_Status", bFaceplateInvalid);			
						unGenericObject_DisplayValue("", "", "", "SHUNTVOLMEAS", "unDisplayValue_Status", bFaceplateInvalid);							
						unGenericObject_DisplayValue("", "", "", "SHUNTREFVOLVALUE", "unDisplayValue_Status", bFaceplateInvalid);			
						unGenericObject_DisplayValue("", "", "", "SHUNTREFVOLMEAS", "unDisplayValue_Status", bFaceplateInvalid);							
						setValue("SHUNTREFVOLTYPICAL","text","");
                                                Text_ShuntRefTypical.visible=false;
					}

				if(sHeaterType == ciet_EH_TYPE_500W_25W || sHeaterType == ciet_EH_TYPE_500W)	
						{setValue("THCOUPLEOFFSET.display", "enabled", TRUE);
						 setValue("THVOLVALUE.display", "enabled", TRUE);	
						 setValue("THVOLMEAS.display", "enabled", TRUE);							
						 setValue("THREFVOLVALUE.display", "enabled", TRUE);	
						 setValue("THREFVOLMEAS.display", "enabled", TRUE);							
						 setValue("THREFVOLTYPICAL", "enabled", TRUE);
						}
				else if(sHeaterType == ciet_EH_TYPE_25W || sHeaterType == ciet_EH_TYPE_100W)						
					{
						setValue("THCOUPLEOFFSET.display", "enabled", FALSE);
						setValue("THVOLVALUE.display", "enabled", FALSE);	
						setValue("THVOLMEAS.display", "enabled", FALSE);							
						setValue("THREFVOLVALUE.display", "enabled", FALSE);	
						setValue("THREFVOLMEAS.display", "enabled", FALSE);							
						setValue("THREFVOLTYPICAL", "enabled", FALSE);
					}	
						
				if(sOperationMode == "DC")	
					{
						setValue("HEATVOLVALUE.display", "enabled", TRUE);		
						setValue("HEATVOLMEAS.display", "enabled", TRUE);										
						setValue("HEATREFVOLVALUE.display", "enabled", TRUE);	
						setValue("HEATREFVOLMEAS.display", "enabled", TRUE);							
						setValue("HEATREFVOLTYPICAL", "enabled", TRUE);
											
						setValue("SHUNTVOLVALUE.display", "enabled", TRUE);	
						setValue("SHUNTVOLMEAS.display", "enabled", TRUE);							
						setValue("SHUNTREFVOLVALUE.display", "enabled", TRUE);	
						setValue("SHUNTREFVOLMEAS.display", "enabled", TRUE);							
						setValue("SHUNTREFVOLTYPICAL", "enabled", TRUE);						
					}
				else if	(sOperationMode == "AC")	
					{
						setValue("HEATVOLVALUE.display", "enabled", FALSE);		
						setValue("HEATVOLMEAS.display", "enabled", FALSE);										
						setValue("HEATREFVOLVALUE.display", "enabled", FALSE);	
						setValue("HEATREFVOLMEAS.display", "enabled", FALSE);							
						setValue("HEATREFVOLTYPICAL", "enabled", FALSE);
											
						setValue("SHUNTVOLVALUE.display", "enabled", FALSE);	
						setValue("SHUNTVOLMEAS.display", "enabled", FALSE);							
						setValue("SHUNTREFVOLVALUE.display", "enabled", FALSE);	
						setValue("SHUNTREFVOLMEAS.display", "enabled", FALSE);							
						setValue("SHUNTREFVOLTYPICAL", "enabled", FALSE);	
					}						
			}

	if(g_bSystemConnected)
		{
			fthVolValue=(ciet_CONFIG_EH_FULLSCALE_thVolMeas*fthVolMeas)/ciet_CONFIG_EH_FULLSCALE_MAXDATA_thVolMeas;	
			unGenericObject_DisplayValue("", ciet_CONFIG_EH_FULLSCALE_UNIT_thVolMeas, fthVolValue, "THVOLVALUE", "unDisplayValue_Status", bFaceplateInvalid);			
		}
	if(g_bSystemConnected)				
		unGenericObject_DisplayValue(g_sThVolMeasFormat, g_sThVolMeasUnit, fthVolMeas, "THVOLMEAS", "unDisplayValue_Status", bFaceplateInvalid);
	if(g_bSystemConnected)				
		unGenericObject_DisplayValue(g_sThRefVolMeasFormat, g_sThVolMeasUnit, fthRefVolMeas, "THREFVOLMEAS", "unDisplayValue_Status", bFaceplateInvalid);
	if(g_bSystemConnected)
		{
			fthRefVolValue=(ciet_CONFIG_EH_FULLSCALE_thRefVolMeas*fthRefVolMeas)/ciet_CONFIG_EH_FULLSCALE_MAXDATA_thRefVolMeas;	
			unGenericObject_DisplayValue("", ciet_CONFIG_EH_FULLSCALE_UNIT_thRefVolMeas, fthRefVolValue, "THREFVOLVALUE", "unDisplayValue_Status", bFaceplateInvalid);			
		}
	if(g_bSystemConnected)
		setValue("THREFVOLTYPICAL","text",ciet_CONFIG_EH_TYPICAL_VALUE_thRefVolMeas + " " + ciet_CONFIG_EH_TYPICAL_VALUE_UNIT_thRefVolMeas);

//  --- Constants  --- 

	if(g_bSystemConnected)					
		cietGenericObject_DisplayInt(iHysteresis, "HYSTERESIS", "unDisplayValue_Parameter", FALSE);
	if(g_bSystemConnected)				
		cietEH_displayValueCyclePeriod(icyclePeriod,"CYCLEPERIOD", "unDisplayValue_Parameter", FALSE);
	if(g_bSystemConnected)				
		unGenericObject_DisplayValue(g_sThresholdFormat, g_sThresholdUnit, fThreshold, "THRESHOLD", "unDisplayValue_Parameter", FALSE);
	if(g_bSystemConnected)					
		unGenericObject_DisplayValue(g_sThCoupleOffsetFormat, g_sThCoupleOffsetUnit, fthCoupleOffset, "THCOUPLEOFFSET", "unDisplayValue_Parameter", FALSE);
	if(g_bSystemConnected)		
		unGenericObject_DisplayValue(g_sheaterResistanceFormat, g_sheaterResistanceUnit, fheaterResistance, "HEATERRESISTANCE", "unDisplayValue_Parameter", FALSE);

// --- Calculations  --- 

	if(g_bSystemConnected)
		unGenericObject_DisplayValue(g_sSuppliedVoltageCalcFormat, g_sSuppliedVoltageCalcUnit, fsuppliedVoltageCalc, "SUPPLIEDVOLTAGECALC", "unDisplayValue_Parameter", FALSE);
	if(g_bSystemConnected)				
		unGenericObject_DisplayValue(g_sSuppliedCurrentCalcFormat, g_sSuppliedCurrentCalcUnit, fsuppliedCurrentCalc, "SUPPLIEDCURRENTCALC", "unDisplayValue_Parameter", FALSE);
	if(g_bSystemConnected)				
		unGenericObject_DisplayValue(g_sThCoupleVoltageCalcFormat, g_sThCoupleVoltageCalcUnit, fthCoupleVoltageCalc, "THCOUPLEVOLTAGECALC", "unDisplayValue_Parameter", FALSE);	
	if(g_bSystemConnected)	
		unGenericObject_DisplayValue(g_sThCoupleTemperatureCalcFormat, g_sThCoupleTemperatureCalcUnit, fthCoupleTemperatureCalc, "THCOUPLETEMPERATURECALC", "unDisplayValue_Parameter", FALSE);	
  if(g_bSystemConnected){	
   unGenericObject_DisplayValue(g_sPowerCalcFormat, g_sPowerCalcUnit, fpowerCalc, "POWERCALC", "unDisplayValue_Parameter", FALSE);
  }
  if(g_bSystemConnected)	{			
    if(!bForcedProductionRawData){  		
      unGenericObject_DisplayValue(g_sSetPointFormat, g_sSetPointUnit, fSetPoint, "SETPOINT", "unDisplayValue_Parameter", FALSE);
    }
    else{
      cietGenericObject_DisplayString( "Not Significant", "SETPOINT", "unDisplayValue_Parameter", FALSE);
    }  
 }
  if(g_bSystemConnected)		{
    if(!bForcedProductionRawData){	
    	unGenericObject_DisplayValue(g_sForcedSetPointFormat, g_sForcedSetPointUnit, fforcedSetPoint, "FORCEDSETPOINT", "unDisplayValue_Parameter", FALSE);
  	 }
    else{
     cietGenericObject_DisplayString( "Not Significant", "FORCEDSETPOINT", "unDisplayValue_Parameter", FALSE);
    }
  } 
//  --- Configuration ---
	if(g_bSystemConnected)				
		cietGenericObject_DisplayString( sHeaterType, "HEATERTYPE", "unDisplayValue_Parameter", FALSE);	
  if(g_bSystemConnected){		
    if(!bForcedProductionRawData){		
		    cietGenericObject_DisplayString( sOperationMode, "MODE_AC_DC", "unDisplayValue_Parameter", FALSE);		
  	 }
    else{
      cietGenericObject_DisplayString( "Not Significant", "MODE_AC_DC", "unDisplayValue_Parameter", FALSE);		
    }
  }
// --- WFIP Infos  ---
	if(g_bSystemConnected)			
		cietGenericObject_DisplayString(g_sGTWName, "GTWNAME", "unDisplayValue_Parameter", FALSE);
	if(g_bSystemConnected)				
		cietGenericObject_DisplayInt(iagNumber, "AGENTNUMBER", "unDisplayValue_Parameter", FALSE);	
	if(g_bSystemConnected)				
		cietGenericObject_DisplayInt(ichNumber, "AGENTCHANNEL", "unDisplayValue_Parameter", FALSE);	
	if(g_bSystemConnected)				
		{
			sBusName=cietGenericFuncitons_getSegmentName(sDp_sgNumber);
			// DebugN("sDp_sgNumber="+sDp_sgNumber);
			// DebugN("sBusName="+sBusName);
			cietGenericObject_DisplayString(sBusName, "BUSNAME", "unDisplayValue_Parameter", FALSE);
			cietGenericObject_DisplayInt(isgNumber, "AGENTBUS", "unDisplayValue_Parameter", FALSE);	
		}
		
//  --- RawData	 ---	

	// case 1st row, place the two bytes in the right position
	db32sensorByte=dsVariable[1];	
	
	// DebugN("--------------- begin -----------------");
	for (i=0;i<=6;i=i+2)
		{ 			
			j=i+1;		
			
			if ((i==2*ichNumber) && (g_bSystemConnected))
				{	
					// DebugN("i="+i+" j="+j+" ichNumber="+ichNumber);
					
					cietGenericObject_DisplayToCell(cietGenericObject_toHexa(db32sensorByte[1]), "rawDatas",0, "B"+i, "unDisplayValue_Parameter", FALSE);
					cietGenericObject_DisplayToCell(cietGenericObject_toHexa(db32sensorByte[2]), "rawDatas",0, "B"+j, "unDisplayValue_Parameter", FALSE);
					
					// DebugN("db32sensorByte[1]"+db32sensorByte[1]+"hexa i+1="+cietGenericObject_toHexa(db32sensorByte[1]));
					// DebugN("db32sensorByte[2]"+db32sensorByte[2]+"hexa j+1="+cietGenericObject_toHexa(db32sensorByte[2]));
				}
			else
				{
					cietGenericObject_DisplayToCell("--", "rawDatas",0, "B"+i, "unDisplayValue_Parameter", FALSE);
					cietGenericObject_DisplayToCell("--", "rawDatas",0, "B"+j, "unDisplayValue_Parameter", FALSE);
				}
		}						
	// DebugN("-------------- end ----------------");
	// for the last two rows
	for(int k=2;k<=dynlen(dsVariable);k++)
		{
			db32sensorByte = dsVariable[k];
			for(i=1;i<=8;i++)
				{
					int j =i-1;
					
					if ((k==3) && (i<=4))
						{
						if (j!=ichNumber)
							rawDatas.cellBackColRC(2,"B"+j,"_3DFace");
						else
							rawDatas.cellBackColRC(2,"B"+j,"_Window");
						}
					sTemp = cietGenericObject_toHexa(db32sensorByte[i]);
					// DebugN("["+k+","+j+"] (sTemp)="+sTemp);
					if(g_bSystemConnected)		
						cietGenericObject_DisplayToCell(sTemp, "rawDatas",k-1, "B"+j, "unDisplayValue_Parameter", FALSE);
				}
		}	
	
	// Not relevant to the information
	rawDatas.cellBackColRC(2,"B5","_3DFace");
	rawDatas.cellBackColRC(2,"B6","_3DFace");
	rawDatas.cellBackColRC(2,"B7","_3DFace");


//  --- Calibration Curve  --- 
	
	 // string sShowIdCurve, sShowIdCard;
		
		//sCurveId = cietGenericFunctions_getCurveId ($sDpName, "EH", 1);
		
		cietGenericObject_DisplayString(sCurveId, "idCurve", "unDisplayValue_Parameter", FALSE);
}
 

//---------------------------------------------------------------------------------------------------------------------------------------

// cietEH_FaceplateStatusDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietEH_FaceplateStatusDisconnection()
{
	//unGenericObject_ColorBoxDisconnect(makeDynString("IO_ERROR","RESET", "AG_IOERROR", /*"OPERATION_AC","OPERATION_DC",*/ "FORCED_MODE" ));

	/*unGenericObject_DisplayValueDisconnect(makeDynString("MODE_AC_DC", "GTWNAME", "AGENTNUMBER", "AGENTBUS", "AGENTCHANNEL", "HEATVOLMEAS", 
	"HEATREFVOLMEAS", "SHUNTVOLMEAS", "SHUNTREFVOLMEAS", "THVOLMEAS", "THREFVOLMEAS", "HYSTERESIS", "CYCLEPERIOD", "HEATERTYPE","THRESHOLD", 
	"SETPOINT", "FORCEDSETPOINT", "SENSORSTATUS",	"COMMANDIN","SUPPLIEDVOLTAGECALC", "SUPPLIEDCURRENTCALC","THCOUPLEVOLTAGECALC",*//*"THCOUPLETEMPERATURECALC",*//*"POWERCALC", "THCOUPLEOFFSET", 
	"HEATERRESISTANCE",	"B11_sensorByte","B10_sensorByte","B9_sensorByte","B8_sensorByte","B7_sensorByte", "B6_sensorByte", "B5_sensorByte", 
	"B4_sensorByte", "B3_sensorByte", "B2_sensorByte", "B1_sensorByte", "B0_sensorByte"));*/
	
  //put in the top textbox the DATA not connected in purple
  cietGenericFunctions_HeaderDisconnect();
}



//---------------------------------------------------------------------------------------------------------------------------------------

// cietEH_ButtonUserAccess
/**
Purpose: User access for EH buttons

Parameters: 
	- sDpName, string, input, device name
	- sDpType, string, input, dptype
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietEH_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
	bool operator, expert, admin;
	int i, length;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
        string sOperAction, sExpAction, sAdminAction;
        bool bActionDefined = false;

        unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo,CIET_ACCESS_CONTROL_DOMAIN_NAME);
        if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) ||
           (sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME))
          bActionDefined=true;
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString();
	if (operator)
	{
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
            dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_SELECT, ciet_POPUPMENU_HELP,
                                              ciet_POPUPMENU_WIDGET_SIMULATOR,ciet_POPUPMENU_OPEN_WFIPAGENT, 
                                              ciet_POPUPMENU_OPEN_WFIPSEGMENT, ciet_POPUPMENU_OPEN_CRATE));
	}
	if (expert)
	{
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
	    			dynAppend(dsAccess, makeDynString(ciet_FACEPLATE_BUTTON_CALIBRATION,ciet_FACEPLATE_BUTTON_CONFIG,
                                              UN_FACEPLATE_BUTTON_SELECT,ciet_FACEPLATE_BUTTON_RESET,
                                              ciet_FACEPLATE_BUTTON_SETPRODUCTIONRAWDATA,
                                              ciet_FACEPLATE_BUTTON_RAWDATA,ciet_FACEPLATE_BUTTON_DEADBANDS,
                                              ciet_FACEPLATE_BUTTON_CONSTANTS,ciet_FACEPLATE_BUTTON_AUTO_MODE,
                                              ciet_FACEPLATE_BUTTON_FORCED_MODE,ciet_FACEPLATE_BUTTON_SETPOINT,
                                              ciet_FACEPLATE_BUTTON_TENDPARAM, ciet_FACEPLATE_BUTTON_DEBUG, 
                                              ciet_FACEPLATE_BUTTON_DISABLE, ciet_POPUPMENU_HELP,
                                              ciet_POPUPMENU_WIDGET_SIMULATOR,ciet_POPUPMENU_OPEN_WFIPAGENT, 
                                              ciet_POPUPMENU_OPEN_WFIPSEGMENT, ciet_POPUPMENU_OPEN_CRATE));	
	}
        if(admin){
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
            dynAppend(dsAccess, makeDynString(ciet_FACEPLATE_BUTTON_CALIBRATION,ciet_FACEPLATE_BUTTON_CONFIG,
                                              UN_FACEPLATE_BUTTON_SELECT,ciet_FACEPLATE_BUTTON_RESET,
                                              ciet_FACEPLATE_BUTTON_RAWDATA,ciet_FACEPLATE_BUTTON_DEADBANDS,
                                              ciet_FACEPLATE_BUTTON_CONSTANTS,ciet_FACEPLATE_BUTTON_AUTO_MODE,
                                              ciet_FACEPLATE_BUTTON_FORCED_MODE,ciet_FACEPLATE_BUTTON_SETPOINT, 
                                              ciet_FACEPLATE_BUTTON_TENDPARAM, ciet_FACEPLATE_BUTTON_DEBUG, 
                                              ciet_FACEPLATE_BUTTON_DISABLE, ciet_POPUPMENU_HELP,
                                              ciet_POPUPMENU_WIDGET_SIMULATOR,ciet_POPUPMENU_OPEN_WFIPAGENT, 
                                              ciet_POPUPMENU_OPEN_WFIPSEGMENT, ciet_POPUPMENU_OPEN_CRATE));	
        }  
  
  
//........................................................  old
/*	bool operator, expert, admin;
	int i, length;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
	
//	DebugN("ButtonUserAccess");
	unGenericButtonFunctionsHMI_getUnicosPriviledges(dbPermissions, exceptionInfo);
	if (dynlen(exceptionInfo) > 0)
		{
		dynClear(dbPermissions);
		}
	while (dynlen(dbPermissions) < 4)
		{
		dynAppend(dbPermissions, false);
		}
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString();
	if (operator)
		{
		dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_SELECT, ciet_POPUPMENU_HELP,ciet_POPUPMENU_WIDGET_SIMULATOR,ciet_POPUPMENU_OPEN_WFIPAGENT, ciet_POPUPMENU_OPEN_WFIPSEGMENT, ciet_POPUPMENU_OPEN_CRATE));
		
		}
	if (expert)
		{
			dynAppend(dsAccess, makeDynString(ciet_FACEPLATE_BUTTON_CALIBRATION,ciet_FACEPLATE_BUTTON_CONFIG,UN_FACEPLATE_BUTTON_SELECT,ciet_FACEPLATE_BUTTON_RESET,ciet_FACEPLATE_BUTTON_RAWDATA,ciet_FACEPLATE_BUTTON_DEADBANDS,ciet_FACEPLATE_BUTTON_CONSTANTS,ciet_FACEPLATE_BUTTON_AUTO_MODE,ciet_FACEPLATE_BUTTON_FORCED_MODE,ciet_FACEPLATE_BUTTON_SETPOINT, ciet_FACEPLATE_BUTTON_TENDPARAM, ciet_FACEPLATE_BUTTON_DEBUG, ciet_FACEPLATE_BUTTON_DISABLE, ciet_POPUPMENU_HELP,ciet_POPUPMENU_WIDGET_SIMULATOR,ciet_POPUPMENU_OPEN_WFIPAGENT, ciet_POPUPMENU_OPEN_WFIPSEGMENT, ciet_POPUPMENU_OPEN_CRATE));	
		}
	if (admin)
		{  
			dynAppend(dsAccess, makeDynString(ciet_FACEPLATE_BUTTON_CALIBRATION,ciet_FACEPLATE_BUTTON_CONFIG,UN_FACEPLATE_BUTTON_SELECT,ciet_FACEPLATE_BUTTON_RESET,ciet_FACEPLATE_BUTTON_RAWDATA,ciet_FACEPLATE_BUTTON_DEADBANDS,ciet_FACEPLATE_BUTTON_CONSTANTS,ciet_FACEPLATE_BUTTON_AUTO_MODE,ciet_FACEPLATE_BUTTON_FORCED_MODE,ciet_FACEPLATE_BUTTON_SETPOINT, ciet_FACEPLATE_BUTTON_TENDPARAM, ciet_FACEPLATE_BUTTON_DEBUG, ciet_FACEPLATE_BUTTON_DISABLE, ciet_POPUPMENU_HELP,ciet_POPUPMENU_WIDGET_SIMULATOR,ciet_POPUPMENU_OPEN_WFIPAGENT, ciet_POPUPMENU_OPEN_WFIPSEGMENT, ciet_POPUPMENU_OPEN_CRATE));	
		}	
*/                
//........................................................        
        
}


//---------------------------------------------------------------------------------------------------------------------------------------

// cietEH_ButtonRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in DQAMCMB faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietEH_ButtonRegisterCB(string sDpSystemConnected, bool bSystemConnected)
{
	string deviceName, sSystemName, sDpType, sFunction;
	int iAction, iRes, iManReg01;
	dyn_string exceptionInfo, dsFunctions, dsUserAccess;
//	DebugN("ButtonRegisterCB");
	
	deviceName = unGenericDpFunctions_getDpName($sDpName);
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	
	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
//		DebugN("ButtonRegisterCB UN_ACTION_DISCONNECT");
			cietEH_ButtonDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
//		DebugN("ButtonRegisterCB UN_ACTION_DPCONNECT");
			sDpType="EH";
			unGenericObject_GetFunctions(sDpType, dsFunctions, exceptionInfo);
			sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];
						 
			if (sFunction != "")
				{
				evalScript(dsUserAccess, "dyn_string main(string deviceNameTemp, string sDpTypeTemp) {" + 
							 "dyn_string dsUserAccessReturn;" +
							 "if (isFunctionDefined(\"" + sFunction  + "\"))" +
							 "    {" +
					 			sFunction  + "(deviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
							 "    }" +
							 "return dsUserAccessReturn; }", makeDynString(), deviceName, sDpType);
						
				if (dynlen(dsUserAccess) == 0)
					fwException_raise(exceptionInfo,"ERROR", "cietEH_ButtonRegisterCB: " + getCatStr("unGeneration","UNKNOWNFUNCTION"), sFunction);
				else
					g_dsUserAccess=dsUserAccess;	
				}
			else
				fwException_raise(exceptionInfo,"ERROR", "cietEH_ButtonRegisterCB: " + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");


			iRes = dpConnect("cietEH_ButtonAnimationCB",
							 				 deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							 				 deviceName + ".statusInformation.selectedManager",
							 				 deviceName + ".ProcessInput.Data.sensorStatus");
			g_bCallbackConnected = (iRes >= 0);
			
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
//			DebugN("ButtonRegisterCB UN_ACTION_DPDISCONNECT_DISCONNECT");
			iRes = dpDisconnect("cietEH_ButtonAnimationCB",
							 						deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							 						deviceName + ".statusInformation.selectedManager",
							 						deviceName + ".ProcessInput.Data.sensorStatus");
			g_bCallbackConnected = !(iRes >= 0);
			cietEH_ButtonDisconnection();
			break;
		case UN_ACTION_NOTHING:
//				DebugN("ButtonRegisterCB UN_ACTION_NOTHING");
		default:
			break;
		}
}


//---------------------------------------------------------------------------------------------------------------------------------------

// cietEH_ButtonAnimationCB
/**
Purpose: animate buttons

Parameters: 
	- dbPermissions, dyn_bool, input, list of permissions, length = 4, order : 1=monitor, 2=operator, 3=expert, 4=admin
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietEH_ButtonAnimationCB(string sDpLock, bool bLocked, 
														 string sDpSelect, string sSelectedManager, 
														 string sDpSensorStatus, unsigned uSensorStatus)
{
//	DebugN("ButtonAnimationCB");

	bool bForcedMode, bStatusRawData, bDisable;
	string sDisableText;

	bDisable = getBit (uSensorStatus, 12);
	
	if (bDisable ==1)
			sDisableText=ciet_FACEPLATE_BUTTON_ENABLE;
	else if (bDisable ==0)
			sDisableText=ciet_FACEPLATE_BUTTON_DISABLE;				
			
	setMultiValue("DISABLE_BUTTON.buttonDisable", "text", sDisableText);		
	
	if(StatusRawDataText.text=="true")
				bStatusRawData = 1;
	else if(StatusRawDataText.text=="false")				
				bStatusRawData = 0;
	// DebugN("bStatusRawData=" + bStatusRawData);
		
					
	bForcedMode = getBit(uSensorStatus,3);
	// DebugN("bForcedMode=" + bForcedMode);
	
	cietEH_ButtonSetState(unGenericDpFunctions_getDpName(sDpLock), "EH", g_dsUserAccess, makeDynString(bLocked, sSelectedManager, bForcedMode,bStatusRawData));
}



//---------------------------------------------------------------------------------------------------------------------------------------

// cietEH_ButtonSetState
/**
Purpose: set button state

Parameters: 
	- deviceName, string, input, device
	- sDpType, string, input, dptype
	- dsUserAccess, dyn_string, input, user access
	- dsData, dyn_string, parameter
		
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietEH_ButtonSetState(string deviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
	bool bSelected, buttonEnabled, bit1, bit2, bForcedMode, bStatusRawData;
	dyn_string dsButtons;
	int i, length, alertType;
	string localManager;
	bool bLocked, ListEnabled;
	string sSelectedManager;
	unsigned uSensorStatus;
	
//	DebugN("ButtonSetState");

	bLocked = dsData[1];
	sSelectedManager = dsData[2];
	bForcedMode = dsData[3];
	bStatusRawData = dsData[4];

	
// 1. Selection state
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = (localManager == "S");
	
	dsButtons = makeDynString(ciet_FACEPLATE_BUTTON_CALIBRATION,ciet_FACEPLATE_BUTTON_CONSTANTS,ciet_FACEPLATE_BUTTON_CONFIG,ciet_FACEPLATE_BUTTON_RESET,ciet_FACEPLATE_BUTTON_RAWDATA, ciet_FACEPLATE_BUTTON_SETPRODUCTIONRAWDATA,ciet_FACEPLATE_BUTTON_DEADBANDS, ciet_FACEPLATE_BUTTON_FORCED_MODE,ciet_FACEPLATE_BUTTON_AUTO_MODE,ciet_FACEPLATE_BUTTON_SETPOINT, ciet_FACEPLATE_BUTTON_TENDPARAM, ciet_FACEPLATE_BUTTON_DEBUG, ciet_FACEPLATE_BUTTON_DISABLE); // Each button except "Select"
	length = dynlen(dsButtons);
	
// 1. Selection state
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = (localManager == "S");
	
// 2. Button state


	for(i=1;i<=length;i++)
		{
		buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0);	// User access
		switch (dsButtons[i])
			{
			case ciet_FACEPLATE_BUTTON_RESET:
				buttonEnabled = buttonEnabled && bSelected;				
				break;
				
			case ciet_FACEPLATE_BUTTON_CONSTANTS:
				buttonEnabled = buttonEnabled && bSelected;
				break;

			case ciet_FACEPLATE_BUTTON_CALIBRATION:
				buttonEnabled = buttonEnabled && bSelected;
				break;

			case ciet_FACEPLATE_BUTTON_DEADBANDS:
				buttonEnabled = buttonEnabled && bSelected;
				break;				
				
			case ciet_FACEPLATE_BUTTON_CONFIG:
				buttonEnabled = buttonEnabled && bSelected;
				break;
			case ciet_FACEPLATE_BUTTON_SETPRODUCTIONRAWDATA:
				buttonEnabled = buttonEnabled && bSelected;
				break;		
			case ciet_FACEPLATE_BUTTON_RAWDATA:
				buttonEnabled = buttonEnabled && bSelected && !bStatusRawData;
			//	DebugN ("Button Status = " + bStatusRawData );
				break;							
				
			case ciet_FACEPLATE_BUTTON_FORCED_MODE:
				buttonEnabled = buttonEnabled && bSelected && !bForcedMode; 
				break;				
				
			case ciet_FACEPLATE_BUTTON_AUTO_MODE:
				buttonEnabled = buttonEnabled && bSelected && bForcedMode;
				break;				

			case ciet_FACEPLATE_BUTTON_SETPOINT:
				buttonEnabled = buttonEnabled && bSelected && bForcedMode;
				break;		
							
			case ciet_FACEPLATE_BUTTON_TENDPARAM:
				buttonEnabled = buttonEnabled && bSelected; 
				break;				
				
			case ciet_FACEPLATE_BUTTON_DEBUG:
				buttonEnabled = buttonEnabled && bSelected;
				break;				

			case ciet_FACEPLATE_BUTTON_DISABLE:
				buttonEnabled = buttonEnabled && bSelected;
				break;		
								
			default:
				break;
			}
		setValue(ciet_FACEPLATE_BUTTON_PREFIX + dsButtons[i], "enabled", buttonEnabled);
		}
// 3. Button Select state
	unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
}


//---------------------------------------------------------------------------------------------------------------------------------------

// cietEH_UserLoginGetButtonState
/**
Purpose: function called before the button set state when a user logs in. 

Parameters:
	deviceName: string, input, the device name
	dsReturnData: dyn_string, output, the data are returned here

Usage: Internal function

PVSS manager usage:  Ctrl, NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
		-> $sDpName, string, the device DP name
		-> g_sFaceplateButtonType, string, faceplate type
	. PVSS version: 3.0 
	. operating system: NT, W2000 and Linux, but tested only under W2000.
	. distributed system: yes but not tested.
*/

cietEH_UserLoginGetButtonState(string deviceName, string sType, dyn_string &dsReturnData)
{
	
		int iRes;
	string sDpLock, sDpSelectedManager, sSelectedManager, sSystemName,sSensorStatusDp;
	bool bLocked;
	unsigned uSensorStatus;
	
//	DebugN("UserLoginGetButtonState");
// 2. Get parameters to call the set button function
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);

	sDpLock = deviceName + ".statusInformation.selectedManager:_lock._original._locked";
	sDpSelectedManager = deviceName + ".statusInformation.selectedManager";
	sSensorStatusDp = deviceName + ".ProcessInput.Data.sensorStatus";
	
	if (dpExists(deviceName))
	{
		iRes = dpGet(sDpLock, bLocked, sDpSelectedManager, sSelectedManager, sSensorStatusDp,uSensorStatus);
	}
	else
	{
		iRes = -1;
	}
	if (iRes < 0) // Errors during dpget
	{
		sDpLock = "";
		bLocked = false;
		sDpSelectedManager = "";
		sSelectedManager = "";
		sInvalidDp = "";
		bInvalid = false;
	}
	dsReturnData=makeDynString(bLocked, sSelectedManager, getBit(uSensorStatus,3), "", "");
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietEH_ButtonDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietEH_ButtonDisconnection()
{
			   
//	DebugN("ButtonDisconnection");
		unGenericObject_ButtonDisconnect(makeDynString(ciet_FACEPLATE_BUTTON_CALIBRATION,ciet_FACEPLATE_BUTTON_RESET,ciet_FACEPLATE_BUTTON_RAWDATA, ciet_FACEPLATE_BUTTON_SETPRODUCTIONRAWDATA, ciet_FACEPLATE_BUTTON_CONFIG,ciet_FACEPLATE_BUTTON_DEADBANDS, ciet_FACEPLATE_BUTTON_CALIBRATION, ciet_FACEPLATE_BUTTON_CONSTANTS, ciet_FACEPLATE_BUTTON_FORCED_MODE, ciet_FACEPLATE_BUTTON_AUTO_MODE, ciet_FACEPLATE_BUTTON_SETPOINT,UN_FACEPLATE_BUTTON_SELECT, ciet_FACEPLATE_BUTTON_TENDPARAM, ciet_FACEPLATE_BUTTON_DEBUG, ciet_FACEPLATE_BUTTON_DISABLE));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietEH_WidgetRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in PT widget
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietEH_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, deviceType;
	int iAction,iRes;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName, sFrontEnd;
	bool bRemote;

	//DebugN("FUNCTION: cietEH_WidgetRegisterCB");
	//DebugN("g_bDisconnectCB="+g_bDisconnectCB);
	
	deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);

	g_bUnSystemAlarmPlc = false;
// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	
	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;

	
	if(sPlcName != "") {
		if (dpExists(sSystemName+c_CRYO_FRONTEND_UNSYSTEMALARM+c_cietSystemIntegrity_DS+sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}

	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			cietEH_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:
			g_sDpName = deviceName;
			g_bDisconnectCB = false;
		//cietEH_WidgetConnect(deviceName, sPlcName); 
	
		if(g_bUnSystemAlarmPlc) {
					deviceType = dpTypeName(deviceName);
				if(deviceType == "EH")
					iRes = dpConnect("cietEH_WidgetLockCallBack", deviceName + ".ProcessInput.Data.powerCalc:_lock._alert_hdl._locked");
				else{
					g_bUnSystemAlarmPlc = false;
					cietEH_WidgetDisconnection(g_sWidgetType);
				}
	
				g_bCallbackConnected = (iRes >= 0);
			}
			else
					cietEH_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:

			if(g_bUnSystemAlarmPlc) 
				iRes = dpDisconnect("cietEH_WidgetLockCallBack", deviceName + ".ProcessInput.Data.powerCalc:_lock._alert_hdl._locked");

			if(g_bDisconnectCB) {
						cietEH_WidgetDisconnect(g_sDpName, sPlcName);
				}
			g_bCallbackConnected = !(iRes >= 0);
			g_bDisconnectCB = false;

			cietEH_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}
//---------------------------------------------------------------------------------------------------------------------------------------
// cietEH_WidgetLockCallBack
/**
Purpose: set up the lock callback function for the widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in PT widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietEH_WidgetLockCallBack(string sdpe, bool bLockState)
{
	int iRes;
	string sPlcName;

	if(bLockState) 
		{ // lock set so disconnect if not already disconnected
		if(g_bDisconnectCB) 
			{
				cietEH_WidgetDisconnect(g_sDpName, iRes);
			}
		g_bDisconnectCB = false;
		cietEH_WidgetDisconnection(g_sWidgetType);
		}
	else 
		{ // lock not set so connect if not already connected
		g_bDisconnectCB = true;
		sPlcName = unGenericObject_GetPLCNameFromDpName(sdpe);
		cietEH_WidgetConnect(g_sDpName,sPlcName);
		}
}
//---------------------------------------------------------------------------------------------------------------------------------------

// cietEH_WidgetConnect
/**
Purpose: Connect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in EH faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietEH_WidgetConnect(string deviceName, string sPlcName)
{
	int iRes;
	string sSystemName, sAlias,sAgentName;
	
	
	//DebugN("FUNCTION: cietEH_WidgetConnect(deviceName="+deviceName+" sPlcName="+sPlcName);
	
	dyn_string dsDeviceNameSplit;
//  check if the device name is of type "PT" if not like of no PLC. -> not available.
	if(dpTypeName(deviceName) != "EH")
		g_bUnSystemAlarmPlc = false;
	
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	
	if(g_bUnSystemAlarmPlc)
	{
		sAgentName = cietGenericFunctions_getAgentName(deviceName);
//		sAlias = unGenericDpFunctions_getAlias(sAgentName);
	//	dsDeviceNameSplit=strsplit(sAlias, ".");
	
		iRes = dpConnect("cietEH_WidgetAnimationCB", 
						 					deviceName + ".ProcessInput.Data.powerCalc",
						 					deviceName + ".ProcessInput.Data.sensorStatus",
						 					deviceName + ".ST_GTWDEV",
						 					deviceName + ".ProcessInput.Data.setPoint",
						 					deviceName + ".ProcessInput.Data.forcedSetPoint",						 											 											 					
				 							sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".enabled",
								/*			sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".enabled",*/
							/*							sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+sAlias+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+sAlias+".enabled",*/
											deviceName + ".ProcessInput.Data.powerCalc:_online.._invalid", 
									 		deviceName + ".statusInformation.selectedManager:_lock._original._locked",
						 					deviceName + ".statusInformation.selectedManager");
		g_bCallbackConnected = (iRes >= 0);		
	}else{
		cietEH_WidgetDisconnection(g_sWidgetType);
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietEH_WidgetDisconnect
/**
Purpose: Disconnect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in EH faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietEH_WidgetDisconnect(string deviceName, string sPlcName)
{
	int iRes;
	string sSystemName, sAlias,sAgentName;
	dyn_string dsDeviceNameSplit;
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);

		
	if(g_bUnSystemAlarmPlc)
	{
		sAgentName = cietGenericFunctions_getAgentName(deviceName);
//		sAlias = unGenericDpFunctions_getAlias(sAgentName);
//		dsDeviceNameSplit=strsplit(sAlias, ".");
		iRes = dpDisconnect("cietEH_WidgetAnimationCB", 
						 					deviceName + ".ProcessInput.Data.powerCalc",
						 					deviceName + ".ProcessInput.Data.sensorStatus",
						 					deviceName + ".ST_GTWDEV",
						 					deviceName + ".ProcessInput.Data.setPoint",
						 					deviceName + ".ProcessInput.Data.forcedSetPoint",							 					
			 							sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".enabled",
/*											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".enabled",*/
/*												sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+sAlias+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+sAlias+".enabled",*/
											deviceName + ".ProcessInput.Data.powerCalc:_online.._invalid", 
									 		deviceName + ".statusInformation.selectedManager:_lock._original._locked",
						 					deviceName + ".statusInformation.selectedManager");
		
		g_bCallbackConnected = (iRes >= 0);
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietEH_WidgetAnimationCB
/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in EH faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietEH_WidgetAnimationCB(string sDp_powerCalc, float fpowerCalc,
												string sDp_sensorStatus, unsigned usensorStatus,
												string sDp_ST_GTWDEV, bool bST_GTWDEV,
												string sDp_setPoint, float fsetPoint,
												string sDp_forcedSetPoint, float fforcedSetPoint,
											 string sDpSystemIntegrityAlarmValue, int iSystemIntegrityAlarmValue, 
											 string sDpSystemIntegrityAlarmEnabled, bool bSystemIntegrityAlarmEnabled,
									/*		 string sDpSystemAlarmValue, int iSystemAlarmValue,
											 string sDpSystemAlarmEnabled, bool bSystemAlarmEnabled,*/
											 string sDpCodeInvalid, bool bCodeInvalid,
											 string sDpLocked, bool bLocked,
											 string sDpSelectedManager, string sSelectedManager)
{

	string formattedValue, sFormat, sUnit;
	string warningLetter, warningColor, alarmLetter, alarmColor, controlLetter, controlColor, selectColor;
	string display1Text, display1Color, display2Text, display2Color; 
	string Body1Backcolor, Body1Forecolor, Body2Backcolor, Body3Backcolor, Body4Backcolor,	Body4Forecolor, Body5Backcolor,	 Body5Forecolor, controlStateColor, overTemperatureColor, overTemperatureLetter;
	bool lockVisible, controlStateVisible,warningVisible, alarmTextVisible, eventStateVisible, overTemperatureVisible;
	
	//DebugN("FUNCTION: cietEH_WidgetAnimationCB(usensorStatus="+usensorStatus+")");

	sFormat	= dpGetFormat(sDp_forcedSetPoint);
	sUnit  	= dpGetUnit(sDp_forcedSetPoint);		
	display1Color = "unDisplayValue_Status";														// Display 1 -> setPoint / Forced setPoint
	
  if(getBit(usensorStatus, 3)==1)
  	formattedValue = unGenericObject_FormatValue(sFormat, fforcedSetPoint);
  else
  	formattedValue = unGenericObject_FormatValue(sFormat, fsetPoint);
  	
  // DebugN("HEATER: fsetPoint= "+fsetPoint+" ;  fforcedSetPoint= "+fforcedSetPoint+"  ::: formattedValue= "+formattedValue);
  if(getBit(usensorStatus, 13)==0){	
    display1Text=formattedValue+ " "+sUnit;	
  }
  //Forced Production RawData
  else{
    display1Text="N/S";
  }

	sFormat	= dpGetFormat(sDp_powerCalc);
	sUnit  	= dpGetUnit(sDp_powerCalc);		
	formattedValue = unGenericObject_FormatValue(sFormat, fpowerCalc);
  display2Text = formattedValue + " " + sUnit;		
										
	display2Color = "unDisplayValue_Output";				

  if(getBit(usensorStatus, 3)==1 || getBit(usensorStatus, 13)==1)																	// Control State Text -> Forced Mode / Normal Mode
  	{
  	 controlStateVisible=true; 
  	 controlStateColor="unWidget_ControlStateForced";
  	} 
  else
  	controlStateVisible=false;
  	
  	
  if(getBit(usensorStatus, 4)==0 && getBit(usensorStatus, 6)==0)	// Card Protection and Heater Over Temperature
  	{
  	 overTemperatureVisible=false; 
  	 overTemperatureColor="cietWidget_CardProtection";
  	 overTemperatureLetter=" ";
  	} 
  else if (getBit(usensorStatus, 4)==1 && getBit(usensorStatus, 6)==0)	
  	{
  	 overTemperatureVisible=true; 
  	 overTemperatureColor="cietWidget_CardProtection";
  	 overTemperatureLetter=ciet_EH_WIDGET_ANIMATION_CARD_OVER_TEMPERATURE_LETTER;
  	}
  else if (getBit(usensorStatus, 4)==0 && getBit(usensorStatus, 6)==1)
  	{
  	 overTemperatureVisible=true; 
  	 overTemperatureColor="cietWidget_CardProtection";
  	 overTemperatureLetter=ciet_EH_WIDGET_ANIMATION_HEATER_OVER_TEMPERATURE_LETTER;
  	}  
  else if (getBit(usensorStatus, 4)==1 && getBit(usensorStatus, 6)==1)
  	{
  	 overTemperatureVisible=true; 
  	 overTemperatureColor="cietWidget_CardProtection";
  	 overTemperatureLetter=ciet_EH_WIDGET_ANIMATION_HEATER_AND_CARD_OVER_TEMPERATURE_LETTER;
  	}
   
  // Staff not used
  eventStateVisible=false;																			  // event State -> Not used
  alarmTextVisible=false;																			  	// Alarm       -> Not used 
	alarmColor="unFaceplate_AlarmActive";
	alarmLetter=" ";
 
  // BODY ANIMATION 

 		 Body1Backcolor="unWidget_ControlStateForced"; /* unWidget_Background */		
 		 
	// case for forced production RawData, just one case because we do not take in account the power to animate the widget (empty, half full, full()
 if(getBit(usensorStatus, 13)==1 )
  	{
  		//DebugN("Heater animation -> 20w<powerCalc && Forced Mode");
  		Body1Backcolor="unWidget_ControlStateForced";
 		  Body1Forecolor="unWidget_ControlStateForced"; 
  		Body2Backcolor="unWidget_ControlStateForced";
  		Body3Backcolor="unWidget_ControlStateForced"; 
 		  Body4Forecolor="unWidget_Background"; 
 		  Body5Forecolor="unWidget_Background";   		 		
  	}
  	  // Heater animation -> 20w<=powerCalc && Forced Mode
	else if(fpowerCalc>=ciet_EH_WIDGET_ANIMATION_MAXVALUE_powerCalc && getBit(usensorStatus, 3)==1 )
  	{
  		//DebugN("Heater animation -> 20w<powerCalc && Forced Mode");
  		Body1Backcolor="unWidget_ControlStateForced";
 		  Body1Forecolor="unWidget_ControlStateForced"; 
  		Body2Backcolor="unWidget_ControlStateForced";
  		Body3Backcolor="unWidget_ControlStateForced"; 
 		  Body4Forecolor="unWidget_Background"; 
 		  Body5Forecolor="unWidget_Background";   		 		
  	}
  	  	
  // Heater animation -> 20w<=powerCalc && Normal Mode
	else if(fpowerCalc>=ciet_EH_WIDGET_ANIMATION_MAXVALUE_powerCalc && getBit(usensorStatus, 3)==0 )
  	{
  		//DebugN("Heater animation -> 20w<powerCalc && Normal Mode");
  		Body1Backcolor="unWidget_ControlStateAuto";
 		  Body1Forecolor="unWidget_ControlStateAuto";   		
  		Body2Backcolor="unWidget_ControlStateAuto";
  		Body3Backcolor="unWidget_ControlStateAuto";
  		Body4Forecolor="unWidget_Background"; 
 		  Body5Forecolor="unWidget_Background";   		 		
  	}
  	  	
  // Heater animation -> 5w<powerCalc<20w && Forced Mode  	
	else if(fpowerCalc>ciet_EH_WIDGET_ANIMATION_MINVALUE_powerCalc && fpowerCalc<ciet_EH_WIDGET_ANIMATION_MAXVALUE_powerCalc && getBit(usensorStatus, 3)==1 ) 
	 	{
	 		//DebugN("Heater animation -> 5w<powerCalc>20w && Forced Mode");
	 		Body1Backcolor="unWidget_ControlStateForced";
 		  Body1Forecolor="unWidget_ControlStateForced"; 	 		
  		Body2Backcolor="unWidget_ControlStateForced";
  		Body3Backcolor="unWidget_Background";
 		  Body4Forecolor="unWidget_Background"; 
 		  Body5Forecolor="unWidget_ControlStateForced";   		  		
  	}
  	
  // Heater animation -> 5w<powerCalc<20w && Normal Mode  	
	else if(fpowerCalc>ciet_EH_WIDGET_ANIMATION_MINVALUE_powerCalc && fpowerCalc<ciet_EH_WIDGET_ANIMATION_MAXVALUE_powerCalc && getBit(usensorStatus, 3)==0 ) 
	 	{
	 		//DebugN("Heater animation -> 5w<powerCalc>20w && Normal Mode");
  		Body1Backcolor="unWidget_ControlStateAuto";
 		  Body1Forecolor="unWidget_ControlStateAuto";	 		
  		Body2Backcolor="unWidget_ControlStateAuto";
  		Body3Backcolor="unWidget_Background";
 		  Body4Forecolor="unWidget_Background"; 
 		  Body5Forecolor="unWidget_ControlStateAuto";   		  		
  	}  	
  	
  // Heater animation -> powerCalc<=5w && Forced Mode  	
	else if(fpowerCalc<=ciet_EH_WIDGET_ANIMATION_MINVALUE_powerCalc && getBit(usensorStatus, 3)==1)										
  	{
  		//DebugN("Heater animation -> powerCalc<5w && Forced Mode");
  		Body1Backcolor="unWidget_ControlStateForced";
 		  Body1Forecolor="unWidget_ControlStateForced";   		
  		Body2Backcolor="unWidget_Background";
  		Body3Backcolor="unWidget_Background";
 		  Body4Forecolor="unWidget_ControlStateForced"; 
 		  Body5Forecolor="unWidget_ControlStateForced";   		  		
  	}
  	
  // Heater animation -> powerCalc<=5w && Normal Mode	  
	else if(fpowerCalc<=ciet_EH_WIDGET_ANIMATION_MINVALUE_powerCalc && getBit(usensorStatus, 3)==0)										
  	{
  		//DebugN("Heater animation -> powerCalc<5w && Normal Mode");
  		Body1Backcolor="unWidget_ControlStateAuto";  		
 		  Body1Forecolor="unWidget_ControlStateAuto";  		
  		Body2Backcolor="unWidget_Background";
  		Body3Backcolor="unWidget_Background";
 		  Body4Forecolor="unWidget_ControlStateAuto"; 
 		  Body5Forecolor="unWidget_ControlStateAuto";   		  		
  	}  		  	 
	
	// WARNING status
	
		if (getBit(usensorStatus, 12)==1)			// Disable
		{
			display1Color = "cietWidget_Disabled";
			display2Color = "cietWidget_Disabled";		
		 	Body1Backcolor="unWidget_Background";  		
		  Body1Forecolor= "cietWidget_Disabled";		
			Body2Backcolor="unWidget_Background";
			Body3Backcolor="unWidget_Background";
		  Body4Forecolor= "cietWidget_Disabled";
		  Body5Forecolor= "cietWidget_Disabled"; 				

			warningLetter = ciet_WIDGET_TEXT_WARNING_DISABLE;
			warningColor = "cietWidget_Warning";	
			warningVisible=true;
										
			if (getBit(usensorStatus, 0)==1)			// I/O Error
				{
					warningLetter = UN_WIDGET_TEXT_WARNING_ERROR + ciet_WIDGET_TEXT_WARNING_DISABLE;
					warningColor = "cietWidget_Warning";	
					warningVisible=true;
				}	
	  			
		}
	else
		{
			if (bCodeInvalid)																						// Invalid 
				{
					warningLetter = UN_WIDGET_TEXT_INVALID;
					warningColor = "unDataNotValid";
					warningVisible=true;
					display1Color = "unDataNotValid";
					display2Color = "unDataNotValid";
		  		Body1Backcolor="unDataNotValid";  		
		 		  Body1Forecolor="unDataNotValid";  		
		  		Body2Backcolor="unWidget_Background";
		  		Body3Backcolor="unWidget_Background";
		 		  Body4Forecolor="unDataNotValid"; 
		 		  Body5Forecolor="unDataNotValid";  			
				}	
		
			else	
				{
				  if (getBit(usensorStatus, 0)==1)													// I/O Error
						{
							warningLetter = UN_WIDGET_TEXT_WARNING_ERROR;
							warningColor = "cietWidget_Warning";	
							warningVisible=true;
							display1Color = "unFaceplate_AlarmActive";  	
				 			display2Color = "unFaceplate_AlarmActive";					
						}
					else
						{
				  	warningVisible=false;																		// OK
						display1Color = "unDisplayValue_Output";
						display2Color = "unDisplayValue_Status";
						}					  	
		
			  	if ((!bSystemIntegrityAlarmEnabled) || (iSystemIntegrityAlarmValue >= CRYO_SYSTEM_INGETRITY_ALARM_LIMIT))  // Old Data
						{
							warningLetter = UN_WIDGET_TEXT_OLD_DATA;
							warningColor = "unDataNotValid";
							warningVisible=true;
							overTemperatureColor = "unDataNotValid";
							overTemperatureVisible=true;			
						}
					
				  if (bST_GTWDEV==0)																				// Status Gateway Device 
						{
					 	 warningLetter= ciet_WIDGET_TEXT_STATUS_GATEWAY_DEVICE;  	   	 
				  	 warningColor="cietWidget_Warning";
				  	 warningVisible=true; 
						}					
			
				}		
		}
	 
	// 3. set the widget.
	unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, selectColor, lockVisible);

	//DebugN("bLocked, sSelectedManager, selectColor,lockVisible", bLocked, sSelectedManager, selectColor,lockVisible);

	// 8. Animate widget
	//DebugN("body1Text (setpoint)= "+body1Text);
	//DebugN("display2Text (read value)= "+display2Text);
	if(g_bSystemConnected)			
		setMultiValue("Display1", "text", display1Text,
					"Display1", "foreCol", display1Color, 				
					"eventState", "visible", eventStateVisible,						
					"AlarmText", "visible", alarmTextVisible,
		      "Display2", "text", display2Text,
		      "Display2", "foreCol", display2Color, 
 		      "Border1", "backCol", Body1Backcolor,		      
 		      "Border1", "foreCol", Body1Forecolor, 		      
 		      "Body2", "fill", "[solid]",
 		      "Body2", "backCol", Body2Backcolor,
 		      "Body3", "fill", "[solid]",
 		      "Body3", "backCol", Body3Backcolor,
 		      "Body4", "backCol", Body4Backcolor,		       		      
 		      "Body4", "foreCol", Body4Forecolor, 		       		      
 		      "Body5", "backCol", Body5Backcolor,		      
 		      "Body5", "foreCol", Body5Forecolor, 		    
		      "ControlStateText", "visible", controlStateVisible,
		      "ControlStateText", "foreCol", controlStateColor,
		      "WarningText", "visible", warningVisible,
				  "WarningText", "text", warningLetter, 
				  "WarningText", "foreCol", warningColor,
				  "OverTemperatureText", "text", overTemperatureLetter,				  
				  "OverTemperatureText", "foreCol", overTemperatureColor,
				  "OverTemperatureText", "visible", overTemperatureVisible,
				  "SelectArea", "foreCol", selectColor, 
				  "LockBmp", "visible", lockVisible, 
				  "WidgetArea", "visible", true);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietEH_WidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
	- sWidgetType
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietEH_WidgetDisconnection(string sWidgetType)
{
								
//DebugN("WidgetDisconnection EH");
								
		setMultiValue(
					"Display1", "foreCol", "unDataNoAccess", 				
					"eventState", "visible", false,						
					"AlarmText", "visible", false,
		      "Display2", "foreCol", "unDataNoAccess", 
 		      "Border1", "backCol", "unWidget_Background",		      
 		      "Border1", "foreCol", "unDataNoAccess", 					    		      
 		      "Body2", "fill", "[solid]",
 		      "Body2", "backCol", "unWidget_Background",
 		      "Body3", "fill", "[solid]",
 		      "Body3", "backCol", "unWidget_Background",
 		      "Body4", "backCol", "unWidget_Background",		       		      
 		      "Body4", "foreCol", "unDataNoAccess", 		       		      
 		      "Body5", "backCol","unWidget_Background",		      
 		      "Body5", "foreCol", "unDataNoAccess", 		    
		      "ControlStateText", "visible", false,
		      "WarningText", "visible",false,
				  "WarningText", "text", "", 
				  "OverTemperatureText", "visible", false,
				  "LockBmp", "visible", false, 
	  			"WidgetArea", "visible", false
	  			);
				  						
}								


//@}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietEH_PanelWidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
	- sWidgetType
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietEH_PanelWidgetDisconnection(string sWidgetType)
{
	string warningText;
//DebugN("PanelWidgetDisconnection");
	warningText = UN_WIDGET_TEXT_INVALID;

	setMultiValue("Body1", "text", "",
								"WarningText", "text", warningText,
								"LockBmp", "visible", false);
}								

//------------------------------------------------------------------------------------------------------------------------
//@}

// cietEH_MenuConfiguration
/**
Purpose: configuration of popup menu

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsAccessOk, dyn_string, input, authorized action
	- menuList, dyn_string, output, menu to pop-up
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 3.0 
	. distributed system: yes.
*/

cietEH_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{

dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, 
								 ciet_POPUPMENU_RESET,ciet_POPUPMENU_CONFIG,ciet_POPUPMENU_DEADBANDS, 
								 ciet_POPUPMENU_FORCED_MODE,ciet_POPUPMENU_AUTO_MODE, ciet_POPUPMENU_SETPOINT,
								 ciet_POPUPMENU_CONSTANTS, ciet_POPUPMENU_CALIBRATION,ciet_POPUPMENU_DISABLE, ciet_POPUPMENU_TENDPARAM, ciet_POPUPMENU_DEBUG,
								 UN_POPUPMENU_FACEPLATE_TEXT, ciet_POPUPMENU_HELP, ciet_POPUPMENU_WIDGET_SIMULATOR, UN_POPUPMENU_TREND_TEXT,
								 ciet_POPUPMENU_OPEN_WFIPAGENT, ciet_POPUPMENU_OPEN_WFIPSEGMENT, ciet_POPUPMENU_OPEN_CRATE);
int granted;
bool bForcedMode;
unsigned uSensorStatus;
string deviceName;

int iUnicosPopupMenuMin = UN_POPUPMENU_MIN;

string localManager, sSelectedManager;
bool bSelected, bLocked;

//  Selection state

	dpGet(sDpName + ".statusInformation.selectedManager:_lock._original._locked", bLocked);
	dpGet(sDpName + ".statusInformation.selectedManager",sSelectedManager);
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = (localManager == "S");
								 
// Select
	unGenericObject_addSelectToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	dynAppend(menuList, "SEPARATOR");

	// 1. Reset
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_RESET) > 0)
	{
		granted = 0;
		if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_RESET) > 0) && bSelected)
		{
			granted = 1;
		}
		dynAppend(menuList,"PUSH_BUTTON, Reset, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_RESET)+", " + granted);
	}

	
	// 2. Config
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_CONFIG) > 0)
		{
			granted = 0;
			if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_CONFIG) > 0) && bSelected)
				{
					granted = 1;
				}
			dynAppend(menuList, "PUSH_BUTTON, WFip Config,"+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_CONFIG)+", " + granted);
		}
		
	// 3. Disable
	
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_DISABLE) > 0)
	{
		granted = 0;
		if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_DISABLE) > 0 ) && bSelected)
		{
			granted = 1;
		}
		dynAppend(menuList, "PUSH_BUTTON, Enable/Disable, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_DISABLE)+", " + granted);
	}				
		
	// 4. Debug
	
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_DEBUG) > 0)
	{
		granted = 0;
		if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_DEBUG) > 0 ) && bSelected)
		{
			granted = 1;
		}
		dynAppend(menuList, "PUSH_BUTTON, Debug, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_DEBUG)+", " + granted);
	}				
		
  // Separator
	if (dynlen(menuList) > 0)
	{
		if (menuList[dynlen(menuList)] != "SEPARATOR")
		{
			dynAppend(menuList, "SEPARATOR");
		}
	}
			
	// 5. DeadBands
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_DEADBANDS) > 0)
	{
		granted = 0;
		if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_DEADBANDS) > 0) && bSelected)
		{
			granted = 1;
		}
		dynAppend(menuList, "PUSH_BUTTON, DeadBands, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_DEADBANDS)+", " + granted);
	}		
	
	// 6. Set Constants	
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_CONSTANTS) > 0)
	{
		granted = 0;
		if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_CONSTANTS) > 0) && bSelected)
		{
			granted = 1;
		}
		dynAppend(menuList, "PUSH_BUTTON, Set Constants, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_CONSTANTS)+", " + granted);
	}	
	
	// 7. Get Calibration
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_CALIBRATION) > 0)
	{
		granted = 0;
		if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_CALIBRATION) > 0) && bSelected)
		{
			granted = 1;
		}
		dynAppend(menuList, "PUSH_BUTTON, Get Calibration, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_CALIBRATION)+", " + granted);
	}		
	
	//11. Tendency
	
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_TENDPARAM) > 0)
	{
		granted = 0;
		if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_TENDPARAM) > 0) && bSelected)
		{
			granted = 1;
		}
		dynAppend(menuList, "PUSH_BUTTON, Tendency, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_TENDPARAM)+", " + granted);
	}	
	
  // Separator
	if (dynlen(menuList) > 0)
	{
		if (menuList[dynlen(menuList)] != "SEPARATOR")
		{
			dynAppend(menuList, "SEPARATOR");
		}
	}	
	
	// 8. ForcedMode
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_FORCED_MODE) > 0)
	{
		granted = 0;
		if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_FORCED_MODE) > 0) && bSelected)
		{
			granted = 1;
		}
		dynAppend(menuList, "PUSH_BUTTON, Forced Mode, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_FORCED_MODE)+", " + granted);
	}		
	
	// 9. Auto Mode
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_AUTO_MODE) > 0)
	{
		granted = 0;
		if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_AUTO_MODE) > 0) && bSelected)
		{
			granted = 1;
		}
		dynAppend(menuList, "PUSH_BUTTON, Auto Mode, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_AUTO_MODE)+", " + granted);
	}			
	
	// 10. SetPoint
	
	deviceName = unGenericDpFunctions_getDpName(sDpName);								
	dpGet(deviceName + ".ProcessInput.Data.sensorStatus",uSensorStatus);
	bForcedMode = getBit (uSensorStatus, 3);
	
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_SETPOINT) > 0)
	{
		granted = 0;
		if (dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_SETPOINT) > 0 && bForcedMode && bSelected)
		{
			granted = 1;
		}
		dynAppend(menuList, "PUSH_BUTTON, SetPoint, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_SETPOINT)+", " + granted);
	}	
	
  // Separator
	if (dynlen(menuList) > 0)
	{
		if (menuList[dynlen(menuList)] != "SEPARATOR")
		{
			dynAppend(menuList, "SEPARATOR");
		}
	}
// Open Agent
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_OPEN_WFIPAGENT) > 0)
		{
			granted = 0;
			if (dynContains(dsAccessOk, ciet_POPUPMENU_OPEN_WFIPAGENT) > 0)
				{
					granted = 1;
				}
			dynAppend(menuList,"PUSH_BUTTON, WFIPAgent, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_OPEN_WFIPAGENT)+", " + granted);
		}
		
// Open Segment	
if (dynContains(dsMenuConfig, ciet_POPUPMENU_OPEN_WFIPSEGMENT) > 0)
{
	granted = 0;
	if (dynContains(dsAccessOk, ciet_POPUPMENU_OPEN_WFIPSEGMENT) > 0)
		{
			granted = 1;
		}
	dynAppend(menuList,"PUSH_BUTTON, WFIPSegment, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_OPEN_WFIPSEGMENT)+", " + granted);
}
	
// Open Crate	
if (dynContains(dsMenuConfig, ciet_POPUPMENU_OPEN_CRATE) > 0)
{
	granted = 0;
	if (dynContains(dsAccessOk, ciet_POPUPMENU_OPEN_CRATE) > 0)
		{
			granted = 1;
		}
	dynAppend(menuList,"PUSH_BUTTON, Crate, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_OPEN_CRATE)+", " + granted);
}
	
// Default Unicos action
	unGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	if (dynlen(menuList) > 0)
	{
		if (menuList[dynlen(menuList)] != "SEPARATOR")
		{
			dynAppend(menuList, "SEPARATOR");
		}
	}
	
// 13. Help

	if (dynlen(menuList) > 0)
		{
			if (menuList[dynlen(menuList)] != "SEPARATOR")
			{
				dynAppend(menuList, "SEPARATOR");
			}
		}

	if (dynContains(dsMenuConfig, ciet_POPUPMENU_HELP) > 0)
		{
			granted = 0;
			if (dynContains(dsAccessOk, ciet_POPUPMENU_HELP) > 0)
				{
					granted = 1;
				}
			dynAppend(menuList,"PUSH_BUTTON, Help, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_HELP)+", " + granted);
		}	

	if (dynContains(dsMenuConfig, ciet_POPUPMENU_WIDGET_SIMULATOR) > 0)
		{
			granted = 0;
			if (dynContains(dsAccessOk, ciet_POPUPMENU_WIDGET_SIMULATOR) > 0)
				{
					granted = 1;
				}
			dynAppend(menuList,"PUSH_BUTTON, Widget Simulator, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_WIDGET_SIMULATOR)+", " + granted);
		}		
	
// Trend Action
	unGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);


	
	if (dynlen(menuList) > 0)
	{
		if (menuList[dynlen(menuList)] != "SEPARATOR")
		{
			dynAppend(menuList, "SEPARATOR");
		}
	}

}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietEH_HandleMenu
/**
Purpose: handle the answer of the popup menu

Parameters:
	- deviceName, string, input, the device dp name
	- sDpType, string, input, the device dp type
	- menuList, dyn_string, input, list of requested action
	- menuAnswer, int, input, selected action from the menuList
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 3.0 
	. distributed system: yes.
*/
cietEH_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
{

dyn_string exceptionInfo;
string descripName, sDpAgentName,sDpGTWName/*, deviceName*/;
string sAgentName,sBusName;
dyn_string dsFIPInfos;
bool bRes;
int iAgentNumber, iRes;
float fCurrentSetPoint;

int iUnicosPopupMenuMin = UN_POPUPMENU_MIN;

//DebugN("HandleMenu", deviceName, sDpType, menuList, menuAnswer);

//deviceName = unGenericDpFunctions_getDpName($sDpName); 
// handle the select
	unGenericObject_handleSelect(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);
// handle the UNICOS default (open faceplate, diagnostic and info)
	unGenericObject_handleDefaultUnicos(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);
// handle the ciet device actions
	switch (menuAnswer-iUnicosPopupMenuMin)
	{

		case ciet_POPUPMENU_ORDER_RESET:					// Reset
		
   		cietGenericObject_resetDevice(deviceName, exceptionInfo);
			break;


		case ciet_POPUPMENU_ORDER_CONFIG:					// set config
			//DebugN("Right - Click Set Config");
  		 /// open Set Config Panel			
			ChildPanelOnModal("vision/cietDevices/General/cietSetConfig.pnl",
						  "WFip Conf: " + unGenericDpFunctions_getAlias(deviceName),
						  makeDynString("$sDpName:"+deviceName),0, 0);			
			
		break;

		case ciet_POPUPMENU_ORDER_DISABLE:					// Disable
			//DebugN("Right -  Disable");
   		 /// open  Disable Panel
				ChildPanelOnModal("vision/cietDevices/General/cietDisable.pnl",
								  "Disable " + unGenericDpFunctions_getAlias(deviceName),
								  makeDynString("$sDpName:"+deviceName),0,0);	
		break;   		 	
		
		case ciet_POPUPMENU_ORDER_DEBUG:					// Debug
			   cietGenericObject_OpenDebug(deviceName);
		break; 
		
		case ciet_POPUPMENU_ORDER_DEADBANDS: 					// DeadBands
			//DebugN("Right - Click DeadBands");
			 /// open Deadbands Panel
			ChildPanelOnModal("vision/cietDevices/General/cietSetDeadBandEH.pnl",
						  "DeadBands: " + unGenericDpFunctions_getAlias(deviceName),
						  makeDynString("$sDpName:"+deviceName), 0, 0);		
		break;

		
		case ciet_POPUPMENU_ORDER_CONSTANTS:					// Set Constants
			//DebugN("Right - Click Set Constants");
			
   		 /// open Set Constants Panel
			ChildPanelOnModal("vision/cietDevices/General/cietSetConstantsEH.pnl",
						  "Set Constants : " + unGenericDpFunctions_getAlias(deviceName),
						  makeDynString("$sDpName:"+deviceName), 0, 0);	   		
		break;
		
		case ciet_POPUPMENU_ORDER_CALIBRATION:					// Get Calibration
			//DebugN("Right -  Get Calibration");
   		 /// open  Get Calibration Panel
			 ChildPanelOnModal("vision/cietDevices/General/cietGetCalibration.pnl",
								  "Calibration Curve:" +  + unGenericDpFunctions_getAlias(deviceName),
								  makeDynString("$sDpName:"+deviceName),0,0);										  
		break;

		case ciet_POPUPMENU_ORDER_FORCED_MODE:					// Forced Mode
			//DebugN("Right -  Forced Mode");
			// Set state to forced mode
			if (dpSetWait(deviceName + ".ProcessOutput.Command.command",ciet_COMMAND_SET_FORCEDMODE)<0) bRes = false; 
		  
		  // Set forced current value
			dpGet(deviceName + ".ProcessInput.Data.setPoint",fCurrentSetPoint);
			if (dpSetWait(deviceName + ".ProcessOutput.Command.param1",fCurrentSetPoint)<0) bRes = false; 
			dpSet(deviceName + ".ProcessOutput.Command.command",ciet_COMMAND_SET_FORCEDMODE); 
		  
		break;
		
		case ciet_POPUPMENU_ORDER_AUTO_MODE:					// Auto Mode
			//DebugN("Right -  Auto Mode");
			if (dpSetWait(deviceName + ".ProcessOutput.Command.command",ciet_COMMAND_SET_NORMALMODE)<0) bRes = false; 	
			//DebugN("Auto Mode - bRes=" + bRes);
		break;

		case ciet_POPUPMENU_ORDER_SETPOINT:					// SetPoint
			//DebugN("Right -  SetPoint");
   		 /// open  SetPoint Panel
				ChildPanelOnModal("vision/cietDevices/General/cietSetPointEH.pnl",
								  "SetPoint " + unGenericDpFunctions_getAlias(deviceName),
								  makeDynString("$sDpName:"+deviceName),0,0);	
		break;   		 

		case ciet_POPUPMENU_ORDER_TENDPARAM:					// TendParam
			//DebugN("Right -  TendParam");
   		 /// open  TendParam Panel
				ChildPanelOnModal("vision/cietDevices/General/cietTendParam.pnl",
								  "Tendency " + unGenericDpFunctions_getAlias(deviceName),
								  makeDynString("$sDpName:"+deviceName),0,0);	
		break;   	
		
		case ciet_POPUPMENU_ORDER_OPEN_WFIPAGENT:							// Open Agent
					sAgentName=unGenericDpFunctions_getSystemName(deviceName) + cietGenericFunctions_getAgentName(deviceName);
				 unGenericObject_OpenFaceplate(unGenericDpFunctions_getWidgetDpName(sAgentName), exceptionInfo);
		break;	
		
		case ciet_POPUPMENU_ORDER_OPEN_WFIPSEGMENT:					// Open Segment
					sBusName=unGenericDpFunctions_getSystemName(deviceName) + cietGenericFuncitons_getSegmentName(deviceName);
					unGenericObject_OpenFaceplate(unGenericDpFunctions_getWidgetDpName(sBusName), exceptionInfo);
		break;	

		case ciet_POPUPMENU_ORDER_OPEN_CRATE:										// Open Crate
					cietGenericObject_openCratePanel (deviceName);
		break;	
				
		case ciet_POPUPMENU_ORDER_HELP:					// Help
					cietGenericObject_OpenHtmlFile("Help.html");
		break;			
		
		case ciet_POPUPMENU_ORDER_WIDGET_SIMULATOR:					// Widget Simulator
			ChildPanelOnRelativ("vision/cietDevices/General/cietWidgetSimulator.pnl",
			"Widget Simulator","",0, 0);			  
		break;					  			 
   		 
		default:
		break;
	}
// handle the trend action
	unGenericObject_handleTrendAction(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);

	if (dynlen(exceptionInfo) > 0)
	{
		unSendMessage_toExpertException("Right Click",exceptionInfo);
	}
	
}
