/**@name LIBRARY: cietExportDevice.ctl

@author: Herve Milcent (AB/CO)   

Creation Date: 25/07/2005

Modification History: 

                16/02/2009      Quique: Include the Calib ID in the LT devices 
 		23/03/2007	Pablo: Added CryoSgtPLC Export.
 		26/03/2007	Pablo: Added crate Level parameter for reset Agent.
		21/03/2007	Pablo: Added parameters for reset Agent.
		21/02/2007	Pablo: Changed curveId from description to Dp.
		24/01/2007	Pablo: Adding PVSS version on CryoGTW configuration.
		20/09/2006	Pablo: Changing Non Common Properties on all Ciet Devices.
	
version 1.0

External Function : 
		
Internal Functions :
	
	
Purpose: 
This library contains the function used to Export Devices Config

Usage: Public

PVSS manager usage: Ctrl, NV, NG

Constraints:
	. JCOP framework
	. PVSS version: 3.0.1
	. operating system: Wds2000 & wds XP
	. distributed system: No
*/



//@{

//------------------------------------------------------------------------------------------------------------------------
// CRYOGTW_Com_ExportConfig
/**
Purpose: Export the CRYOGTW front-end config 

Parameters:

Usage: External function

PVSS manager usage: NG, NV 
*/
CRYOGTW_Com_ExportConfig(dyn_string dsParam, dyn_string &exceptionInfo)
{
	int iRes, i, j, iLen, digitalOKRange, iNbr, alertRanges;
	dyn_int limitNumbers;
	dyn_float alertLimits;
	string sAlarmAddress, sAlarmType, sAlertText, sPlcName, sPlcDrv, sSplitModPlc;
	string sType, sDp, sCounterAddress, sCommandInterface, sCMWAlias;
	dyn_string dsDpParameters, dsAlarmDps, dsAlertText; 
	dyn_string dsAnalogArchive, dsEventArchive, dsBoolArchive;
	string sFunction;
	dyn_string dsTemp;

	////DebugN("FUNCTION: CRYOGTW_Com_ExportConfig(dsParam= "+dsParam);
	
	if (dynlen(dsParam)==UN_CONFIG_EXPORT_LENGTH)
	{
		sPlcName = dsParam[UN_CONFIG_EXPORT_PLCNAME];
		sDp = c_cietSystemIntegrity_CRYOGTW + sPlcName;
		iRes = dpGet(sDp + ".configuration.archive_bool", dsBoolArchive,
									sDp + ".configuration.archive_analog", dsAnalogArchive,
									sDp + ".configuration.archive_event", dsEventArchive,
									sDp + ".communication.driver_num", sPlcDrv,
									sDp + ".version.PVSS", dsParam[CRYO_CONFIG_CRYOGTW_CIET_VERSION]);
		TextFieldDriver.text = sPlcDrv;
	
		if (dynlen(dsBoolArchive)>0 && dynlen(dsAnalogArchive)>0 && dynlen(dsEventArchive)>0)
		{
			TextFieldBoolArchive.text = dsBoolArchive[1];
			TextFieldAnalogArchive.text = dsAnalogArchive[1];
			TextFieldEventArchive.text = dsEventArchive[1];
		}

		//Delete
		dynClear(dsDpParameters);
		dynAppend(dsDpParameters, "#" + UN_DELETE_COMMAND);
		dynAppend(dsDpParameters, dsParam[UN_CONFIG_EXPORT_PLCNAME]);
		dynAppend(dsDpParameters, dsParam[UN_CONFIG_EXPORT_APPLICATION]);				
		unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
	
		//PLCCONFIG
		dynClear(dsDpParameters);
		dynAppend(dsDpParameters, UN_PLC_COMMAND);
		dynAppend(dsDpParameters, CRYO_CRYOGTW_DPTYPE);
		dynAppend(dsDpParameters, dsParam[UN_CONFIG_EXPORT_PLCNAME]);
		dynAppend(dsDpParameters, dsParam[UN_CONFIG_EXPORT_APPLICATION]);
		dynAppend(dsDpParameters, dsTemp);

		//Get counter Address and CMW Alias
		dpGet(sDp + ".communication.counter:_address.._drv_ident" , sCounterAddress);
		sFunction = "DRV_"+sCounterAddress+"_convertToUnicosAddress";
		if(isFunctionDefined(sFunction)) {
			evalScript(sCounterAddress, "string main(string sDeviceDpeName, string sDeviceType) {"+
													"	string temp;"+
													"	temp = "+sFunction+"(sDeviceDpeName, sDeviceType);"+
													"	return temp;"+
													"}", makeDynString(), 
													sDp + ".communication.counter", 
													CRYO_CRYOGTW_DPTYPE);
			sCounterAddress = substr(sCounterAddress,0, strpos(sCounterAddress, "$"));
		}
		else
			sCounterAddress = UN_CONFIG_EXPORT_UNDEF; //
		
			// CMW device Alias
		sCMWAlias = substr(sCounterAddress,0, strpos(sCounterAddress, "."));
		dynAppend(dsDpParameters, sCMWAlias);
			
			//Counter Address
  	sCounterAddress = substr(sCounterAddress, strpos(sCounterAddress, ".")+1); 
		dynAppend(dsDpParameters, sCounterAddress);

			//PVSS Version
		dynAppend(dsDpParameters, dsParam[CRYO_CONFIG_CRYOGTW_CIET_VERSION]); 
		
// write to file
		unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);

// front-end system alarm
		fputs("#" + "\n" , g_fileToExport);
		dsAlarmDps = dpNames(c_unSystemAlarm_dpPattern + FE_pattern + c_cietSystemIntegrity_CRYOGTW + "*", c_unSystemAlarm_dpType);
		
		iLen=dynlen(dsAlarmDps);
		for(i=1; i<=iLen; i++)
		{
			if (dpExists(dsAlarmDps[i]))
			{
				fputs("#For " + CRYO_CRYOGTW_DPTYPE + " " + i + "\n" , g_fileToExport);
					
				sSplitModPlc = substr(dsAlarmDps[i], strpos(dsAlarmDps[i], dsParam[UN_CONFIG_EXPORT_PLCNAME]) + strlen(dsParam[UN_CONFIG_EXPORT_PLCNAME]) + 1);
				
				//Get Alarm Address
				dpGet(dsAlarmDps[i] + ".alarm:_address.._drv_ident" , sAlarmAddress);
				sFunction = "DRV_"+sAlarmAddress+"_convertToUnicosAddress";
				if(isFunctionDefined(sFunction)) {
					evalScript(sAlarmAddress, "string main(string sDeviceDpeName, string sDeviceType) {"+
															"	string temp;"+
															"	temp = "+sFunction+"(sDeviceDpeName, sDeviceType);"+
															"	return temp;"+
															"}", makeDynString(), 
															dsAlarmDps[i] + ".alarm", 
															c_unSystemAlarm_dpType);
					sAlarmAddress = substr(sAlarmAddress,0, strpos(sAlarmAddress, "$"));
				}
				else
					sAlarmAddress = UN_CONFIG_EXPORT_UNDEF;
					
				//Get alarm type
				dpGet(dsAlarmDps[i] + ".alarm:_alert_hdl.._type", sAlarmType);
				if(sAlarmType == DPCONFIG_ALERT_NONBINARYSIGNAL)
				{
					fwAlertConfig_getLimits(dsAlarmDps[i] + ".alarm", limitNumbers, alertLimits, exceptionInfo);
					alertRanges = (dynMax(limitNumbers) + 1);
					for(j=1; j<=alertRanges; j++)
					{
						dpGet(dsAlarmDps[i] + ".alarm:_alert_hdl." + j + "._text", sAlertText);
						dynAppend(dsAlertText, sAlertText);
					}
																				
					if(dynlen(dsAlertText) > 0)
					{
						if (dsAlertText[1]==UN_SYSTEMINTEGRITY_OK)
						{
							sType = "TRUE";
						}
						else
						{
							sType = "FALSE";
						}
					}
					else
						sType = UN_CONFIG_EXPORT_UNDEF;
				}
				else
					sType = UN_CONFIG_EXPORT_UNDEF;	
									
				dynClear(dsDpParameters);
				dynAppend(dsDpParameters, UN_FESYSTEMALARM_COMMAND);
				dynAppend(dsDpParameters, i);																															//???? nbr
				dynAppend(dsDpParameters, sSplitModPlc);																									//identifier
				dynAppend(dsDpParameters, i);																															//address (only for MODBUS)
				dynAppend(dsDpParameters, sType);																													//alarm type
				dynAppend(dsDpParameters, unGenericDpFunctions_getDescription(dsAlarmDps[i] + ".alarm"));	//description
				dynAppend(dsDpParameters, sAlarmAddress);																									//address
	// write to file
				unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
			}
		}
	}
	else
		fwException_raise(exceptionInfo, "ERROR", "CRYOGTW_Com_ExportConfig(): Wrong Parameters", "");		
}

//------------------------------------------------------------------------------------------------------------------------
// DRV_CMW_convertToUnicosAddress
/**
Purpose: convert the S7 address to the UNICOS format

Parameters:

Usage: Internal function

PVSS manager usage: NG, NV
*/
string DRV_CMW_convertToUnicosAddress(string sDeviceDpeName, string sDeviceType)
{
	string temp, sResult;
	int pos;

	dpGet(sDeviceDpeName + ":_address.._reference" , temp);
	return temp;
}

//------------------------------------------------------------------------------------------------------------------------
// CRYOGTW_WFIPSegment_ExportConfig
/**
Purpose: Export the CRYOGTW front-end config 

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/

CRYOGTW_WFIPSegment_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{

// //DebugN("CRYOGTW_WFIPSegment_ExportConfig");

	dyn_string dsAllCommonParameters, dsDpParameters, dsArchiveNames, dsAddressDpParameters, dsTemp, dExceptionInfo, dAddress;
	string sDeviceType, sArchiveActive, sArchiveName, sDescription, sUnit, sQWFIPAGENT, sFIPBUS, sDpName;
	float fArchiveTimeFilter; 
	int i,iLeni, j, iLenj,iSmoothingType, iDollarPosition;	 		
	dyn_float dfDeadBand; 
	bool bConfigExists, bAddress, bIsActive; 
	
	sDeviceType="WFIPSegment"; 
	
	iLeni=dynlen(dsDpList);
	if(iLeni > 0) {
		cietExportDevice_writeDeviceTypeLineFormat(sDeviceType, "");
	}
	for(i=1; i<=iLeni; i++)
	{									
		if (dpExists(dsDpList[i]))
		{
			dynClear(dsAllCommonParameters);
			dynClear(dsDpParameters);
			dynClear(dsArchiveNames);
				
			//Get all common Parameters
			unExportDevice_getAllCommonParameters(dsDpList[i], sDeviceType ,  dsAllCommonParameters);
			dsDpParameters = dsAllCommonParameters;
			
						
			//Get FIPBus;
			sFIPBUS= cietGenericFuncitons_getSegmentSlotRef(dsDpList[i]);
			dynAppend(dsDpParameters, sFIPBUS);	

			//Get QWFIPAGENT device alias;Channel Number;
			
			fwPeriphAddress_get(dsDpList[i] + ".ProcessInput.Data.statusFIP", bConfigExists, dAddress, bIsActive, dExceptionInfo);
			sQWFIPAGENT=dAddress[fwPeriphAddress_REFERENCE];
			iDollarPosition = strtok (sQWFIPAGENT, "$");
			sQWFIPAGENT = substr (sQWFIPAGENT, 0, iDollarPosition);  // , strlen(sQWFIPAGENT)-iDollarPosition 
			
			dynAppend(dsDpParameters, sQWFIPAGENT);			
				

			//Get Archive Parameters
			fArchiveTimeFilter =0;
			cietConfigGenericFunctions_getArchiveParameters(dsDpList[i]+".ProcessInput.Data.statusFIP", sArchiveActive, fArchiveTimeFilter);
			dynAppend(dsDpParameters, sArchiveActive);
			dynAppend(dsDpParameters, fArchiveTimeFilter);						

			unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
		}//dp doesn't exist
	}//end for
}


//------------------------------------------------------------------------------------------------------------------------
// CRYOGTW_CryoSgtPLC_ExportConfig
/**
Purpose: Export the CRYOGTW front-end config 

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
CRYOGTW_CryoSgtPLC_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	dyn_string dsAllCommonParameters, dsDpParameters, dsArchiveNames, dsAddressDpParameters,dsTemp, dExceptionInfo, dAddress;
	string sDeviceType, sArchiveActive, sArchiveName, sDescription, sQWFIPAGENT;
	float fArchiveTimeFilter;
	int i,iLeni, j, iLenj, iDollarPosition;;
	dyn_int diSmoothingType;
	dyn_float dfDeadBand;
	bool bConfigExists, bAddress, bIsActive;

/*##CryoSgtPLC;DeviceNumber;Alias (Device Name);Description;[Diagnostic];[Html];[DefaultPanel];domaine;Nature;widget;QWFIPAGENT device alias;ArchiveActive;ArchiveTimeFilter;[boolean];[analog];[event]*/
	sDeviceType="CryoSgtPLC";
	
	iLeni=dynlen(dsDpList);
	if(iLeni > 0) {
		cietExportDevice_writeDeviceTypeLineFormat(sDeviceType, "");
	}
	for(i=1; i<=iLeni; i++)
	{									
		if (dpExists(dsDpList[i]))
		{
			dynClear(dsAllCommonParameters);
			dynClear(dsDpParameters);
			dynClear(dsArchiveNames);
																
			//Get all common Parameters
			unExportDevice_getAllCommonParameters(dsDpList[i], sDeviceType ,  dsAllCommonParameters);
			dsDpParameters = dsAllCommonParameters;

			//Get QWFIPAGENT device alias;Channel Number;
			fwPeriphAddress_get(dsDpList[i] + ".ProcessInput.Status.mode", bConfigExists, dAddress, bIsActive, dExceptionInfo);
 			sQWFIPAGENT=dAddress[fwPeriphAddress_REFERENCE];
	 		iDollarPosition = strtok (sQWFIPAGENT, "$");
	 		sQWFIPAGENT = substr (sQWFIPAGENT, 0, iDollarPosition);  // , strlen(sQWFIPAGENT)-iDollarPosition 
 			dynAppend(dsDpParameters, sQWFIPAGENT);
												
			//Get Archive Parameters
			fArchiveTimeFilter =0;			
			cietConfigGenericFunctions_getArchiveParameters(dsDpList[i]+".ProcessInput.Status.mode", sArchiveActive, fArchiveTimeFilter);
			dynAppend(dsDpParameters, sArchiveActive);
			dynAppend(dsDpParameters, fArchiveTimeFilter);			

			//Get Archive Names for ciet devices
			cietConfigGenericFunctions_getArchiveNames(dsDpList[i], sDeviceType, dsArchiveNames);
			iLenj=dynlen(dsArchiveNames);
			for(j=1; j<=iLenj ; j++)
			{
				dynAppend(dsDpParameters, dsArchiveNames[j]);
			}	
			
			unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
		}//dp doesn't exist
	}//end for
}

//------------------------------------------------------------------------------------------------------------------------
// CRYOGTW_DI_ExportConfig
/**
Purpose: Export the CRYOGTW front-end config 

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
CRYOGTW_DI_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	dyn_string dsAllCommonParameters, dsDpParameters, dsArchiveNames, dsAddressDpParameters,dsTemp, dExceptionInfo, dAddress;
	string sDeviceType, sArchiveActive, sArchiveName, sDescription, sQWFIPAGENT;
	float fArchiveTimeFilter;
	int i,iLeni, j, iLenj, iDollarPosition;;
	dyn_int diSmoothingType;
	dyn_float dfDeadBand;
	bool bConfigExists, bAddress, bIsActive;

/*##DI;DeviceNumber;Alias (Device Name);Description;[Diagnostic];[Html];[DefaultPanel];domaine;Nature;widget;QWFIPAGENT device alias;
#Channel Number;FIPPosition;ArchiveActive;ArchiveTimeFilter;[boolean];[analog];[event]*/
	sDeviceType="DI";
	
	iLeni=dynlen(dsDpList);
	if(iLeni > 0) {
		cietExportDevice_writeDeviceTypeLineFormat(sDeviceType, "");
	}
	for(i=1; i<=iLeni; i++)
	{									
		if (dpExists(dsDpList[i]))
		{
			dynClear(dsAllCommonParameters);
			dynClear(dsDpParameters);
			dynClear(dsArchiveNames);
				
			//Get all common Parameters
			unExportDevice_getAllCommonParameters(dsDpList[i], sDeviceType ,  dsAllCommonParameters);
			dsDpParameters = dsAllCommonParameters;


			//Get QWFIPAGENT device alias;Channel Number;
			
			fwPeriphAddress_get(dsDpList[i] + ".ProcessInput.Config.sgNumber", bConfigExists, dAddress, bIsActive, dExceptionInfo);
			sQWFIPAGENT=dAddress[fwPeriphAddress_REFERENCE];
			iDollarPosition = strtok (sQWFIPAGENT, "$");
			sQWFIPAGENT = substr (sQWFIPAGENT, 0, iDollarPosition);  // , strlen(sQWFIPAGENT)-iDollarPosition 
			
			dynAppend(dsDpParameters, sQWFIPAGENT);
	
			//Get Archive Parameters
			fArchiveTimeFilter =0;			
			cietConfigGenericFunctions_getArchiveParameters(dsDpList[i]+".ProcessInput.Data.digitalInput", sArchiveActive, fArchiveTimeFilter);
			dynAppend(dsDpParameters, sArchiveActive);
			dynAppend(dsDpParameters, fArchiveTimeFilter);			
												
			//Get Archive Names for ciet devices
			cietConfigGenericFunctions_getArchiveNames(dsDpList[i], sDeviceType, dsArchiveNames);
			iLenj=dynlen(dsArchiveNames);
			for(j=1; j<=iLenj ; j++)
			{
				dynAppend(dsDpParameters, dsArchiveNames[j]);
			}	
			
			unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
		}//dp doesn't exist
	}//end for
}

//------------------------------------------------------------------------------------------------------------------------
// CRYOGTW_LT_ExportConfig
/**
Purpose: Export the DI device CRYOGTW front-end config 

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/

CRYOGTW_LT_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	dyn_string dsAllCommonParameters, dsDpParameters, dsArchiveNames, dsAddressDpParameters, dExceptionInfo, dAddress;
	string sDeviceType, sCalibrationID, sArchiveActive, sArchiveName, sDescription, sUnit, sQWFIPAGENT, sExceptionInfo;
	float fArchiveTimeFilter,  fMin, fMax;
	int i,iLeni, j, iLenj,iSmoothingType, iDollarPosition;			
	dyn_float dfDeadBand;
	bool bConfigExists, bAddress, bIsActive;
	
	sDeviceType="LT";
	
	iLeni=dynlen(dsDpList);
	if(iLeni > 0) {
		cietExportDevice_writeDeviceTypeLineFormat(sDeviceType, "");
	}
	for(i=1; i<=iLeni; i++)
	{									
		if (dpExists(dsDpList[i]))
		{
			dynClear(dsAllCommonParameters);
			dynClear(dsDpParameters);
			dynClear(dsArchiveNames);
				
			//Get all common Parameters
			unExportDevice_getAllCommonParameters(dsDpList[i], sDeviceType ,  dsAllCommonParameters);
			dsDpParameters = dsAllCommonParameters;
			
			//Get Unit			
			sUnit=dpGetUnit (dsDpList[i]+".ProcessInput.Data.levelCalc");
			dynAppend(dsDpParameters, sUnit);	
		
			//Get PvRange
			
			cietConfigGenericDpFunctions_getPvRange(dsDpList[i]+".ProcessInput.Data.levelCalc", fMin, fMax, bConfigExists, sExceptionInfo);
		
			dynAppend(dsDpParameters, fMin);
			dynAppend(dsDpParameters, fMax);
                        
                        //Get Calibration Curve						
			dpGet(dsDpList[i]+".ProcessInput.Curve.curveId",sCalibrationID);
			dynAppend(dsDpParameters, sCalibrationID);

			//Get QWFIPAGENT device alias;Channel Number;
			
			fwPeriphAddress_get(dsDpList[i] + ".ProcessInput.Config.sgNumber", bConfigExists, dAddress, bIsActive, dExceptionInfo);
			sQWFIPAGENT=dAddress[fwPeriphAddress_REFERENCE];
			iDollarPosition = strtok (sQWFIPAGENT, "$");
			sQWFIPAGENT = substr (sQWFIPAGENT, 0, iDollarPosition);  // , strlen(sQWFIPAGENT)-iDollarPosition 
			
			dynAppend(dsDpParameters, sQWFIPAGENT);			
			
			//Get Archive Parameters
			fArchiveTimeFilter =0;
			cietConfigGenericFunctions_getArchiveParameters(dsDpList[i]+".ProcessInput.Data.senVolMeas", sArchiveActive, fArchiveTimeFilter);
			dynAppend(dsDpParameters, sArchiveActive);
			dynAppend(dsDpParameters, fArchiveTimeFilter);						
						
			//Get Archive Names for ciet devices
			cietConfigGenericFunctions_getArchiveNames(dsDpList[i], sDeviceType, dsArchiveNames);
			iLenj=dynlen(dsArchiveNames);
			for(j=1; j<=iLenj ; j++)
			{
				dynAppend(dsDpParameters, dsArchiveNames[j]);
			}	
			
			unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
		}//dp doesn't exist
	}//end for
}

//------------------------------------------------------------------------------------------------------------------------
// CRYOGTW_PT_ExportConfig
/**
Purpose: Export the CRYOGTW front-end config 

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
CRYOGTW_PT_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
  dyn_string dsAllCommonParameters, dsDpParameters, dsArchiveNames, dsAddressDpParameters, dExceptionInfo, dAddress;
	string sDeviceType, sArchiveActive, sArchiveName,sDescription, sUnit, sCalibrationID, sQWFIPAGENT, sExceptionInfo;
	float fArchiveTimeFilter, fMin, fMax;
	int i,iLeni, j, iLenj, iSmoothingType, iDollarPosition;
	dyn_float dfDeadBand;
	bool bConfigExists, bAddress, bIsActive;

	sDeviceType="PT";
	
	iLeni=dynlen(dsDpList);
	if(iLeni > 0) {
		cietExportDevice_writeDeviceTypeLineFormat(sDeviceType, "");
	}
	for(i=1; i<=iLeni; i++)
	{									
		if (dpExists(dsDpList[i]))
		{
			dynClear(dsAllCommonParameters);
			dynClear(dsDpParameters);
			dynClear(dsArchiveNames);
				
			//Get all common Parameters
			unExportDevice_getAllCommonParameters(dsDpList[i], sDeviceType ,  dsAllCommonParameters);
			dsDpParameters = dsAllCommonParameters;

			//Get Unit			
			sUnit=dpGetUnit (dsDpList[i]+".ProcessInput.Data.pressureCalc");
			dynAppend(dsDpParameters, sUnit);	

			//Get PvRange
			cietConfigGenericDpFunctions_getPvRange(dsDpList[i]+".ProcessInput.Data.pressureCalc", fMin, fMax, bConfigExists, sExceptionInfo);
			
			dynAppend(dsDpParameters, fMin);
			dynAppend(dsDpParameters, fMax);
		
			
			//Get Calibration Curve						
			dpGet(dsDpList[i]+".ProcessInput.Curve.curveId",sCalibrationID);
			dynAppend(dsDpParameters, sCalibrationID);			


			//Get QWFIPAGENT device alias;Channel Number;
			
			fwPeriphAddress_get(dsDpList[i] + ".ProcessInput.Config.sgNumber", bConfigExists, dAddress, bIsActive, dExceptionInfo);
			sQWFIPAGENT=dAddress[fwPeriphAddress_REFERENCE];
			iDollarPosition = strtok (sQWFIPAGENT, "$");
			sQWFIPAGENT = substr (sQWFIPAGENT, 0, iDollarPosition);  // , strlen(sQWFIPAGENT)-iDollarPosition 
			
			dynAppend(dsDpParameters, sQWFIPAGENT);			
			
			//Get Archive Parameters
			fArchiveTimeFilter =0;
			cietConfigGenericFunctions_getArchiveParameters(dsDpList[i]+".ProcessInput.Data.senVolMeas", sArchiveActive, fArchiveTimeFilter);
			dynAppend(dsDpParameters, sArchiveActive);
			dynAppend(dsDpParameters, fArchiveTimeFilter);						
						
			//Get Archive Names for ciet devices
			cietConfigGenericFunctions_getArchiveNames(dsDpList[i], sDeviceType, dsArchiveNames);
			iLenj=dynlen(dsArchiveNames);
			for(j=1; j<=iLenj ; j++)
			{
				dynAppend(dsDpParameters, dsArchiveNames[j]);
			}	
			unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
		}//dp doesn't exist
	}//end for
}
//------------------------------------------------------------------------------------------------------------------------
// CRYOGTW_TT_ExportConfig
/**
Purpose: Export the DI device CRYOGTW front-end config 

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/

CRYOGTW_TT_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{

  dyn_string dsAllCommonParameters, dsDpParameters, dsArchiveNames, dsAddressDpParameters, dExceptionInfo, dAddress;
	string sDeviceType, sArchiveActive, sArchiveName,sDescription, sUnit,sTemp;
	string sCalibrationID1, sCalibrationID2, sCalibrationID1FALSE, sCalibrationID2FALSE;
	string sQWFIPAGENT, sExceptionInfo;
	float fArchiveTimeFilter, fMin, fMax;
	int i,iLeni, j, iLenj, iSmoothingType, iDollarPosition;
	dyn_float dfDeadBand;
	bool bConfigExists, bIsActive;
			
	sDeviceType="TT";
	
	iLeni=dynlen(dsDpList);
	if(iLeni > 0) {
		cietExportDevice_writeDeviceTypeLineFormat(sDeviceType, "");
	}
	for(i=1; i<=iLeni; i++)
	{									
		if (dpExists(dsDpList[i]))
		{
			dynClear(dsAllCommonParameters);
			dynClear(dsDpParameters);
			dynClear(dsArchiveNames);
				
			//Get all common Parameters
			//DI;DeviceNumber;Alias (Device Name);Description;[Diagnostic];[Html];[DefaultPanel];domaine;Nature;widget;
			
			unExportDevice_getAllCommonParameters(dsDpList[i], sDeviceType ,  dsAllCommonParameters);
			dsDpParameters = dsAllCommonParameters;
			
			//Get Unit			
			sUnit=dpGetUnit (dsDpList[i]+".ProcessInput.Data.temperatureCalc");
			dynAppend(dsDpParameters, sUnit);	

			//Get PvRange
			
			cietConfigGenericDpFunctions_getPvRange(dsDpList[i]+".ProcessInput.Data.temperatureCalc", fMin, fMax, bConfigExists, sExceptionInfo);
			
			dynAppend(dsDpParameters, fMin);
			dynAppend(dsDpParameters, fMax);

			
		//	unGenericDpFunctions_getFaceplateRange(dsDpList[i]+".ProcessInput.Data.temperatureCalc", dyn_string &dsDpe, dyn_float &dfMax, dyn_float &dfMin)	

			// Get Calibration Curve ID
			//LBBRD_12L8_TT822;			
	
			dpGet (dsDpList[i]+".ProcessInput.Curve.curveId1",sCalibrationID1);
			dpGet (dsDpList[i]+".ProcessInput.Curve.curveId2",sCalibrationID2);
			
                        if (sCalibrationID2 =="")  // case no second calibration curve (optional)
                            sTemp =sCalibrationID1;
                        else
                            sTemp =sCalibrationID1 + "," + sCalibrationID2;
			dynAppend(dsDpParameters, sTemp);

			//Get QWFIPAGENT device alias;Channel Number;
			//CRYO.LBBRD_12L8_TT822
			
			fwPeriphAddress_get(dsDpList[i] + ".ProcessInput.Config.sgNumber", bConfigExists, dAddress, bIsActive, dExceptionInfo);
			sQWFIPAGENT=dAddress[fwPeriphAddress_REFERENCE];
			iDollarPosition = strtok (sQWFIPAGENT, "$");
			sQWFIPAGENT = substr (sQWFIPAGENT, 0, iDollarPosition);  // , strlen(sQWFIPAGENT)-iDollarPosition 
			
			dynAppend(dsDpParameters, sQWFIPAGENT);				
	
			//Get Archive Parameters
			//archive (Y/A/N); ArchiveTimeFilter;
			fArchiveTimeFilter =0;
			cietConfigGenericFunctions_getArchiveParameters(dsDpList[i]+".ProcessInput.Data.senVolMeas", sArchiveActive, fArchiveTimeFilter);
			dynAppend(dsDpParameters, sArchiveActive);
			dynAppend(dsDpParameters, fArchiveTimeFilter);						
						
			//Get Archive Names for ciet devices
			//[boolean];[analog];[event]
			cietConfigGenericFunctions_getArchiveNames(dsDpList[i], sDeviceType, dsArchiveNames);
			iLenj=dynlen(dsArchiveNames);
			for(j=1; j<=iLenj ; j++)
			{
				dynAppend(dsDpParameters, dsArchiveNames[j]);
			}	
			
			unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
		}//dp doesn't exist
	}//end for
}

//------------------------------------------------------------------------------------------------------------------------
// CRYOGTW_WFIPEvenAgent_ExportConfig
/**
Purpose: Export the CRYOGTW front-end config 

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
CRYOGTW_WFIPEvenAgent_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	char cCrateLevel;
dyn_string dsAllCommonParameters, dsDpParameters, dsArchiveNames, dsAddressDpParameters,dsTemp, dAddress, dExceptionInfo;
	string sDeviceType, sArchiveActive, sArchiveName,sDescription, sQWFIPAGENT,sOppositeNet;
	float fArchiveTimeFilter;
	int i,iLeni, j, iLenj, iDollarPosition, iPartner,iOppositeOdd,iOppositeEven;
	dyn_int diSmoothingType;
	dyn_float dfDeadBand;
  bool bConfigExists, bIsActive;		
	
	sDeviceType="WFIPEvenAgent";
	
	iLeni=dynlen(dsDpList);
	if(iLeni > 0) {
		cietExportDevice_writeDeviceTypeLineFormat(sDeviceType, "");
	}
	for(i=1; i<=iLeni; i++)
	{									
		if (dpExists(dsDpList[i]))
		{
			dynClear(dsAllCommonParameters);
			dynClear(dsDpParameters);
			dynClear(dsArchiveNames);
				
			//Get all common Parameters
			unExportDevice_getAllCommonParameters(dsDpList[i], sDeviceType ,  dsAllCommonParameters);
			dsDpParameters = dsAllCommonParameters;
			
			//Get QWFIPAGENT device alias;Channel Number;
			
			fwPeriphAddress_get(dsDpList[i] + ".ProcessInput.Config.sgNumber", bConfigExists, dAddress, bIsActive, dExceptionInfo);
			sQWFIPAGENT=dAddress[fwPeriphAddress_REFERENCE];
			iDollarPosition = strtok (sQWFIPAGENT, "$");
			sQWFIPAGENT = substr (sQWFIPAGENT, 0, iDollarPosition);  // , strlen(sQWFIPAGENT)-iDollarPosition 
			
			dynAppend(dsDpParameters, sQWFIPAGENT);				

			//Set Reset Info               

			dpGet(dsDpList[i]+".Config.crateLevel",cCrateLevel);
			dynAppend(dsDpParameters, cCrateLevel);	
			dpGet(dsDpList[i]+".Config.partner",iPartner);
			dynAppend(dsDpParameters, iPartner);
			dpGet(dsDpList[i]+".Config.oppositeOdd",iOppositeOdd);
			dynAppend(dsDpParameters, iOppositeOdd);
			dpGet(dsDpList[i]+".Config.oppositeEven",iOppositeEven);
			dynAppend(dsDpParameters, iOppositeEven);			
			dpGet(dsDpList[i]+".Config.oppositeNet",sOppositeNet);
			dynAppend(dsDpParameters, sOppositeNet);	

			//Get Archive Parameters
			fArchiveTimeFilter =0;
			cietConfigGenericFunctions_getArchiveParameters(dsDpList[i]+".ProcessInput.Data.agentStatus", sArchiveActive, fArchiveTimeFilter);
			dynAppend(dsDpParameters, sArchiveActive);
			dynAppend(dsDpParameters, fArchiveTimeFilter);						

			unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
		}//dp doesn't exist
	}//end for
}

//------------------------------------------------------------------------------------------------------------------------
// CRYOGTW_WFIPOddAgent_ExportConfig
/**
Purpose: Export the CRYOGTW front-end config 

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
CRYOGTW_WFIPOddAgent_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	char cCrateLevel;
dyn_string dsAllCommonParameters, dsDpParameters, dsArchiveNames, dsAddressDpParameters,dsTemp, dAddress, dExceptionInfo;
	string sDeviceType, sArchiveActive, sArchiveName,sDescription, sQWFIPAGENT,sOppositeNet;
	float fArchiveTimeFilter;
	int i,iLeni, j, iLenj, iDollarPosition, iPartner,iOppositeOdd,iOppositeEven;
	dyn_int diSmoothingType;
	dyn_float dfDeadBand;
	bool bConfigExists, bIsActive;	
	
	sDeviceType="WFIPOddAgent";
	
	iLeni=dynlen(dsDpList);
	if(iLeni > 0) {
		cietExportDevice_writeDeviceTypeLineFormat(sDeviceType, "");
	}
	for(i=1; i<=iLeni; i++)
	{									
		if (dpExists(dsDpList[i]))
		{
			dynClear(dsAllCommonParameters);
			dynClear(dsDpParameters);
			dynClear(dsArchiveNames);
				
			//Get all common Parameters
			unExportDevice_getAllCommonParameters(dsDpList[i], sDeviceType ,  dsAllCommonParameters);
			dsDpParameters = dsAllCommonParameters;

			//Get QWFIPAGENT device alias;Channel Number;
			//CRYO.LBBRD_12L8_TT822
			
			fwPeriphAddress_get(dsDpList[i] + ".ProcessInput.Config.sgNumber", bConfigExists, dAddress, bIsActive, dExceptionInfo);
			sQWFIPAGENT=dAddress[fwPeriphAddress_REFERENCE];
			iDollarPosition = strtok (sQWFIPAGENT, "$");
			sQWFIPAGENT = substr (sQWFIPAGENT, 0, iDollarPosition);  // , strlen(sQWFIPAGENT)-iDollarPosition 
			
			dynAppend(dsDpParameters, sQWFIPAGENT);	

			//Set Reset Info

			dpGet(dsDpList[i]+".Config.crateLevel",cCrateLevel);
			dynAppend(dsDpParameters, cCrateLevel);	
			dpGet(dsDpList[i]+".Config.partner",iPartner);
			dynAppend(dsDpParameters, iPartner);
			dpGet(dsDpList[i]+".Config.oppositeOdd",iOppositeOdd);
			dynAppend(dsDpParameters, iOppositeOdd);
			dpGet(dsDpList[i]+".Config.oppositeEven",iOppositeEven);
			dynAppend(dsDpParameters, iOppositeEven);			
			dpGet(dsDpList[i]+".Config.oppositeNet",sOppositeNet);
			dynAppend(dsDpParameters, sOppositeNet);								

			//Get Archive Parameters
			fArchiveTimeFilter =0;
			cietConfigGenericFunctions_getArchiveParameters(dsDpList[i]+".ProcessInput.Data.agentStatus", sArchiveActive, fArchiveTimeFilter);
			dynAppend(dsDpParameters, sArchiveActive);
			dynAppend(dsDpParameters, fArchiveTimeFilter);			

			unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
		}//dp doesn't exist
	}//end for
}

//------------------------------------------------------------------------------------------------------------------------
// CRYOGTW_EH_ExportConfig
/**
Purpose: Export DQAMSRQ Devices for DQGTW front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/

void CRYOGTW_EH_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	dyn_string dsAllCommonParameters, dsDpParameters, dsArchiveNames, dsAddressDpParameters,dsTemp, dExceptionInfo, dAddress;
	string sDeviceType, sArchiveActive, sArchiveName,sDescription, sUnit, sCalibrationID, sQWFIPAGENT, sExceptionInfo;
	float fArchiveTimeFilter, fMin, fMax;
	int i,iLeni, j, iLenj, iSmoothingType,iDollarPosition;
	dyn_float dfDeadBand;
	bool bConfigExists, bAddress, bIsActive;
	
	sDeviceType="EH";
	
	iLeni=dynlen(dsDpList);
	if(iLeni > 0) {
		cietExportDevice_writeDeviceTypeLineFormat(sDeviceType, "");
	}
	for(i=1; i<=iLeni; i++)
	{									
		if (dpExists(dsDpList[i]))
		{
			dynClear(dsAllCommonParameters);
			dynClear(dsDpParameters);
			dynClear(dsArchiveNames);
				
			//Get all common Parameters
			unExportDevice_getAllCommonParameters(dsDpList[i], sDeviceType ,  dsAllCommonParameters);
			dsDpParameters = dsAllCommonParameters;

			//Get Unit			
			sUnit=dpGetUnit (dsDpList[i]+".ProcessInput.Data.powerCalc");
			dynAppend(dsDpParameters, sUnit);	
			
			//Get PvRange
			
			cietConfigGenericDpFunctions_getPvRange(dsDpList[i]+".ProcessInput.Data.powerCalc", fMin, fMax, bConfigExists, sExceptionInfo);
			
			dynAppend(dsDpParameters, fMin);
			dynAppend(dsDpParameters, fMax);

			
			//Get Calibration Curve						
			dpGet(dsDpList[i]+".ProcessInput.Curve.curveId",sCalibrationID);
			dsTemp = sCalibrationID;

			dynAppend(dsDpParameters, dsTemp);		
			

			//Get QWFIPAGENT device alias;Channel Number;
			
			fwPeriphAddress_get(dsDpList[i] + ".ProcessInput.Config.sgNumber", bConfigExists, dAddress, bIsActive, dExceptionInfo);
			sQWFIPAGENT=dAddress[fwPeriphAddress_REFERENCE];
			iDollarPosition = strtok (sQWFIPAGENT, "$");
			sQWFIPAGENT = substr (sQWFIPAGENT, 0, iDollarPosition);  // , strlen(sQWFIPAGENT)-iDollarPosition 
			
			dynAppend(dsDpParameters, sQWFIPAGENT);					

			//Get Archive Parameters
			fArchiveTimeFilter =0;
			cietConfigGenericFunctions_getArchiveParameters(dsDpList[i]+".ProcessInput.Data.heatVolMeas", sArchiveActive, fArchiveTimeFilter);
			dynAppend(dsDpParameters, sArchiveActive);
			dynAppend(dsDpParameters, fArchiveTimeFilter);						
						
			//Get Archive Names for ciet devices
			cietConfigGenericFunctions_getArchiveNames(dsDpList[i], sDeviceType, dsArchiveNames);
			iLenj=dynlen(dsArchiveNames);
			for(j=1; j<=iLenj ; j++)
			{
				dynAppend(dsDpParameters, dsArchiveNames[j]);
			}	
			
			unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
		}//dp doesn't exist
	}//end for
}

//------------------------------------------------------------------------------------------------------------------------
// CRYOGTW_EHBS_ExportConfig
/**
Purpose: Export DQAMSRQ Devices for DQGTW front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/

void CRYOGTW_EHBS_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	dyn_string dsAllCommonParameters, dsDpParameters, dsArchiveNames, dsAddressDpParameters,dsTemp, dExceptionInfo, dAddress;
	string sDeviceType, sArchiveActive, sArchiveName,sDescription, sUnit, sCalibrationID, sQWFIPAGENT, sExceptionInfo;
	float fArchiveTimeFilter, fMin, fMax;
	int i,iLeni, j, iLenj, iSmoothingType,iDollarPosition;
	dyn_float dfDeadBand;
	bool bConfigExists, bAddress, bIsActive;
	
	sDeviceType="EHBS";
	
	iLeni=dynlen(dsDpList);
	if(iLeni > 0) {
		cietExportDevice_writeDeviceTypeLineFormat(sDeviceType, "");
	}
	for(i=1; i<=iLeni; i++)
	{									
		if (dpExists(dsDpList[i]))
		{
			dynClear(dsAllCommonParameters);
			dynClear(dsDpParameters);
			dynClear(dsArchiveNames);
				
			//Get all common Parameters
			unExportDevice_getAllCommonParameters(dsDpList[i], sDeviceType ,  dsAllCommonParameters);
			dsDpParameters = dsAllCommonParameters;

			//Get Unit			
			sUnit=dpGetUnit(dsDpList[i]+".ProcessInput.Data.powerCalc");
			dynAppend(dsDpParameters, sUnit);	
			
			//Get PvRange
			
			cietConfigGenericDpFunctions_getPvRange(dsDpList[i]+".ProcessInput.Data.powerCalc", fMin, fMax, bConfigExists, sExceptionInfo);
			
			dynAppend(dsDpParameters, fMin);
			dynAppend(dsDpParameters, fMax);

			
			//Get Calibration Curve						
			dpGet(dsDpList[i]+".ProcessInput.Curve.curveId",sCalibrationID);
			dsTemp = sCalibrationID;

			dynAppend(dsDpParameters, dsTemp);		
			

			//Get QWFIPAGENT device alias;Channel Number;
			
			fwPeriphAddress_get(dsDpList[i] + ".ProcessInput.Config.sgNumber", bConfigExists, dAddress, bIsActive, dExceptionInfo);
			sQWFIPAGENT=dAddress[fwPeriphAddress_REFERENCE];
			iDollarPosition = strtok (sQWFIPAGENT, "$");
			sQWFIPAGENT = substr (sQWFIPAGENT, 0, iDollarPosition);  // , strlen(sQWFIPAGENT)-iDollarPosition 
			
			dynAppend(dsDpParameters, sQWFIPAGENT);					

			//Get Archive Parameters
			fArchiveTimeFilter =0;
			cietConfigGenericFunctions_getArchiveParameters(dsDpList[i]+".ProcessInput.Data.heatVolMeas", sArchiveActive, fArchiveTimeFilter);
			dynAppend(dsDpParameters, sArchiveActive);
			dynAppend(dsDpParameters, fArchiveTimeFilter);						
						
			//Get Archive Names for ciet devices
			cietConfigGenericFunctions_getArchiveNames(dsDpList[i], sDeviceType, dsArchiveNames);
			iLenj=dynlen(dsArchiveNames);
			for(j=1; j<=iLenj ; j++)
			{
				dynAppend(dsDpParameters, dsArchiveNames[j]);
			}	
			
			unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
		}//dp doesn't exist
	}//end for
}

//------------------------------------------------------------------------------------------------------------------------
// cietExportDevice_writeDeviceTypeLineFormat
/**
Purpose: insert comment in the export file

Parameters:
	sDeviceType, input 
	sCommand, input

Usage: External function

PVSS manager usage: NG, NV
*/

cietExportDevice_writeDeviceTypeLineFormat(string sDeviceType, string sCommand)
{
	string sFormat, sMandatory, sDeviceConfig;
	
	sMandatory = "\n#"+ sDeviceType+"; Device number (unique, WorldFipAgent Number); Device name (Device Identifier); description; Diagnostic panel;"+
	" \n#HTML page; Default panel; subsystem1 (Domain); subsystem2 (Nature); Widget Name";
	switch(sDeviceType) 
	{
		case "EH":
			sDeviceConfig ="Unit; Range Min; Range Max; Calibration Curve ID; QWFIPAGENT device alias;"
			+"archive (Y/A/N); ArchiveTimeFilter; [boolean]; [analog]; [event];\n#";
			sFormat = sMandatory+";"+sDeviceConfig;
			fputs("#format: " +sFormat + "\n" , g_fileToExport);
			break;
		case "EHBS":
			sDeviceConfig ="Unit; Range Min; Range Max; Calibration Curve ID; QWFIPAGENT device alias;"
			+"archive (Y/A/N); ArchiveTimeFilter; [boolean]; [analog]; [event];\n#";
			sFormat = sMandatory+";"+sDeviceConfig;
			fputs("#format: " +sFormat + "\n" , g_fileToExport);
			break;
		case "LT":
			sDeviceConfig =+"Unit; Range Min; Range Max; Calibration Curve ID; QWFIPAGENT device alias;"			
			+"archive (Y/A/N); ArchiveTimeFilter; [boolean]; [analog]; [event]\n#";
			sFormat = sMandatory+"; "+sDeviceConfig;
			fputs("#format: " +sFormat + "\n" , g_fileToExport);
		break;
		case "PT":
			sDeviceConfig =+"Unit; Range Min; Range Max; Calibration Curve ID; QWFIPAGENT device alias;"			
			+"archive (Y/A/N); ArchiveTimeFilter; [boolean]; [analog]; [event]\n#";
			sFormat = sMandatory+"; "+sDeviceConfig;
			fputs("#format: " +sFormat + "\n" , g_fileToExport);
		break;
		case "TT":
			sDeviceConfig =+"Unit; Range Min; Range Max; Calibration Curve ID1, Calibration Curve ID2; QWFIPAGENT device alias;"			
			+"archive (Y/A/N); ArchiveTimeFilter; [boolean]; [analog]; [event]\n#";
			sFormat = sMandatory+"; "+sDeviceConfig;
			fputs("#format: " +sFormat + "\n" , g_fileToExport);
		break;
		case "WFIPSegment":
			sDeviceConfig ="FIP BUS;	QWFIPAGENT device alias;	archive (Y/A/N);	ArchiveTimeFilter";
			sFormat = sMandatory+"; "+sDeviceConfig;
			fputs("#format: " +sFormat + "\n" , g_fileToExport);
		break;		
		case "WFIPEvenAgent":
			sDeviceConfig ="QWFIPAGENT device alias; Crate; Partner; Opposite Odd; Opposite Even; Opposite Net; archive (Y/A/N);	ArchiveTimeFilter";
			sFormat = sMandatory+";"+sDeviceConfig;
			fputs("#format: " +sFormat + "\n" , g_fileToExport);
		break;
		case "WFIPOddAgent":
			sDeviceConfig ="QWFIPAGENT device alias; Crate; Partner; Opposite Odd; Opposite Even; Opposite Net; archive (Y/A/N);	ArchiveTimeFilter";
			sFormat = sMandatory+"; "+sDeviceConfig;
			fputs("#format: " +sFormat + "\n" , g_fileToExport);
		break;
		case "DI":
			sDeviceConfig ="QWFIPAGENT device alias; ArchiveActive; \n#ArchiveTimeFilter; [boolean]; [analog]; [event]\n#";
			sFormat = sMandatory+"; "+sDeviceConfig;
			fputs("#format: " +sFormat + "\n" , g_fileToExport);
		break;
		case "CryoSgtPLC":
			sDeviceConfig ="QWFIPAGENT device alias; ArchiveActive; \n#ArchiveTimeFilter; [boolean]; [analog]; [event]\n#";
			sFormat = sMandatory+"; "+sDeviceConfig;
			fputs("#format: " +sFormat + "\n" , g_fileToExport);
		break;

		default:
			fputs("#no format: \n" , g_fileToExport);
			break;
	}
}
//@}








