/**@name LIBRARY: cietGenericFunctions.ctl

@author: Celine VUILLERMOZ (AB/CO)

Creation Date: 24-08-2005

Modification History: 
   . 16/03/09  Quique  cietGenericFunctions_setDefaultUser modified. Not any more hardcoded passwords
   . 06/02/09: Frederic updates for the new fwAccessControl (cietGenericFunctions_setDefaultUser())
   . 29/09/08: Quique
               - Bug: cietGenericFunctions_getAgentNumber: variable sRef not defined (taken out)
   . 22/07/08: Quique
		- Bug: adapt to 3.6 the dpe reference: _online..stime to "_online.._stime"
		
  . 30/05/2008: Herve
    - in function cietGenericFunctions_getCIETDefaultFileAccessControl
      vision/systemIntegrity/unConfigMail.pnl for expert instead of admin


  . 13/05/08  Pablo: New function cietGenericFunctions_setDefaultUser. 
  . 14/01/08  Pablo: New functions:
  											-	cietGenericFunctions_getWFIPConfigAssigned.
  											-	cietGenericFunctions_getFreeWFIPAgents.
  											-	cietGenericFunctions_getFreeWFIPChannels.
  											-	cietGenericFunctions_getFreeWFIPDIPosition.
  . 13/11/07  Pablo: 	Changed warning messages in log to message or pop un window.
  . 18/09/07  Quique: New function cietGenericFuncitons_getPlcName
	. 18/06/07	Pablo:	Adapted to distributed system.
	. 17/04/07	Pablo: 	Adapted cietGenericFunctions_getResetInfos to specs
	. 21/02/07	Pablo: 	Changed curveId from description to Dp on cietGenericFunctions_getCurveId.
  . 19/12/06	Pablo:	New function
  										  -	cietGenericFunctions_DecodeFIPDiagReferences
  . 01/11/06	Pablo:	New function
  										  -	cietGenericFunctions_getCurveId
	. 21/09/06 Pablo		Identification card source changed from sensorStatus to ConfigFlag.
	. 08/09/06 Quique		New function
												- cietGenericFunctions_getResetInfos
	. 06/09/06 Quique		New functions: 
												-	cietGenericFuncitons_getSegmentName 
												- cietGenericFuncitons_getSegmentSlotRef
												- cietGenericFunctions_getCrateName
	. 01/09/06 Pablo		cietGenericFunctions_getDeviceId included.

version 1.0

External Function : 
	. cietGenericFunctions_DecodeFIPDiagReferences
	. cietGenericFuncitons_getSegmentName: (from the segment number CRYO: 1,3)
	. cietGenericFunctions_getCurveId
	. cietGenericFunctions_getDeviceId
	. cietGenericFunctions_getChannel : get channel number
	. cietGenericFunctions_getAgentName: get agent name.
	. cietGenericFunctions_getFIPAgentInfos : get all FIP infos
	. cietGenericFunctions_Header : get frontend status to put in faceplate header
	. cietGenericFunctions_getWFIPConfigAssigned : Get all the WFIP Config already assigned to CIET devices, from a system [Segment, Agent, Channel, DIPosition, Type]
	. cietGenericFunctions_getFreeWFIPAgents : Return Agent Numbers availables (not used) to asign to a Dp, from System Name, DpType and Segment Number assigned to it.
	. cietGenericFunctions_getFreeWFIPChannels : Return Channel Numbers availables (not used) to asign to a Dp, from System Name, DpType, Segment Number and Agent number assigned to it.
	. cietGenericFunctions_getFreeWFIPDIPosition : Return DIPositions availables (not used) to asign to a DI Dp, from System Name, Segment Number, Agent numberand Channel assigned to it.
	
Internal Functions :
	.	cietGenericFunctions_getSegmentNumber: get segment number
	. cietGenericFuncitons_getSegmentSlotRef: get the assigned number to the segment (slot number 1,3)
Purpose: This library contains Analog object functions.

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. cietTT object (DPType etc.)
	. Global variables and $parameters defined in Analog faceplate and widget
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

// constant declaration

//------------------------------------------------------------------------------------------------------------------------
// cietGenericFunctions_getCrateName
/**
Purpose: get the CRYO CRATE name from a Device (TT,PT,LT,EH,DI,Agent)
				 the number is included in the Process Input Description in the configuration (import) 
				 
				 Example: INPUT: dist_1:un-CFC_SH8_QL8CE__E-ARC78-PT-00001.
				 					OUTPUT: QYCA.12L8
				 					
Parameters:
				@Input: sDpeName 	Get CRYO FIP CRATE name
				
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: XP.
	. distributed system: yes.
*/
string cietGenericFunctions_getCrateName(string sDpName)
	{
		dyn_string dsParameters,dsNatureComponents;
		string sNature,sCrate;
		
		const int CIET_PARAMETER_CRATE=1;
		
		// DebugN("FUNCTION: cietGenericFunctions_getCrateName( sDpName= " +sDpName);
		
		unGenericDpFunctions_getParameters(sDpName, dsParameters);
		// DebugN("dsParameters= "+dsParameters);
		
		sNature=dsParameters[UN_PARAMETER_NATURE];
		dsNatureComponents = strsplit(sNature,",");
		
		if (dynlen(dsNatureComponents)>1)
			sCrate=dsNatureComponents[1];
		else
			sCrate="unknown";
		
		// DebugN("sCrate="+sCrate);
		return sCrate;
		
	}

//------------------------------------------------------------------------------------------------------------------------
// cietGenericFuncitons_getSegmentSlotRef
/**
Purpose: get the assigned number to the segment (slot number 1,3)
				 the number is included in the dpeName in the configuration (import) 
				 
				 Example: INPUT: dist_1:un-CFC_SH8_QL8CE__C-ARC78-WFIPSegment-00001.
				 					OUTPUT: 1
				 					
Parameters:

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: XP.
	. distributed system: yes.
*/
int cietGenericFuncitons_getSegmentSlotRef(string sDpName)
{
 dyn_string dsConfig;

	// DebugN("FUNCTION: cietGenericFuncitons_getSegmentSlotRef()");
	// DebugN("sDpName= "+sDpName);
	dsConfig=strsplit(sDpName, UN_DPNAME_SEPARATOR);
	// DebugN("dsConfig[UN_DPNAME_FIELD_NUMBER]= "+dsConfig[UN_DPNAME_FIELD_NUMBER]);
	
	if (dynlen(dsConfig)>=UN_DPNAME_FIELD_LENGTH)
		return (int) dsConfig[UN_DPNAME_FIELD_NUMBER];	
	else
		return 0;
	
}

//------------------------------------------------------------------------------------------------------------------------
// cietGenericFuncitons_getPlcName
/**
Purpose: get PLC name from the WFIP segment name

Parameters:

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: XP.
	. distributed system: yes.
*/
string cietGenericFuncitons_getPlcName(string sDpName)
{
	string sSegmentName;
	string sSegmentNumber;
	string sGatewayName;
	string sTemp;
	dyn_string dsDpes;

	// Based in the fact of having a fip net by gateway. (virtual concept)	
	sGatewayName= cietGenericFunctions_getGTWName(sDpName);
	// sSegmentNumber=cietGenericFunctions_getSegmentNumber(sDpName);
	
	// 1- Extract the gateway name from the sDpName
	// 2- Search for a PLC device dpe with the gateway in the name 
	// 3- OPTIONAL: Check if the number included in the segment dpe name is equal to the one used.
		
	dsDpes = dpNames("*:*"+sGatewayName+"*"+"CryoSgtPLC"+"*");
					
	for (int i=1;i<=dynlen(dsDpes);i++)
		{	
			if (dpTypeName(dsDpes[i])=="CryoSgtPLC")		// Only PLC devices
				{
					sTemp= unGenericDpFunctions_getAlias(dsDpes[i]);
					return sTemp;
				}
		}	
	return "";
} 

//------------------------------------------------------------------------------------------------------------------------
// cietGenericFuncitons_getSegmentName
/**
Purpose: get Segment number from its number

Parameters:

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: XP.
	. distributed system: yes.
*/
string cietGenericFuncitons_getSegmentName(string sDpName)
{
	string sSegmentName;
	string sSegmentNumber;
	string sGatewayName;
	string sTemp;
	dyn_string dsDpes;

	// DebugN("FUNCTION: cietGenericFuncitons_getSegmentName(sDpName="+sDpName+")");
	
	// Based in the fact of having a fip net by gateway. (virtual concept)	
	sGatewayName= cietGenericFunctions_getGTWName(sDpName);
	//DebugN("sGatewayName= "+sGatewayName);
	
	// sSegmentNumber=cietGenericFunctions_getSegmentNumber(sDpName);
	
	// 1- Extract the gateway name from the sDpName
	// 2- Search for a FIp segment dpe with the gateway in the name 
	// 3- OPTIONAL: Check if the number included in the segment dpe name is equal to the one used.
		
	dsDpes = dpNames("*:*"+sGatewayName+"*");
	// DebugN("FUNCTION: cietGenericFuncitons_getSegmentName --> dsDpes="+dsDpes);
	for (int i=1;i<=dynlen(dsDpes);i++)
		{	
			if (dpTypeName(dsDpes[i])=="WFIPSegment")		// Only WFIP segments
				{
					sTemp= unGenericDpFunctions_getAlias(dsDpes[i]);
					return sTemp;
				}
		}	
	return "";
} 

//------------------------------------------------------------------------------------------------------------------------
// cietGenericFunctions_getChannelNumber
/**
Purpose: get channel number

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
string cietGenericFunctions_getChannelNumber(string sDpName)		
{
	string sChannelNumber;
	int iRes;
	if(dpExists (sDpName+".ProcessInput.Config.chNumber"))
	{
		iRes = dpGet(sDpName+".ProcessInput.Config.chNumber", sChannelNumber);
	}
	else iRes=-1;
		
	if (iRes ==0)
		return sChannelNumber;
	else return "";	
}
//------------------------------------------------------------------------------------------------------------------------
// cietGenericFunctions_getSegmentNumber
/**
Purpose: get SegmentNumber

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
string cietGenericFunctions_getSegmentNumber(string sDpName)
{
	string sSgNumber, sSgName;
	int iRes;
	if(dpExists (sDpName+".ProcessInput.Config.sgNumber"))
	{
		iRes = dpGet(sDpName+".ProcessInput.Config.sgNumber", sSgNumber);
		sSgName = sSgNumber;
	}
	else iRes=-1;
		
	if (iRes ==0)
		return sSgName;
	else return "";	
}

//------------------------------------------------------------------------------------------------------------------------
// cietGenericFunctions_getAgentNumber
/**
Purpose: get agent number

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
string cietGenericFunctions_getAgentNumber(string sDpName)		
{
	string sAgentNumber;
	int iRes;
        dyn_string exceptionInfo;
	// DebugN("FUNCTION: cietGenericFunctions_getAgentNumber(sDpName="+sDpName);
	
	if(dpExists (sDpName+".ProcessInput.Config.agNumber"))
		{
			iRes = dpGet(sDpName+".ProcessInput.Config.agNumber", sAgentNumber);
		}
	else 
		{
			iRes=-1;
			// unMessageText_send(getSystemName(), myManNum(), "unicosHMI initialisation", "user", "*", "INFO", getCatStr("unApplication",  "sDpName ="+sDpName+".ProcessInput.Config.agNumber"+ " does not exist") +": ", exceptionInfo);
			//DebugN("ERROR: function cietGenericFunctions_getAgentNumber();");			
		}
	if (iRes ==0)
		return sAgentNumber;
	else return "";	
}

//------------------------------------------------------------------------------------------------------------------------
// cietGenericFunctions_getAgentName
/**
Purpose: get AgentNumber

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
string cietGenericFunctions_getAgentName(string sDpName)
{
	string  sAgName, sQuery1, sQuery2, sType, sTemp,sGatewayName;
	int iRes, iAgNumber, iSgNumber;
	dyn_dyn_anytype queryTab1, queryTab2;
	dyn_string 	dsQueryTab1, dsQueryTab2, dsQueryIntersect, dsCandidates;

	//DebugN("--- cietGenericFunctions_getAgentName ---");
	//DebugN("sDpName = "+ sDpName);  //*********************************************************
	
	iAgNumber=cietGenericFunctions_getAgentNumber(sDpName);
	iSgNumber=cietGenericFunctions_getSegmentNumber(sDpName);
	sGatewayName=cietGenericFunctions_getGTWName(sDpName);
	
	//DebugN("iAgNumber= "+iAgNumber);
	//DebugN("iSgNumber= "+iSgNumber);	
	//DebugN("sGatewayName= "+sGatewayName);	

	if(iAgNumber%2 == 0)
		sType = "WFIPEvenAgent";
	else 
		sType = "WFIPOddAgent";

  string sSystemName;//------------------------- NEW !!!
	sSystemName = unGenericDpFunctions_getSystemName(sDpName);//------------------------- NEW !!!
	dsCandidates=dpNames(sSystemName+"*"+sGatewayName+"*",sType);  // gives back at most 2   --------------- NEW!!	

	// dsCandidates=dpNames("*"+sGatewayName+"*",sType);  // gives back at most 2
	
	//DebugN("Inside dp Names || sGatewayName = " + sGatewayName);
	//DebugN("Inside dp Names || sType = " + sType);
		
	//DebugN("dsCandidates= "+dsCandidates);
	for (int i=1;i<=dynlen(dsCandidates);i++)
		{	
			// DebugN("dsCandidates["+i+"]= "+dsCandidates[i]);
			// DebugN("unGenericDpFunctions_getAlias(dsCandidates["+i+"])= "+unGenericDpFunctions_getAlias(dsCandidates[i]));
			if ((cietGenericFunctions_getAgentNumber(dsCandidates[i])==iAgNumber) &&
					(cietGenericFunctions_getSegmentNumber(dsCandidates[i])==iSgNumber))
				{
					sTemp= unGenericDpFunctions_getAlias(dsCandidates[i]);
					// DebugN("sTemp="+sTemp);
					return sTemp;
				}
		}

//DebugN("sTemp = " +  sTemp);		
		
	return "";
	
//DebugN("--------------");	
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietGenericFunctions_getGTWName
/**
Purpose: this function returns GTWName according to a device (DpName)

Parameter:
	-> dsDp, string input, datapoint name = device

Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, XP
	. distributed system: yes.
*/

string cietGenericFunctions_getGTWName(string sDevice)
{
dyn_string dsTemp;

dsTemp=strsplit(sDevice, UN_DPNAME_SEPARATOR);

if (dynlen(dsTemp)>=UN_DPNAME_FIELD_LENGTH)
	return dsTemp[UN_DPNAME_FIELD_PLCNAME];	
else
	return "";
}
 
//------------------------------------------------------------------------------------------------------------------------
// cietGenericFunctions_getResetInfos
/**
Purpose: get all needed RESET infos

Parameters:
	input	

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: XP.
	. distributed system: yes.
*/
cietGenericFunctions_getResetInfos(string sDpName,dyn_string &dsRESETInfos,dyn_string &exceptionInfo)
{
	dyn_string dsTemp,dsFIPInfos;
	string deviceSystemName,deviceName,sDpGTWName,sDpAgentName, deviceAgentName, sAgPrefix;
	int iDpAgentNumber,iDpSegmentNumber,iDpChannelNumber;

	string sTopEvenAgent, sBottomEvenAgent, sTopOddAgent, sBottomOddAgent;
	string sCrateName;
	string sTopSegment, sBottomSegment;

	char	cCrateLevel;
	int	iPartner, iOppositeOdd, iOppositeEven;
	string sPartner, sOppositeOdd, sOppositeEven, sOppositeNet;

		// --   G E T T I N G   I N F O   --
		
		deviceSystemName = unGenericDpFunctions_getSystemName(sDpName);
		
		deviceName = unGenericDpFunctions_getDpName(sDpName);							// Device
		sCrateName = cietGenericFunctions_getCrateName(sDpName);					// Crate Name

			// From FIP Infos
		
		cietGenericFunctions_getFIPAgentInfos(deviceName,dsFIPInfos, exceptionInfo);		
		if(dynlen(exceptionInfo)>0)			// in case of problems
			{	
				DebugN("ERROR: Problem getting FIP infos");
			return;
			}
	
		sDpAgentName = dsFIPInfos[ciet_PARAMETER_AGENTNAME];							// Agent Name
  	iDpAgentNumber = dsFIPInfos[ciet_PARAMETER_AGENTNUMBER];
		iDpSegmentNumber = dsFIPInfos[ciet_PARAMETER_BUS];
		iDpChannelNumber = dsFIPInfos[ciet_PARAMETER_CHANNELNUMBER];

		sDpGTWName = dsFIPInfos[ciet_PARAMETER_GTWNAME];									// Gateway
	
		sAgPrefix = strrtrim(sDpAgentName, "_"+iDpAgentNumber); 					// Agent Prefix
		
			// From specs
			
		deviceAgentName = dpAliasToName (deviceSystemName +sDpAgentName);	// Device Agent Name

		dpGet(deviceAgentName + "Config.partner", iPartner);							// Partner
		sPartner = sAgPrefix + "_" + iPartner;								
		dpGet(deviceAgentName + "Config.oppositeNet", sOppositeNet);			// Opposite Net
		strreplace (sOppositeNet, "-","_");
		dpGet(deviceAgentName + "Config.oppositeOdd", iOppositeOdd);			// Opposite Odd
		sOppositeOdd = sOppositeNet + "_" + iOppositeOdd;
		dpGet(deviceAgentName + "Config.oppositeEven", iOppositeEven);		// Opposite Even
		sOppositeEven = sOppositeNet + "_" + iOppositeEven;		
		dpGet(deviceAgentName + "Config.crateLevel", cCrateLevel);				// Crate Level				

		// --   A G E N T S   /   S E G M E N T S --
		
		switch(iDpAgentNumber%2)
			{
				case 0:											// Even
					
					if (cCrateLevel=="A") 			// Top
						{
							sTopEvenAgent = sDpAgentName;
							sTopOddAgent = sPartner;
							sBottomEvenAgent = sOppositeEven;
							sBottomOddAgent = sOppositeOdd;
							sTopSegment = sAgPrefix;
							sBottomSegment = sOppositeNet;
						}
					else if (cCrateLevel=="B") // Bottom
						{
							sTopEvenAgent = sOppositeEven;
							sTopOddAgent = sOppositeOdd;
							sBottomEvenAgent = sDpAgentName;
							sBottomOddAgent = sPartner;
							sTopSegment = sOppositeNet;
							sBottomSegment = sAgPrefix;						
						}
					break;	

				case 1:											// Odd

					if (cCrateLevel=="A") 			// Top
						{
							sTopEvenAgent = sPartner;
							sTopOddAgent = sDpAgentName;
							sBottomEvenAgent = sOppositeEven;
							sBottomOddAgent = sOppositeOdd;
							sTopSegment = sAgPrefix;
							sBottomSegment = sOppositeNet;
						}
					else if (cCrateLevel=="B") // Bottom
						{
							sTopEvenAgent = sOppositeEven;
							sTopOddAgent = sOppositeOdd;
							sBottomEvenAgent = sPartner;
							sBottomOddAgent = sDpAgentName;
							sTopSegment = sOppositeNet;
							sBottomSegment = sAgPrefix;
						}
					break;
			}		
		
		// - - -   R E S U L T S   - - -  
		
		dsRESETInfos[ciet_PARAM_RESET_DEVICE] = deviceName;
		dsRESETInfos[ciet_PARAM_RESET_GATEWAY] = sDpGTWName;
		dsRESETInfos[ciet_PARAM_RESET_TOPEVEN_AGENT] = sTopEvenAgent;
		dsRESETInfos[ciet_PARAM_RESET_TOPODD_AGENT] = sTopOddAgent;
		dsRESETInfos[ciet_PARAM_RESET_BOTTONEVEN_AGENT] = sBottomEvenAgent;
		dsRESETInfos[ciet_PARAM_RESET_BOTTONODD_AGENT] = sBottomOddAgent;
		dsRESETInfos[ciet_PARAM_RESET_CRATE] = sCrateName;
		dsRESETInfos[ciet_PARAM_RESET_TOPSEGMENT] = sTopSegment;
		dsRESETInfos[ciet_PARAM_RESET_BOTTONSEGMENT] = sBottomSegment;

}

//------------------------------------------------------------------------------------------------------------------------
// cietGenericFunctions_getFIPAgentInfos
/**
Purpose: get all FIP infos

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietGenericFunctions_getFIPAgentInfos(string sDpName,dyn_string &dsFIPInfos,dyn_string &exceptionInfo)		
{

//DebugN("---  cietGenericFunctions_getFIPAgentInfos   ---");
//DebugN("sDpName = " + sDpName);


	dsFIPInfos[ciet_PARAMETER_GTWNAME] = cietGenericFunctions_getGTWName(sDpName);
	dsFIPInfos[ciet_PARAMETER_BUS] = cietGenericFunctions_getSegmentNumber(sDpName);
	dsFIPInfos[ciet_PARAMETER_AGENTNAME] = cietGenericFunctions_getAgentName(sDpName);
	dsFIPInfos[ciet_PARAMETER_AGENTNUMBER] = cietGenericFunctions_getAgentNumber(sDpName);
	dsFIPInfos[ciet_PARAMETER_CHANNELNUMBER] = cietGenericFunctions_getChannelNumber(sDpName);
//DebugN("GTWName - SegmentName - AgentName - ChannelNumber", dsFIPInfos);
//DebugN("---------------------------------");	
	
}


//------------------------------------------------------------------------------------------------------------------------
// cietGenericFunctions_cietGenericFunctions_Header
/**
Purpose: get the status of the system integrity to put in the faceplate header

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

bool cietGenericFunctions_Header(const string sDp, const int iAlarm, const bool bSystemAlarmEnable, const bool bInvalid=FALSE)
{
bool bOk=false, bFrontEnd=false;
time tLast;
string sBackColor, sForeColor, sText, sTime, sName;

	sName=unGenericDpFunctions_getAlias(sDp);
	if (sName=="")
		sName="?";
	else if (sName!=sDp)
		bFrontEnd=true;
		
//	DebugN(sDp,sName);
	
	
	if (bInvalid)
		{
		sBackColor="unDataNotValid";
		sForeColor="_3DText";
		sTime="";
		sText="DATA INVALID";
		}
	else if (!bSystemAlarmEnable)
		{
		//SystemAlarm disabled
		sBackColor="unAlarmMasked";
		sForeColor="_3DText";
		sTime="";
		sText=sName + " DATA Connection DISABLED";
		}
	else
		{
		if (iAlarm > c_unSystemIntegrity_no_alarm_value)
			{
			//Connection stopped
			sBackColor="unDataNotValid";
			sForeColor="_3DText";
			sText=sName + " NO DATA Connection";
			
			if(bFrontEnd)
				{
				if (dpExists(sDp))
					{
					dpGet(sDp + ".communication.counter:_online.._stime", tLast);
					sTime=" since: " + (string)tLast; 
					}
				else
					{
					sTime="?";
//					DebugN("cietGenericFunctions_Header", sDp + " Datapoint Name doesn't exist! Contact UNICOS support");
					}		
				}
			else
				sTime="";
			}
		else
			{
			//Connection OK
			sBackColor="_3DFace";
			sForeColor="_3DText";
			sText=sName + " DATA Connection OK";
			sTime="";			
			bOk=true;
			}
		}
	
	sText=sText + sTime;
	if(shapeExists("txtPLCstatus"))
		{
		setMultiValue("txtPLCstatus", "backCol", sBackColor,
									"txtPLCstatus", "foreCol", sForeColor,
									"txtPLCstatus", "text", sText);
		}
	else
//		DebugN("cietGenericFunctions_Header", "\"txtPLCstatus\" shape doesn't exist! Contact UNICOS support");

return bOk;		

}

//------------------------------------------------------------------------------------------------------------------------
// cietGenericFunctions_cietGenericFunctions_Header
/**
Purpose: get the status of the system integrity to put in the faceplate header

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietGenericFunctions_HeaderDisconnect()
{
	string sBackColor, sForeColor, sText;

	sBackColor="_3DFace";
	sForeColor="unDataNoAccess";
	sText="DATA not connected";
				
	if(shapeExists("txtPLCstatus"))
		{
		setMultiValue("txtPLCstatus", "backCol", sBackColor,
									"txtPLCstatus", "foreCol", sForeColor,
									"txtPLCstatus", "text", sText);
		}
	else
		unMessageText_send(getSystemName(), myManNum(), "unicosHMI initialisation", "user", "*", "INFO", getCatStr("unApplication", "\"txtPLCstatus\" shape doesn't exist! Contact UNICOS support") +": "+sRef, exceptionInfo);
		//DebugN("cietGenericFunctions_HeaderDisconnect", "\"txtPLCstatus\" shape doesn't exist! Contact UNICOS support");
}


//------------------------------------------------------------------------------------------------------------------------
// cietGenericFunctions_cietGenericFunctions_getDeviceId
/**
Purpose: get a string within the type of device (TT,LT,PT or DI) from sensorByte[5].

Parameters:

Return a string.
Needs $sDpName.

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

string cietGenericFunctions_getDeviceId (string sDpName)	
  
{
			int  iIdCard, iMascIdCard;
			string sShowIdCard, deviceName;
	
	
		  deviceName = unGenericDpFunctions_getDpName($sDpName);
    	dpGet(deviceName + ".ProcessInput.Data.configFlag" ,iIdCard);

	  	iMascIdCard = 30; 										// 00011110b
 	  	iIdCard = iIdCard & iMascIdCard;			// 00011110b AND XXXXXXXXb = 000XXXX0b

			switch (iIdCard)
			{
				case 16:									// B4 FipRawData = XXX1000Xb -> DI
				sShowIdCard="DI";
				break;
		
				case 8:										// B4 FipRawData = XXX0100Xb -> LT
				sShowIdCard="LT";
				break;
		
				case 4:										// B4 FipRawData = XXX0010Xb -> PT
				sShowIdCard="PT";		
				break;
		
				case 2:										// B4 FipRawData = XXX0001Xb -> TT
				sShowIdCard="TT";		
				break;

				default:									// B4 FipRawData = XXXXXXXXb -> ERROR						
				sShowIdCard="ERROR";		
			}
		return sShowIdCard;		
}	  

//------------------------------------------------------------------------------------------------------------------------
// cietGenericFunctions_getCurveId
/**
Purpose: get a string within the identifier of the Calibration Curve(EH, PT or TT).

Parameters:

Return a string.
Needs $sDpName, sDeviceType, iCurveNumber

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

string cietGenericFunctions_getCurveId (string sDpName, string sDeviceType, int iCurveNumber)	
{
			string sIdCurve, deviceName, sDpe;

			deviceName = unGenericDpFunctions_getDpName($sDpName);

			if (sDeviceType == "PT" || sDeviceType == "EH") 
				{
					sDpe=".ProcessInput.Curve.curveId";
    		}
    	else if	(sDeviceType == "TT" && iCurveNumber == 1)
				{
					sDpe=".ProcessInput.Curve.curveId1";
				}	
			else if (sDeviceType == "TT" && iCurveNumber == 2)
				{
					sDpe=".ProcessInput.Curve.curveId2";
				}											

  		//sIdCurve= dpGetDescription(deviceName + sDpe);  
  		dpGet(deviceName + sDpe,sIdCurve);

   	//	DebugN ("sDpe=", deviceName + sDpe);
		//	DebugN ("sIdCurve=", sIdCurve);
		//	DebugN ("---------------------------------------");  		 	
			
 			return sIdCurve;		
}

		
//------------------------------------------------------------------------------------------------------------------------		
//cietGenericFunctions_DecodeFIPDiagReferences(sfipDiagConstructor, sfipDiagModel, sfipDiagLocation, sfipDiagSystem, sfipDiagSegment);
/**
Purpose: Decode Constructor and Model Bytes to get Fip Diag References for Segment.

Parameters:

Needs string sfipDiagConstructor, string sfipDiagModel.
Modifies: string sfipDiagLocation, string sfipDiagSystem, string sfipDiagSegment.
Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietGenericFunctions_DecodeFIPDiagReferences (string sfipDiagConstructor, string sfipDiagModel, string &sfipDiagLocation, string &sfipDiagSystem, string &sfipDiagSegment)
{

string sAllData, sSystem, sSpare, sLocation, sSegment;
int iAllData, ifipDiagConstructor, ifipDiagModel, iSystem, iSpare, iLocation, iSegment;
bit32 BAllData, BfipDiagConstructor, BfipDiagModel, BSystem, BSpare, BLocation, BSegment;

dyn_string dsSystem = makeDynString ("NULL","CRYO","RECTIFER(PO)", "QPS","BLM/BPM","SURVEY","DOSIMETRY","EXPERIMENT","RF");
dyn_string dsSegmentLetter = makeDynString ("A","B","C", "D","E","F","G","H","I","J", "K","L","M", "N","U","Z","O","P");
dyn_string dsLocation = makeDynString ("IP8","IP1","IP2","IP3","IP4","IP5","IP6","IP7","EX4","BA7");

ifipDiagConstructor=sfipDiagConstructor;							// To B32
ifipDiagModel=sfipDiagModel;
BfipDiagConstructor=ifipDiagConstructor;
BfipDiagModel=ifipDiagModel;

sfipDiagConstructor=BfipDiagConstructor;							// To String within 32B
sfipDiagConstructor=sfipDiagConstructor+"00000000";		// DiagConstructor Shifted
BfipDiagConstructor=sfipDiagConstructor;							// Diag Constructor Shifted to  32B
BAllData=BfipDiagConstructor|BfipDiagModel;  					// Or bit a bit Diag Constructor shifted and DiagModel
sAllData = BAllData;																	// All Data well presented to string
sAllData = substr (sAllData, 16);											// All Data to string

sSystem = substr (sAllData, 0, 4);										// Associate bits to info
sSpare = substr (sAllData, 4,3);
sLocation = substr (sAllData, 7,4);
sSegment = substr (sAllData, 11,5);

BSystem = sSystem;																		// Info bits to int
BSpare = sSpare;
BLocation = sLocation;
BSegment = sSegment;
iSystem = BSystem;																		
iSpare = BSpare;
iLocation = BLocation;
iSegment = BSegment;

sfipDiagLocation = dsLocation[iLocation+1];							// Show info :)
sfipDiagSystem = dsSystem[iSystem+1];
sfipDiagSegment = dsSegmentLetter[iSegment+1];

}
		
//------------------------------------------------------------------------------------------------------------------------		


//------------------------------------------------------------------------------------------------------------------------		
//cietGenericFunctions_opendeviceCIETConsole (string sDeviceDpName, string sDeviceDpType, dyn_string &exceptionInfo);
/*
Purpose: User access for CIET buttons

Parameters: 
	- sDeviceDpName, string, input, device name DP name
	- sDeviceDpType, string, input, device DP type
	- exceptionInfo, dyn_string, output, exception
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietGenericFunctions_opendeviceCIETConsole (string sDeviceDpName)
{

	bool bConfigExists, bIsActive;
	string sPLCName, sCMWAddress;
	string sDeviceName, sFECName, sCMWName;
	dyn_string ds_DeviceDpNameFields, ds_CMWFields, ds_PLCFields, ds_CMWAddress, dExceptionInfo;


	//  - - Build Device Name - -
			
			ds_DeviceDpNameFields = strsplit (sDeviceDpName, "-");											// Ex: dist_1:un | CFC_SH8_QL8CE__E | AppliTest | WFIPSegment | 00001
			sDeviceName = ds_DeviceDpNameFields[4]+ds_DeviceDpNameFields[5];						// Ex: WFIPSegment00001

	//  - - Build CMW Name - -

			fwPeriphAddress_get(sDeviceDpName + ".ProcessInput.Data.statusFIP", bConfigExists, ds_CMWAddress, bIsActive, dExceptionInfo);
																																									// Ex: ds_CMWAddress = CMW |  | CIET.CBWMB-SH8-QL8E$Data$statusFIP/timeStamp |  | 1004 | TRUE |  |  |  |  | TRUE | 0 | 1970.01.01 01:00:00.000 | 1970.01.01 01:00:00.000 |  | B		
			sCMWAddress = ds_CMWAddress[3];																							// Ex: sCMWAddress = CIET.CBWMB-SH8-QL8E$Data$statusFIP/timeStamp
			ds_CMWFields = strsplit (sCMWAddress, "$"); 																// Ex: CIET.CBWMB-SH8-QL8E | Data$statusFIP/timeStamp																																
			sCMWName = ds_CMWFields[1];																									// Ex: CIET.CBWMB-SH8-QL8E

	//  - - Build FEC Name - -
				
			sPLCName = unGenericObject_GetPLCNameFromDpName(sDeviceDpName); 						// Ex: CRYOGTW_CFC_SH8_QL8CE__E 										
			ds_PLCFields = strsplit (sPLCName, "_"); 																		// Ex: CRYOGTW | CFC | SH8 | QL8CE | | E
			sFECName = ds_PLCFields[2] + "-" + ds_PLCFields[3] + "-" + ds_PLCFields[4]; // Ex: CFC-SH8-QL8CE			
			
	// 	- - Call function UNICOS OpenConsole- -
	
			unFipDiag_openFipDiagExpert (sFECName, sCMWName, sDeviceName);	

}
//------------------------------------------------------------------------------------------------------------------------
// cietGenericFunctions_getWFIPConfigAssigned
/**
Purpose: get all the WFIP Config already assigned to CIET devices, from a system [Segment, Agent, Channel, DIPosition] 

				 					
Parameters:
				Input: string sSystemName: System Name
				Output: dyn_dyn_int: Existing Config [i]= [Segment[i], Agent[i], Channel[i], DIPosition[i], Type[1]]
				
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: XP.
	. distributed system: yes.
*/
dyn_dyn_int cietGenericFunctions_getWFIPConfigAssigned(string sSystemName)
{
	
	dyn_dyn_int ddiExistingConfig; 								// ddiExistingConfig = [Segment, Agent, Channel, Position, Type]
	dyn_string dsDps, dsDpAgents;
	int iExistingSegmentNumber,iExistingAgentNumber,iExistingChannelNumber,iExistingDINumber;
	int i, iType;

	dsDpAgents = dpNames(sSystemName + "*.ProcessInput.Config.agNumber");	// DpAgents

	for (i=1;i<=dynlen(dsDpAgents);i++)
	{
		dsDps[i]=unGenericDpFunctions_getDpName(dsDpAgents[i]); // Each dp with channel number

		if (dpExists(dsDps[i]+".ProcessInput.Config.sgNumber")) 
			dpGet(dsDps[i]+".ProcessInput.Config.sgNumber", iExistingSegmentNumber);

		if (dpExists(dsDps[i]+".ProcessInput.Config.agNumber")) 
			dpGet(dsDps[i]+".ProcessInput.Config.agNumber", iExistingAgentNumber);
		else
		{
				iType=1;	// Segment		
				iExistingSegmentNumber=0;				
		}
					
		if (dpExists(dsDps[i]+".ProcessInput.Config.chNumber")) 
			dpGet(dsDps[i]+".ProcessInput.Config.chNumber", iExistingChannelNumber);
		else
			{
				iType=2;	// Agent		
				iExistingChannelNumber=0;				
			}
			
		if (dpExists(dsDps[i]+".ProcessInput.Config.diNumber")) 
			{
				dpGet(dsDps[i]+".ProcessInput.Config.diNumber", iExistingDINumber); 
				iType=4;		// TT, PT, LT
			}		
		else
			{
				iType=3;	// DI
				iExistingDINumber=0;
			}


			
		ddiExistingConfig[i]=makeDynInt(iExistingSegmentNumber,iExistingAgentNumber,iExistingChannelNumber,iExistingDINumber, iType);
	}
	
	return(ddiExistingConfig);
}

//------------------------------------------------------------------------------------------------------------------------
// cietGenericFunctions_getFreeWFIPAgents
/**
Purpose: Return Agent Numbers availables (not used) to asign to a Dp, from System Name, DpType and Segment Number assigned to it.

				 					
Parameters:
	Input:
			string sSystemName: System Name
			string sDpType: Dp Type
			int iSegment Number: Segment Number assigned to Dp.
								
Output: dyn_int: diDpAgentNumbers= List of Agent Numbers availables
				
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: XP.
	. distributed system: yes.
*/

dyn_int cietGenericFunctions_getFreeWFIPAgents(string sSystemName, string sDpType, int iSegmentNumber)
{

	int iPosition;
	int i, iAgents, iAgentNumber;
	dyn_int diExistingConfig, diDpAgentNumbers;
  dyn_dyn_int ddiExistingConfig;
  
  // Config used by all Dps in this system

	ddiExistingConfig=cietGenericFunctions_getWFIPConfigAssigned(sSystemName); 						// ddiExistingConfig[i] = [Segment, Agent, Channel]
  
	// Possible Agents.
	
	  dynClear(diDpAgentNumbers);
	  switch (sDpType)
	  			{
	  			 case "EH":
	  			 case "WFIPEvenAgent":	  			 
											for (iAgents=2;iAgents<=98;iAgents=iAgents+2) 
													dynAppend(diDpAgentNumbers, iAgents);
	  			 break;
	  			 
 	  			 case "DI":	  			 
	  			 case "LT":
 	  			 case "PT":
 	  			 case "TT":
	  			 case "WFIPOddAgent":  			 
											for (iAgents=1;iAgents<=99;iAgents=iAgents+2) 
													dynAppend(diDpAgentNumbers, iAgents); 	  			 
	  			 break;
	  			}
				
  // Remove existing Agent Numbers already full (ones with same system name and segment number).	
/*
		for (i=1;i<=dynlen(ddiExistingConfig);i++)																						
					{
						diExistingConfig=ddiExistingConfig[i];
						if(diExistingConfig[1]==iSegmentNumber)
							{
								iAgentNumber=diExistingConfig[2];						// Get Agent Number used if Segment number is the same				//*****																	
								if (iAgentNumber>=1 && iAgentNumber<=99) 		// Remove from possible list if it is valid one.
										{
											iPosition=dynContains(diDpAgentNumbers,iAgentNumber);
											if (iPosition>=1)
												dynRemove(diDpAgentNumbers, iPosition);
										}	

							}
					}
*/				
	return(diDpAgentNumbers);
}

//------------------------------------------------------------------------------------------------------------------------
// cietGenericFunctions_getFreeWFIPChannels
/**
Purpose: Return Channel Numbers availables (not used) to asign to a Dp, from System Name, DpType, Segment Number and Agent number assigned to it.

				 					
Parameters:
	Input:
			string sSystemName: System Name
			string sDpType: Dp Type
			int iSegment Number: Segment Number assigned to Dp.
			int iAgent Number: Agent Number assigned to Dp.
								
Output: dyn_int: diDpChannelNumbers= List of Channel Numbers availables
				
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: XP.
	. distributed system: yes.
*/


dyn_int cietGenericFunctions_getFreeWFIPChannels(string sSystemName, string sDpType, int iSegmentNumber, int iAgentNumber)
{
	int iPosition;
	int i, iChannels, iChannelNumber;
	dyn_int diExistingConfig, diDpChannelNumbers;
  dyn_dyn_int ddiExistingConfig;
  
  // Config used by all Dps in this system

	ddiExistingConfig=cietGenericFunctions_getWFIPConfigAssigned(sSystemName); 						// ddiExistingConfig[i] = [Segment, Agent, Channel]

	// Possible channels

	  dynClear(diDpChannelNumbers);
	  
	  switch (sDpType)
	  			{
	  			 case "EH":
											diDpChannelNumbers=makeDynInt(0,1,2,3); 
	  			 break;
	  			 
 	  			 case "DI":
											diDpChannelNumbers=makeDynInt(14); 	  			 	  	
 	  			 break;		 
	  			 case "LT":
 	  			 case "PT":
 	  			 case "TT":
											for (iChannels=0;iChannels<=13;iChannels++)
												dynAppend(diDpChannelNumbers, iChannels); 	  			 
	  			 break;
	  			}
	  				  
  // Remove existing Channel Numbers already used from Possible Channel list (ones with same system name, segment number and agent number).	
					
		for (i=1;i<=dynlen(ddiExistingConfig);i++)																						
					{
						diExistingConfig=ddiExistingConfig[i];
						if(diExistingConfig[1]==iSegmentNumber && diExistingConfig[2]==iAgentNumber)
							{
								iChannelNumber=diExistingConfig[3];								// Get Channel Number used if Segment and Agent number are the same																													
									if (iChannelNumber>=0 && iChannelNumber<=14 && sDpType!="DI") 		// Remove from possible list if it is valid one.
										{
											iPosition=dynContains(diDpChannelNumbers,iChannelNumber);
											if (iPosition>=1)
												dynRemove(diDpChannelNumbers, iPosition);
										}	
							}
					}					
					
	return(diDpChannelNumbers);			
}

//------------------------------------------------------------------------------------------------------------------------
// cietGenericFunctions_getFreeWFIPDIPosition
/**
Purpose: Return DIPositions availables (not used) to asign to a DI Dp, from System Name, Segment Number, Agent numberand Channel assigned to it.

				 					
Parameters:
	Input:
			string sSystemName: System Name
			int iSegment Number: Segment Number assigned to Dp.
			int iAgent Number: Agent Number assigned to Dp.
			int iChannel Number: Channel Number assigned to Dp.
								
Output: dyn_int: diDpChannelNumbers= List of DI Positions availables
				
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: XP.
	. distributed system: yes.
*/
dyn_int cietGenericFunctions_getFreeWFIPDIPosition(string sSystemName, int iSegmentNumber, int iAgentNumber, int iChannelNumber)
{
	int iPosition;
	int i, iDIPosition, iDIPositionNumber;
	dyn_int diExistingConfig, diDpDIPositionNumbers;
  dyn_dyn_int ddiExistingConfig;
  
  
	// Config used by all Dps in this system

	ddiExistingConfig=cietGenericFunctions_getWFIPConfigAssigned(sSystemName); 						// ddiExistingConfig[i] = [Segment, Agent, Channel, DIPosition]

	// Possible channels

	  dynClear(diDpDIPositionNumbers);
		for (iDIPosition=0;iDIPosition<=11;iDIPosition++)
			dynAppend(diDpDIPositionNumbers, iDIPosition);
		
  // Remove existing Channel Numbers already used from Possible Channel list (ones with same system name, segment number and agent number).	
					
		for (i=1;i<=dynlen(ddiExistingConfig);i++)																						
					{
						diExistingConfig=ddiExistingConfig[i];

						if(diExistingConfig[1]==iSegmentNumber && diExistingConfig[2]==iAgentNumber && diExistingConfig[3]==iChannelNumber && diExistingConfig[5]==4)
							{
								iDIPositionNumber=diExistingConfig[4];									// Get DIPosition Number used if Segment, Agent and Channel number are the same																													

									if (iDIPositionNumber>=0 && iDIPositionNumber<=11) 		// Remove from possible list if it is valid one.
											{
												iPosition=dynContains(diDpDIPositionNumbers,iDIPositionNumber);
												if (iPosition>=1)
													dynRemove(diDpDIPositionNumbers, iPosition);
											}	

							}
					}					
			
	return(diDpDIPositionNumbers);		
}

//------------------------------------------------------------------------------------------------------------------------
// cietGenericFunctions_setDefaultUser
/**
Purpose: Create/reset default user, password not change if already set.

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
cietGenericFunctions_setDefaultUser(bool bForceDefaultPassword = false)
{
string acComponent="fwAccessControl";
string acVersion="3.2.7", sPasswordMonitor, sPasswordOperator, sPasswordExpert, sPasswordAdmin;
int acInstalled, iPos;
dyn_string exceptionInfo, dsUserName, dsPassword;
bool bIntegratedMode;  
  
  // Check correct version
      fwInstallation_componentInstalled(acComponent, acVersion, acInstalled);
      if (!acInstalled) 
      	{
        DebugTN("cietGenericFunctions_setDefaultUser: Access Control component " + acVersion + " not installed!");	
        return;
        }

  bIntegratedMode=_fwAccessControl_integratedMode();
  if(!bIntegratedMode)
    { 
  // -  -  -  C R E A T E   D O M A I N   -  -  -  
         
  		// CIET domain      
      if (!dynlen(exceptionInfo))
         	fwAccessControl_checkAddDomain(CIET_ACCESS_CONTROL_DOMAIN_NAME, 
                                       	 makeDynString("monitor", "operator", "expert","admin"),
      		 																exceptionInfo, 
                                       	 CIET_ACCESS_CONTROL_DOMAIN_NAME,
                                         "CIET domain name");
      
  // -  -  -  C R E A T E   G R O U P S   -  -  - 
  	
      // CIET_MONITOR_GROUPNAME      
      if (!dynlen(exceptionInfo)) 
      		{
        		fwAccessControl_checkAddGroup(CIET_MONITOR_GROUPNAME,
      		 																makeDynString("SYSTEM:Visualize","SYSTEM:Normal user level", 
              	                                        "SYSTEM:Extended user level","SYSTEM:Administration","SYSTEM:Acknowledge",
                	                                      CIET_ACCESS_CONTROL_DOMAIN_NAME+":monitor"),
      		 																exceptionInfo,
                    	                   	CIET_MONITOR_GROUPNAME,
    				  		 												"CIET monitor group");
        		DebugN("CIET group: monitor updated");
      		}

  		// CIET_OPERATOR_GROUPNAME      
      if (!dynlen(exceptionInfo))
      	{
          fwAccessControl_checkAddGroup(CIET_OPERATOR_GROUPNAME,
        		 														makeDynString("SYSTEM:Visualize","SYSTEM:Normal user level",
                                                      "SYSTEM:Extended user level","SYSTEM:Administration","SYSTEM:Acknowledge",
                                                      CIET_ACCESS_CONTROL_DOMAIN_NAME+":monitor",
                                                      CIET_ACCESS_CONTROL_DOMAIN_NAME+":operator"),
        		 														exceptionInfo,
        		 														CIET_OPERATOR_GROUPNAME,
        		 														"CIET operator group");
          DebugN("CIET group: operator updated");
      	}      
        
  		// CIET_EXPERT_GROUPNAME      
      if (!dynlen(exceptionInfo))
      	{
          fwAccessControl_checkAddGroup(CIET_EXPERT_GROUPNAME,
        		 														makeDynString("SYSTEM:Visualize","SYSTEM:Normal user level",
                                                      "SYSTEM:Extended user level","SYSTEM:Administration","SYSTEM:Acknowledge",
                                                      CIET_ACCESS_CONTROL_DOMAIN_NAME+":monitor",
                                                      CIET_ACCESS_CONTROL_DOMAIN_NAME+":operator",
                                                      CIET_ACCESS_CONTROL_DOMAIN_NAME+":expert"),
        		 														exceptionInfo,
        		 														CIET_EXPERT_GROUPNAME,
        		 														"CIET expert group");
          DebugN("CIET group: expert updated");
      	}

  		// CIET_ADMIN_GROUPNAME      
      if (!dynlen(exceptionInfo))
      	{
          fwAccessControl_checkAddGroup(CIET_ADMIN_GROUPNAME,
        		 														makeDynString("SYSTEM:Visualize","SYSTEM:Normal user level",
                                                      "SYSTEM:Extended user level","SYSTEM:Administration","SYSTEM:Acknowledge",
                                                      CIET_ACCESS_CONTROL_DOMAIN_NAME+":monitor",
                                                      CIET_ACCESS_CONTROL_DOMAIN_NAME+":operator",
                                                      CIET_ACCESS_CONTROL_DOMAIN_NAME+":expert",
                                                      CIET_ACCESS_CONTROL_DOMAIN_NAME+":admin"),
        		 														exceptionInfo,
        		 														CIET_ADMIN_GROUPNAME,
        		 														"CIET admin group");
          DebugN("CIET group: admin updated");
      	}      
        
  // -  -  -  C R E A T E   U S E R   N A M E   -  -  - 
            
  	// Get all password      
      dpGet("_Users.UserName:_online.._value", dsUserName, "_Users.Password:_online.._value", dsPassword);
    			
      if(!bForceDefaultPassword) // if not bForceDefaultPassword recuperate what it was configureed 										// else if password already defined, use it, if not set password to "" = no password
      	{ 	
				// CIET_MONITOR_USERNAME		
         	iPos = dynContains(dsUserName, CIET_MONITOR_USERNAME); 
        	if(iPos > 0) 
         	 	sPasswordMonitor = dsPassword[iPos];
        	else
          	sPasswordMonitor = CIET_PASSWORD_DEFAULT;
                
				// CIET_OPERATOR_USERNAME		
         	iPos = dynContains(dsUserName, CIET_OPERATOR_USERNAME); 
        	if(iPos > 0) 
         	 	sPasswordOperator = dsPassword[iPos];
        	else
          	sPasswordOperator = CIET_PASSWORD_OPERATOR;
                
				// CIET_EXPERT_USERNAME		
         	iPos = dynContains(dsUserName, CIET_EXPERT_USERNAME); 
        	if(iPos > 0) 
         	 	sPasswordExpert = dsPassword[iPos];
        	else
          	sPasswordExpert = CIET_PASSWORD_EXPERT;
                
				// CIET_ADMIN_USERNAME		
         	iPos = dynContains(dsUserName, CIET_ADMIN_USERNAME); 
        	if(iPos > 0) 
         	 	sPasswordAdmin = dsPassword[iPos];
        	else
          	sPasswordAdmin = CIET_PASSWORD_ADMIN;
    		}
  	// Create users
       
  		// CIET_MONITOR_USERNAME      
      if (!dynlen(exceptionInfo)) 
        	fwAccessControl_checkAddUser(CIET_MONITOR_USERNAME, makeDynString(CIET_MONITOR_GROUPNAME), exceptionInfo,
    																	 CIET_MONITOR_USERNAME,"CIET monitor user name",sPasswordMonitor, TRUE, 3072,TRUE,TRUE);
  
  		// CIET_OPERATOR_USERNAME      
      if (!dynlen(exceptionInfo)) 
        	fwAccessControl_checkAddUser(CIET_OPERATOR_USERNAME, makeDynString(CIET_OPERATOR_GROUPNAME), exceptionInfo,
    																	 CIET_OPERATOR_USERNAME,"CIET operator user name",sPasswordOperator, TRUE, 3072,TRUE,TRUE);
      
  		// CIET_EXPERT_USERNAME      
      if (!dynlen(exceptionInfo)) 
        	fwAccessControl_checkAddUser(CIET_EXPERT_USERNAME, makeDynString(CIET_EXPERT_GROUPNAME), exceptionInfo,
    																	 CIET_EXPERT_USERNAME,"CIET expert user name",sPasswordExpert, TRUE, 3072,TRUE,TRUE);
      
  		// CIET_ADMIN_USERNAME      
      if (!dynlen(exceptionInfo)) 
        	fwAccessControl_checkAddUser(CIET_ADMIN_USERNAME, makeDynString(CIET_ADMIN_GROUPNAME), exceptionInfo,
    																	 CIET_ADMIN_USERNAME,"CIET admin user name",sPasswordAdmin, TRUE, 3072,TRUE,TRUE);
      
  // -  -  -  R E S U L T S   -  -  - 
      
      if(dynlen(exceptionInfo) > 0) // error --> print to PVSS console log
        DebugTN("cietGenericFunctions_setDefaultUser: CIET domain, group and user failed", exceptionInfo);
      else 													// no error --> print to PVSS console log
        DebugTN("cietGenericFunctions_setDefaultUser: CIET domain, group and user updated");
	}
	else
		DebugTN("cietGenericFunctions_setDefaultUser: fwAccessControl mode activated, the changes are automatically populated from the AC server.");	
		
		
}
//------------------------------------------------------------------------------------------------------------------------


// cietGenericFunctions_getCIETDefaultFileAccessControl
/**
Purpose:
Get the default CIET file access control.

	@param dsFileAccess: dyn_string, output: default CIET file access

Usage: Public

PVSS manager usage: UI (PVSS00NV) and UI extended (PVSS00NG)

Constraints:
	. PVSS version: 3.6 
	. operating system: WXP and Linux.
	. distributed system: yes.
*/
cietGenericFunctions_getCIETDefaultFileAccessControl(dyn_string &dsFileAccess)
{
  string sDomain, sOperator, sExpert,sAdmin;
  
  sDomain = CIET_UNICOS_ACCESS_CONTROL_DOMAIN_NAME+","+ CIET_ACCESS_CONTROL_DOMAIN_NAME;
  
  // operator access
  sOperator = UN_USER_OPERATOR;
  sExpert = "";
  sAdmin = "";
  dynAppend(dsFileAccess, "vision/unAlertPanel/unAlertScreen.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unEventList/unEventList.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unEventList/unEventList_SelectFilter.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unObjectList/unObjectList.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "fwTrending/fwTrendingPlotConfPanel.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "fwTrending/fwTrendingPlotsPage.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "fwTrending/fwTrendingPage.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "fwTrending/fwTrendingHistogramConfig.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "fwTrending/fwTrendingTrend.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "fwTrending/fwTrendingHistogram.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  
  // expert access
  sOperator = "";
  sExpert = UN_USER_EXPERT;
  sAdmin = "";
  dynAppend(dsFileAccess, "vision/windowTree/unWindowTreeConfiguration.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/trendTree/unTrendTreeConfiguration.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/graphicalFrame/unicosSystemDPs.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/graphicalFrame/unTreeDeleteDps.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unLHCLogging/unLHCLoggingStatus.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unLaser/unLaserStatus.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unCMWClient/unCMWClientStatus.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unCMWServer/unCMWServerStatus.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/systemIntegrity/unConfigMail.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unicosObject/General/unModifyConfig.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  
  // admin access
  sOperator = "";
  sExpert = "";
  sAdmin = UN_USER_ADMIN;
  dynAppend(dsFileAccess, "vision/graphicalFrame/applicationConfiguration.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/graphicalFrame/applicationConfigurationSetDefaultSettings.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/graphicalFrame/addSystemIntegrity.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/systemIntegrity/systemIntegrityConfiguration.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/systemIntegrity/unSystemIntegrityConfigurationHMI.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/systemIntegrity/stopDriverAndCtrl.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unEventList/unEventList_Configure.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unAlertPanel/unAlertPanel_ConfigureNames.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unicosObject/General/unObjectConfigurationHMI.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unicosObject/General/unObjectTrendConfiguration.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unicosObject/General/unDeviceTypeExtendedTrendConfiguration.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unLHCLogging/unLHCLoggingManager.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unLHCLogging/unLHCLoggingConfiguration.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unLHCLogging/unLHCLoggingConfigFile.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unLHCLogging/unLHCLoggingConfigFile_Check.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unLHCLogging/unLHCLoggingManagers_ExpertSetting.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unLHCLogging/unLHCLoggingArchives_ExpertSetting.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unLaser/unLaserManager.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unLaser/unLaserConfiguration.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unLaser/unLaserConfigFile.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unLaser/unLaserConfigFile_Check.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unLaser/unLaserExpertSetting.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unCMWClient/unCMWClientDriver.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unCMWServer/unCMWServerManager.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unCMWServer/unCMWServerConfiguration.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unCMWServer/unCMWServerConfigFile.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unCMWServer/unCMWServerConfigFile_Check.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unCMWServer/unCMWServerExpertSetting.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unicosObject/General/unFrontEndConfiguration.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/distributedControl/unDistributedControl_configuration.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unicosObject/textLabel/unTextLabel_modify.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unicosObject/General/unOptionModes_Configure.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unGeneration/unGenerateDatabase.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/unGeneration/unImportTree.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/graphicalFrame/unFileAccessControlConfiguration.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  dynAppend(dsFileAccess, "vision/graphicalFrame/unMenuConfiguration.pnl"+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sDomain+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sOperator+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sExpert+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR+sAdmin+UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
}

//------------------------------------------------------------------------------------------------------------------------
