/**@name LIBRARY: cietTT.ctl

@author: Celine VUILLERMOZ (AB/CO)

Creation Date: 24-08-2005

Modification History:
   04/11/11 achiron: add a condition in cietGenericObject_getCrate function in case of the Nature field would be empty
 . 01/04/09 Quique: cietGenericObject_getCrate() not working whenever the NATURE is not multiple nature.
	.	05/11/07 Pablo: New function cietGenericObject_OpenDebug to open debug console.
	.	29/08/07 Pablo: New function cietGenericObject_OpenHtmlFile to open html file with Browser from data/html directory or internet.
	. 08/09/06 Quique: New function (cietGenericObject_resetDevice) to call the RESET panel  with the right parameters
	. 08/08/06 Quique: Include new function to display integeres:  cietGenericObject_DisplayInt()

version 1.0

External Function : 

	.	cietGenericObject_OpenHtmlFile: open html file with Browser from data/html directory or internet.
	. cietGenericObject_resetDevice: call the RESET panel  with the right parameters
	
	. cietGenericObject_toHexa : format an int value to a string in hexa
	. cietGenericObject_DisplayString : display a string
	. cietGenericObject_DisplayInt: display an integer
	.	cietGenericObject_OpenDebug: Open debug Console
	
Internal Functions :
		
Purpose: This library contains Analog object functions.

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. cietTT object (DPType etc.)
	. Global variables and $parameters defined in Analog faceplate and widget
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

// constant declaration

//---------------------------------------------------------------------------------------------------------------------------------------
//@{

// cietGenericObject_resetDevice
/**
Purpose: RESET a defined device (Open panel to RESET with right parameters

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.1
	. operating system: XP.
	. distributed system: yes.
*/
cietGenericObject_resetDevice(string sDpName, dyn_string exceptionInfo)
	{
	int xPos, yPos;
	dyn_int diPanelSize;
	bool bRes;
	
	dyn_string dsRESETInfos;
	string sDpGTWName,sCrateName,sTopSegment,sBottonSegment,deviceName;
	string sTopEvenAgent, sBottonEvenAgent, sTopOddAgent, sBottonOddAgent;

	// DebugN("FUNCTION: cietGenericObject_resetDevice(sDpName="+sDpName+")");
	
	// Recuperate RESET information
	cietGenericFunctions_getResetInfos(sDpName,dsRESETInfos,exceptionInfo);
	
	deviceName=dsRESETInfos[ciet_PARAM_RESET_DEVICE];
	sDpGTWName=dsRESETInfos[ciet_PARAM_RESET_GATEWAY];
	sTopEvenAgent=dsRESETInfos[ciet_PARAM_RESET_TOPEVEN_AGENT];
	sTopOddAgent=dsRESETInfos[ciet_PARAM_RESET_TOPODD_AGENT];
	sBottonEvenAgent=dsRESETInfos[ciet_PARAM_RESET_BOTTONEVEN_AGENT];
	sBottonOddAgent=dsRESETInfos[ciet_PARAM_RESET_BOTTONODD_AGENT];
	sCrateName=dsRESETInfos[ciet_PARAM_RESET_CRATE];
	sTopSegment=dsRESETInfos[ciet_PARAM_RESET_TOPSEGMENT];
	sBottonSegment=dsRESETInfos[ciet_PARAM_RESET_BOTTONSEGMENT];
	
		getValue("", "position", xPos, yPos);
		diPanelSize = getPanelSize("vision/cietDevices/General/cietResetDevice.pnl");
		if (dynlen(diPanelSize) == 2)
			{
			xPos = xPos - diPanelSize[1];
			yPos = yPos - diPanelSize[2] - 15;
			}
		unGenericObject_NeedSelect(dpTypeName(deviceName), bRes, exceptionInfo);
		if (bRes)
			{
			unSelectDeselectHMI_select(deviceName, true, exceptionInfo);
			}
			
//		DebugN(" ---- $ parameters --------");
//		DebugN("$deviceName:"+deviceName);
//		DebugN("$sDpGTWName:"+sDpGTWName);
//		DebugN("$sTopEvenAgent:"+sTopEvenAgent);
//		DebugN("$sTopOddAgent:"+sTopOddAgent);
//		DebugN("$sBottonEvenAgent:"+sBottonEvenAgent);
//		DebugN("$sBottonOddAgent:"+sBottonOddAgent);
//		DebugN("$sCrateName:"+sCrateName);
//		DebugN("$sTopSegment:"+sTopSegment);
//		DebugN("$sBottonSegment:"+sBottonSegment);
										  								
		ChildPanelOnModal("vision/cietDevices/General/cietResetDevice.pnl",
						  				"Crate " + sCrateName,
										  makeDynString("$deviceName:"+deviceName,
										  							"$sDpGTWName:"+sDpGTWName,
										  							"$sTopEvenAgent:"+sTopEvenAgent,
										  							"$sTopOddAgent:"+sTopOddAgent,
										  							"$sBottonEvenAgent:"+sBottonEvenAgent,
										  							"$sBottonOddAgent:"+sBottonOddAgent,
										  							"$sCrateName:"+sCrateName,
										  							"$sTopSegment:"+sTopSegment,
										  							"$sBottonSegment:"+sBottonSegment
										  						 ),
						  				xPos, yPos);
	

	}



//---------------------------------------------------------------------------------------------------------------------------------------
//@{

// cietGenericObject_toHexa
/**
Purpose: format an int value to a string in hexa

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
char toHexa(int valeur) {

  if(valeur<10) return (char) valeur+'0';
  return (char) valeur-10+'a';
}
string cietGenericObject_toHexa(int valeur) {

	char temp1, temp2;
	int i;
	
	i = (int)valeur/16;
 	temp1 = toHexa(i);
  i= (int)valeur%16;
 	temp2 = toHexa(i);
  return strtoupper(temp1+""+temp2);
}


//------------------------------------------------------------------------------------------------------------------------
// cietGenericObject_DisplayString
/**
Purpose: display a string value and check the validity

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietGenericObject_DisplayString(string sValue, string sShape, string sForeCol, bool bInvalid)
{
	string sColor;

	sColor = sForeCol;
	if (bInvalid)
		{
		sColor = "unDataNotValid";
		}
	setMultiValue(sShape + ".display", "foreCol", sColor, sShape + ".display", "text", sValue);

}


//---------------------------------------------------------------------------------------------------------------------------------------

// cietGenericObject_DisplayInt
/**
Purpose: animate a display value with an integer value

Parameters:
	- iValue, integer, input, value to display
	- sShape, string, input, shape that displays the value
	- sForeCol, string, input, display color
	- bInvalid, bool, input, is value invalid ?
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.1 
	. operating system: WinXP.
	. distributed system: yes.
*/

cietGenericObject_DisplayInt(int iValue, string sShape, string sForeCol, bool bInvalid)
{
	string formattedValue, sColor;

	// DebugN("FUNCTION: cietGenericObject_DisplayInt(iValue= "+iValue);
	sprintf(formattedValue, "%d", iValue);
	// DebugN("----- = formattedValue= "+formattedValue);
	sColor = sForeCol;
	if (bInvalid)
		{
		sColor = "unDataNotValid";
		}
	setMultiValue(sShape + ".display", "foreCol", sColor, sShape + ".display", "text", formattedValue);
}



//------------------------------------------------------------------------------------------------------------------------
// cietGenericObject_DisplayToCell
/**
Purpose: display a string value and check the validity in a table cell

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietGenericObject_DisplayToCell(string sValue, string sShape, int iRow, string sColumn, string sForeCol, bool bInvalid)
{
	string sColor;

	sColor = sForeCol;
	if (bInvalid)
		{
		sColor = "unDataNotValid";
		}
//	setMultiValue(sShape + ".display", "foreCol", sColor, sShape + ".display", "text", sValue);
		setMultiValue(sShape, "cellValueRC", iRow,sColumn, sValue,sShape,"cellForeColRC",iRow, sColumn, sColor);
}
//setValue (string shape, "cellBackColRC", int n, string Name, string Color );

//------------------------------------------------------------------------------------------------------------------------
// cietGenericObject_OpenHtmlFile
/**
Purpose: Open a HTML File with browser from data/html directory or internet.

Parameters:

Usage: Internal

Parameters:
	- iRes, integer, output, error info.
	- sLink, string, input, link where file is.
	
PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: WXP, Linux.
	. distributed system: yes.
*/

cietGenericObject_OpenHtmlFile(string sLinkToOpen)			
{
/*		string sPath;
		int iRes;
		
			sPath = getPath(DATA_REL_PATH, "VbScript/unLaunchIE.vbs");
			if (sPath == "")
				sPath = "explorer";
				
			if (substr(sLink,0,3) == "www")
				sLink = "http://" + sLink;
			else	
				sLink = getPath(DATA_REL_PATH, "html/"+ sLink);				

			strreplace (sLink,"//", "\\\\");
			
			iRes=system(sPath + " " + sLink);		
			return(iRes);
			*/
   string sLink=sLinkToOpen;
   if (substr(sLink,0,3) == "www")
    sLink = "http://" + sLink;
   else 
    sLink = getPath(DATA_REL_PATH, "html/"+ sLink);    
   
   if (sLink == "")
		{
			//ChildPanelOnCentralModal("vision/MessageWarning","Warning",makeDynString("$1:" + alias + getCatStr("unDB","EMPTYHTML")));
			ChildPanelOnCentralModal("vision/MessageWarning","Warning",makeDynString("$1:" + "Page not found"));
		}
		else
    unGenericObject_OpenHtmlFile(sLink);	
}		


//--------------------------------------------------------------------------------------------------------------------------

// cietGenericObject_getCrate
/**
Purpose: Return Crate from dpName

Parameters:

Usage: Internal

Parameters:
	- sDp, string, input, DpName.
	- sCrate, string, output, Crate Name
	
PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
string cietGenericObject_getCrate (string sDpName)
{
      string sNature, sCrate;
      dyn_string dsParameters;

      unGenericDpFunctions_getParameters (sDpName, dsParameters);
      sNature=dsParameters[5];
      dyn_string dsNature=strsplit(sNature,",");    // Case multinature
        //  if dynlen(dsNature) 
        //sCrate=substr(sNature,0,strpos(sNature, ","));
      if (dynlen(dsNature)>=1){     
        sCrate=dsNature[1];
      }
      return (sCrate);
}

//--------------------------------------------------------------------------------------------------------------------------

// cietGenericObject_openCratePanel
/**
Purpose: Open Ciet Crate Panel from dpe

Parameters:

Usage: Internal

Parameters:
	- sDp, string, input, DpName.
	
PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietGenericObject_openCratePanel (string sDp)
{
	// Get Crate from Nature
string sCrate, sRef;

sCrate = cietGenericObject_getCrate (sDp);
strreplace(sCrate,".","_");
sRef="fwTN_"+sCrate;

cietGenericObject_openTreePanelOnMainHMIPanel(sRef);
}

//--------------------------------------------------------------------------------------------------------------------------

// cietGenericObject_openTreePanelOnMainHMIPanel(sRef);
/**
Purpose: Open Window Tree Panel on Main HMI Panel.

Parameters:

Usage: Internal

Parameters:
	- sRef, string, input, panel reference name.
	
PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
	
*/

cietGenericObject_openTreePanelOnMainHMIPanel(string sRef)
{

	char c_unPanelConfiguration_maxPanelNavigation;
	dyn_string dsUserData, exceptionInfo, buttonPanel;
	string panelDp, sDpType;
	int len;
 string panelFile;  
	
// Open panel from window tree in main HMI panel

		if(dpExists(sRef)) 
		{ //check if of type unTreeWidget_treeNodeDPT
			if(dpTypeName(sRef) == unTreeWidget_treeNodeDPT)
				{
					fwTree_getNodeUserData(_fwTree_getNodeName(sRef), dsUserData, exceptionInfo);
					fwTree_getNodeDevice(_fwTree_getNodeName(sRef), panelDp, sDpType, exceptionInfo);
					panelFile = unPanel_convertDpPanelNameToPanelFileName(panelDp);
					len = dynlen(dsUserData);
						if(len <= 0)
							{ // case no userData
								dsUserData[1] = "";
								len = dynlen(dsUserData);
							}
						if((getPath(PANELS_REL_PATH, panelFile) != "") && (len > 0) && (sDpType == UN_PANEL_DPTYPE) && dpExists(panelDp))
								{
								unGenericDpFunctions_getContextPanel(panelDp, buttonPanel, exceptionInfo);
									if(len > c_unPanelConfiguration_maxPanelNavigation) 
										{// 11, give the sDpNodeName
											unGraphicalFrame_showUserPanel(panelFile, dsUserData[1], sRef);
											unGraphicalFrame_showContextualPanel(buttonPanel, sRef);
										}
									else 
										{
											unGraphicalFrame_showUserPanel(panelFile, dsUserData[1], panelDp);
											unGraphicalFrame_showContextualPanel(buttonPanel, panelDp);
										}
								}
						else
								{
									// if the file is not existing, send on info message
								unMessageText_send(getSystemName(), myManNum(), "unicosHMI initialisation", "user", "*", "INFO", 
									getCatStr("unApplication", "WRONGPANEL") +": "+sRef, exceptionInfo);
								}
			}
		}
}

//--------------------------------------------------------------------------------------------------------------------------

// cietGenericObject_OpenDebug(deviceName)
/**
Purpose: Open Window Tree Panel on Main HMI Panel.

Parameters:

Usage: Internal

Parameters:
	- deviceName, string, input, device namme.
	
PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: NT and WXP.
	. distributed system: yes.
	
*/
cietGenericObject_OpenDebug(string deviceName)
{
bool bConfigExists, bIsActive, iRes;
int iDollarPosition;
string sApplication, sFEC, sCMWDevice, sLogFile, sCommand;
dyn_string dExceptionInfo, dAddress;

	// Application
	
			sApplication = getPath(BIN_REL_PATH, "LogClient.exe");;

	// FEC.
	
			sFEC = cietGenericFunctions_getGTWName(deviceName);
			sFEC = strtolower(sFEC);
			strreplace(sFEC,"_","-");
			sFEC = substr (sFEC,0, strpos(sFEC,"--"));
			sFEC = "-g " + sFEC;

	// Device
	
			fwPeriphAddress_get(deviceName+".ST_GTWDEV", bConfigExists, dAddress, bIsActive, dExceptionInfo);
			sCMWDevice = dAddress[fwPeriphAddress_REFERENCE];
			iDollarPosition = strtok (sCMWDevice, "$");
			sCMWDevice = substr (sCMWDevice, 0, iDollarPosition);			
			sCMWDevice = "-d " + sCMWDevice;

	// Path Log File
 
		  sLogFile = "-f " + getPath(LOG_REL_PATH);

	// Open Console
			
			sCommand = sApplication + " " + sFEC + " " + sCMWDevice + " " + sLogFile;
			iRes=system(sCommand);
}

//------------------------------------------------------------------------------------------------
