/**@name LIBRARY: cietPT.ctl

@author: Celine VUILLERMOZ (AB/CO)

Creation Date: 20-07-2005

Modification History: 
 
 04/11/11  achiron: call only cietGenericFunctions_HeaderDisconnect(); in the ciet-DPT-_FaceplateStatusDisconnection()
 02/11/11  achiron: Corrected the call to the function ciet-DTD-_WidgetDisconnect iRes was argument instead of a PlcName, sCompleteAgentName has been change in global variable g_sCompleteAgentName in order tu use it in disconnect
	15/05/08	Pablo:	Filter samples shown on faceplate.
 	09/05/08 	Pablo: 	Button UserAccess adapted to QT.
 	09/11/07 	Pablo: 	Measurements animation on connection and disconnection.
 	05/11/07 	Pablo: 	Open wfipagent, wfipsegment and crate from popup menu.
 										debug application included. 	
 	23/08/07 	Pablo: 	Help on handle menu.
 										Disable some option on handle menu when widget is not selected.
 	20/06/07 	Pablo: 	Disable widget animation.
	18/06/07	Pablo:	Adapted to distributed system.
	07/03/07  Pablo:	Added filtering animation on widget.
	21/02/07  Pablo:	Added curveId dp animation on faceplate.
	19/02/07  Pablo:	Added constants button on faceplate.
										Added offset box on faceplate.
	15/02/07  Pablo:	Changed typical value from box to text.
	14/02/07  Pablo:	Added typical value on references values.
	08/02/07	Pablo:	Added FILTERING box animation on Faceplate.
	05/02/07 	Pablo:	Added full scale value on references values.
	14/12/06	Pablo:	Reorder Warnings Widgets Animation.
	27/11/06	Quique:	Include in faceplate the IOError of the agent and a button to open it
	17/11/06	Pablo:	Red color for values displayed on widget when error happens.
	14/11/06	Pablo:	Modified Resistance Text color on widget.
	13/11/06	Pablo:	Modified CalibrationCurveId and IdCard on faceplate to transparent background.
	13/11/06	Pablo:	Modified CalibrationCurveId and IdCard on faceplate to transparent background.
	10/11/06	Pablo:	Resistance show on widget, as a second value.
	01/11/06	Pablo:	Added Calibration Curve ID on Faceplate.
	24/10/06	Pablo:	Modified ButtonSetState & ButtonAnimationCB to allow click on UpdateRawData Button only when there is no updating.
	10/10/06	Pablo:	Modified widgetDisconnection function to visualize right behaviour.
	02/10/06	Quique: Corrected Get calibration call ($param name changed)
	26/09/06  Pablo:  Change I/O Error letter color on widget from Cyan to Orange.
	25/09/06  Pablo:  Included STATUS GATEWAY DEVICE for Widget Warning Letter Animation.
	12/09/06  Pablo:  Included I/O Error for Widget Warning Letter Animation.
	09/09/06	Quique	Modified popup reset menu
	06/09/06	Quique	Added in the faceplate the name of the segment
	01/09/06  Pablo:  Identification Card Configuration code changed.
  31/08/06  Pablo: 	Identification Card Configuration added.
  28/08/06  Pablo: 	Added links for popup Menu.
                    Arranging it.
	25/08/06	Quique: Add button fip config.
										Visualize the right format in the widget.
  22/08/06 	Pablo: 	Added control DeadBands Buttons.
  10/08/06 	Pablo:	Added animation for warning boxes: I/O Error, Reset, BadConfig.
										Changed WFIP Infos display to be done by cietGenericObject_DisplayInt(), instead of cietGenericObject_DisplayInt().


version 1.0

External Function : 
	. cietPT_FaceplateStatusDisconnection : animate faceplate disconnection
	. cietPT_ButtonUserAccess : User access for Analog buttons
	. cietPT_ButtonSetState : set button state
	. cietPT_WidgetDisconnection : animate widget disconnection
	. cietPT_MenuConfiguration : configuration of popup menu

Internal Functions :
	. cietPT_FaceplateStatusRegisterCB : register callback function
	. cietPT_ButtonRegisterCB : register callback function
	. cietPT_ButtonAnimationCB : animate buttons
	. cietPT_ButtonDisconnection : animate buttons disconnection
	. cietPT_WidgetRegisterCB : register callback function
	. cietPT_WidgetAnimationCB : animate widget
		
Purpose: This library contains Analog object functions.

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. cietPT object (DPType etc.)
	. Global variables and $parameters defined in Analog faceplate and widget
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

// constant declaration

//---------------------------------------------------------------------------------------------------------------------------------------

// cietPT_ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 3.1
	. operating system: WinXP
	. distributed system: yes.
*/

cietPT_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
	float value,value2;
	string sTime;
	  
	dpGet(sDeviceName+".ProcessInput.Data.pressureCalc", value, sDeviceName+".ProcessInput.Data.pressureCalc:_online.._stime", sTime);
	dpGet(sDeviceName+".ProcessInput.Data.resistanceCalc", value2, sDeviceName+".ProcessInput.Data.resistanceCalc:_online.._stime", sTime);
	dsReturnData[1] = sTime;
	dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(sDeviceName+".ProcessInput.Data.pressureCalc"), value) + " " 
							+ dpGetUnit(sDeviceName+".ProcessInput.Data.pressureCalc") + " , "  +
							unGenericObject_FormatValue(dpGetFormat(sDeviceName+".ProcessInput.Data.resistanceCalc"), value2) + " " 
							+ dpGetUnit(sDeviceName+".ProcessInput.Data.resistanceCalc");
}

//---------------------------------------------------------------------------------------------------------------------------------------
//@{

// cietPT_FaceplateStatusRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in cietPT faceplate
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietPT_FaceplateStatusRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, sSystemName, sPlcName;
	int iAction, iRes, iAlertType1;
	dyn_string exceptionInfo;
	bool bRemote;
	
	deviceName = unGenericDpFunctions_getDpName($sDpName);
// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	if(sPlcName != "") {
		if (dpExists(sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;
		unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;																			 
	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	
//	DebugN("FaceplateStatusRegisterCB", deviceName,iAction);
	
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
//DebugN("cietPT_FaceplateStatusDisconnection();");
			cietPT_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
		
			g_bDisconnectCB = false;
			if(g_bUnSystemAlarmPlc) {
				iRes = dpConnect("cietPT_FaceplateLockCallBack", deviceName + ".ProcessInput.Data.pressureCalc:_lock._alert_hdl._locked");

				g_bCallbackConnected = (iRes >= 0);
			}
			else
				cietPT_FaceplateStatusDisconnection();
			
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
		
			if(g_bUnSystemAlarmPlc) 
				iRes = dpDisconnect("cietPT_FaceplateLockCallBack", deviceName + ".ProcessInput.Data.pressureCalc:_lock._alert_hdl._locked");

			if(g_bDisconnectCB)
				cietPT_FaceplateDisconnect(deviceName, iRes);

			g_bCallbackConnected = !(iRes >= 0);
			g_bDisconnectCB = false;
				cietPT_FaceplateStatusDisconnection();
			
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
	}
//---------------------------------------------------------------------------------------------------------------------------------------
// cietPT_FaceplateLockCallBack
/**
Purpose: set up the lock callback function for the faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in PT widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietPT_FaceplateLockCallBack(string sdpe, bool bLockState)
{
	int iRes;
	string deviceName;

	deviceName = unGenericDpFunctions_getDpName(sdpe);
	if(bLockState) { 
		if(g_bDisconnectCB) {
			cietPT_FaceplateDisconnect(deviceName, iRes);
			g_bDisconnectCB = false;
		}
		cietPT_FaceplateStatusDisconnection();
	}
	else { 
		g_bDisconnectCB = true;
		cietPT_FaceplateConnect(deviceName);
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietPT_FaceplateConnect
/**
Purpose: connect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in cietPT faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietPT_FaceplateConnect(string deviceName)
{
int iAction, iRes, iAlertType1, iAlertType2, iRangeType;
	dyn_string exceptionInfo, dsDeviceNameSplit,dsTemp,dsFIPInfos;
	string sPlcName, sSystemName, sAlias,sDescrip,sAgentName;

//DebugN("cietPT_FaceplateConnect");

// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	
	if(sPlcName != "") {
		if (dpExists(sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;
		
		g_ssenVolMeasFormat 		= dpGetFormat(deviceName + ".ProcessInput.Data.senVolMeas");
		g_ssenVolMeasUnit 			= dpGetUnit(deviceName + ".ProcessInput.Data.senVolMeas");
		g_srefVolMeasFormat 		= dpGetFormat(deviceName + ".ProcessInput.Data.refVolMeas");
		g_srefVolMeasUnit				= dpGetUnit(deviceName + ".ProcessInput.Data.refVolMeas");							
		g_sgainValueFormat 			= dpGetFormat(deviceName + ".ProcessInput.Data.gainValue");
		g_sgainValueUnit  				= dpGetUnit(deviceName + ".ProcessInput.Data.gainValue");
		g_sresistanceCalcFormat 	= dpGetFormat(deviceName + ".ProcessInput.Data.resistanceCalc");
		g_sresistanceCalcUnit  	= dpGetUnit(deviceName + ".ProcessInput.Data.resistanceCalc");
		g_spressureCalcFormat 		= dpGetFormat(deviceName + ".ProcessInput.Data.pressureCalc");
		g_spressureCalcUnit  		= dpGetUnit(deviceName + ".ProcessInput.Data.pressureCalc");		
		g_soffsetFormat	= dpGetFormat(deviceName + ".ProcessInput.Data.offset");
		g_soffsetUnit  	= dpGetUnit(deviceName + ".ProcessInput.Data.offset");			
		
		
		//GTWName
		cietGenericFunctions_getFIPAgentInfos(deviceName, dsFIPInfos, exceptionInfo);		
	
		
		
		sAgentName=cietGenericFunctions_getAgentName(deviceName);
		g_sCompleteAgentName=unGenericDpFunctions_getWidgetDpName(sAgentName);
		
		
	if (dynlen(exceptionInfo) <= 0)	
	{	
		g_sGTWName = dsFIPInfos[ciet_PARAMETER_GTWNAME];
		g_sAgentBus = dsFIPInfos[ciet_PARAMETER_BUS]; 
		g_sAgentNumber = dsFIPInfos[ciet_PARAMETER_AGENTNUMBER];
		sAgentName =dsFIPInfos[ciet_PARAMETER_AGENTNAME];
		g_sAgentChannel = dsFIPInfos[ciet_PARAMETER_CHANNELNUMBER];

		// Get Agent name to display also ioerror info
		g_sCompleteAgentName=unGenericDpFunctions_getWidgetDpName(sSystemName + sAgentName);
	}
	
		rawDatas.appendLines(1);
		rawDatas.sortOnClick(FALSE);
		flags.appendLines(4);
		flags.sortOnClick(FALSE);
		
	if(g_bUnSystemAlarmPlc) {
	
		sAlias = unGenericDpFunctions_getAlias(sAgentName);
		dsDeviceNameSplit=strsplit(sAlias, ".");
		
		iRes = dpConnect("cietPT_FaceplateStatusAnimationCB",
										 	deviceName + ".ProcessInput.Config.sgNumber", 		
						 					deviceName + ".ProcessInput.Config.agNumber", 		
						 					deviceName + ".ProcessInput.Config.chNumber", 	
							 				deviceName + ".ProcessInput.Data.senVolMeas", 
							 				deviceName + ".ProcessInput.Data.refVolMeas",
						 					deviceName + ".ProcessInput.Data.configFlag",
						 					deviceName + ".ProcessInput.Data.gainFlag",
						 					deviceName + ".ProcessInput.Data.diagHF",
						 					deviceName + ".ProcessInput.Data.diagLF",
						 					deviceName + ".ProcessInput.Data.sensorStatus",
						 					g_sCompleteAgentName + ".ProcessInput.Data.agentStatus",
						 					deviceName + ".ProcessInput.Data.command",
						 					deviceName + ".ProcessInput.Data.gainValue",
						 					deviceName + ".ProcessInput.Data.resistanceCalc",
						 					deviceName + ".ProcessInput.Data.pressureCalc",
						 					deviceName + ".ProcessInput.Data.offset",
						 					deviceName + ".ProcessInput.Curve.curveId",
						 					deviceName + ".ProcessInput.Data.filterSamples",                                                                                          
						 					deviceName + ".ProcessInput.Data.sensorByte[1]",
						 					deviceName + ".ProcessInput.Data.sensorByte[2]",
						 					deviceName + ".ProcessInput.Data.sensorByte[3]",
						 					deviceName + ".ProcessInput.Data.sensorByte[4]",
						 					deviceName + ".ProcessInput.Data.sensorByte[5]",
						 					deviceName + ".ProcessInput.Data.sensorByte[6]",
						 					deviceName + ".ProcessInput.Data.sensorByte[7]",
						 					deviceName + ".ProcessInput.Data.sensorByte[8]",		
						 					sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".enabled",
									/*		sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".enabled",*/
											deviceName + ".ProcessInput.Data.pressureCalc:_online.._invalid");
	}
	else
		cietPT_FaceplateStatusDisconnection();
	g_bCallbackConnected = (iRes >= 0);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietPT_FaceplateDisconnect
/**
Purpose: disconnect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in cietPT faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietPT_FaceplateDisconnect(string deviceName)
{
	int iRes;
	string sPlcName, sSystemName, sAlias,sAgentName;
	dyn_string dsDeviceNameSplit;
	
//	DebugN("FaceplateDisconnect");
	if(g_bUnSystemAlarmPlc) {
// get the PLC name 
		sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
		sSystemName = unGenericDpFunctions_getSystemName(deviceName);
		sAgentName = cietGenericFunctions_getAgentName(deviceName);
		sAlias = unGenericDpFunctions_getAlias(sAgentName);
		dsDeviceNameSplit=strsplit(sAlias, ".");
	
		iRes = dpDisconnect("cietPT_FaceplateStatusAnimationCB",
										 	deviceName + ".ProcessInput.Config.sgNumber", 		
						 					deviceName + ".ProcessInput.Config.agNumber", 		
						 					deviceName + ".ProcessInput.Config.chNumber", 	
						 					deviceName + ".ProcessInput.Data.senVolMeas", 
							 				deviceName + ".ProcessInput.Data.refVolMeas",
						 					deviceName + ".ProcessInput.Data.configFlag",
						 					deviceName + ".ProcessInput.Data.gainFlag",
						 					deviceName + ".ProcessInput.Data.diagHF",
						 					deviceName + ".ProcessInput.Data.diagLF",
						 					deviceName + ".ProcessInput.Data.sensorStatus",
						 					g_sCompleteAgentName + ".ProcessInput.Data.agentStatus",
						 					deviceName + ".ProcessInput.Data.command",
						 					deviceName + ".ProcessInput.Data.gainValue",
						 					deviceName + ".ProcessInput.Data.resistanceCalc",
						 					deviceName + ".ProcessInput.Data.pressureCalc",
						 					deviceName + ".ProcessInput.Data.offset",				
						 					deviceName + ".ProcessInput.Curve.curveId",
						 					deviceName + ".ProcessInput.Data.filterSamples",                                                                                         						 							 					
						 					deviceName + ".ProcessInput.Data.sensorByte[1]",
						 					deviceName + ".ProcessInput.Data.sensorByte[2]",
						 					deviceName + ".ProcessInput.Data.sensorByte[3]",
						 					deviceName + ".ProcessInput.Data.sensorByte[4]",
						 					deviceName + ".ProcessInput.Data.sensorByte[5]",
						 					deviceName + ".ProcessInput.Data.sensorByte[6]",
						 					deviceName + ".ProcessInput.Data.sensorByte[7]",
						 					deviceName + ".ProcessInput.Data.sensorByte[8]",						 											 											 											 											 											 											 					
						 					sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".enabled",
									/*		sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".enabled",*/
											deviceName + ".ProcessInput.Data.pressureCalc:_online.._invalid");
	}
	g_bCallbackConnected = !(iRes >= 0);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietPT_FaceplateStatusAnimationCB
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in cietPT faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietPT_FaceplateStatusAnimationCB(string sDp_sgNumber, int isgNumber,
																				string sDp_agNumber, int iagNumber,
																				string sDp_chNumber, int ichNumber,
																				string sDp_senVolMeas, float fsenVolMeas,
																				string sDp_refVolMeas, float frefVolMeas,
																				string sDp_configFlag, int iconfigFlag,
																				string sDp_gainFlag, int igainFlag,
																				string sDp_diagHF, int idiagHF,
																				string sDp_diagLF, int idiagLF,
																				string sDp_sensorStatus, unsigned uSensorStatus,
																				string sDp_AgentStatus, unsigned uAgentStatus,
																				string sDp_commandIn, unsigned uCommandIn,
																				string sDp_gainValue, float fgainValue,
																				string sDp_resistanceCalc, float fresistanceCalc,
																				string sDp_pressureCalc, float fpressureCalc,
																				string sDp_offset, float foffset,		
																				string sDp_curveId, string sCurveId,	
																				string sDp_filterSamples, int iFilterSamples,                                                                                                                                                                 																	
																				string sDp_sensorByte1, bit32 sensorByte1,
																				string sDp_sensorByte2, bit32 sensorByte2,
																				string sDp_sensorByte3, bit32 sensorByte3,
																				string sDp_sensorByte4, bit32 sensorByte4,
																				string sDp_sensorByte5, bit32 sensorByte5,
																				string sDp_sensorByte6, bit32 sensorByte6,
																				string sDp_sensorByte7, bit32 sensorByte7,
																				string sDp_sensorByte8, bit32 sensorByte8,																																																																																																																																												
																				string sDpSystemIntegrityAlarmValue, int iSystemIntegrityAlarmValue, 
																			 	string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
																			/* 	string sDpSystemAlarmValue, int iSystemAlarmValue,
																			 	string sDpSystemAlarmEnabled, bool bSystemAlarmEnabled,*/
																			 	string sDpInvalid, bool bCodeInvalid)
{

int iAlarmValue,i,iStart, iIdCard,iFiltering;
bool bFaceplateInvalid, bDisable, bIoError, bReset, bBadConfig,bAgIoError, bFiltering, bFiltering0, bFiltering1;
string sTemp,sDpPLCName, sShowIdCard, deviceName, sBusName,sCrate;
dyn_bit32 db32sensorByte;
dyn_string dsFlags = makeDynString((bit32)iconfigFlag, (bit32)igainFlag, (bit32)idiagHF, (bit32)idiagLF);	
float frefVolValue,fsenVolValue;
bit32 b32Filtering;

	if ((iSystemIntegrityAlarmValue >= CRYO_SYSTEM_INGETRITY_ALARM_LIMIT)
		/*	||
			(iSystemAlarmValue < CRYO_SYSTEM_INGETRITY_ALARM_LIMIT)*/)
//	if (iSystemIntegrityAlarmValue >= CRYO_SYSTEM_INGETRITY_ALARM_LIMIT)
			iAlarmValue=c_unSystemIntegrity_alarm_value_level1;
	else
		iAlarmValue=c_unSystemIntegrity_no_alarm_value;
		
		iStart=strpos(sDpSystemIntegrityAlarmValue, c_cietSystemIntegrity_DS) + strlen(c_cietSystemIntegrity_DS);
	sDpPLCName=substr(sDpSystemIntegrityAlarmValue,  iStart, strpos(sDpSystemIntegrityAlarmValue, ".alarm") - iStart);
	
	cietGenericFunctions_Header(sDpPLCName, iAlarmValue, bSystemIntegrityAlarmEnabled /* && bSystemAlarmEnabled*/, bCodeInvalid);	
		
		bFaceplateInvalid = (bCodeInvalid || !bSystemIntegrityAlarmEnabled || (iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value));
////////////////////////////////
	
		bIoError = getBit(uSensorStatus, 0);
		bReset = getBit(uSensorStatus, 1);
		bBadConfig = getBit(uSensorStatus, 2);

		bFiltering0 = getBit(uSensorStatus,8);
		bFiltering1 = getBit(uSensorStatus,9);
		bDisable = getBit (uSensorStatus, 12);		
		setBit(b32Filtering,0, bFiltering0);
  	setBit(b32Filtering,1, bFiltering1);
		iFiltering = b32Filtering;
		bFiltering = bFiltering0 || bFiltering1;	

/////////////////////////////////

	
// - - - WARNINGS - - - 
	
	// bIoError = bReset || bAgError;
	
		if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate ("DISABLE", !bDisable, 1, "", FALSE, "cietFaceplate_Warning");	
	if(g_bSystemConnected)		
		unGenericObject_ColorBoxAnimate ("IO_ERROR", !bIoError, 1, "", FALSE, "cietFaceplate_Warning");
	if(g_bSystemConnected)		
		unGenericObject_ColorBoxAnimate ("RESET", !bReset, 1, "", FALSE, "cietFaceplate_Warning");
	if(g_bSystemConnected)		
		unGenericObject_ColorBoxAnimate ("BADCONFIG", !bBadConfig, 1, "", FALSE, "cietFaceplate_Warning");
		
		
	if(g_bSystemConnected)			
		{
		unGenericObject_ColorBoxAnimate("FILTERING", !bFiltering, 1, "", FALSE, "cietFaceplate_ModeActive");		
		switch (iFiltering)
			{
					case 0:
					setValue("FilteringBoxText","text","Filtering Active");
					break;
					
					case 1:
					setValue("FilteringBoxText","text","Filtering Active: Median. " + iFilterSamples+ " Sp");
					break;

					case 2:
					setValue("FilteringBoxText","text","Filtering Active: Average. " + iFilterSamples+ " Sp");
					break;	
					
					default:
					setValue("FilteringBoxText","text","Filtering Active: Error");
					break;				

			}
		}			

// - - - MEASUREMENTS - - -  

	if(g_bSystemConnected)		
		unGenericObject_DisplayValue(g_ssenVolMeasFormat, g_ssenVolMeasUnit, fsenVolMeas, "senVolMeas", "unDisplayValue_Status", bFaceplateInvalid);
	if(g_bSystemConnected)				
		unGenericObject_DisplayValue(g_srefVolMeasFormat, g_srefVolMeasUnit, frefVolMeas, "refVolMeas", "unDisplayValue_Status", bFaceplateInvalid);
	if(g_bSystemConnected)
		{
			frefVolValue=(ciet_CONFIG_PT_FULLSCALE_refVolMeas*frefVolMeas)/ciet_CONFIG_PT_FULLSCALE_MAXDATA_refVolMeas;	
			unGenericObject_DisplayValue("", ciet_CONFIG_PT_FULLSCALE_UNIT_refVolMeas, frefVolValue, "refVolValue", "unDisplayValue_Status", bFaceplateInvalid);			
		}
	if(g_bSystemConnected)
		{
			setValue("refVolTypical","text", ciet_CONFIG_PT_TYPICAL_VALUE_refVolMeas + " " + ciet_CONFIG_PT_TYPICAL_VALUE_UNIT_refVolMeas);		
		}
		
	if(g_bSystemConnected)
		{
			fsenVolValue=(ciet_CONFIG_PT_FULLSCALE_senVolMeas*fsenVolMeas)/ciet_CONFIG_PT_FULLSCALE_MAXDATA_senVolMeas;	
			unGenericObject_DisplayValue("", ciet_CONFIG_PT_FULLSCALE_UNIT_senVolMeas, fsenVolValue, "senVolValue", "unDisplayValue_Status", bFaceplateInvalid);			
		}		

// - - - FLAGS - - - 

	for(int k=1;k<= dynlen(dsFlags);k++)
	{
		sTemp = dsFlags[k];
		int j=7;
		for(i=24;i<=31;i++)
			{
				string sTemp1;
				sTemp1	= substr(sTemp,i,1);	
//				DebugN("k, sTemp, sTemp1, i, j",k, sTemp, sTemp1, i, j);
				if(g_bSystemConnected)		
					cietGenericObject_DisplayToCell(sTemp1, "flags", k-1, "b"+j, "unDisplayValue_Parameter", FALSE);
				j=j-1;	
			}
	}
	
// - - - CALCULATIONS - - -  
	
	if(g_bSystemConnected)			
		unGenericObject_DisplayValue(g_spressureCalcFormat, g_spressureCalcUnit, fpressureCalc, "pressureCalc", "unDisplayValue_Parameter", FALSE);
	if(g_bSystemConnected)			
		unGenericObject_DisplayValue(g_sresistanceCalcFormat, g_sresistanceCalcUnit, fresistanceCalc, "resistanceCalc", "unDisplayValue_Parameter", FALSE);
	if(g_bSystemConnected)			
		unGenericObject_DisplayValue(g_soffsetFormat, g_soffsetUnit, foffset, "offset", "unDisplayValue_Parameter", FALSE);	
		
	if(g_bSystemConnected)			
		unGenericObject_DisplayValue(g_sgainValueFormat, g_sgainValueUnit, fgainValue, "gainValue", "unDisplayValue_Parameter", FALSE);

// - - - RAW DATA - - - 

	db32sensorByte = makeDynChar(sensorByte1, sensorByte2, sensorByte3, sensorByte4, sensorByte5, sensorByte6,	sensorByte7, sensorByte8);
		
	for(i=1;i<=8;i++)
		{
			string sTemp;
			int j=i-1;
			sTemp = cietGenericObject_toHexa(db32sensorByte[i]);
			if(g_bSystemConnected)		
				cietGenericObject_DisplayToCell(sTemp, "rawDatas", 0, "B"+j, "unDisplayValue_Parameter", FALSE);
		}
		
// Identify Card Configuration
	
		string sShowIdCard;
		
		sShowIdCard = cietGenericFunctions_getDeviceId ($sDpName);
		
		cietGenericObject_DisplayString(sShowIdCard, "idCard", "unDisplayValue_Parameter", FALSE);
	
// - - - WFIP INFOS - - - 	
	
if(g_bSystemConnected)			
		cietGenericObject_DisplayString(g_sGTWName, "GTWNAME", "unDisplayValue_Parameter", FALSE);
	if(g_bSystemConnected)				
		cietGenericObject_DisplayInt(iagNumber, "AGENTNUMBER", "unDisplayValue_Parameter", FALSE);	
	if(g_bSystemConnected)				
		cietGenericObject_DisplayInt(ichNumber, "AGENTCHANNEL", "unDisplayValue_Parameter", FALSE);	
	if(g_bSystemConnected)				
		{
			sBusName=cietGenericFuncitons_getSegmentName(sDp_sgNumber);
			// DebugN("sDp_sgNumber="+sDp_sgNumber);
			// DebugN("sBusName="+sBusName);
			cietGenericObject_DisplayString(sBusName, "BUSNAME", "unDisplayValue_Parameter",FALSE);
			cietGenericObject_DisplayInt(isgNumber, "AGENTBUS", "unDisplayValue_Parameter", FALSE);	
		}

  g_sGTWName = cietGenericFunctions_getGTWName(sDp_sgNumber);
	
	if(g_bSystemConnected)	
		{
			// Agent io_error
			bAgIoError=getBit(uAgentStatus,0);
			unGenericObject_ColorBoxAnimate ("AG_IOERROR", !bAgIoError, 1, "", FALSE, "cietFaceplate_Warning");
		}
	
	sCrate= cietGenericObject_getCrate ($sDpName);
	if(g_bSystemConnected)
		cietGenericObject_DisplayString(sCrate, "CRATE", "unDisplayValue_Parameter", FALSE);
		

		
// Calibration Curve Configuration		
	
		//string idCurve;
		
		//idCurve = cietGenericFunctions_getCurveId ($sDpName, "PT", 1);	
		
		cietGenericObject_DisplayString(sCurveId, "idCurve", "unDisplayValue_Parameter", FALSE);
}
 

//---------------------------------------------------------------------------------------------------------------------------------------

// cietPT_FaceplateStatusDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietPT_FaceplateStatusDisconnection()
{

//DebugN("FaceplateStatusDisconnection");
//	unGenericObject_ColorBoxDisconnect(makeDynString("IO_ERROR","RESET", "AG_ERROR" ));

	/*unGenericObject_DisplayValueDisconnect(makeDynString("senVolMeas", "refVolMeas", "configFlag", "gainFlag", "diagLF", 
	"diagHF", "gainValue", "resistanceCalc", "pressureCalc","B7_sensorByte", "B6_sensorByte", "B5_sensorByte", "B4_sensorByte", 
	"B3_sensorByte", "B2_sensorByte", "B1_sensorByte", "B0_sensorByte"));*/

 //put in the top textbox the DATA not connected in purple
	cietGenericFunctions_HeaderDisconnect();
}



//---------------------------------------------------------------------------------------------------------------------------------------

// cietPT_ButtonUserAccess
/**
Purpose: User access for DQAMCMB buttons

Parameters: 
	- sDpName, string, input, device name
	- sDpType, string, input, dptype
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietPT_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
	bool operator, expert, admin;
	int i, length;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
        string sOperAction, sExpAction, sAdminAction;
        bool bActionDefined = false;

        unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo,CIET_ACCESS_CONTROL_DOMAIN_NAME);
        if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) ||
           (sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME))
          bActionDefined=true;
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString();
	if (operator)
	{
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
            dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_SELECT, ciet_POPUPMENU_HELP,
                                              ciet_POPUPMENU_WIDGET_SIMULATOR,ciet_POPUPMENU_OPEN_WFIPAGENT, 
                                              ciet_POPUPMENU_OPEN_WFIPSEGMENT, ciet_POPUPMENU_OPEN_CRATE));
	}
	if (expert)
	{
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
	    			dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_SELECT,ciet_FACEPLATE_BUTTON_RESET,
                                              ciet_FACEPLATE_BUTTON_RAWDATA,ciet_FACEPLATE_BUTTON_DEADBANDS,
                                              ciet_FACEPLATE_BUTTON_CONFIG,ciet_FACEPLATE_BUTTON_CALIBRATION,
                                              ciet_FACEPLATE_BUTTON_FILTERING,ciet_FACEPLATE_BUTTON_CONSTANTS, 
                                              ciet_FACEPLATE_BUTTON_TENDPARAM, ciet_FACEPLATE_BUTTON_DEBUG, 
                                              ciet_FACEPLATE_BUTTON_DISABLE,ciet_POPUPMENU_HELP,ciet_POPUPMENU_WIDGET_SIMULATOR,
                                              ciet_POPUPMENU_OPEN_WFIPAGENT, ciet_POPUPMENU_OPEN_WFIPSEGMENT, 
                                              ciet_POPUPMENU_OPEN_CRATE));	
	}
        if(admin){
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
            dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_SELECT,ciet_FACEPLATE_BUTTON_RESET,
                                              ciet_FACEPLATE_BUTTON_RAWDATA,ciet_FACEPLATE_BUTTON_DEADBANDS,
                                              ciet_FACEPLATE_BUTTON_CONFIG,ciet_FACEPLATE_BUTTON_CALIBRATION,
                                              ciet_FACEPLATE_BUTTON_FILTERING,ciet_FACEPLATE_BUTTON_CONSTANTS, 
                                              ciet_FACEPLATE_BUTTON_TENDPARAM, ciet_FACEPLATE_BUTTON_DEBUG, 
                                              ciet_FACEPLATE_BUTTON_DISABLE,ciet_POPUPMENU_HELP,ciet_POPUPMENU_WIDGET_SIMULATOR,
                                              ciet_POPUPMENU_OPEN_WFIPAGENT, ciet_POPUPMENU_OPEN_WFIPSEGMENT, 
                                              ciet_POPUPMENU_OPEN_CRATE));	 
        }
  //..........................................
  /*		bool operator, expert, admin;
	int i, length;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
	
//	DebugN("ButtonUserAccess");
	unGenericButtonFunctionsHMI_getUnicosPriviledges(dbPermissions, exceptionInfo);
	if (dynlen(exceptionInfo) > 0)
		{
		dynClear(dbPermissions);
		}
	while (dynlen(dbPermissions) < 4)
		{
		dynAppend(dbPermissions, false);
		}
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString();
	if (operator)
		{
		dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_SELECT, ciet_POPUPMENU_HELP,ciet_POPUPMENU_WIDGET_SIMULATOR,ciet_POPUPMENU_OPEN_WFIPAGENT, ciet_POPUPMENU_OPEN_WFIPSEGMENT, ciet_POPUPMENU_OPEN_CRATE));
		
		}
	if (expert)
		{
			dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_SELECT,ciet_FACEPLATE_BUTTON_RESET,ciet_FACEPLATE_BUTTON_RAWDATA,ciet_FACEPLATE_BUTTON_DEADBANDS,ciet_FACEPLATE_BUTTON_CONFIG,ciet_FACEPLATE_BUTTON_CALIBRATION,ciet_FACEPLATE_BUTTON_FILTERING,ciet_FACEPLATE_BUTTON_CONSTANTS, ciet_FACEPLATE_BUTTON_TENDPARAM, ciet_FACEPLATE_BUTTON_DEBUG, ciet_FACEPLATE_BUTTON_DISABLE,ciet_POPUPMENU_HELP,ciet_POPUPMENU_WIDGET_SIMULATOR,ciet_POPUPMENU_OPEN_WFIPAGENT, ciet_POPUPMENU_OPEN_WFIPSEGMENT, ciet_POPUPMENU_OPEN_CRATE));	
		}
	if (admin)
		{
			dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_SELECT,ciet_FACEPLATE_BUTTON_RESET,ciet_FACEPLATE_BUTTON_RAWDATA,ciet_FACEPLATE_BUTTON_DEADBANDS,ciet_FACEPLATE_BUTTON_CONFIG,ciet_FACEPLATE_BUTTON_CALIBRATION,ciet_FACEPLATE_BUTTON_FILTERING,ciet_FACEPLATE_BUTTON_CONSTANTS, ciet_FACEPLATE_BUTTON_TENDPARAM, ciet_FACEPLATE_BUTTON_DEBUG, ciet_FACEPLATE_BUTTON_DISABLE,ciet_POPUPMENU_HELP,ciet_POPUPMENU_WIDGET_SIMULATOR,ciet_POPUPMENU_OPEN_WFIPAGENT, ciet_POPUPMENU_OPEN_WFIPSEGMENT, ciet_POPUPMENU_OPEN_CRATE));	
		}	
                */
} 


//---------------------------------------------------------------------------------------------------------------------------------------

// cietPT_ButtonRegisterCB  
/**
Purpose: register callback function

Parameters: 

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in DQAMCMB faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietPT_ButtonRegisterCB(string sDpSystemConnected, bool bSystemConnected)
{
	string deviceName, sSystemName, sDpType, sFunction;
	int iAction, iRes, iManReg01;
	dyn_string exceptionInfo, dsUserAccess,dsFunctions ;
//	DebugN("ButtonRegisterCB");
	
	deviceName = unGenericDpFunctions_getDpName($sDpName);
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	
	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
//		DebugN("ButtonRegisterCB UN_ACTION_DISCONNECT");
			cietPT_ButtonDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
//		DebugN("ButtonRegisterCB UN_ACTION_DPCONNECT");
			sDpType="PT";	
			unGenericObject_GetFunctions(sDpType, dsFunctions, exceptionInfo);
			sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];
						 
			if (sFunction != "")
				{
				evalScript(dsUserAccess, "dyn_string main(string deviceNameTemp, string sDpTypeTemp) {" + 
							 "dyn_string dsUserAccessReturn;" +
							 "if (isFunctionDefined(\"" + sFunction  + "\"))" +
							 "    {" +
					 			sFunction  + "(deviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
							 "    }" +
							 "return dsUserAccessReturn; }", makeDynString(), deviceName, sDpType);
						
				if (dynlen(dsUserAccess) == 0)
					fwException_raise(exceptionInfo,"ERROR", "cietPT_ButtonRegisterCB: " + getCatStr("unGeneration","UNKNOWNFUNCTION"), sFunction);
				else
					g_dsUserAccess=dsUserAccess;	
				}
			else
				fwException_raise(exceptionInfo,"ERROR", "cietPT_ButtonRegisterCB: " + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");



			iRes = dpConnect("cietPT_ButtonAnimationCB",
							 				 deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							 				 deviceName + ".statusInformation.selectedManager",
							 				 deviceName + ".ProcessInput.Data.sensorStatus");
			g_bCallbackConnected = (iRes >= 0);
			
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
//			DebugN("ButtonRegisterCB UN_ACTION_DPDISCONNECT_DISCONNECT");
			iRes = dpDisconnect("cietPT_ButtonAnimationCB",
							 						deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							 						deviceName + ".statusInformation.selectedManager",
 									 				deviceName + ".ProcessInput.Data.sensorStatus");
			g_bCallbackConnected = !(iRes >= 0);
			cietPT_ButtonDisconnection();
			break;
		case UN_ACTION_NOTHING:
//				DebugN("ButtonRegisterCB UN_ACTION_NOTHING");
		default:
			break;
		}
}


//---------------------------------------------------------------------------------------------------------------------------------------

// cietPT_ButtonAnimationCB
/**
Purpose: animate buttons

Parameters: 
	- dbPermissions, dyn_bool, input, list of permissions, length = 4, order : 1=monitor, 2=operator, 3=expert, 4=admin
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietPT_ButtonAnimationCB(string sDpLock, bool bLocked, 
														 string sDpSelect, string sSelectedManager, 
														 string sDpSensorStatus, unsigned uSensorStatus)
{
bool bStatusRawData, bDisable;
string sDisableText;

	bDisable = getBit (uSensorStatus, 12);
	
	if (bDisable ==1)
			sDisableText=ciet_FACEPLATE_BUTTON_ENABLE;
	else if (bDisable ==0)
			sDisableText=ciet_FACEPLATE_BUTTON_DISABLE;				
			
	setMultiValue("DISABLE_BUTTON.buttonDisable", "text", sDisableText);
	
	if(StatusRawDataText.text=="true")
				bStatusRawData = 1;
	else if(StatusRawDataText.text=="false")				
				bStatusRawData = 0;
	// DebugN("bStatusRawData=" + bStatusRawData);
	
//	DebugN("ButtonAnimationCB");
	cietPT_ButtonSetState(unGenericDpFunctions_getDpName(sDpLock), "PT", g_dsUserAccess, makeDynString(bLocked, sSelectedManager, bStatusRawData));
}



//---------------------------------------------------------------------------------------------------------------------------------------

// cietPT_ButtonSetState
/**
Purpose: set button state

Parameters: 
	- deviceName, string, input, device
	- sDpType, string, input, dptype
	- dsUserAccess, dyn_string, input, user access
	- dsData, dyn_string, parameter
		
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietPT_ButtonSetState(string deviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
	bool bSelected, buttonEnabled, bit1, bit2, bStatusRawData;
	dyn_string dsButtons;
	int i, length, alertType;
	string localManager;
	bool bLocked, ListEnabled;
	string sSelectedManager;
//	DebugN("ButtonSetState");

	bLocked = dsData[1];
	sSelectedManager = dsData[2];
	bStatusRawData = dsData[3];
	
// 1. Selection state
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = (localManager == "S");
	
	dsButtons = makeDynString(ciet_FACEPLATE_BUTTON_CALIBRATION, ciet_FACEPLATE_BUTTON_RESET,ciet_FACEPLATE_BUTTON_RAWDATA,ciet_FACEPLATE_BUTTON_CONFIG,ciet_FACEPLATE_BUTTON_DEADBANDS,ciet_FACEPLATE_BUTTON_FILTERING,ciet_FACEPLATE_BUTTON_CONSTANTS, ciet_FACEPLATE_BUTTON_TENDPARAM, ciet_FACEPLATE_BUTTON_DEBUG, ciet_FACEPLATE_BUTTON_DISABLE); // Each button except "Select"
	length = dynlen(dsButtons);
// 1. Selection state
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = (localManager == "S");
// 2. Button state
	for(i=1;i<=length;i++)
		{
		buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0);	// User access
		switch (dsButtons[i])
			{
			case ciet_FACEPLATE_BUTTON_RESET:
/*				bit1 = !(getBit(stsReg01Value, UN_STSREG01_FOMOST) == 0) && !(bStsReg01Bad || bStsReg02Bad);	
				bit2 = !(getBit(stsReg01Value, UN_STSREG01_AUINHFMO) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				buttonEnabled = buttonEnabled && bSelected && !(bit1 || bit2);*/
				buttonEnabled = buttonEnabled && bSelected;				
				break;
			case ciet_FACEPLATE_BUTTON_CALIBRATION:
/*				bit1 = !(getBit(stsReg01Value, UN_STSREG01_AUMOST) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				buttonEnabled = buttonEnabled && bSelected && !(bit1);*/
				buttonEnabled = buttonEnabled && bSelected;
				break;
			case ciet_FACEPLATE_BUTTON_CONFIG:
				buttonEnabled = buttonEnabled && bSelected;
				break;	
			case ciet_FACEPLATE_BUTTON_RAWDATA:
				buttonEnabled = buttonEnabled && bSelected  && !bStatusRawData;
								// DebugN ("Button State = " + bStatusRawData );
				break;			
			case ciet_FACEPLATE_BUTTON_DEADBANDS:
				buttonEnabled = buttonEnabled && bSelected;
				break;				
			case ciet_FACEPLATE_BUTTON_FILTERING:
				buttonEnabled = buttonEnabled && bSelected  && !bStatusRawData;
								// DebugN ("Button State = " + bStatusRawData );
			case ciet_FACEPLATE_BUTTON_CONSTANTS:
				buttonEnabled = buttonEnabled && bSelected  && !bStatusRawData;
								// DebugN ("Button State = " + bStatusRawData );		
			case ciet_FACEPLATE_BUTTON_TENDPARAM:
				buttonEnabled = buttonEnabled && bSelected; 
				break;				
				
			case ciet_FACEPLATE_BUTTON_DEBUG:
				buttonEnabled = buttonEnabled && bSelected;
				break;				

			case ciet_FACEPLATE_BUTTON_DISABLE:
				buttonEnabled = buttonEnabled && bSelected;
				break;																
			default:
				break;
			}
		setValue(ciet_FACEPLATE_BUTTON_PREFIX + dsButtons[i], "enabled", buttonEnabled);
		}
// 3. Button Select state
	unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
}


//---------------------------------------------------------------------------------------------------------------------------------------

// cietPT_UserLoginGetButtonState
/**
Purpose: function called before the button set state when a user logs in. 

Parameters:
	deviceName: string, input, the device name
	dsReturnData: dyn_string, output, the data are returned here

Usage: Internal function

PVSS manager usage:  Ctrl, NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
		-> $sDpName, string, the device DP name
		-> g_sFaceplateButtonType, string, faceplate type
	. PVSS version: 3.0 
	. operating system: NT, W2000 and Linux, but tested only under W2000.
	. distributed system: yes but not tested.
*/

cietPT_UserLoginGetButtonState(string deviceName, string sType, dyn_string &dsReturnData)
{
	
		int iRes;
	string sDpLock, sDpSelectedManager, sSelectedManager, sSystemName;
	bool bLocked;
//	DebugN("UserLoginGetButtonState");
// 2. Get parameters to call the set button function
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);

	sDpLock = deviceName + ".statusInformation.selectedManager:_lock._original._locked";
	sDpSelectedManager = deviceName + ".statusInformation.selectedManager";
	
	if (dpExists(deviceName))
	{
		iRes = dpGet(sDpLock, bLocked, sDpSelectedManager, sSelectedManager);
	}
	else
	{
		iRes = -1;
	}
	if (iRes < 0) // Errors during dpget
	{
		sDpLock = "";
		bLocked = false;
		sDpSelectedManager = "";
		sSelectedManager = "";
	}
	dsReturnData=makeDynString(bLocked, sSelectedManager, "", "", "");
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietPT_ButtonDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietPT_ButtonDisconnection()
{
			   
//	DebugN("ButtonDisconnection");
		unGenericObject_ButtonDisconnect(makeDynString(ciet_FACEPLATE_BUTTON_RESET,ciet_FACEPLATE_BUTTON_RAWDATA, ciet_FACEPLATE_BUTTON_CALIBRATION, ciet_FACEPLATE_BUTTON_CONFIG,ciet_FACEPLATE_BUTTON_DEADBANDS, ciet_FACEPLATE_BUTTON_FILTERING,ciet_FACEPLATE_BUTTON_CONSTANTS, UN_FACEPLATE_BUTTON_SELECT, ciet_FACEPLATE_BUTTON_TENDPARAM, ciet_FACEPLATE_BUTTON_DEBUG, ciet_FACEPLATE_BUTTON_DISABLE));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietPT_WidgetRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in PT widget
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietPT_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, deviceType;
	int iAction,iRes;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName;
	bool bRemote;
	
//		DebugN("WidgetRegisterCB");
	deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);

	if (deviceName == "")	// In case of disconnection
		{
		deviceName = g_sDpName;
		}
		// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	
	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;

	if(sPlcName != "") {
		if (dpExists(sSystemName+c_CRYO_FRONTEND_UNSYSTEMALARM+c_cietSystemIntegrity_DS+sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;

	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			cietPT_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:
			g_sDpName = deviceName;
			g_bDisconnectCB = false;
	
		if(g_bUnSystemAlarmPlc) {
					deviceType = dpTypeName(deviceName);
				if(deviceType == "PT")
					iRes = dpConnect("cietPT_WidgetLockCallBack", deviceName + ".ProcessInput.Data.pressureCalc:_lock._alert_hdl._locked");
				else{
					g_bUnSystemAlarmPlc = false;
					cietPT_WidgetDisconnection(g_sWidgetType);
				}
	
				g_bCallbackConnected = (iRes >= 0);
			}
			else
					cietPT_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			if(g_bUnSystemAlarmPlc) 
				iRes = dpDisconnect("cietPT_WidgetLockCallBack", deviceName + ".ProcessInput.Data.pressureCalc:_lock._alert_hdl._locked");

			if(g_bDisconnectCB) {
						cietPT_WidgetDisconnect(g_sDpName, sPlcName);
				}
			g_bCallbackConnected = !(iRes >= 0);
			g_bDisconnectCB = false;

			cietPT_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}
//---------------------------------------------------------------------------------------------------------------------------------------
// cietPT_WidgetLockCallBack
/**
Purpose: set up the lock callback function for the widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in PT widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietPT_WidgetLockCallBack(string sdpe, bool bLockState)
{
	int iRes;
	string sPlcName;

	if(bLockState) 
		{ // lock set so disconnect if not already disconnected
		if(g_bDisconnectCB) 
			{
				cietPT_WidgetDisconnect(g_sDpName, sPlcName);
			}
		g_bDisconnectCB = false;
		cietPT_WidgetDisconnection(g_sWidgetType);
		}
	else 
		{ // lock not set so connect if not already connected
		g_bDisconnectCB = true;
		sPlcName = unGenericObject_GetPLCNameFromDpName(sdpe);
		cietPT_WidgetConnect(g_sDpName,sPlcName);
		}
}
//---------------------------------------------------------------------------------------------------------------------------------------

// cietPT_WidgetConnect
/**
Purpose: Connect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Analog faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietPT_WidgetConnect(string deviceName, string sPlcName)
{
	int iRes;
	string sSystemName, sAlias,sAgentName;
	dyn_string dsDeviceNameSplit;
//			DebugN("WidgetConnect");
//  check if the device name is of type "PT" if not like of no PLC. -> not available.
	if(dpTypeName(deviceName) != "PT")
		g_bUnSystemAlarmPlc = false;
	
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);

			
	if(g_bUnSystemAlarmPlc)
	{
		
	sAgentName = cietGenericFunctions_getAgentName(deviceName);
	sAlias = unGenericDpFunctions_getAlias(sAgentName);
	dsDeviceNameSplit=strsplit(sAlias, ".");

//		iRes = dpConnect("cietPT_WidgetCB", 
		iRes = dpConnect("cietPT_WidgetAnimationCB", 
						 					deviceName + ".ProcessInput.Data.pressureCalc",
						 					deviceName + ".ProcessInput.Data.resistanceCalc",						 					
				 							sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".enabled",
						 					deviceName + ".ProcessInput.Data.sensorStatus",		
											deviceName + ".ST_GTWDEV",						 														
									/*		sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".enabled",*/
											deviceName + ".ProcessInput.Data.pressureCalc:_online.._invalid", 
									 		deviceName + ".statusInformation.selectedManager:_lock._original._locked",
						 					deviceName + ".statusInformation.selectedManager");
		g_bCallbackConnected = (iRes >= 0);		
	}else{
		cietPT_WidgetDisconnection(g_sWidgetType);
	}
}
//---------------------------------------------------------------------------------------------------------------------------------------

// cietPT_WidgetDisconnect
/**
Purpose: Disconnect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Analog faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietPT_WidgetDisconnect(string deviceName, string sPlcName)
{
	int iRes;
	string sSystemName, sAlias,sAgentName;
	dyn_string dsDeviceNameSplit;
//				DebugN("WidgetDisconnect");
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);

		
	if(g_bUnSystemAlarmPlc)
	{
	sAgentName = cietGenericFunctions_getAgentName(deviceName);
	sAlias = unGenericDpFunctions_getAlias(sAgentName);
	dsDeviceNameSplit=strsplit(sAlias, ".");

		iRes = dpDisconnect("cietPT_WidgetAnimationCB", 
						 					deviceName + ".ProcessInput.Data.pressureCalc",
						 					deviceName + ".ProcessInput.Data.resistanceCalc",						 											 					
			 								sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".enabled",
						 					deviceName + ".ProcessInput.Data.sensorStatus",					
											deviceName + ".ST_GTWDEV",						 																				 											
									/*		sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".enabled",*/
											deviceName + ".ProcessInput.Data.pressureCalc:_online.._invalid", 
									 		deviceName + ".statusInformation.selectedManager:_lock._original._locked",
						 					deviceName + ".statusInformation.selectedManager");
		
		g_bCallbackConnected = (iRes >= 0);
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietPT_WidgetAnimationCB
/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Analog faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietPT_WidgetAnimationCB(string sDp_pressureCalc, float fpressureCalc,
											 string sDp_resistanceCalc, float fresistanceCalc,
											 string sDpSystemIntegrityAlarmValue, int iSystemIntegrityAlarmValue, 
											 string sDpSystemIntegrityAlarmEnabled, bool bSystemIntegrityAlarmEnabled,
											 string sDp_sensorStatus, unsigned usensorStatus,			
											 string sDp_ST_GTWDEV, bool bST_GTWDEV,
											/* string sDpSystemAlarmValue, int iSystemAlarmValue,
											 string sDpSystemAlarmEnabled, bool bSystemAlarmEnabled,*/
											 string sDpCodeInvalid, bool bCodeInvalid,
											 string sDpLocked, bool bLocked,
											 string sDpSelectedManager, string sSelectedManager)
{

	string formattedValue,sFormat,sUnit;
	string warningLetter, warningColor, alarmLetter, alarmColor, controlLetter, controlColor, body1Text, body1Color, body2Text, body2Color, selectColor, filteringColor, filteringLetter;
	bool lockVisible, warningVisible, filteringVisible;

// Pressure
	sFormat	= dpGetFormat(sDp_pressureCalc);
	sUnit  	= dpGetUnit(sDp_pressureCalc);		
	formattedValue = unGenericObject_FormatValue(sFormat, fpressureCalc);
	body1Text = formattedValue + " " + sUnit;
	body1Color = "unDisplayValue_Status";
	// body1Text = fpressureCalc;
	
// Resistance
	sFormat	= dpGetFormat(sDp_resistanceCalc);
	sUnit  	= dpGetUnit(sDp_resistanceCalc);		
	formattedValue = unGenericObject_FormatValue(sFormat, fresistanceCalc);
	body2Text = formattedValue + " " + sUnit;
	body2Color = "unDisplayValue_Status";

	if(getBit(usensorStatus, 8)==1 )																	// Filtering
  	{
  	 filteringVisible=true; 
  	 filteringColor="white";
  	 filteringLetter=ciet_WIDGET_TEXT_FILTERING_MEDIAN;  	 
  	} 
  else if (getBit(usensorStatus, 9)==1)
  	{
  	 filteringVisible=true; 
  	 filteringColor="white";
  	 filteringLetter=ciet_WIDGET_TEXT_FILTERING_AVERAGE;
  	}  
  else
  	filteringVisible=false;
	
	// WARNING status	
	
	if (getBit(usensorStatus, 12)==1)			// Disable
		{
			body1Color = "cietWidget_Disabled";		
			body2Color = "cietWidget_Disabled";		
			
			warningLetter = ciet_WIDGET_TEXT_WARNING_DISABLE;
			warningColor = "cietWidget_Warning";	
			warningVisible=true;
										
			if (getBit(usensorStatus, 0)==1)			// I/O Error
				{
					warningLetter = UN_WIDGET_TEXT_WARNING_ERROR + ciet_WIDGET_TEXT_WARNING_DISABLE;
					warningColor = "cietWidget_Warning";	
					warningVisible=true;
				}				
		}
	else
		{
			if (bCodeInvalid)																		// Invalid
				{
					warningLetter = UN_WIDGET_TEXT_INVALID;
					warningColor = "unDataNotValid";
					warningVisible=true;	
					body1Color = "unDataNotValid";
					body2Color = "unDataNotValid";	
				}	
			else
				{
						if (getBit(usensorStatus, 0)==1)							// I/O Error
							{
								warningLetter = UN_WIDGET_TEXT_WARNING_ERROR;
								warningColor = "cietWidget_Warning";	
								warningVisible=true;
								body1Color = "unFaceplate_AlarmActive";  	
								body2Color = "unFaceplate_AlarmActive";
							}
						else																					// OK
							{		
		  					warningVisible=false;		
								body1Color = "unDisplayValue_Status";  	
								body2Color = "unDisplayValue_Status";
							}		
				
						if ( (!bSystemIntegrityAlarmEnabled) || (iSystemIntegrityAlarmValue >= CRYO_SYSTEM_INGETRITY_ALARM_LIMIT)) //Old Data
							{			
								warningLetter = UN_WIDGET_TEXT_OLD_DATA;
								warningColor = "unDataNotValid";
								warningVisible=true;		
							}
		
						if (bST_GTWDEV==0)														// Status Gateway Device 
							{
				 				warningLetter= ciet_WIDGET_TEXT_STATUS_GATEWAY_DEVICE;  	   	 
		  	 				warningColor="cietWidget_Warning";
		   	 				warningVisible=true; 
							}	
				}	
		}	
						
	// 3. set the widget.
	unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, selectColor, lockVisible);

//	DebugN("bLocked, sSelectedManager, selectColor,lockVisible", bLocked, sSelectedManager, selectColor,lockVisible);
	// 8. Animate widget
	
if (g_sWidgetType=="PT")	
	{
	if(g_bSystemConnected)			
		setMultiValue("Body1", "text", body1Text,
									"Body1", "foreCol", body1Color, 
									"Body2", "visible", TRUE,									
									"Body2", "text", body2Text,
									"Body2", "foreCol", body2Color, 
		      				"WarningText", "visible", warningVisible,									
				  				"WarningText", "text", warningLetter,
				  				"WarningText", "foreCol", warningColor,
									"Filtering", "visible", filteringVisible,
				  				"Filtering", "foreCol", filteringColor,		
									"Filtering", "text", filteringLetter,					  							  				
				  				"SelectArea", "foreCol", selectColor,
				  				"LockBmp", "visible", lockVisible, 
				  				"WidgetArea", "visible", true);
	}
else if(g_sWidgetType=="PT_onlyP")
	{
	if(g_bSystemConnected)			
		setMultiValue("Body1", "text", body1Text,
									"Body1", "foreCol", body1Color, 
									"Body2", "visible", FALSE,
		      				"WarningText", "visible", warningVisible,									
				  				"WarningText", "text", warningLetter,
				  				"WarningText", "foreCol", warningColor,
									"Filtering", "visible", filteringVisible,
				  				"Filtering", "foreCol", filteringColor,		
									"Filtering", "text", filteringLetter,					  							  				
				  				"SelectArea", "foreCol", selectColor,
				  				"LockBmp", "visible", lockVisible, 
				  				"WidgetArea", "visible", true);
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietPT_WidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
	- sWidgetType
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietPT_WidgetDisconnection(string sWidgetType)
{

	//DebugN("WidgetDisconnection PT");

	setMultiValue(	"Body1", "foreCol", "unDataNoAccess",
									"Body1", "backCol", "unWidget_Background",
									"Body1", "text", "???",									
									"Body2", "foreCol", "unDataNoAccess",
									"Body2", "backCol", "unWidget_Background",
									"Body2", "text", "???",
									"WarningText", "text", "",
									"Filtering", "visible", "false",
									"Filtering", "text","",										
									"LockBmp", "visible", false,
									"WidgetArea", "visible", false);									
								
}								


//@}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietPT_PanelWidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
	- sWidgetType
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietPT_PanelWidgetDisconnection(string sWidgetType)
{
	string warningText;
//DebugN("PanelWidgetDisconnection");
	warningText = UN_WIDGET_TEXT_INVALID;

	setMultiValue("Body1", "text", "",
								"Body2", "text", "",
								"WarningText", "text", warningText,
								"LockBmp", "visible", false,
								"Filtering", "visible", false,
								"Filtering", "text","");
}								

//------------------------------------------------------------------------------------------------------------------------
//@}

// cietPT_MenuConfiguration
/**
Purpose: configuration of popup menu

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsAccessOk, dyn_string, input, authorized action
	- menuList, dyn_string, output, menu to pop-up
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 3.0 
	. distributed system: yes.
*/

cietPT_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{

dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, ciet_POPUPMENU_CALIBRATION,ciet_POPUPMENU_CONFIG, 
																				ciet_POPUPMENU_DEADBANDS, ciet_POPUPMENU_FILTERING, ciet_POPUPMENU_RESET, 
																				UN_POPUPMENU_FACEPLATE_TEXT, UN_POPUPMENU_TREND_TEXT, ciet_POPUPMENU_CONSTANTS, 
																				ciet_POPUPMENU_DISABLE, ciet_POPUPMENU_TENDPARAM, ciet_POPUPMENU_DEBUG,
																				ciet_POPUPMENU_HELP,ciet_POPUPMENU_WIDGET_SIMULATOR,
																				ciet_POPUPMENU_OPEN_WFIPAGENT, ciet_POPUPMENU_OPEN_WFIPSEGMENT, ciet_POPUPMENU_OPEN_CRATE);
int granted;
								 
string localManager, sSelectedManager;
bool bSelected, bLocked;

int iUnicosPopupMenuMin = UN_POPUPMENU_MIN;	

//  Selection state

	dpGet(sDpName + ".statusInformation.selectedManager:_lock._original._locked", bLocked);
	dpGet(sDpName + ".statusInformation.selectedManager",sSelectedManager);
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = (localManager == "S");								 

								 
//1. Select
	unGenericObject_addSelectToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	dynAppend(menuList, "SEPARATOR");

// 2. Reset
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_RESET) > 0)
	{
		granted = 0;
		if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_RESET) > 0) && bSelected)
		{
			granted = 1;
		}
		dynAppend(menuList,"PUSH_BUTTON, Reset, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_RESET)+", " + granted);
	}
	
// 4. WFip constants
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_CONFIG) > 0)
	{
		granted = 0;
		if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_CONFIG) > 0) && bSelected)
		{
			granted = 1;
		}
		dynAppend(menuList, "PUSH_BUTTON, WFIP Configuration, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_CONFIG)+", " + granted);
	}

	// 11. Disable
	
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_DISABLE) > 0)
	{
		granted = 0;
		if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_DISABLE) > 0 ) && bSelected)
		{
			granted = 1;
		}
		dynAppend(menuList, "PUSH_BUTTON, Enable/Disable, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_DISABLE)+", " + granted);
	}	
		
	// 10. Debug
	
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_DEBUG) > 0)
	{
		granted = 0;
		if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_DEBUG) > 0 ) && bSelected)
		{
			granted = 1;
		}
		dynAppend(menuList, "PUSH_BUTTON, Debug, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_DEBUG)+", " + granted);
	}		
	
	// Separator
	if (dynlen(menuList) > 0)
	{
		if (menuList[dynlen(menuList)] != "SEPARATOR")
		{
			dynAppend(menuList, "SEPARATOR");
		}
	}		
	
	
// 6. DeadBands
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_DEADBANDS) > 0)
	{
		granted = 0;
		if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_DEADBANDS) > 0) && bSelected)
		{
			granted = 1;
		}
		dynAppend(menuList, "PUSH_BUTTON, DeadBands, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_DEADBANDS)+", " + granted);
	}

// 3. Get calibration	
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_CALIBRATION) > 0)
	{
		granted = 0;
		if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_CALIBRATION) > 0) && bSelected)
		{
			granted = 1;
		}
		dynAppend(menuList, "PUSH_BUTTON, Get Calibration, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_CALIBRATION)+", " + granted);
	}
	
// 3. Constants	
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_CONSTANTS) > 0)
	{
		granted = 0;
		if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_CONSTANTS) > 0) && bSelected)
		{
			granted = 1;
		}
		dynAppend(menuList, "PUSH_BUTTON, Constants, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_FILTERING)+", " + granted);
	}	
	
// 3. Filtering	
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_FILTERING) > 0)
	{
		granted = 0;
		if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_FILTERING) > 0) && bSelected)
		{
			granted = 1;
		}
		dynAppend(menuList, "PUSH_BUTTON, Filtering, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_FILTERING)+", " + granted);
	}

	//9. Tendency
	
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_TENDPARAM) > 0)
	{
		granted = 0;
		if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_TENDPARAM) > 0) && bSelected)
		{
			granted = 1;
		}
		dynAppend(menuList, "PUSH_BUTTON, Tendency, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_TENDPARAM)+", " + granted);
	}		
	
	// Separator
	if (dynlen(menuList) > 0)
	{
		if (menuList[dynlen(menuList)] != "SEPARATOR")
		{
			dynAppend(menuList, "SEPARATOR");
		}
	}		
	
// Open Agent
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_OPEN_WFIPAGENT) > 0)
		{
			granted = 0;
			if (dynContains(dsAccessOk, ciet_POPUPMENU_OPEN_WFIPAGENT) > 0)
				{
					granted = 1;
				}
			dynAppend(menuList,"PUSH_BUTTON, WFIPAgent, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_OPEN_WFIPAGENT)+", " + granted);
		}
		
// Open Segment	
if (dynContains(dsMenuConfig, ciet_POPUPMENU_OPEN_WFIPSEGMENT) > 0)
{
	granted = 0;
	if (dynContains(dsAccessOk, ciet_POPUPMENU_OPEN_WFIPSEGMENT) > 0)
		{
			granted = 1;
		}
	dynAppend(menuList,"PUSH_BUTTON, WFIPSegment, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_OPEN_WFIPSEGMENT)+", " + granted);
}
	
// Open Crate	
if (dynContains(dsMenuConfig, ciet_POPUPMENU_OPEN_CRATE) > 0)
{
	granted = 0;
	if (dynContains(dsAccessOk, ciet_POPUPMENU_OPEN_CRATE) > 0)
		{
			granted = 1;
		}
	dynAppend(menuList,"PUSH_BUTTON, Crate, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_OPEN_CRATE)+", " + granted);
}
	
//7. Default Unicos action
	unGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	if (dynlen(menuList) > 0)
	{
		if (menuList[dynlen(menuList)] != "SEPARATOR")
		{
			dynAppend(menuList, "SEPARATOR");
		}
	}
	
// 13. Help

	if (dynlen(menuList) > 0)
		{
			if (menuList[dynlen(menuList)] != "SEPARATOR")
			{
				dynAppend(menuList, "SEPARATOR");
			}
		}

	if (dynContains(dsMenuConfig, ciet_POPUPMENU_HELP) > 0)
		{
			granted = 0;
			if (dynContains(dsAccessOk, ciet_POPUPMENU_HELP) > 0)
				{
					granted = 1;
				}
			dynAppend(menuList,"PUSH_BUTTON, Help, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_HELP)+", " + granted);
		}
		
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_WIDGET_SIMULATOR) > 0)
		{
			granted = 0;
			if (dynContains(dsAccessOk, ciet_POPUPMENU_WIDGET_SIMULATOR) > 0)
				{
					granted = 1;
				}
			dynAppend(menuList,"PUSH_BUTTON, Widget Simulator, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_WIDGET_SIMULATOR)+", " + granted);
		}	
			
//5. Trend Action
	unGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);

}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietPT_HandleMenu
/**
Purpose: handle the answer of the popup menu

Parameters:
	- deviceName, string, input, the device dp name
	- sDpType, string, input, the device dp type
	- menuList, dyn_string, input, list of requested action
	- menuAnswer, int, input, selected action from the menuList
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 3.0 
	. distributed system: yes.
*/
cietPT_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
{

dyn_string exceptionInfo,dsTemp, dsFIPInfos;
string descripName,	sDpAgentName,	sDpGTWName;
string sAgentName,sBusName;
bool bRes;
int iAgentNumber, iRes;
int iUnicosPopupMenuMin = UN_POPUPMENU_MIN;

//DebugN("HandleMenu", deviceName, sDpType, menuList, menuAnswer);
//	unGenericObject_HandleUnicosMenu(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);

// handle the select
	unGenericObject_handleSelect(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);
// handle the UNICOS default (open faceplate, diagnostic and info)
	unGenericObject_handleDefaultUnicos(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);
// handle the ciet device actions
	switch (menuAnswer-iUnicosPopupMenuMin)
	{
		case ciet_POPUPMENU_ORDER_RESET:					// Reset
			cietGenericObject_resetDevice(deviceName, exceptionInfo);
			break;

		case ciet_POPUPMENU_ORDER_CONFIG:					// set config
			// DebugN("Right - Click Set Config");
			

   		 /// open Set Config Panel			
			// DebugN("Right click: WFip COnfig");
			ChildPanelOnModal("vision/cietDevices/General/cietSetConfig.pnl",
						  "WFip Conf: " + unGenericDpFunctions_getAlias(deviceName),
						  makeDynString("$sDpName:"+deviceName),0, 0);			
			
		break;

		case ciet_POPUPMENU_ORDER_DISABLE:					// Disable
			//DebugN("Right -  Disable");
   		 /// open  Disable Panel
				ChildPanelOnModal("vision/cietDevices/General/cietDisable.pnl",
								  "Disable " + unGenericDpFunctions_getAlias(deviceName),
								  makeDynString("$sDpName:"+deviceName),0,0);	
		break;
		
		case ciet_POPUPMENU_ORDER_DEBUG:					// Debug
				 cietGenericObject_OpenDebug(deviceName);	
		break;   		 

		case ciet_POPUPMENU_ORDER_DEADBANDS: 					// DeadBands
			// DebugN("Right - Click DeadBands");
			
   		 /// open Deadbands Panel
			ChildPanelOnModal("vision/cietDevices/General/cietSetDeadBandPT.pnl",
						  "DeadBands: " + unGenericDpFunctions_getAlias(deviceName),
						  makeDynString("$sDpName:"+deviceName), 0, 0);		
		break;

		case ciet_POPUPMENU_ORDER_CONSTANTS:					// Set Constants
			//DebugN("Right - Click Constants");
			
   		 /// open Constants Panel
			ChildPanelOnModal("vision/cietDevices/General/cietSetConstantsPT.pnl",
						  "Constants : " + unGenericDpFunctions_getAlias(deviceName),
						  makeDynString("$sDpName:"+deviceName), 0, 0);		
		break;

		case ciet_POPUPMENU_ORDER_CALIBRATION:					// Get Calibration
			// DebugN("Right -  Get Calibration");
   		 /// open  Get Calibration Panel
   		 /// open  Get Calibration Panel
				ChildPanelOnModal("vision/cietDevices/General/cietGetCalibration.pnl",
								  "Calibration Curve:" +  + unGenericDpFunctions_getAlias(deviceName),
								  makeDynString("$sDpName:"+deviceName),0,0);									  
								  
		break;	
		
					  	
		case ciet_POPUPMENU_ORDER_FILTERING:					// Set Filtering
			//DebugN("Right - Click Filtering");
			
   		 /// open Filtering Panel
			ChildPanelOnModal("vision/cietDevices/General/cietSetFiltering.pnl",
						  "Filtering : " + unGenericDpFunctions_getAlias(deviceName),
						  makeDynString("$sDpName:"+deviceName), 0, 0);	   		
		break;	
		
		case ciet_POPUPMENU_ORDER_TENDPARAM:					// Tendency
			//DebugN("Right -  Tendency");
   		 /// open  Tendency Panel
				ChildPanelOnModal("vision/cietDevices/General/cietTendParam.pnl",
								  "Tendency " + unGenericDpFunctions_getAlias(deviceName),
								  makeDynString("$sDpName:"+deviceName),0,0);	
		break;   		 
		
		case ciet_POPUPMENU_ORDER_OPEN_WFIPAGENT:							// Open Agent
					sAgentName=unGenericDpFunctions_getSystemName(deviceName) + cietGenericFunctions_getAgentName(deviceName);
				 unGenericObject_OpenFaceplate(unGenericDpFunctions_getWidgetDpName(sAgentName), exceptionInfo);
		break;	
		
		case ciet_POPUPMENU_ORDER_OPEN_WFIPSEGMENT:					// Open Segment
					sBusName=unGenericDpFunctions_getSystemName(deviceName) + cietGenericFuncitons_getSegmentName(deviceName);
					unGenericObject_OpenFaceplate(unGenericDpFunctions_getWidgetDpName(sBusName), exceptionInfo);
		break;	

		case ciet_POPUPMENU_ORDER_OPEN_CRATE:										// Open Crate
					cietGenericObject_openCratePanel (deviceName);
		break;	
		
		case ciet_POPUPMENU_ORDER_HELP:					// Help
					cietGenericObject_OpenHtmlFile("Help.html");
		break;	
		
		case ciet_POPUPMENU_ORDER_WIDGET_SIMULATOR:					// Widget Simulator
			ChildPanelOnRelativ("vision/cietDevices/General/cietWidgetSimulator.pnl",
			"Widget Simulator","",0, 0);			  
		break;			

		default:
		break;
	}
// handle the trend action
	unGenericObject_handleTrendAction(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);

	if (dynlen(exceptionInfo) > 0)
	{
		unSendMessage_toExpertException("Right Click",exceptionInfo);
	}
}







