/**@name LIBRARY: cietProfileGraph.ctl

@author: Enrique Blanco (AB-CO-IS)

Creation Date: 21/11/2006

Modification History: 

	. 29/11/2006	Pablo:	Adapted to CIET working.

version 1.0

External Function : 
	
Internal Functions :
	
Purpose: 
This library contains functions used to by the profile graph for the CIET devices

Usage: Public

PVSS manager usage: Ctrl, NV, NG

Constraints:
	. JCOP framework
	. PVSS version: 3.0
	. operating system: Linux and WXP but tested only on WXP.
	. distributed system: yes.
*/

//Constants


//@{

//---------------------------------------------------------------------------------------------------------------------------------------

// cietProfileGraph_systemRegister
/**
Purpose: register function for each device

Parameters:
	- sDp, string, input, system name
	- bConnected, bool, input, state of the remote system

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 3.0 
	. operating system: WXP.
	. distributed system: yes.
*/
cietProfileGraph_systemRegister(string sDp, bool bConnected)
{
	dyn_string dsDeviceName;
// get the dp to connect to.	
	string sSystemName, sSysDp = dpSubStr(sDp, DPSUB_SYS_DP), sDpName;
	int i, len, iIndex;
	int iAction, iRes;
	dyn_string exceptionInfo, dsResult;
	bool isRemoteSystem, localConnected;
	string sPVSSDpe;

	sSystemName = substr(sSysDp, strpos(sSysDp, c_unDistributedControl_dpName) + strlen(c_unDistributedControl_dpName), strlen(sSysDp)) + ":";
	dsDeviceName = g_m_dsSystemName[sSystemName];
//DebugN("unProfileGraph_systemRegister", sDp, bConnected, sSystemName);

	unDistributedControl_isRemote(isRemoteSystem, sSystemName);
	localConnected = bConnected;
	if(!isRemoteSystem)
	{
		localConnected = true;
	}

	if(localConnected && !g_bSystemNameInitialized[sSystemName]) {
// check if g_mDeviceToIndex was built before. If not build it and call cietProfileGraph_initDpe for each DP otherwise do nothing;
		len = dynlen(dsDeviceName);
		for(i=1;i<=len;i++) {
			unGenericDpFunctions_convert_UNICOSDPE_to_PVSSDPE(dsDeviceName[i], sPVSSDpe);
			iIndex = g_mDeviceToIndex[dsDeviceName[i]];
			g_dsDeviceDpe[iIndex] = sPVSSDpe;
			g_mDpToIndex[sPVSSDpe] = iIndex;
// call the init function
			cietProfileGraph_initDpe(sPVSSDpe, g_dbDeviceUseAlarm[iIndex], g_diDeviceConnectType[iIndex], g_dsDeviceType[iIndex]);
		}
//DebugTN("INIT "+sSystemName, "lodev:", dsDeviceName, "dev:", g_dsDeviceName, "devDP:", g_dsDeviceDpe, "mapdev:", g_mDeviceToIndex, "mapDp:", g_mDpToIndex, "END "+sSystemName);
		g_bSystemNameInitialized[sSystemName] = true;
	}
	len = dynlen(dsDeviceName);
	for(i=1;i<=len;i++) {
		iIndex = g_mDeviceToIndex[dsDeviceName[i]];
		sPVSSDpe = g_dsDeviceDpe[iIndex];
		sDpName = unGenericDpFunctions_getDpName(sPVSSDpe);
////DebugN(sDpName, dsDeviceName[i], iIndex);
		unGenericObject_Connection(sDpName, g_dbCallbackConnected[iIndex], bConnected, iAction, exceptionInfo);
// if deviceData does not exists set iAction = UN_ACTION_DISCONNECT

		switch (iAction)
		{
			case UN_ACTION_DISCONNECT:
//DebugTN("UN_ACTION_DISCONNECT", sDpName, dsDeviceName[i], iIndex);
				unProfileGraph_setDisconnectionDp(dsDeviceName[i]);
				break;
			case UN_ACTION_DPCONNECT:
// dpConnect
				cietProfileGraph_connectDisconnect(true, sDpName, sPVSSDpe, g_diDeviceConnectType[iIndex], dsDeviceName[i], iRes);

				g_dbCallbackConnected[iIndex] = (iRes >= 0);
//DebugTN("*************UN_ACTION_DPCONNECT", sDpName, sPVSSDpe, iIndex, g_dbCallbackConnected[iIndex], g_diDeviceConnectType[iIndex], g_dbDeviceUseAlarm[iIndex], g_dsDeviceType[iIndex]);
				break;
			case UN_ACTION_DPDISCONNECT_DISCONNECT:
//DebugTN("*************UN_ACTION_DPDISCONNECT_DISCONNECT", sDpName, sPVSSDpe, iIndex, g_dbCallbackConnected[iIndex], g_diDeviceConnectType[iIndex], g_dbDeviceUseAlarm[iIndex], g_dsDeviceType[iIndex]);
// dpDisconnect
				cietProfileGraph_connectDisconnect(false, sDpName, sPVSSDpe, g_diDeviceConnectType[iIndex], dsDeviceName[i], iRes);
				
				g_dbCallbackConnected[iIndex] = !(iRes >= 0);
////DebugN("UN_ACTION_DPDISCONNECT_DISCONNECT", sDpName, sPVSSDpe, iIndex, g_dbCallbackConnected[iIndex], g_diDeviceConnectType[iIndex], g_dbDeviceUseAlarm[iIndex], g_dsDeviceType[iIndex]);

// set data
				unProfileGraph_setDisconnectionDp(dsDeviceName[i]);
				break;
			case UN_ACTION_NOTHING:
			default:
//DebugTN("UN_ACTION_NOTHING", sDpName, dsDeviceName[i], iIndex);
				break;
		}
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietProfileGraph_initDpe
/**
Purpose: initialise the type of dpConnect dpDisconnect call

Parameters:
	- sPVSSDpe, string, input, dpe name
	- bUseAlarm, bool, input, use PVSS alert_hdl if any
	- sDpName, string, input, dp name
	- iType, int, input, type of dpConnect/dpDisconnect
	- sDeviceType, string, output, the the type of the device is returned here

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 3.0 
	. operating system: WXP.
	. distributed system: yes.
*/
cietProfileGraph_initDpe(string sPVSSDpe, bool bUseAlarm, int &iType, string &sDeviceType)
{
	int type=UN_PROFILE_GRAPH_NODP;
	bool bStsReg01;
	string sStsReg01;
	string sType = UN_PROFILE_GRAPH_NO_VALUE;
	
	if(dpExists(sPVSSDpe)) 
		{
			type=UN_PROFILE_GRAPH_NODP;
	
			sStsReg01=dpSubStr(sPVSSDpe, DPSUB_SYS_DP);
			sType = dpTypeName(sStsReg01);
	
			switch(sType) 
				{
					case "TT":
//DebugN("TT");	
						type = CIET_PROFILE_GRAPH_TT;
						break;
					case "PT":
//DebugN("PT");						
						type = CIET_PROFILE_GRAPH_PT;
						break;
					case "LT":
//DebugN("LT");						
						type = CIET_PROFILE_GRAPH_LT;
						break;
					case "EH":
//DebugN("EH");	
						type = CIET_PROFILE_GRAPH_EH;
						break;
					case "DI":
//DebugN("DI");	
						type = CIET_PROFILE_GRAPH_DI;
						break;
					default: 
//DebugN("default");						
						break;
				}
		}

	iType = type;
	sDeviceType = sType;
////DebugN("cietProfileGraph_initDpe", sPVSSDpe, sStsReg01, sType, dpTypeName(sStsReg01), iType);	
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietProfileGraph_connectDisconnect
/**
Purpose: setup dpConnect dpDisconnect call

Parameters:
	- bConnect, bool, input, true dpConnect/false dpDisconnect
	- sDpName, string, input, dp name
	- sDpe, string, input, dpe name
	- iType, int, input, type of dpConnect/dpDisconnect
	- sDeviceName, string, input, device name
	- iResult, int, output, the result of dpConnect/dpDisconnect is returned here

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 3.0 
	. operating system: WXP.
	. distributed system: yes.
*/
cietProfileGraph_connectDisconnect(bool bConnect, string sDpName, string sDpe, int iType, string sDeviceName, int &iResult)
{
//DebugN("cietProfileGraph_connectDisconnect");	
	int iRes=0;
	dyn_string dsDpe;

	switch(iType)
	 { 

			// - - - TT - - - 

			case CIET_PROFILE_GRAPH_TT:
//DebugN("case CIET_PROFILE_GRAPH_TT:");
									dsDpe = makeDynString(sDpName + ".ST_GTWDEV",
								  											sDpName + ".ProcessInput.Data.temperatureCalc:_online.._invalid",
								  											sDpName + ".ProcessInput.Data.sensorStatus");  

	  				if(bConnect) // TT Connected
						{
								if(dynContains(dsDpe, sDpe) <= 0) 																// TT Connected with DPE
								{
										//DebugN("****************************************************");																														
										//DebugN("TT Connected with DPE");
										//DebugN("========================================");																				
										//DebugN("Selected Manager Locked = " + sDpName + ".statusInformation.selectedManager:_lock._original._locked");
										//DebugN("Selected Manager = "+sDpName + ".statusInformation.selectedManager");
										//DebugN("ST_GTWDEV = " + sDpName + ".ST_GTWDEV");
										//DebugN("Invalid = " + sDpName + ".ProcessInput.Data.temperatureCalc:_online.._invalid");
										//DebugN("sDpe = "+ sDpe);
										//DebugN("Time = "+	sDpe + ":_online.._stime");
										//DebugN("========================================");		
										//DebugN("****************************************************");																						
										
										iRes = dpConnect("cietProfileGraph_animationCB", 
														sDpName + ".statusInformation.selectedManager:_lock._original._locked",
														sDpName + ".statusInformation.selectedManager",
														sDpName + ".ST_GTWDEV",
								  					sDpName + ".ProcessInput.Data.temperatureCalc:_online.._invalid",
								  					sDpName + ".ProcessInput.Data.sensorStatus",
													 	sDpe,
														sDpe + ":_online.._stime");
								}
								else 																															// TT Connected without DPE
								{
										//DebugN("****************************************************");		
										//DebugN("TT Connected without DPE");
										//DebugN("========================================");																				
										//DebugN("Selected Manager Locked = " + sDpName + ".statusInformation.selectedManager:_lock._original._locked");
										//DebugN("Selected Manager = "+sDpName + ".statusInformation.selectedManager");
										//DebugN("ST_GTWDEV = " + sDpName + ".ST_GTWDEV");
										//DebugN("Invalid = " + sDpName + ".ProcessInput.Data.temperatureCalc:_online.._invalid");
										//DebugN("sDpe = "+ sDpe);
										//DebugN("Time = "+	sDpe + ":_online.._stime");
										//DebugN("========================================");		
										//DebugN("****************************************************");												
										
										iRes = dpConnect("cietProfileGraph_animationCB", 
														sDpName + ".statusInformation.selectedManager:_lock._original._locked",
														sDpName + ".statusInformation.selectedManager",
														sDpName + ".ST_GTWDEV",
								  					sDpName + ".ProcessInput.Data.temperatureCalc:_online.._invalid",
								  					sDpName + ".ProcessInput.Data.sensorStatus",
														sDpe + ":_online.._stime");
								}
						}
						else 					// TT Disconnected
						{
								if(dynContains(dsDpe, sDpe) <= 0) 																// TT Disconnected with DPE 	
								{
										//DebugN("****************************************************");	
										//DebugN("TT Disconnected with DPE");		
										//DebugN("========================================");																				
										//DebugN("Selected Manager Locked = " + sDpName + ".statusInformation.selectedManager:_lock._original._locked");
										//DebugN("Selected Manager = "+sDpName + ".statusInformation.selectedManager");
										//DebugN("ST_GTWDEV = " + sDpName + ".ST_GTWDEV");
										//DebugN("Invalid = " + sDpName + ".ProcessInput.Data.temperatureCalc:_online.._invalid");
										//DebugN("sDpe = "+ sDpe);
										//DebugN("Time = "+	sDpe + ":_online.._stime");
										//DebugN("========================================");		
										//DebugN("****************************************************");										
														
										iRes = dpDisconnect("cietProfileGraph_animationCB", 
														sDpName + ".statusInformation.selectedManager:_lock._original._locked",
														sDpName + ".statusInformation.selectedManager",
													 	sDpName + ".ST_GTWDEV",
								  					sDpName + ".ProcessInput.Data.temperatureCalc:_online.._invalid",
								  					sDpName + ".ProcessInput.Data.sensorStatus",
													 	sDpe,
														sDpe + ":_online.._stime");
								}
								else 																															// TT Disconnected without DPE
								{
										//DebugN("****************************************************");											
										//DebugN("TT Disconnected without DPE");
										//DebugN("========================================");																				
										//DebugN("Selected Manager Locked = " + sDpName + ".statusInformation.selectedManager:_lock._original._locked");
										//DebugN("Selected Manager = "+sDpName + ".statusInformation.selectedManager");
										//DebugN("ST_GTWDEV = " + sDpName + ".ST_GTWDEV");
										//DebugN("Invalid = " + sDpName + ".ProcessInput.Data.temperatureCalc:_online.._invalid");
										//DebugN("sDpe = "+ sDpe);
										//DebugN("Time = "+	sDpe + ":_online.._stime");
										//DebugN("========================================");		
										//DebugN("****************************************************");												
										
										iRes = dpDisconnect("cietProfileGraph_animationCB", 
														sDpName + ".statusInformation.selectedManager:_lock._original._locked",
														sDpName + ".statusInformation.selectedManager",
														sDpName + ".ST_GTWDEV",
								  					sDpName + ".ProcessInput.Data.temperatureCalc:_online.._invalid",
								  					sDpName + ".ProcessInput.Data.sensorStatus",
														sDpe + ":_online.._stime");
								}
						}
						break;


  	// - - - PT - - - 

			case CIET_PROFILE_GRAPH_PT:
//DebugN("case CIET_PROFILE_GRAPH_PT:");
									dsDpe = makeDynString(sDpName + ".ST_GTWDEV",
								  											sDpName + ".ProcessInput.Data.pressureCalc:_online.._invalid",
								  											sDpName + ".ProcessInput.Data.sensorStatus");  
								  											
	  				if(bConnect) // PT Connected
						{
								if(dynContains(dsDpe, sDpe) <= 0) 																// PT Connected with DPE
								{
										//DebugN("****************************************************");	
										//DebugN("PT Connected with DPE");
										//DebugN("========================================");																				
										//DebugN("Selected Manager Locked = " + sDpName + ".statusInformation.selectedManager:_lock._original._locked");
										//DebugN("Selected Manager = "+sDpName + ".statusInformation.selectedManager");
										//DebugN("ST_GTWDEV = " + sDpName + ".ST_GTWDEV");
										//DebugN("Invalid = " + sDpName + ".ProcessInput.Data.pressureCalc:_online.._invalid");
										//DebugN("sDpe = "+ sDpe);
										//DebugN("Time = "+	sDpe + ":_online.._stime");
										//DebugN("========================================");		
										//DebugN("****************************************************");											
										
										iRes = dpConnect("cietProfileGraph_animationCB", 
														sDpName + ".statusInformation.selectedManager:_lock._original._locked",
														sDpName + ".statusInformation.selectedManager",
														sDpName + ".ST_GTWDEV",
								  					sDpName + ".ProcessInput.Data.pressureCalc:_online.._invalid",
								  					sDpName + ".ProcessInput.Data.sensorStatus",
													 	sDpe,
														sDpe + ":_online.._stime");
								}
								else 																															// PT Connected without DPE
								{
										//DebugN("****************************************************");		
										//DebugN("PT Connected without DPE");
										//DebugN("========================================");																				
										//DebugN("Selected Manager Locked = " + sDpName + ".statusInformation.selectedManager:_lock._original._locked");
										//DebugN("Selected Manager = "+sDpName + ".statusInformation.selectedManager");
										//DebugN("ST_GTWDEV = " + sDpName + ".ST_GTWDEV");
										//DebugN("Invalid = " + sDpName + ".ProcessInput.Data.pressureCalc:_online.._invalid");
										//DebugN("sDpe = "+ sDpe);
										//DebugN("Time = "+	sDpe + ":_online.._stime");
										//DebugN("========================================");		
										//DebugN("****************************************************");												
										
										iRes = dpConnect("cietProfileGraph_animationCB", 
														sDpName + ".statusInformation.selectedManager:_lock._original._locked",
														sDpName + ".statusInformation.selectedManager",
														sDpName + ".ST_GTWDEV",
								  					sDpName + ".ProcessInput.Data.pressureCalc:_online.._invalid",
								  					sDpName + ".ProcessInput.Data.sensorStatus",
														sDpe + ":_online.._stime");
								}
						}
						else 					// PT Disconnected
						{
								if(dynContains(dsDpe, sDpe) <= 0) 																// PT Disconnected with DPE 	
								{
										//DebugN("****************************************************");	
										//DebugN("PT Disconnected with DPE");
										//DebugN("========================================");																				
										//DebugN("Selected Manager Locked = " + sDpName + ".statusInformation.selectedManager:_lock._original._locked");
										//DebugN("Selected Manager = "+sDpName + ".statusInformation.selectedManager");
										//DebugN("ST_GTWDEV = " + sDpName + ".ST_GTWDEV");
										//DebugN("Invalid = " + sDpName + ".ProcessInput.Data.pressureCalc:_online.._invalid");
										//DebugN("sDpe = "+ sDpe);
										//DebugN("Time = "+	sDpe + ":_online.._stime");
										//DebugN("========================================");		
										//DebugN("****************************************************");												

										iRes = dpDisconnect("cietProfileGraph_animationCB", 
														sDpName + ".statusInformation.selectedManager:_lock._original._locked",
														sDpName + ".statusInformation.selectedManager",
													 	sDpName + ".ST_GTWDEV",
								  					sDpName + ".ProcessInput.Data.pressureCalc:_online.._invalid",
								  					sDpName + ".ProcessInput.Data.sensorStatus",
													 	sDpe,
														sDpe + ":_online.._stime");
								}
								else 																															// PT Disconnected without DPE
								{
										//DebugN("****************************************************");									
										//DebugN("PT Disconnected without DPE");
										//DebugN("========================================");																				
										//DebugN("Selected Manager Locked = " + sDpName + ".statusInformation.selectedManager:_lock._original._locked");
										//DebugN("Selected Manager = "+sDpName + ".statusInformation.selectedManager");
										//DebugN("ST_GTWDEV = " + sDpName + ".ST_GTWDEV");
										//DebugN("Invalid = " + sDpName + ".ProcessInput.Data.pressureCalc:_online.._invalid");
										//DebugN("sDpe = "+ sDpe);
										//DebugN("Time = "+	sDpe + ":_online.._stime");
										//DebugN("========================================");										
										//DebugN("****************************************************");											

										iRes = dpDisconnect("cietProfileGraph_animationCB", 
														sDpName + ".statusInformation.selectedManager:_lock._original._locked",
														sDpName + ".statusInformation.selectedManager",
														sDpName + ".ST_GTWDEV",
								  					sDpName + ".ProcessInput.Data.pressureCalc:_online.._invalid",
								  					sDpName + ".ProcessInput.Data.sensorStatus",
														sDpe + ":_online.._stime");
								}
						}
						break;

							
			// - - - LT - - - 

			case CIET_PROFILE_GRAPH_LT:
//DebugN("case CIET_PROFILE_GRAPH_LT:");
									dsDpe = makeDynString(sDpName + ".ST_GTWDEV",
								  											sDpName + ".ProcessInput.Data.levelCalc:_online.._invalid",
								  											sDpName + ".ProcessInput.Data.sensorStatus");  
								  											
	  				if(bConnect) // LT Connected
						{
								if(dynContains(dsDpe, sDpe) <= 0) 																// LT Connected with DPE
								{
										//DebugN("LT Connected with DPE");
										//DebugN("========================================");																				
										//DebugN("Selected Manager Locked = " + sDpName + ".statusInformation.selectedManager:_lock._original._locked");
										//DebugN("Selected Manager = "+sDpName + ".statusInformation.selectedManager");
										//DebugN("ST_GTWDEV = " + sDpName + ".ST_GTWDEV");
										//DebugN("Invalid = " + sDpName + ".ProcessInput.Data.levelCalc:_online.._invalid");
										//DebugN("sDpe = "+ sDpe);
										//DebugN("Time = "+	sDpe + ":_online.._stime");
										//DebugN("========================================");										
																																																												
										iRes = dpConnect("cietProfileGraph_animationCB", 
														sDpName + ".statusInformation.selectedManager:_lock._original._locked",
														sDpName + ".statusInformation.selectedManager",
														sDpName + ".ST_GTWDEV",
								  					sDpName + ".ProcessInput.Data.levelCalc:_online.._invalid",
								  					sDpName + ".ProcessInput.Data.sensorStatus",
													 	sDpe,
														sDpe + ":_online.._stime");
								}
								else 																															// LT Connected without DPE
								{
										//DebugN("****************************************************");	
										//DebugN("LT Connected without DPE");
										//DebugN("========================================");																				
										//DebugN("Selected Manager Locked = " + sDpName + ".statusInformation.selectedManager:_lock._original._locked");
										//DebugN("Selected Manager = "+sDpName + ".statusInformation.selectedManager");
										//DebugN("ST_GTWDEV = " + sDpName + ".ST_GTWDEV");
										//DebugN("Invalid = " + sDpName + ".ProcessInput.Data.levelCalc:_online.._invalid");
										//DebugN("sDpe = "+ sDpe);
										//DebugN("Time = "+	sDpe + ":_online.._stime");
										//DebugN("========================================");										
										//DebugN("****************************************************");											
																		
										iRes = dpConnect("cietProfileGraph_animationCB", 
														sDpName + ".statusInformation.selectedManager:_lock._original._locked",
														sDpName + ".statusInformation.selectedManager",
														sDpName + ".ST_GTWDEV",
								  					sDpName + ".ProcessInput.Data.levelCalc:_online.._invalid",
								  					sDpName + ".ProcessInput.Data.sensorStatus",
														sDpe + ":_online.._stime");
								}
						}
						else 					// LT Disconnected
						{
								if(dynContains(dsDpe, sDpe) <= 0) 																// LT Disconnected with DPE 	
								{
										//DebugN("****************************************************");										
										//DebugN("LT Disconnected with DPE");		
										//DebugN("========================================");																				
										//DebugN("Selected Manager Locked = " + sDpName + ".statusInformation.selectedManager:_lock._original._locked");
										//DebugN("Selected Manager = "+sDpName + ".statusInformation.selectedManager");
										//DebugN("ST_GTWDEV = " + sDpName + ".ST_GTWDEV");
										//DebugN("Invalid = " + sDpName + ".ProcessInput.Data.levelCalc:_online.._invalid");
										//DebugN("sDpe = "+ sDpe);
										//DebugN("Time = "+	sDpe + ":_online.._stime");
										//DebugN("========================================");										
										//DebugN("****************************************************");																						
																
										iRes = dpDisconnect("cietProfileGraph_animationCB", 
														sDpName + ".statusInformation.selectedManager:_lock._original._locked",
														sDpName + ".statusInformation.selectedManager",
													 	sDpName + ".ST_GTWDEV",
								  					sDpName + ".ProcessInput.Data.levelCalc:_online.._invalid",
								  					sDpName + ".ProcessInput.Data.sensorStatus",
													 	sDpe,
														sDpe + ":_online.._stime");
								}
								else 																															// LT Disconnected without DPE
								{
										//DebugN("****************************************************");									
										//DebugN("LT Disconnected without DPE");	
										//DebugN("========================================");																				
										//DebugN("Selected Manager Locked = " + sDpName + ".statusInformation.selectedManager:_lock._original._locked");
										//DebugN("Selected Manager = "+sDpName + ".statusInformation.selectedManager");
										//DebugN("ST_GTWDEV = " + sDpName + ".ST_GTWDEV");
										//DebugN("Invalid = " + sDpName + ".ProcessInput.Data.levelCalc:_online.._invalid");
										//DebugN("sDpe = "+ sDpe);
										//DebugN("Time = "+	sDpe + ":_online.._stime");
										//DebugN("========================================");										
										//DebugN("****************************************************");												
																		
										iRes = dpDisconnect("cietProfileGraph_animationCB", 
														sDpName + ".statusInformation.selectedManager:_lock._original._locked",
														sDpName + ".statusInformation.selectedManager",
														sDpName + ".ST_GTWDEV",
								  					sDpName + ".ProcessInput.Data.levelCalc:_online.._invalid",
								  					sDpName + ".ProcessInput.Data.sensorStatus",
														sDpe + ":_online.._stime");
								}
						}
						break;

							
	// - - - EH - - - 

			case CIET_PROFILE_GRAPH_EH:
//DebugN("case CIET_PROFILE_GRAPH_EH:");
									dsDpe = makeDynString(sDpName + ".ST_GTWDEV",
								  											sDpName + ".ProcessInput.Data.powerCalc:_online.._invalid",
								  											sDpName + ".ProcessInput.Data.sensorStatus");  

	  				if(bConnect) // EH Connected
						{
								if(dynContains(dsDpe, sDpe) <= 0) 																// EH Connected with DPE
								{
										//DebugN("****************************************************");									
										//DebugN("EH Connected with DPE");
										//DebugN("========================================");																				
										//DebugN("Selected Manager Locked = " + sDpName + ".statusInformation.selectedManager:_lock._original._locked");
										//DebugN("Selected Manager = "+sDpName + ".statusInformation.selectedManager");
										//DebugN("ST_GTWDEV = " + sDpName + ".ST_GTWDEV");
										//DebugN("Invalid = " + sDpName + ".ProcessInput.Data.powerCalc:_online.._invalid");
										//DebugN("sDpe = "+ sDpe);
										//DebugN("Time = "+	sDpe + ":_online.._stime");
										//DebugN("========================================");										
										//DebugN("****************************************************");												
																																																												
										iRes = dpConnect("cietProfileGraph_animationCB", 
														sDpName + ".statusInformation.selectedManager:_lock._original._locked",
														sDpName + ".statusInformation.selectedManager",
														sDpName + ".ST_GTWDEV",
								  					sDpName + ".ProcessInput.Data.powerCalc:_online.._invalid",
								  					sDpName + ".ProcessInput.Data.sensorStatus",
													 	sDpe,
														sDpe + ":_online.._stime");
								}
								else 																															// EH Connected without DPE
								{
										//DebugN("****************************************************");		
										//DebugN("EH Connected without DPE");
										//DebugN("========================================");																				
										//DebugN("Selected Manager Locked = " + sDpName + ".statusInformation.selectedManager:_lock._original._locked");
										//DebugN("Selected Manager = "+sDpName + ".statusInformation.selectedManager");
										//DebugN("ST_GTWDEV = " + sDpName + ".ST_GTWDEV");
										//DebugN("Invalid = " + sDpName + ".ProcessInput.Data.powerCalc:_online.._invalid");
										//DebugN("sDpe = "+ sDpe);
										//DebugN("Time = "+	sDpe + ":_online.._stime");
										//DebugN("========================================");										
										//DebugN("****************************************************");												
										
										iRes = dpConnect("cietProfileGraph_animationCB", 
														sDpName + ".statusInformation.selectedManager:_lock._original._locked",
														sDpName + ".statusInformation.selectedManager",
														sDpName + ".ST_GTWDEV",
								  					sDpName + ".ProcessInput.Data.powerCalc:_online.._invalid",
								  					sDpName + ".ProcessInput.Data.sensorStatus",
														sDpe + ":_online.._stime");
								}
						}
						else 					// EH Disconnected
						{
								if(dynContains(dsDpe, sDpe) <= 0) 																// EH Disconnected with DPE 	
								{
										//DebugN("****************************************************");	
										//DebugN("EH Disconnected with DPE");
										//DebugN("========================================");																				
										//DebugN("Selected Manager Locked = " + sDpName + ".statusInformation.selectedManager:_lock._original._locked");
										//DebugN("Selected Manager = "+sDpName + ".statusInformation.selectedManager");
										//DebugN("ST_GTWDEV = " + sDpName + ".ST_GTWDEV");
										//DebugN("Invalid = " + sDpName + ".ProcessInput.Data.powerCalc:_online.._invalid");
										//DebugN("sDpe = "+ sDpe);
										//DebugN("Time = "+	sDpe + ":_online.._stime");
										//DebugN("========================================");										
										//DebugN("****************************************************");										
										
										iRes = dpDisconnect("cietProfileGraph_animationCB", 
														sDpName + ".statusInformation.selectedManager:_lock._original._locked",
														sDpName + ".statusInformation.selectedManager",
													 	sDpName + ".ST_GTWDEV",
								  					sDpName + ".ProcessInput.Data.powerCalc:_online.._invalid",
								  					sDpName + ".ProcessInput.Data.sensorStatus",
													 	sDpe,
														sDpe + ":_online.._stime");
								}
								else 																															// EH Disconnected without DPE
								{
										//DebugN("****************************************************");			
										//DebugN("EH Disconnected without DPE");
										//DebugN("========================================");																				
										//DebugN("Selected Manager Locked = " + sDpName + ".statusInformation.selectedManager:_lock._original._locked");
										//DebugN("Selected Manager = "+sDpName + ".statusInformation.selectedManager");
										//DebugN("ST_GTWDEV = " + sDpName + ".ST_GTWDEV");
										//DebugN("Invalid = " + sDpName + ".ProcessInput.Data.powerCalc:_online.._invalid");
										//DebugN("sDpe = "+ sDpe);
										//DebugN("Time = "+	sDpe + ":_online.._stime");
										//DebugN("========================================");										
										//DebugN("****************************************************");													
										
										iRes = dpDisconnect("cietProfileGraph_animationCB", 
														sDpName + ".statusInformation.selectedManager:_lock._original._locked",
														sDpName + ".statusInformation.selectedManager",
														sDpName + ".ST_GTWDEV",
								  					sDpName + ".ProcessInput.Data.powerCalc:_online.._invalid",
								  					sDpName + ".ProcessInput.Data.sensorStatus",
														sDpe + ":_online.._stime");
								}
						}
						break;

							
		// - - - DI - - - 

			case CIET_PROFILE_GRAPH_DI:
//DebugN("case CIET_PROFILE_GRAPH_DI:");
									dsDpe = makeDynString(sDpName + ".ST_GTWDEV",
								  											sDpName + ".ProcessInput.Data.digitalInput:_online.._invalid",
								  											sDpName + ".ProcessInput.Data.sensorStatus");  

	  				if(bConnect) // DI Connected
						{
								if(dynContains(dsDpe, sDpe) <= 0) 																// DI Connected with DPE
								{
										//DebugN("****************************************************");		
										//DebugN("DI Connected with DPE");
										//DebugN("========================================");																				
										//DebugN("Selected Manager Locked = " + sDpName + ".statusInformation.selectedManager:_lock._original._locked");
										//DebugN("Selected Manager = "+sDpName + ".statusInformation.selectedManager");
										//DebugN("ST_GTWDEV = " + sDpName + ".ST_GTWDEV");
										//DebugN("Invalid = " + sDpName + ".ProcessInput.Data.digitalInput:_online.._invalid");
										//DebugN("sDpe = "+ sDpe);
										//DebugN("Time = "+	sDpe + ":_online.._stime");
										//DebugN("========================================");										
										//DebugN("****************************************************");											
										
										iRes = dpConnect("cietProfileGraph_animationCB", 
														sDpName + ".statusInformation.selectedManager:_lock._original._locked",
														sDpName + ".statusInformation.selectedManager",
														sDpName + ".ST_GTWDEV",
								  					sDpName + ".ProcessInput.Data.digitalInput:_online.._invalid",
								  					sDpName + ".ProcessInput.Data.sensorStatus",
													 	sDpe,
														sDpe + ":_online.._stime");
								}
								else 																															// DI Connected without DPE
								{
										//DebugN("****************************************************");											
										//DebugN("DI Connected without DPE");		
										//DebugN("========================================");																				
										//DebugN("Selected Manager Locked = " + sDpName + ".statusInformation.selectedManager:_lock._original._locked");
										//DebugN("Selected Manager = "+sDpName + ".statusInformation.selectedManager");
										//DebugN("ST_GTWDEV = " + sDpName + ".ST_GTWDEV");
										//DebugN("Invalid = " + sDpName + ".ProcessInput.Data.digitalInput:_online.._invalid");
										//DebugN("sDpe = "+ sDpe);
										//DebugN("Time = "+	sDpe + ":_online.._stime");
										//DebugN("========================================");										
										//DebugN("****************************************************");																					
																
										iRes = dpConnect("cietProfileGraph_animationCB", 
														sDpName + ".statusInformation.selectedManager:_lock._original._locked",
														sDpName + ".statusInformation.selectedManager",
														sDpName + ".ST_GTWDEV",
								  					sDpName + ".ProcessInput.Data.digitalInput:_online.._invalid",
								  					sDpName + ".ProcessInput.Data.sensorStatus",
														sDpe + ":_online.._stime");
								}
						}
						else 					// DI Disconnected
						{
								if(dynContains(dsDpe, sDpe) <= 0) 																// DI Disconnected with DPE 	
								{
										//DebugN("****************************************************");																					
										//DebugN("DI Disconnected with DPE");
										//DebugN("========================================");																				
										//DebugN("Selected Manager Locked = " + sDpName + ".statusInformation.selectedManager:_lock._original._locked");
										//DebugN("Selected Manager = "+sDpName + ".statusInformation.selectedManager");
										//DebugN("ST_GTWDEV = " + sDpName + ".ST_GTWDEV");
										//DebugN("Invalid = " + sDpName + ".ProcessInput.Data.digitalInput:_online.._invalid");
										//DebugN("sDpe = "+ sDpe);
										//DebugN("Time = "+	sDpe + ":_online.._stime");
										//DebugN("========================================");										
										//DebugN("****************************************************");																					

										iRes = dpDisconnect("cietProfileGraph_animationCB", 
														sDpName + ".statusInformation.selectedManager:_lock._original._locked",
														sDpName + ".statusInformation.selectedManager",
													 	sDpName + ".ST_GTWDEV",
								  					sDpName + ".ProcessInput.Data.digitalInput:_online.._invalid",
								  					sDpName + ".ProcessInput.Data.sensorStatus",
													 	sDpe,
														sDpe + ":_online.._stime");
								}
								else 																															// DI Disconnected without DPE
								{
										//DebugN("****************************************************");											
										//DebugN("DI Disconnected without DPE");
										//DebugN("========================================");																				
										//DebugN("Selected Manager Locked = " + sDpName + ".statusInformation.selectedManager:_lock._original._locked");
										//DebugN("Selected Manager = "+sDpName + ".statusInformation.selectedManager");
										//DebugN("ST_GTWDEV = " + sDpName + ".ST_GTWDEV");
										//DebugN("Invalid = " + sDpName + ".ProcessInput.Data.digitalInput:_online.._invalid");
										//DebugN("sDpe = "+ sDpe);
										//DebugN("Time = "+	sDpe + ":_online.._stime");
										//DebugN("========================================");										
										//DebugN("****************************************************");																					

										iRes = dpDisconnect("cietProfileGraph_animationCB", 
														sDpName + ".statusInformation.selectedManager:_lock._original._locked",
														sDpName + ".statusInformation.selectedManager",
														sDpName + ".ST_GTWDEV",
								  					sDpName + ".ProcessInput.Data.digitalInput:_online.._invalid",
								  					sDpName + ".ProcessInput.Data.sensorStatus",
														sDpe + ":_online.._stime");
								}
						}
						break;


		// - - - - - - - 							
							
		default:
//DebugN("case default");
			unProfileGraph_setDisconnectionDp(sDeviceName); // set data
			break;
	}
	iResult = iRes;
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietProfileGraph_animationCB
/**
Purpose: Set the color, the select state and the value of a bar

Parameters:
	- bLocked, bool, input, the state of the lock: true=locked/selected, false=notLocked/deselected
	- sDpManager, string, input, the Dpe of the manager that took the lock
	- sSelectedManager, string, input, contains the manager that locked the device
	- iST_DQAMX, int, input, the state of the DQAMx 
	- bOkInvalid, bool, input, the invalid value 
	- bPwrPerm, bool, input, the ST_PWR_PERM value 
	- bStLogging, bool, input, the ST_LOGGING value 
	- bStFip, bool, input, the ST_FIP value 
	- sDpeValue, string, input, the Dpe
	- fValue, float, input, the value in float format: boolean are converted to float, true=1.0, false=0.0
	- tTime, time, input, the time of the last modification of the value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 3.0 
	. operating system: WXP.
	. distributed system: yes.
*/
cietProfileGraph_animationCB( string sDpLocked, bool bLocked, 
															string sDpSelectedManager, string sSelectedManager,
															string sDpST_GTWDEV, bool bST_GTWDEV, 
															string sDpOkInvalid, bool bOkInvalid, 
															string sDpSensorStatus, unsigned uSensorStatus, 
															string sDpeValue, float fValue, 
															string sDpTime, time tTime) 

{

	int iRes;
	string sDpe = dpSubStr(sDpeValue, DPSUB_SYS_DP_EL), sDeviceName;
	int iIndex = g_mDpToIndex[sDpe];
	string sColor, s;
	bool bFlashing=false;

//DebugN("cietProfileGraph_animationCB");
	
// keep the value and time
	g_dsDeviceValue[iIndex] = fValue;
	g_dtDeviceTimeValue[iIndex] = tTime;
	sDeviceName = g_dsDeviceName[iIndex];
	
// Error Control

	if(bOkInvalid)										// Invalid Value
		{
			sColor = "unDataNotValid";
		  //DebugN("Invalid Value - sColor=" + sColor);
		}
	else if(!bST_GTWDEV)  							// Status Gateway
		{
			sColor = "unEventList_Warning";
		  //DebugN("Status Gateway - sColor=" + sColor);
		}
	else if(getBit(uSensorStatus,0))	// I/O Error
		{
			sColor = "unEventList_AlarmActive";
	  	//DebugN("I/O Error - sColor=" + sColor);
	  }
	else if((dpTypeName(sDpeValue)=="EH") && getBit(uSensorStatus,3))		// Forced Set Point - only for EH //
		{
			sColor = "unEventList_ForcedMode";		
		 	//DebugN("Forced Set Point - sColor=" + sColor);
		  //DebugN("sDpST_GTWDEV=" + sDpST_GTWDEV);
		}	
	else														  // No error
		{
			sColor = "unDisplayValue_Status";
	    //DebugN("No error - sColor=" + sColor);
	  }

// set the DeviceValue and the color state of the bar.
	bFlashing = sColor == "unWidget_AlarmNotAck" ? true:false;
	iRes = this.SetDeviceValue(sDeviceName, fValue, true, OLEColor(sColor), bFlashing);

// Select if needed
	if(sDpSelectedManager != UN_PROFILE_GRAPH_NO_VALUE)
	{
		unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, sColor, bFlashing);
		if(sColor == "unWidget_Select")
			this.SelectDevice(sDeviceName, false);
		else
			this.UnselectDevice(sDeviceName);
	}
}

//------------------------------------------------------------------------------------------------------------------------

//@}