/**@name LIBRARY: cietSystemIntegrity_CRYOGTW.ctl

@author: Herve Milcent (AB-CO)

Creation Date: 11/04/2005

Modification History: 
	05/03/2007: Herve
		- add GQ

	25/07/2005: Herve
		- in function cietSystemIntegrity_CRYOGTWTimeCallback: check if the timestamp of the counter is not too far in the past
		
version 1.0

External Functions: 
	
Internal Functions: 
	
Purpose: 
	Library of the CRYOGTW component used by the systemIntegrity.
	 	
Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. global variable: the following variables are global to the script
	. data point type needed: _UnSystemIntegrity
	. data point: CRYOGTW_systemIntegrity
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/

const int C_CRYOGTW_WAIT_BEFORE_GQ = 60;
global int g_cietSystemIntegrity_CRYOGTWWaitBeforeGQ;
global bool g_bCRYOGTWCNotStarted=true;
mapping g_cietSystemIntegrity_CRYOGTW_TimeAlarm;

//@{

//------------------------------------------------------------------------------------------------------------------------

// cietSystemIntegrity_CRYOGTW_Initialize
/**
Purpose:
Get the list of CRYOGTW check by the systemIntegrity. The ones that are enabled.

@param dsResult: dyn_string, output, the list of enabled CRYOGTW that are checked

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
cietSystemIntegrity_CRYOGTW_Initialize(dyn_string &dsResult)
{
	dyn_string dsList;
	int len, i;
	string dpToCheck;
	bool enabled;

	dpGet(UN_SYSTEM_INTEGRITY_CRYOGTW+UN_SYSTEMINTEGRITY_EXTENSION+".config.data", dsList);
	cietSystemIntegrity_CRYOGTW_DataCallback("", dsList);

	dsList = dpNames(c_unSystemAlarm_dpPattern+DS_pattern+"*", c_unSystemAlarm_dpType);
	len = dynlen(dsList);
	for(i = 1; i<=len; i++) {
		dpToCheck = dpSubStr(dsList[i], DPSUB_DP);
		dpToCheck = substr(dpToCheck, strlen(c_unSystemAlarm_dpPattern+DS_pattern),strlen(dpToCheck));
		if(dpExists(dpToCheck)){
			if(dpTypeName(dpToCheck) == CRYO_CRYOGTW_DPTYPE) {
				dpGet(dsList[i]+".enabled", enabled);
				if(enabled)
					dynAppend(dsResult, dpToCheck);
				unSystemIntegrity_CMW_InitializeGQThread(dpToCheck);
			}
		}	
	}
//DebugN("CRYOGTW_DS:", dsList, dsResult);
}

//------------------------------------------------------------------------------------------------------------------------

// cietSystemIntegrity_CRYOGTW_DataCallback
/**
Purpose:
handle any systemIntegrity command.

@param sDpe1: string, input, dpe
@param sConfigData: string, input, the config data

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
cietSystemIntegrity_CRYOGTW_DataCallback(string sDpe1, dyn_string dsConfigData)
{
	dyn_string dsTemp=dsConfigData;

	while(dynlen(dsTemp) <=2)
		dynAppend(dsTemp, "0");

	g_cietSystemIntegrity_CRYOGTWCheckingDelay = dsTemp[1];
	if(g_cietSystemIntegrity_CRYOGTWCheckingDelay <= 0)
		g_cietSystemIntegrity_CRYOGTWCheckingDelay = c_cietSystemIntegrity_defaultCRYOGTWCheckingDelay;

	g_cietSystemIntegrity_CRYOGTWWaitBeforeGQ = dsTemp[2];
	if(g_cietSystemIntegrity_CRYOGTWWaitBeforeGQ <= C_CRYOGTW_WAIT_BEFORE_GQ)
		g_cietSystemIntegrity_CRYOGTWWaitBeforeGQ = C_CRYOGTW_WAIT_BEFORE_GQ;
		
//DebugN(dsConfigData, g_cietSystemIntegrity_CRYOGTWCheckingDelay);
}

//------------------------------------------------------------------------------------------------------------------------

// cietSystemIntegrity_CRYOGTW_HandleCommand
/**
Purpose:
handle any systemIntegrity command.

@param sDpe1: string, input, dpe
@param command: int, input, the systemIntegrity command
@param sDpe2: string, input, dpe
@param parameters: dyn_string, input, the list of parameters of the systemIntegrity command

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
cietSystemIntegrity_CRYOGTW_HandleCommand(string sDpe1, int command, string sDpe2, dyn_string parameters)
{
	dyn_string exceptionInfo;
	int i, len =dynlen(parameters);
  string sMessage;
	
	switch(command) {
		case UN_SYSTEMINTEGRITY_ADD: // add the _UnSystemAlarm DP
			for(i=1; i<=len; i++) {
				cietSystemIntegrity_CRYOGTW_checking(parameters[i], true, true, exceptionInfo);
			}
			break;
		case UN_SYSTEMINTEGRITY_DELETE: // delete the _UnSystemAlarm DP
			for(i=1; i<=len; i++) {
				cietSystemIntegrity_CRYOGTW_checking(parameters[i], false, false, exceptionInfo);
				// remove dp from the alert list of applicationDP if it is in
				_unSystemIntegrity_modifyApplicationAlertList(c_unSystemAlarm_dpPattern+DS_pattern+parameters[i], false, exceptionInfo);
				// delete the _UnSystemAlarm dps
				if(dpExists(c_unSystemAlarm_dpPattern+DS_pattern+parameters[i]))
					dpDelete(c_unSystemAlarm_dpPattern+DS_pattern+parameters[i]);

				// remove dp from the alert list of applicationDP if it is in
				_unSystemIntegrity_modifyApplicationAlertList(c_unSystemAlarm_dpPattern+DS_Time_pattern+parameters[i], false, exceptionInfo);
				// delete the _UnSystemAlarm dps
				if(dpExists(c_unSystemAlarm_dpPattern+DS_Time_pattern+parameters[i]))
					dpDelete(c_unSystemAlarm_dpPattern+DS_Time_pattern+parameters[i]);
			}
			break;
		case UN_SYSTEMINTEGRITY_ENABLE: // enable the _UnSystemAlarm DP
			for(i=1; i<=len; i++) {
				if(dpExists(c_unSystemAlarm_dpPattern+DS_pattern+parameters[i]))
					cietSystemIntegrity_CRYOGTW_checking(parameters[i], false, true, exceptionInfo);
			}
			break;
		case UN_SYSTEMINTEGRITY_DISABLE: // disable the _UnSystemAlarm DP
			for(i=1; i<=len; i++) {
				if(dpExists(c_unSystemAlarm_dpPattern+DS_pattern+parameters[i]))
					cietSystemIntegrity_CRYOGTW_checking(parameters[i], false, false, exceptionInfo);
			}
			break;
		case UN_SYSTEMINTEGRITY_DIAGNOSTIC: // give the list of _UnSystemAlarm DP and the state
			dpSet(UN_SYSTEM_INTEGRITY_CRYOGTW+UN_SYSTEMINTEGRITY_EXTENSION+".diagnostic.commandResult", command, 
					UN_SYSTEM_INTEGRITY_CRYOGTW+UN_SYSTEMINTEGRITY_EXTENSION+".diagnostic.result", g_cietSystemIntegrity_CRYOGTW_DSList);
			break;
   case UN_SYSTEMINTEGRITY_VERSION:
    sMessage = unGenericDpFunctions_getComponentsVersion(makeDynString("cietDevices"));
    dpSet(UN_SYSTEM_INTEGRITY_CRYOGTW+UN_SYSTEMINTEGRITY_EXTENSION+".diagnostic.commandResult", command,
    UN_SYSTEM_INTEGRITY_CRYOGTW+UN_SYSTEMINTEGRITY_EXTENSION+".diagnostic.result", makeDynString(getCurrentTime(),sMessage));
   break;
		default:
			break;
	}

	if(dynlen(exceptionInfo) > 0) {
		if(isFunctionDefined("unMessageText_sendException")) {
			unMessageText_sendException("*", "*", "cietSystemIntegrity_CRYOGTW_HandleCommand", "user", "*", exceptionInfo);
		}
// handle any error uin case the send message failed
		if(dynlen(exceptionInfo) > 0) {
			DebugN(getCurrentTime(), exceptionInfo);
		}
	}

// do here the dpConnect to the list of drivers
	if(g_bCRYOGTWCNotStarted) {
		dpConnect("cietSystemIntegrity_CRYOGTW_ConnectionDriverCB", CONNECTION_DRIVER);
		g_bCRYOGTWCNotStarted = false;
	}

//DebugN(sDpe1, command, parameters, g_cietSystemIntegrity_CRYOGTW_DSList);
}

//------------------------------------------------------------------------------------------------------------------------

// cietSystemIntegrity_CRYOGTW_ConnectionDriverCB
/**
Purpose:
callback on the list of connected driver.

@param sDpe1: string, input, dpe
@param diDriverNum: dyn_int, input, the list of driver number started

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
cietSystemIntegrity_CRYOGTW_ConnectionDriverCB(string sDp, dyn_int diDriverNum)
{
	dyn_string dsList = g_cietSystemIntegrity_CRYOGTW_DSList;
	string sDpName;
	int i, len;

	len = dynlen(dsList);
//DebugTN("cietSystemIntegrity_CRYOGTW_ConnectionDriverCB GQ", diDriverNum, dsList);
	for(i=1;i<=len;i++) {
		sDpName = substr(dsList[i], strlen(c_unSystemAlarm_dpPattern+DS_pattern), strlen(dsList[i]));
//DebugTN("cietSystemIntegrity_CRYOGTW_ConnectionDriverCB GQ", sDpName, drvNbr);
		unSystemIntegrity_CMW_ConnectionDriver(sDpName, diDriverNum);
	}
}

//------------------------------------------------------------------------------------------------------------------------

// cietSystemIntegrity_CRYOGTW_checking
/**
Purpose:
This function register/de-register the callback funtion of CRYOGTW manager. This function can also create the _unSystemAlarm_CRYOGTW dp 
with the alarm config but it cannot delete it.

	@param sDp: string, input, data point name
	@param bCreate: bool, input, true create the dp and the alarm config if it does not exist, enable it
	@param bRegister: bool, input, true do a dpConnect, false do a dpDisconnect
	@param exceptionInfo: dyn_string, output, exception are returned here

Usage: Internal

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/

cietSystemIntegrity_CRYOGTW_checking(string sDp, bool bCreate, bool bRegister, dyn_string &exceptionInfo)
{
	string sDpToChek, dp, dpTime, dpError, dpPlc, plcHostname, description;
	int res, thId, pos, drvNum, len,i;
	bool bError = false;
	dyn_string plc_plc_list;

	string alertPanel,alertHelp;
	bool configExists, isActive;
	int alertConfigType;
	dyn_string alertTexts,alertClasses,summaryDpeList,alertPanelParameters;
	dyn_float alertLimits;
	
// remove the system name
	res = strpos(sDp, ":");
	if(res >= 0)
		dp = substr(sDp, res+1, strlen(sDp));
	else
		dp = sDp;
	sDpToChek = dp;
	dpTime = c_unSystemAlarm_dpPattern+DS_Time_pattern+dp;
	dp = c_unSystemAlarm_dpPattern+DS_pattern+dp;
//DebugN("cietSystemIntegrity_CRYOGTW_checking", dp, bCreate, bRegister, sDpToChek, dpTime);

// in case the Dp is not of type _UnPlc do nothing. Should not happen, just for security.
	if(dpTypeName(sDpToChek) != CRYO_CRYOGTW_DPTYPE)
		return;
		
	dpGet(sDpToChek+".communication.driver_num", drvNum);

	if(bCreate) {
// get the plc hostname and the driver number
		plcHostname = unGenericDpFunctions_getAlias(sDpToChek);
//		dpGet( sDpToChek+".communication.counter:_distrib.._driver", drvNum);
// create the sDp and its alarm config if it is not existing
		if(!dpExists(dp)) {
			description = getCatStr("unSystemIntegrity", "PLC_DS_DESCRIPTION")+plcHostname+" -> DS driver "+ drvNum;
			unSystemIntegrity_createSystemAlarm(sDpToChek, DS_pattern, description, exceptionInfo);
		}
// create the sDp time and its alarm config if it is not existing
		if(!dpExists(dpTime)) {
			description = getCatStr("unSystemIntegrity", "PLC_DS_TIME_DESCRIPTION")+plcHostname+" -> DS driver "+ drvNum;
			unSystemIntegrity_createSystemAlarm(sDpToChek, DS_Time_pattern, description, exceptionInfo);
		}
	}

	if(bRegister) {
		unSystemIntegrity_CMW_InitializeGQThread(sDpToChek);
// connect to the callback function if not already done
		pos = _unSystemIntegrity_isInList(g_cietSystemIntegrity_CRYOGTW_DSList, dp);
		if(pos <= 0) {
	// add it in the list, because one of the callback function needs it.				
			pos = _unSystemIntegrity_setList(g_cietSystemIntegrity_CRYOGTW_DSList, dp, true);

// start the thread for checking the counter
			thId = startThread(UN_SYSTEM_INTEGRITY_CRYOGTW_check, sDpToChek, drvNum);
			g_cietSystemIntegrity_CRYOGTW_DSThreadId[pos] = thId;
			if(thId<0)
				bError = true;

			if(!bError) {
				g_cietSystemIntegrity_CRYOGTW_TimeAlarm[dpSubStr(sDpToChek,DPSUB_SYS_DP)] = -1;
	// dpConnect to the stime of the counter
				res = dpConnect(c_cietSystemIntegrity_CRYOGTW_Time_callback, 
														sDpToChek+".communication.counter:_online.._stime_inv",
														sDpToChek+".communication.counter:_online.._stime");
				if(res<0)
					bError = true;
			}
			if(bError) {

				pos = _unSystemIntegrity_isInList(g_cietSystemIntegrity_CRYOGTW_DSList, dp);
				if(pos > 0) {
		// kill the threads if they were started
					res = stopThread(g_cietSystemIntegrity_CRYOGTW_DSThreadId[pos]);
// and then remove it from the list
					pos = _unSystemIntegrity_setList(g_cietSystemIntegrity_CRYOGTW_DSList, dp, false);
//DebugN("rmv", res);
					dynRemove(g_cietSystemIntegrity_CRYOGTW_DSThreadId, pos);
				}

				fwException_raise(exceptionInfo, "ERROR", 
							"cietSystemIntegrity_CRYOGTW_checking():"+getCatStr("unSystemIntegrity", "CANNOT_DP_CONNECT") +dp,"");
			}
			else {
//DebugN("Enable", g_cietSystemIntegrity_CRYOGTW_DSThreadId, "end");
			// set the enable to true and activate the alarm if it is not activated.
				dpSet(dp+".enabled", true);
				unAlarmConfig_mask(dp+".alarm", false, exceptionInfo);
				dpSet(dpTime+".enabled", true);
				unAlarmConfig_mask(dpTime+".alarm", false, exceptionInfo);
			// get all the PLC-PLC dp, and enable them
				plc_plc_list = dpNames(c_unSystemAlarm_dpPattern+PLC_PLC_pattern+sDpToChek+"*", c_unSystemAlarm_dpType);
				len = dynlen(plc_plc_list);
				for(i=1;i<=len;i++) {
					dpSet(plc_plc_list[i]+".enabled", true);
					unAlarmConfig_mask(plc_plc_list[i]+".alarm", false, exceptionInfo);
				}

			// get all the FESystemAlarm dp, and enable them, check the Ok state and set the correct value
				plc_plc_list = dpNames(c_unSystemAlarm_dpPattern+FE_pattern+sDpToChek+"*", c_unSystemAlarm_dpType);
				len = dynlen(plc_plc_list);
				for(i=1;i<=len;i++) {
					dpSet(plc_plc_list[i]+".enabled", true);
					unAlarmConfig_mask(plc_plc_list[i]+".alarm", false, exceptionInfo);
				}
			}
		}
	}
	else {
// disconnect the callback function
//DebugN("Disable", g_cietSystemIntegrity_CRYOGTW_DSThreadId, "end");
		pos = _unSystemIntegrity_isInList(g_cietSystemIntegrity_CRYOGTW_DSList, dp);
		if(pos > 0) {
// kill the thread
			res = stopThread(g_cietSystemIntegrity_CRYOGTW_DSThreadId[pos]);
			if(res<0)
				bError = true;

// dpDisconnect stime of the counter
			res = dpDisconnect(c_cietSystemIntegrity_CRYOGTW_Time_callback, 
																	sDpToChek+".communication.counter:_online.._stime_inv",
																	sDpToChek+".communication.counter:_online.._stime");
			if(res<0)
				bError = true;

			if(bError) {
				fwException_raise(exceptionInfo, "ERROR", 
							"cietSystemIntegrity_CRYOGTW_checking():"+getCatStr("unSystemIntegrity", "CANNOT_DP_DISCONNECT") +dp,"");
			}
			else {
			// remove from list
				pos = _unSystemIntegrity_setList(g_cietSystemIntegrity_CRYOGTW_DSList, dp, false);
//DebugN("rmv", res);
				dynRemove(g_cietSystemIntegrity_CRYOGTW_DSThreadId, pos);
			// set the enable to false and de-activate the alarm and acknowledge it if necessary
				dpSet(dp+".enabled", false, dp+".alarm", c_unSystemIntegrity_no_alarm_value);
				unAlarmConfig_mask(dp+".alarm", true, exceptionInfo);
				dpSet(dpTime+".enabled", false, dpTime+".alarm", c_unSystemIntegrity_no_alarm_value);
				unAlarmConfig_mask(dpTime+".alarm", true, exceptionInfo);
			// get all the PLC-PLC dp, and disable them
				plc_plc_list = dpNames(c_unSystemAlarm_dpPattern+PLC_PLC_pattern+sDpToChek+"*", c_unSystemAlarm_dpType);
				len = dynlen(plc_plc_list);
				for(i=1;i<=len;i++) {
// do not reset the alarm value because this DPE should be connected to an address config
					dpSet(plc_plc_list[i]+".enabled", false);
					unAlarmConfig_mask(plc_plc_list[i]+".alarm", true, exceptionInfo);
				}

			// get all the FESystemAlarm dp, and disable them, check the Ok state and set the correct value
				plc_plc_list = dpNames(c_unSystemAlarm_dpPattern+FE_pattern+sDpToChek+"*", c_unSystemAlarm_dpType);
				len = dynlen(plc_plc_list);
				for(i=1;i<=len;i++) {
// do not reset the alarm value because this DPE should be connected to an address config
					dpSet(plc_plc_list[i]+".enabled", false);
					unAlarmConfig_mask(plc_plc_list[i]+".alarm", true, exceptionInfo);
				}
			}
		}
	}
}

//------------------------------------------------------------------------------------------------------------------------

// cietSystemIntegrity_CRYOGTWCheck
/**
Purpose:
This function checks if the counter of the CRYOGTW dp is modified periodically. If not it generates and alarm.

	@param sCRYOGTWDp: string, input, data point name

Usage: Internal

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
cietSystemIntegrity_CRYOGTWCheck(string sCRYOGTWDp, int iDrvNum)
{
	int waitingTime, oldAlarmValue = -1;
	int oldValue = -1, newValue, alarmValue;
	bool bEnabled;
	string sModPlc;
	int res, iPlcNumber;
	
	while(true) {
	// if 0 set it to g_cietSystemIntegrity_CRYOGTWCheckingDelay
		waitingTime = g_cietSystemIntegrity_CRYOGTWCheckingDelay;
		if(waitingTime <= 0)
			waitingTime = c_cietSystemIntegrity_defaultCRYOGTWCheckingDelay;
	// wait
		delay(waitingTime);
	
	// get the value of the counter and the timeout PLCChecking
		dpGet(sCRYOGTWDp+".communication.counter", newValue);
		if(newValue == oldValue) {
		// counter was not modified, set an alarm
			alarmValue = c_unSystemIntegrity_alarm_value_level1;
		}
		else	
		// counter was modified, reset the alarm
			alarmValue = c_unSystemIntegrity_no_alarm_value;

		if(oldAlarmValue != alarmValue) {
			dpSet(c_unSystemAlarm_dpPattern+DS_pattern+sCRYOGTWDp+".alarm", alarmValue);
			unSystemIntegrity_CMW_checkState(alarmValue, iDrvNum, g_cietSystemIntegrity_CRYOGTWWaitBeforeGQ);
		}
		unSystemIntegrity_CMW_checkDriver(iDrvNum, g_cietSystemIntegrity_CRYOGTWWaitBeforeGQ);

//DebugN("Check counter", sCRYOGTWDp, oldValue, newValue, alarmValue);	
		oldValue = newValue;
		oldAlarmValue = alarmValue;
	}
}

//------------------------------------------------------------------------------------------------------------------------

// cietSystemIntegrity_CRYOGTWTimeCallback
/**
Purpose:
Callback function to check if the time is correctly set in the PLC.

	@param sDp1: string, input, data point name
	@param bStimeInvalid: bool, input, state of the stime bit of the counter
	@param sDp2: string, input, data point name
	@param iTime: int, input, the timestamp of the counter

Usage: Internal

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
cietSystemIntegrity_CRYOGTWTimeCallback(string sDp1, bool bStimeInvalid, string sDpe2, int iTime)
{
	string sDp,	sKey = dpSubStr(sDp1, DPSUB_SYS_DP);
	int value, oldValue = -1;
	int iCurrentTime;
	bool bHasKey;
	
//DebugN("stime", sDp1, bStimeInvalid);
	
	if(mappingHasKey(g_cietSystemIntegrity_CRYOGTW_TimeAlarm, sKey))
		{
			oldValue= g_cietSystemIntegrity_CRYOGTW_TimeAlarm[sKey];
			bHasKey = true;
		}


// get the dpname without the systemname
	sDp = dpSubStr(sDp1, DPSUB_DP);
// set the alarm 
	if(bStimeInvalid) { // bad time synchro
		value = c_unSystemIntegrity_alarm_value_level1;
	}
	else { // good time synchro
	// check if too late in the past
		iCurrentTime = getCurrentTime();
		if(iTime <= iCurrentTime) {
//DebugN(iTime, iCurrentTime);
			if((iCurrentTime - iTime) > CONST_CHANGE_DIFF) { // too far in the past
//DebugN("alarm too late");
				value = c_unSystemIntegrity_alarm_value_level1;
			}
			else
				value = c_unSystemIntegrity_no_alarm_value;
		}
		else
			value = c_unSystemIntegrity_no_alarm_value;
	}
	if(oldValue != value)
		dpSet(c_unSystemAlarm_dpPattern+DS_Time_pattern+sDp+".alarm", value);
		
	oldValue = value;
	
	if(bHasKey)
		g_cietSystemIntegrity_CRYOGTW_TimeAlarm[sKey] = value;
}

//------------------------------------------------------------------------------------------------------------------------

//@}

