/**@name LIBRARY: cietWFIPAgent.ctl

@author: Celine VUILLERMOZ (AB/CO)

Creation Date: 11-08-2005

Modification History: 
 
 04/11/11  achiron: call only cietGenericFunctions_HeaderDisconnect(); in the ciet-DPT-_FaceplateStatusDisconnection()
 02/11/11  Adrien: Corrected the call to the function ciet-DTD-_WidgetDisconnect iRes was argument instead of a PlcName
 29/09/08        Quique: Faceplate connect (take out the prefix "un-" and replace by "*")
	15/05/08	Pablo:	Significant, refreshment and promptenss box to orange color.
 	09/05/08 	Pablo: 	Button UserAccess adapted to QT.
 	13/12/08 	Pablo: 	Included FIP Presence box in faceplate.
								 	 	Included Reset box in faceplate.
 	05/11/07 	Pablo: 	Open wfipsegment and crate from popup menu.
 										debug application included. 	
 	23/08/07 	Pablo: 	Help on handle menu.
 						Disable some option on handle menu when widget is not selected.
	22/08/07	Pablo:	Added Sensor Status animation on Table Devices Connected.
 	20/08/07 	Pablo: 	New widget Led.
 	20/06/07 	Pablo: 	Disable widget animation.
	18/06/07	Pablo:	Adapted to distributed system.
	18/12/06	Pablo:	Reordered Warnings on Widget Animation.
	17/11/06	Quique: New functions: cietWFIPEvenAgent_ObjectListGetValueTime, cietWFIPOddAgent_ObjectListGetValueTime
	24/10/06	Pablo:	Modified ButtonSetState & ButtonAnimationCB to allow click on UpdateRawData Button only when there is no updating.
	09/10/06	Quique	Modified widgetDisconnection function to visualize rigth behaviour
	04/10/06	Quique	Modified the way the rawdata is visualize (Bo -> B7)
	26/09/06	Pablo   Added ST_GTWDEV indicator on widget animation.
	09/09/06	Quique  change reset popup call
	06/09/06	Quique	Added in the faceplate the name of the segment
	16/08/06	Quique:	Include WFip Config button allowance
	
version 1.0

External Function : 
	. cietWFIPAgent_FaceplateStatusDisconnection : animate faceplate disconnection
	. cietWFIPAgent_ButtonUserAccess : User access for Analog buttons
	. cietWFIPAgent_ButtonSetState : set button state
	. cietWFIPAgent_WidgetDisconnection : animate widget disconnection
	. cietWFIPAgent_MenuConfiguration : configuration of popup menu

Internal Functions :
	. cietWFIPAgent_FaceplateStatusRegisterCB : register callback function
	. cietWFIPAgent_ButtonRegisterCB : register callback function
	. cietWFIPAgent_ButtonAnimationCB : animate buttons
	. cietWFIPAgent_ButtonDisconnection : animate buttons disconnection
	. cietWFIPAgent_WidgetRegisterCB : register callback function
	. cietWFIPAgent_WidgetAnimationCB : animate widget
		
Purpose: This library contains Analog object functions.
 
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. cietWFIPAgent object (DPType etc.)
	. Global variables and $parameters defined in Analog faceplate and widget
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

// constant declaration

//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPEvenAgent_ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 3.1
	. operating system: WinXP
	. distributed system: yes.
*/
cietWFIPEvenAgent_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
	float value;
	string sTime;
	  
	dpGet(sDeviceName+".ProcessInput.Data.agentStatus", value, sDeviceName+".ProcessInput.Data.agentStatus:_online.._stime", sTime);
	dsReturnData[1] = sTime;
	dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(sDeviceName+".ProcessInput.Data.agentStatus"), value) + " " 
							+ dpGetUnit(sDeviceName+".ProcessInput.Data.agentStatus");
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPOddAgent_ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 3.1
	. operating system: WinXP
	. distributed system: yes.
*/
cietWFIPOddAgent_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
	float value;
	string sTime;
	  
	dpGet(sDeviceName+".ProcessInput.Data.agentStatus", value, sDeviceName+".ProcessInput.Data.agentStatus:_online.._stime", sTime);
	dsReturnData[1] = sTime;
	dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(sDeviceName+".ProcessInput.Data.agentStatus"), value) + " " 
							+ dpGetUnit(sDeviceName+".ProcessInput.Data.agentStatus");
}

//---------------------------------------------------------------------------------------------------------------------------------------
//@{


// cietWFIPAgent_FaceplateStatusRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in cietWFIPAgent faceplate
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietWFIPAgent_FaceplateStatusRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, sSystemName, sPlcName;
	int iAction, iRes, iAlertType1;
	dyn_string exceptionInfo;
	bool bRemote;
	
	deviceName = unGenericDpFunctions_getDpName($sDpName);
// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	if(sPlcName != "") {
		if (dpExists(sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;
		unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;																			 
	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	
//	DebugN("FaceplateStatusRegisterCB", deviceName,iAction);
	
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
//DebugN("cietWFIPAgent_FaceplateStatusDisconnection();");
			cietWFIPAgent_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
		
			g_bDisconnectCB = false;
			if(g_bUnSystemAlarmPlc) {
				iRes = dpConnect("cietWFIPAgent_FaceplateLockCallBack", deviceName + ".ProcessInput.Data.agentStatus:_lock._alert_hdl._locked");

				g_bCallbackConnected = (iRes >= 0);
			}
			else
				cietWFIPAgent_FaceplateStatusDisconnection();
			
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
		
			if(g_bUnSystemAlarmPlc) 
				iRes = dpDisconnect("cietWFIPAgent_FaceplateLockCallBack", deviceName + ".ProcessInput.Data.agentStatus:_lock._alert_hdl._locked");

			if(g_bDisconnectCB)
				cietWFIPAgent_FaceplateDisconnect(deviceName, sPlcName);

			g_bCallbackConnected = !(iRes >= 0);
			g_bDisconnectCB = false;
				cietWFIPAgent_FaceplateStatusDisconnection();
			
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
	}
//---------------------------------------------------------------------------------------------------------------------------------------
// cietWFIPAgent_FaceplateLockCallBack
/**
Purpose: set up the lock callback function for the faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in WFIPEvenAgentwidget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietWFIPAgent_FaceplateLockCallBack(string sdpe, bool bLockState)
{
	int iRes;
	string deviceName;
 string sPlcName;

	deviceName = unGenericDpFunctions_getDpName(sdpe);
	if(bLockState) { 
		if(g_bDisconnectCB) {
			cietWFIPAgent_FaceplateDisconnect(deviceName, sPlcName);
			g_bDisconnectCB = false;
		}
		cietWFIPAgent_FaceplateStatusDisconnection();
	}
	else { 
		g_bDisconnectCB = true;
		cietWFIPAgent_FaceplateConnect(deviceName);
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPAgent_FaceplateConnect
/**
Purpose: connect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in cietWFIPAgent faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietWFIPAgent_FaceplateConnect(string deviceName)
{

	int iAction, iRes, iAlertType1, iAlertType2, iRangeType;
	dyn_string exceptionInfo, dsDeviceNameSplit,dsDpes,dsAliases,dsAlias,dsTypes,dsFIPInfos,dsChannels;
	string sPlcName, sSystemName, sAlias, sDeviceType;
	string sDpeName,sType;
	dyn_string s_tempDsDpes, s_tempDsAliases;
	
	//DebugN("FUNCTION: cietWFIPAgent_FaceplateConnect");
		
	// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	sDeviceType = dpTypeName(deviceName);
	sAlias = /*sSystemName +*/ unGenericDpFunctions_getAlias(deviceName);
	
	// system alarm
	if(sPlcName != "") 
		{
			if (dpExists(sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName)) 
					g_bUnSystemAlarmPlc = true;
			else 
				g_bUnSystemAlarmPlc = false;
		}
	else
		g_bUnSystemAlarmPlc = false;
		
		
//DebugN("deviceName = " + deviceName);		
	// get Fip infos	
	cietGenericFunctions_getFIPAgentInfos(deviceName, dsFIPInfos, exceptionInfo);		
	if (dynlen(exceptionInfo) <= 0)	
		{	
			g_sGTWName = dsFIPInfos[ciet_PARAMETER_GTWNAME];
			g_sAgentBus = dsFIPInfos[ciet_PARAMETER_BUS]; 
			g_sAgentNumber = dsFIPInfos[ciet_PARAMETER_AGENTNUMBER];
			

//                         DebugN("=========================================");
//                         DebugN("g_sGTWName= "+g_sGTWName);
//                         DebugN("g_sAgentBus= "+g_sAgentBus);
//                         DebugN("g_sAgentNumber= "+g_sAgentNumber);
//                         DebugN(" ----------------     -------------------");
//                         DebugN("sPlcName ="+sPlcName);
//                         DebugN("sSystemName ="+sSystemName);
//                         DebugN("sDeviceType ="+sDeviceType);
//                         DebugN("sAlias ="+sAlias);
//                         DebugN("=========================================");
		 	
		
			//get all  devices with same gateway name (virtual gateway= WFip segment)	
						
			dsDpes=dpNames (sSystemName+"*"+g_sGTWName+"*"+"DI"+"*");
			dynAppend(dsDpes,dpNames (sSystemName+"*"+g_sGTWName+"*"+"EH"+"*"));
			dynAppend(dsDpes,dpNames (sSystemName+"*"+g_sGTWName+"*"+"TT"+"*"));
			dynAppend(dsDpes,dpNames (sSystemName+"*"+g_sGTWName+"*"+"PT"+"*"));
			dynAppend(dsDpes,dpNames (sSystemName+"*"+g_sGTWName+"*"+"LT"+"*"));

			// DebugN("dsDpes = "+dsDpes);
			
			// Ini Table
			// for(int i=1;i<=14;i++)
			//	{
			//		dsChannels[i]=i-1;
			//		dsTypes[i]="";
			//		dsAlias[i]="";
			//	}	
			
			//DebugN("dynlen(dsDpes) = " + dynlen(dsDpes) );
			
			for(int i=1,j=0;i<=dynlen(dsDpes);i++)
				{
				
					sDpeName = unGenericDpFunctions_getDpName(dsDpes[i]);
				
					sType = dpTypeName(sDpeName);
					//DebugN("Found device: <Type:>" +sType+" <dpname:> "+sDpeName);
					
					// If agent is the same then take it into account
					
							//DebugN("--> Cadidate device: <Type:>" +sType+" <dpname:> "+sDpeName);
							if (cietGenericFunctions_getAgentNumber(sDpeName)==g_sAgentNumber)
							{
								//DebugN("----> Successfull device: <Type:>" +sType+" <dpname:> "+sDpeName);
								string sChannel=cietGenericFunctions_getChannelNumber(sDpeName);
								int iChannel=(int)sChannel;
								//DebugN("iChannel= "+iChannel);
								if (iChannel==14)
									{
										dsAlias[iChannel+1+j] = dpGetAlias(sDpeName+".");
										dsTypes[iChannel+1+j] = dpTypeName(sDpeName);
										dsChannels[iChannel+1+j] =iChannel;
										j++;
									}
								else
									{
										dsAlias[iChannel+1] = dpGetAlias(sDpeName+".");
										dsTypes[iChannel+1] = dpTypeName(sDpeName);
										dsChannels[iChannel+1] =iChannel;
									}
		
					 }	
				}	// for dynlen(dsDpes) 		
		// Insert in table
		for (int i=1;i<=dynlen(dsAlias);i++)
			{	
				if (dsChannels[i]!="")
					{
						//DebugN("dsChannels["+i+"]= "+dsChannels[i]);
						Table1.appendLine("Ch",dsChannels[i],"Name", dsAlias[i],"Type", dsTypes[i]);
						// Table1.appendLines(dynlen(dsAlias), "Ch",dsChannels,"Name", dsAlias,"Type", dsTypes);
					}
			}	
			
			
		if(g_bUnSystemAlarmPlc) 
			{
				if (sDeviceType == "WFIPEvenAgent")
						{
						startThread("cietWFIPSegment_animateTableStatusSensor", sSystemName,sPlcName);

						iRes = dpConnect("cietWFIPEvenAgent_FaceplateStatusAnimationCB",
													 		deviceName + ".ProcessInput.Config.sgNumber", 		
									 						deviceName + ".ProcessInput.Config.agNumber", 		
											 				deviceName + ".ProcessInput.Data.agentStatus", 
															deviceName + ".ProcessInput.Data.prodVariable1[1]", deviceName + ".ProcessInput.Data.prodVariable1[2]",
											 				deviceName + ".ProcessInput.Data.prodVariable1[3]", deviceName + ".ProcessInput.Data.prodVariable1[4]",
											 				deviceName + ".ProcessInput.Data.prodVariable1[5]", deviceName + ".ProcessInput.Data.prodVariable1[6]",
											 				deviceName + ".ProcessInput.Data.prodVariable1[7]", deviceName + ".ProcessInput.Data.prodVariable1[8]",
											 				deviceName + ".ProcessInput.Data.consVariable[1]", deviceName + ".ProcessInput.Data.consVariable[2]",
											 				deviceName + ".ProcessInput.Data.consVariable[3]", deviceName + ".ProcessInput.Data.consVariable[4]",
											 				deviceName + ".ProcessInput.Data.consVariable[5]", deviceName + ".ProcessInput.Data.consVariable[6]",
											 				deviceName + ".ProcessInput.Data.consVariable[7]", deviceName + ".ProcessInput.Data.consVariable[8]",
											 				deviceName + ".ProcessInput.Data.consVariable[9]", deviceName + ".ProcessInput.Data.consVariable[10]",
											 				deviceName + ".ProcessInput.Data.consVariable[11]", deviceName + ".ProcessInput.Data.consVariable[12]",
											 				deviceName + ".ProcessInput.Data.consVariable[13]", deviceName + ".ProcessInput.Data.consVariable[14]",
											 				deviceName + ".ProcessInput.Data.consVariable[15]", deviceName + ".ProcessInput.Data.consVariable[16]",
											 				deviceName + ".ProcessInput.Data.consVariable[17]", deviceName + ".ProcessInput.Data.consVariable[18]",
											 				deviceName + ".ProcessInput.Data.consVariable[19]", deviceName + ".ProcessInput.Data.consVariable[20]",
											 				deviceName + ".ProcessInput.Data.consVariable[21]", deviceName + ".ProcessInput.Data.consVariable[22]",
											 				deviceName + ".ProcessInput.Data.consVariable[23]", deviceName + ".ProcessInput.Data.consVariable[24]",
											 				deviceName + ".ProcessInput.Data.consVariable[25]", deviceName + ".ProcessInput.Data.consVariable[26]",
											 				deviceName + ".ProcessInput.Data.consVariable[27]", deviceName + ".ProcessInput.Data.consVariable[28]",
											 				deviceName + ".ProcessInput.Data.consVariable[29]", deviceName + ".ProcessInput.Data.consVariable[30]",
											 				deviceName + ".ProcessInput.Data.consVariable[31]", deviceName + ".ProcessInput.Data.consVariable[32]",
											 				deviceName + ".ProcessInput.Data.consVariable[33]", deviceName + ".ProcessInput.Data.consVariable[34]",
											 				deviceName + ".ProcessInput.Data.consVariable[35]", deviceName + ".ProcessInput.Data.consVariable[36]",
											 				deviceName + ".ProcessInput.Data.consVariable[37]", deviceName + ".ProcessInput.Data.consVariable[38]",
											 				deviceName + ".ProcessInput.Data.consVariable[39]", deviceName + ".ProcessInput.Data.consVariable[40]",
											 				deviceName + ".ProcessInput.Data.prodVariable3[1]", deviceName + ".ProcessInput.Data.prodVariable3[2]",
											 				deviceName + ".ProcessInput.Data.prodVariable3[3]", deviceName + ".ProcessInput.Data.prodVariable3[4]",
											 				deviceName + ".ProcessInput.Data.prodVariable3[5]", deviceName + ".ProcessInput.Data.prodVariable3[6]",
											 				deviceName + ".ProcessInput.Data.prodVariable3[7]", deviceName + ".ProcessInput.Data.prodVariable3[8]",
										 					sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".alarm",
															sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".enabled",
													/*	sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+sAlias+".alarm",
															sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+sAlias+".enabled",*/
															deviceName + ".ProcessInput.Data.agentStatus:_online.._invalid");						
						
						}
													
				else if (sDeviceType == "WFIPOddAgent")
						{
								startThread("cietWFIPSegment_animateTableStatusSensor", sSystemName,sPlcName);

								iRes = dpConnect("cietWFIPOddAgent_FaceplateStatusAnimationCB",
												 	deviceName + ".ProcessInput.Config.sgNumber", 		
								 					deviceName + ".ProcessInput.Config.agNumber", 		
									 				deviceName + ".ProcessInput.Data.agentStatus", 
								 					deviceName + ".ProcessInput.Data.consVariable[1]", deviceName + ".ProcessInput.Data.consVariable[2]",
									 				deviceName + ".ProcessInput.Data.consVariable[3]", deviceName + ".ProcessInput.Data.consVariable[4]",
									 				deviceName + ".ProcessInput.Data.consVariable[5]", deviceName + ".ProcessInput.Data.consVariable[6]",
									 				deviceName + ".ProcessInput.Data.consVariable[7]", deviceName + ".ProcessInput.Data.consVariable[8]",
									 				deviceName + ".ProcessInput.Data.consVariable[9]", deviceName + ".ProcessInput.Data.consVariable[10]",
									 				deviceName + ".ProcessInput.Data.consVariable[11]", deviceName + ".ProcessInput.Data.consVariable[12]",
									 				deviceName + ".ProcessInput.Data.consVariable[13]", deviceName + ".ProcessInput.Data.consVariable[14]",
									 				deviceName + ".ProcessInput.Data.consVariable[15]", deviceName + ".ProcessInput.Data.consVariable[16]",
									 				deviceName + ".ProcessInput.Data.consVariable[17]", deviceName + ".ProcessInput.Data.consVariable[18]",
									 				deviceName + ".ProcessInput.Data.consVariable[19]", deviceName + ".ProcessInput.Data.consVariable[20]",
									 				deviceName + ".ProcessInput.Data.consVariable[21]", deviceName + ".ProcessInput.Data.consVariable[22]",
									 				deviceName + ".ProcessInput.Data.consVariable[23]", deviceName + ".ProcessInput.Data.consVariable[24]",
									 				deviceName + ".ProcessInput.Data.consVariable[25]", deviceName + ".ProcessInput.Data.consVariable[26]",
									 				deviceName + ".ProcessInput.Data.consVariable[27]", deviceName + ".ProcessInput.Data.consVariable[28]",
									 				deviceName + ".ProcessInput.Data.consVariable[29]", deviceName + ".ProcessInput.Data.consVariable[30]",
									 				deviceName + ".ProcessInput.Data.consVariable[31]", deviceName + ".ProcessInput.Data.consVariable[32]",
									 				deviceName + ".ProcessInput.Data.consVariable[33]", deviceName + ".ProcessInput.Data.consVariable[34]",
									 				deviceName + ".ProcessInput.Data.consVariable[35]", deviceName + ".ProcessInput.Data.consVariable[36]",
									 				deviceName + ".ProcessInput.Data.consVariable[37]", deviceName + ".ProcessInput.Data.consVariable[38]",
									 				deviceName + ".ProcessInput.Data.consVariable[39]", deviceName + ".ProcessInput.Data.consVariable[40]",
									 				deviceName + ".ProcessInput.Data.consVariable[41]", deviceName + ".ProcessInput.Data.consVariable[42]",
									 				deviceName + ".ProcessInput.Data.consVariable[43]", deviceName + ".ProcessInput.Data.consVariable[44]",
									 				deviceName + ".ProcessInput.Data.consVariable[45]", deviceName + ".ProcessInput.Data.consVariable[46]",
									 				deviceName + ".ProcessInput.Data.consVariable[47]", deviceName + ".ProcessInput.Data.consVariable[48]",
									 				deviceName + ".ProcessInput.Data.consVariable[49]", deviceName + ".ProcessInput.Data.consVariable[50]",
									 				deviceName + ".ProcessInput.Data.consVariable[51]", deviceName + ".ProcessInput.Data.consVariable[52]",
									 				deviceName + ".ProcessInput.Data.consVariable[53]", deviceName + ".ProcessInput.Data.consVariable[54]",
									 				deviceName + ".ProcessInput.Data.consVariable[55]", deviceName + ".ProcessInput.Data.consVariable[56]",
									 				deviceName + ".ProcessInput.Data.consVariable[57]", deviceName + ".ProcessInput.Data.consVariable[58]",
									 				deviceName + ".ProcessInput.Data.consVariable[59]", deviceName + ".ProcessInput.Data.consVariable[60]",
									 				deviceName + ".ProcessInput.Data.consVariable[61]", deviceName + ".ProcessInput.Data.consVariable[62]",
									 				deviceName + ".ProcessInput.Data.consVariable[63]", deviceName + ".ProcessInput.Data.consVariable[64]",
									 				deviceName + ".ProcessInput.Data.consVariable[65]", deviceName + ".ProcessInput.Data.consVariable[66]",
									 				deviceName + ".ProcessInput.Data.consVariable[67]", deviceName + ".ProcessInput.Data.consVariable[68]",
									 				deviceName + ".ProcessInput.Data.consVariable[69]", deviceName + ".ProcessInput.Data.consVariable[70]",
									 				deviceName + ".ProcessInput.Data.consVariable[71]", deviceName + ".ProcessInput.Data.consVariable[72]",
									 				deviceName + ".ProcessInput.Data.consVariable[73]", deviceName + ".ProcessInput.Data.consVariable[74]",
									 				deviceName + ".ProcessInput.Data.consVariable[75]", deviceName + ".ProcessInput.Data.consVariable[76]",
									 				deviceName + ".ProcessInput.Data.consVariable[77]", deviceName + ".ProcessInput.Data.consVariable[78]",
									 				deviceName + ".ProcessInput.Data.consVariable[79]", deviceName + ".ProcessInput.Data.consVariable[80]",
									 				deviceName + ".ProcessInput.Data.consVariable[81]", deviceName + ".ProcessInput.Data.consVariable[82]",
									 				deviceName + ".ProcessInput.Data.consVariable[83]", deviceName + ".ProcessInput.Data.consVariable[84]",
									 				deviceName + ".ProcessInput.Data.consVariable[85]", deviceName + ".ProcessInput.Data.consVariable[86]",
									 				deviceName + ".ProcessInput.Data.consVariable[87]", deviceName + ".ProcessInput.Data.consVariable[88]",
									 				deviceName + ".ProcessInput.Data.consVariable[89]", deviceName + ".ProcessInput.Data.consVariable[90]",
									 				deviceName + ".ProcessInput.Data.consVariable[91]", deviceName + ".ProcessInput.Data.consVariable[92]",
									 				deviceName + ".ProcessInput.Data.consVariable[93]", deviceName + ".ProcessInput.Data.consVariable[94]",
									 				deviceName + ".ProcessInput.Data.consVariable[95]", deviceName + ".ProcessInput.Data.consVariable[96]",
									 				deviceName + ".ProcessInput.Data.consVariable[97]", deviceName + ".ProcessInput.Data.consVariable[98]",
									 				deviceName + ".ProcessInput.Data.consVariable[99]", deviceName + ".ProcessInput.Data.consVariable[100]",
									 				deviceName + ".ProcessInput.Data.consVariable[101]", deviceName + ".ProcessInput.Data.consVariable[102]",
									 				deviceName + ".ProcessInput.Data.consVariable[103]", deviceName + ".ProcessInput.Data.consVariable[104]",
									 				deviceName + ".ProcessInput.Data.consVariable[105]", deviceName + ".ProcessInput.Data.consVariable[106]",
									 				deviceName + ".ProcessInput.Data.consVariable[107]", deviceName + ".ProcessInput.Data.consVariable[108]",
									 				deviceName + ".ProcessInput.Data.consVariable[109]", deviceName + ".ProcessInput.Data.consVariable[110]",
									 				deviceName + ".ProcessInput.Data.consVariable[111]", deviceName + ".ProcessInput.Data.consVariable[112]",
									 				deviceName + ".ProcessInput.Data.consVariable[113]", deviceName + ".ProcessInput.Data.consVariable[114]",
									 				deviceName + ".ProcessInput.Data.consVariable[115]", deviceName + ".ProcessInput.Data.consVariable[116]",
									 				deviceName + ".ProcessInput.Data.consVariable[117]", deviceName + ".ProcessInput.Data.consVariable[118]",
									 				deviceName + ".ProcessInput.Data.consVariable[119]", deviceName + ".ProcessInput.Data.consVariable[120]",
								 					sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".alarm",
													sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".enabled",
											/*	sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+sAlias+".alarm",
													sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+sAlias+".enabled",*/
													deviceName + ".ProcessInput.Data.agentStatus:_online.._invalid");
						}													
					}
		
		}
		else
			cietWFIPAgent_FaceplateStatusDisconnection();
	
	g_bCallbackConnected = (iRes >= 0);
		
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPAgent_FaceplateDisconnect
/**
Purpose: disconnect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in cietWFIPAgent faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietWFIPAgent_FaceplateDisconnect(string deviceName)
{

	int iRes;
	string sPlcName, sSystemName, sAlias, sDeviceType;
	dyn_string dsDeviceNameSplit;
	
  //DebugN("Function:: cietWFIPAgent_FaceplateDisconnect");
	if(g_bUnSystemAlarmPlc) {
// get the PLC name 
		sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
		sSystemName = unGenericDpFunctions_getSystemName(deviceName);
		sDeviceType = dpTypeName(deviceName);
		sAlias = unGenericDpFunctions_getAlias(deviceName);
		dsDeviceNameSplit=strsplit(sAlias, ".");
									
		if (sDeviceType == "WFIPEvenAgent")
		{
		
		stopThread (g_iThreadId);
					
		iRes = dpDisconnect("cietWFIPEvenAgent_FaceplateStatusAnimationCB",
												 	deviceName + ".ProcessInput.Config.sgNumber", 		
						 					deviceName + ".ProcessInput.Config.agNumber", 		
								 				deviceName + ".ProcessInput.Data.agentStatus", 
								 				deviceName + ".ProcessInput.Data.prodVariable1[1]", deviceName + ".ProcessInput.Data.prodVariable1[2]",
								 				deviceName + ".ProcessInput.Data.prodVariable1[3]", deviceName + ".ProcessInput.Data.prodVariable1[4]",
								 				deviceName + ".ProcessInput.Data.prodVariable1[5]", deviceName + ".ProcessInput.Data.prodVariable1[6]",
								 				deviceName + ".ProcessInput.Data.prodVariable1[7]", deviceName + ".ProcessInput.Data.prodVariable1[8]",
								 				deviceName + ".ProcessInput.Data.consVariable[1]", deviceName + ".ProcessInput.Data.consVariable[2]",
								 				deviceName + ".ProcessInput.Data.consVariable[3]", deviceName + ".ProcessInput.Data.consVariable[4]",
								 				deviceName + ".ProcessInput.Data.consVariable[5]", deviceName + ".ProcessInput.Data.consVariable[6]",
								 				deviceName + ".ProcessInput.Data.consVariable[7]", deviceName + ".ProcessInput.Data.consVariable[8]",
								 				deviceName + ".ProcessInput.Data.consVariable[9]", deviceName + ".ProcessInput.Data.consVariable[10]",
								 				deviceName + ".ProcessInput.Data.consVariable[11]", deviceName + ".ProcessInput.Data.consVariable[12]",
								 				deviceName + ".ProcessInput.Data.consVariable[13]", deviceName + ".ProcessInput.Data.consVariable[14]",
								 				deviceName + ".ProcessInput.Data.consVariable[15]", deviceName + ".ProcessInput.Data.consVariable[16]",
								 				deviceName + ".ProcessInput.Data.consVariable[17]", deviceName + ".ProcessInput.Data.consVariable[18]",
								 				deviceName + ".ProcessInput.Data.consVariable[19]", deviceName + ".ProcessInput.Data.consVariable[20]",
								 				deviceName + ".ProcessInput.Data.consVariable[21]", deviceName + ".ProcessInput.Data.consVariable[22]",
								 				deviceName + ".ProcessInput.Data.consVariable[23]", deviceName + ".ProcessInput.Data.consVariable[24]",
								 				deviceName + ".ProcessInput.Data.consVariable[25]", deviceName + ".ProcessInput.Data.consVariable[26]",
								 				deviceName + ".ProcessInput.Data.consVariable[27]", deviceName + ".ProcessInput.Data.consVariable[28]",
								 				deviceName + ".ProcessInput.Data.consVariable[29]", deviceName + ".ProcessInput.Data.consVariable[30]",
								 				deviceName + ".ProcessInput.Data.consVariable[31]", deviceName + ".ProcessInput.Data.consVariable[32]",
								 				deviceName + ".ProcessInput.Data.consVariable[33]", deviceName + ".ProcessInput.Data.consVariable[34]",
								 				deviceName + ".ProcessInput.Data.consVariable[35]", deviceName + ".ProcessInput.Data.consVariable[36]",
								 				deviceName + ".ProcessInput.Data.consVariable[37]", deviceName + ".ProcessInput.Data.consVariable[38]",
								 				deviceName + ".ProcessInput.Data.consVariable[39]", deviceName + ".ProcessInput.Data.consVariable[40]",
								 				deviceName + ".ProcessInput.Data.prodVariable3[1]", deviceName + ".ProcessInput.Data.prodVariable3[2]",
								 				deviceName + ".ProcessInput.Data.prodVariable3[3]", deviceName + ".ProcessInput.Data.prodVariable3[4]",
								 				deviceName + ".ProcessInput.Data.prodVariable3[5]", deviceName + ".ProcessInput.Data.prodVariable3[6]",
								 				deviceName + ".ProcessInput.Data.prodVariable3[7]", deviceName + ".ProcessInput.Data.prodVariable3[8]",
							 					sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".alarm",
												sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".enabled",
/*												sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".alarm",
												sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".enabled",	*/
												deviceName + ".ProcessInput.Data.agentStatus:_online.._invalid");
												
		}
	else if (sDeviceType == "WFIPOddAgent")
		{
				stopThread (g_iThreadId);
							
				iRes = dpDisconnect("cietWFIPOddAgent_FaceplateStatusAnimationCB",
										 	deviceName + ".ProcessInput.Config.sgNumber", 		
						 					deviceName + ".ProcessInput.Config.agNumber", 		
								 				deviceName + ".ProcessInput.Data.agentStatus", 
								 				deviceName + ".ProcessInput.Data.consVariable[1]", deviceName + ".ProcessInput.Data.consVariable[2]",
								 				deviceName + ".ProcessInput.Data.consVariable[3]", deviceName + ".ProcessInput.Data.consVariable[4]",
								 				deviceName + ".ProcessInput.Data.consVariable[5]", deviceName + ".ProcessInput.Data.consVariable[6]",
								 				deviceName + ".ProcessInput.Data.consVariable[7]", deviceName + ".ProcessInput.Data.consVariable[8]",
								 				deviceName + ".ProcessInput.Data.consVariable[9]", deviceName + ".ProcessInput.Data.consVariable[10]",
								 				deviceName + ".ProcessInput.Data.consVariable[11]", deviceName + ".ProcessInput.Data.consVariable[12]",
								 				deviceName + ".ProcessInput.Data.consVariable[13]", deviceName + ".ProcessInput.Data.consVariable[14]",
								 				deviceName + ".ProcessInput.Data.consVariable[15]", deviceName + ".ProcessInput.Data.consVariable[16]",
								 				deviceName + ".ProcessInput.Data.consVariable[17]", deviceName + ".ProcessInput.Data.consVariable[18]",
								 				deviceName + ".ProcessInput.Data.consVariable[19]", deviceName + ".ProcessInput.Data.consVariable[20]",
								 				deviceName + ".ProcessInput.Data.consVariable[21]", deviceName + ".ProcessInput.Data.consVariable[22]",
								 				deviceName + ".ProcessInput.Data.consVariable[23]", deviceName + ".ProcessInput.Data.consVariable[24]",
								 				deviceName + ".ProcessInput.Data.consVariable[25]", deviceName + ".ProcessInput.Data.consVariable[26]",
								 				deviceName + ".ProcessInput.Data.consVariable[27]", deviceName + ".ProcessInput.Data.consVariable[28]",
								 				deviceName + ".ProcessInput.Data.consVariable[29]", deviceName + ".ProcessInput.Data.consVariable[30]",
								 				deviceName + ".ProcessInput.Data.consVariable[31]", deviceName + ".ProcessInput.Data.consVariable[32]",
								 				deviceName + ".ProcessInput.Data.consVariable[33]", deviceName + ".ProcessInput.Data.consVariable[34]",
								 				deviceName + ".ProcessInput.Data.consVariable[35]", deviceName + ".ProcessInput.Data.consVariable[36]",
								 				deviceName + ".ProcessInput.Data.consVariable[37]", deviceName + ".ProcessInput.Data.consVariable[38]",
								 				deviceName + ".ProcessInput.Data.consVariable[39]", deviceName + ".ProcessInput.Data.consVariable[40]",
								 				deviceName + ".ProcessInput.Data.consVariable[41]", deviceName + ".ProcessInput.Data.consVariable[42]",
								 				deviceName + ".ProcessInput.Data.consVariable[43]", deviceName + ".ProcessInput.Data.consVariable[44]",
								 				deviceName + ".ProcessInput.Data.consVariable[45]", deviceName + ".ProcessInput.Data.consVariable[46]",
								 				deviceName + ".ProcessInput.Data.consVariable[47]", deviceName + ".ProcessInput.Data.consVariable[48]",
								 				deviceName + ".ProcessInput.Data.consVariable[49]", deviceName + ".ProcessInput.Data.consVariable[50]",
								 				deviceName + ".ProcessInput.Data.consVariable[51]", deviceName + ".ProcessInput.Data.consVariable[52]",
								 				deviceName + ".ProcessInput.Data.consVariable[53]", deviceName + ".ProcessInput.Data.consVariable[54]",
								 				deviceName + ".ProcessInput.Data.consVariable[55]", deviceName + ".ProcessInput.Data.consVariable[56]",
								 				deviceName + ".ProcessInput.Data.consVariable[57]", deviceName + ".ProcessInput.Data.consVariable[58]",
								 				deviceName + ".ProcessInput.Data.consVariable[59]", deviceName + ".ProcessInput.Data.consVariable[60]",
								 				deviceName + ".ProcessInput.Data.consVariable[61]", deviceName + ".ProcessInput.Data.consVariable[62]",
								 				deviceName + ".ProcessInput.Data.consVariable[63]", deviceName + ".ProcessInput.Data.consVariable[64]",
								 				deviceName + ".ProcessInput.Data.consVariable[65]", deviceName + ".ProcessInput.Data.consVariable[66]",
								 				deviceName + ".ProcessInput.Data.consVariable[67]", deviceName + ".ProcessInput.Data.consVariable[68]",
								 				deviceName + ".ProcessInput.Data.consVariable[69]", deviceName + ".ProcessInput.Data.consVariable[70]",
								 				deviceName + ".ProcessInput.Data.consVariable[71]", deviceName + ".ProcessInput.Data.consVariable[72]",
								 				deviceName + ".ProcessInput.Data.consVariable[73]", deviceName + ".ProcessInput.Data.consVariable[74]",
								 				deviceName + ".ProcessInput.Data.consVariable[75]", deviceName + ".ProcessInput.Data.consVariable[76]",
								 				deviceName + ".ProcessInput.Data.consVariable[77]", deviceName + ".ProcessInput.Data.consVariable[78]",
								 				deviceName + ".ProcessInput.Data.consVariable[79]", deviceName + ".ProcessInput.Data.consVariable[80]",
								 				deviceName + ".ProcessInput.Data.consVariable[81]", deviceName + ".ProcessInput.Data.consVariable[82]",
								 				deviceName + ".ProcessInput.Data.consVariable[83]", deviceName + ".ProcessInput.Data.consVariable[84]",
								 				deviceName + ".ProcessInput.Data.consVariable[85]", deviceName + ".ProcessInput.Data.consVariable[86]",
								 				deviceName + ".ProcessInput.Data.consVariable[87]", deviceName + ".ProcessInput.Data.consVariable[88]",
								 				deviceName + ".ProcessInput.Data.consVariable[89]", deviceName + ".ProcessInput.Data.consVariable[90]",
								 				deviceName + ".ProcessInput.Data.consVariable[91]", deviceName + ".ProcessInput.Data.consVariable[92]",
								 				deviceName + ".ProcessInput.Data.consVariable[93]", deviceName + ".ProcessInput.Data.consVariable[94]",
								 				deviceName + ".ProcessInput.Data.consVariable[95]", deviceName + ".ProcessInput.Data.consVariable[96]",
								 				deviceName + ".ProcessInput.Data.consVariable[97]", deviceName + ".ProcessInput.Data.consVariable[98]",
								 				deviceName + ".ProcessInput.Data.consVariable[99]", deviceName + ".ProcessInput.Data.consVariable[100]",
								 				deviceName + ".ProcessInput.Data.consVariable[101]", deviceName + ".ProcessInput.Data.consVariable[102]",
								 				deviceName + ".ProcessInput.Data.consVariable[103]", deviceName + ".ProcessInput.Data.consVariable[104]",
								 				deviceName + ".ProcessInput.Data.consVariable[105]", deviceName + ".ProcessInput.Data.consVariable[106]",
								 				deviceName + ".ProcessInput.Data.consVariable[107]", deviceName + ".ProcessInput.Data.consVariable[108]",
								 				deviceName + ".ProcessInput.Data.consVariable[109]", deviceName + ".ProcessInput.Data.consVariable[110]",
								 				deviceName + ".ProcessInput.Data.consVariable[111]", deviceName + ".ProcessInput.Data.consVariable[112]",
								 				deviceName + ".ProcessInput.Data.consVariable[113]", deviceName + ".ProcessInput.Data.consVariable[114]",
								 				deviceName + ".ProcessInput.Data.consVariable[115]", deviceName + ".ProcessInput.Data.consVariable[116]",
								 				deviceName + ".ProcessInput.Data.consVariable[117]", deviceName + ".ProcessInput.Data.consVariable[118]",
								 				deviceName + ".ProcessInput.Data.consVariable[119]", deviceName + ".ProcessInput.Data.consVariable[120]",
							 					sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".alarm",
												sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".enabled",
/*												sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".alarm",
												sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".enabled",*/
												deviceName + ".ProcessInput.Data.agentStatus:_online.._invalid");
		}
	}
	g_bCallbackConnected = !(iRes >= 0);

	
}



//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPSegment_animateTableStatusSensor
/*
Purpose: Animate Sensor Status in Table.

Parameters:

Usage: Internal - Thread

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in cietWFIPSegment faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietWFIPSegment_animateTableStatusSensor(string sSystemName, string sPlcName)	
{
		string sSensorDeviceName, sSensorAlias, sColor;
		int j;
		
		unsigned uSensorStatus;
		bool bCodeInvalid, bIO_Error,bST_GTWDEV,bSystemIntegrityAlarmEnabled;
		int iSystemIntegrityAlarmValue;		
		
		
		g_iThreadId = getThreadId ();
		
		// DebugN("FUNCTION::cietWFIPSegment_animateTableStatusSensor () sSystemName="+sSystemName+" sPlcName="+sPlcName);
                // DebugN("g_iThreadId="+g_iThreadId);
                
		for (;;)
		{

					for (j=0;j<Table1.lineCount();j++)
						{
							getValue("Table1", "cellValueRC", j, "Name", sSensorAlias);	
							sSensorDeviceName = unGenericDpFunctions_dpAliasToName(sSystemName+sSensorAlias);
					
							dpGet(sSensorDeviceName + "ProcessInput.Data.sensorStatus",uSensorStatus);
							bIO_Error = getBit (uSensorStatus, 0);
			  			dpGet(sSensorDeviceName + "ProcessInput.Data.sensorStatus:_online.._invalid",bCodeInvalid);
			  			dpGet(sSensorDeviceName + "ST_GTWDEV",bST_GTWDEV);
			  			dpGet(sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".enabled",bSystemIntegrityAlarmEnabled);
							dpGet(sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".alarm",iSystemIntegrityAlarmValue);
			
							
							if (getBit(uSensorStatus, 12)==1)												// Disable
								{
									sColor = "cietWidget_Disabled";
								}
							else
								{
									 if (bCodeInvalid)
									 	{
											 sColor = "unDataNotValid"; 									// Invalid
									 	}	
									 else
									 	{
											if (bIO_Error==1)															// I/O Error
													sColor="cietWidget_Error";
											else
													sColor="unDisplayValue_Status";						// OK
													
											if ((!bSystemIntegrityAlarmEnabled) || (iSystemIntegrityAlarmValue >= CRYO_SYSTEM_INGETRITY_ALARM_LIMIT))
													sColor = "unDataNotValid";								// Old Data								
			
											if (bST_GTWDEV==0)														// Status Gateway Device 
										  	 sColor="cietWidget_Warning";
										 }	
								}					 
				
							setValue ("Table1", "cellBackColRC", j, "Status", sColor);		
						}

						delay(5);
		}		
}


//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPOddAgent_FaceplateStatusAnimationCB
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in cietWFIPAgent faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietWFIPOddAgent_FaceplateStatusAnimationCB(string sDp_sgNumber, int isgNumber,
																				string sDp_agNumber, int iagNumber,
													string sDp_agentStatus, unsigned uAgentStatus,
													string sDp_consVariable_b1, bit32 bconsVariable_1, string sDp_consVariable_b2, bit32 bconsVariable_2,
													string sDp_consVariable_b3, bit32 bconsVariable_3, string sDp_consVariable_b4, bit32 bconsVariable_4,
													string sDp_consVariable_b5, bit32 bconsVariable_5, string sDp_consVariable_b6, bit32 bconsVariable_6,
													string sDp_consVariable_b7, bit32 bconsVariable_7, string sDp_consVariable_b8, bit32 bconsVariable_8,
													string sDp_consVariable_b9, bit32 bconsVariable_9, string sDp_consVariable_b10, bit32 bconsVariable_10,
													string sDp_consVariable_b11, bit32 bconsVariable_11, string sDp_consVariable_b12, bit32 bconsVariable_12,
													string sDp_consVariable_b13, bit32 bconsVariable_13, string sDp_consVariable_b14, bit32 bconsVariable_14,
													string sDp_consVariable_b15, bit32 bconsVariable_15, string sDp_consVariable_b16, bit32 bconsVariable_16,
													string sDp_consVariable_b17, bit32 bconsVariable_17, string sDp_consVariable_b18, bit32 bconsVariable_18,
													string sDp_consVariable_b19, bit32 bconsVariable_19, string sDp_consVariable_b20, bit32 bconsVariable_20,													
													string sDp_consVariable_b21, bit32 bconsVariable_21, string sDp_consVariable_b22, bit32 bconsVariable_22,
													string sDp_consVariable_b23, bit32 bconsVariable_23, string sDp_consVariable_b24, bit32 bconsVariable_24,
													string sDp_consVariable_b25, bit32 bconsVariable_25, string sDp_consVariable_b26, bit32 bconsVariable_26,
													string sDp_consVariable_b27, bit32 bconsVariable_27, string sDp_consVariable_b28, bit32 bconsVariable_28,
													string sDp_consVariable_b29, bit32 bconsVariable_29, string sDp_consVariable_b30, bit32 bconsVariable_30,																
													string sDp_consVariable_b31, bit32 bconsVariable_31, string sDp_consVariable_b32, bit32 bconsVariable_32,
													string sDp_consVariable_b33, bit32 bconsVariable_33, string sDp_consVariable_b34, bit32 bconsVariable_34,
													string sDp_consVariable_b35, bit32 bconsVariable_35, string sDp_consVariable_b36, bit32 bconsVariable_36,
													string sDp_consVariable_b37, bit32 bconsVariable_37, string sDp_consVariable_b38, bit32 bconsVariable_38,
													string sDp_consVariable_b39, bit32 bconsVariable_39, string sDp_consVariable_b40, bit32 bconsVariable_40,
													string sDp_consVariable_b41, bit32 bconsVariable_41, string sDp_consVariable_b42, bit32 bconsVariable_42,
													string sDp_consVariable_b43, bit32 bconsVariable_43, string sDp_consVariable_b44, bit32 bconsVariable_44,
													string sDp_consVariable_b45, bit32 bconsVariable_45, string sDp_consVariable_b46, bit32 bconsVariable_46,
													string sDp_consVariable_b47, bit32 bconsVariable_47, string sDp_consVariable_b48, bit32 bconsVariable_48,
													string sDp_consVariable_b49, bit32 bconsVariable_49, string sDp_consVariable_b50, bit32 bconsVariable_50,														
													string sDp_consVariable_b51, bit32 bconsVariable_51, string sDp_consVariable_b52, bit32 bconsVariable_52,
													string sDp_consVariable_b53, bit32 bconsVariable_53, string sDp_consVariable_b54, bit32 bconsVariable_54,
													string sDp_consVariable_b55, bit32 bconsVariable_55, string sDp_consVariable_b56, bit32 bconsVariable_56,
													string sDp_consVariable_b57, bit32 bconsVariable_57, string sDp_consVariable_b58, bit32 bconsVariable_58,
													string sDp_consVariable_b59, bit32 bconsVariable_59, string sDp_consVariable_b60, bit32 bconsVariable_60,															
													string sDp_consVariable_b61, bit32 bconsVariable_61, string sDp_consVariable_b62, bit32 bconsVariable_62,
													string sDp_consVariable_b63, bit32 bconsVariable_63, string sDp_consVariable_b64, bit32 bconsVariable_64,
													string sDp_consVariable_b65, bit32 bconsVariable_65, string sDp_consVariable_b66, bit32 bconsVariable_66,
													string sDp_consVariable_b67, bit32 bconsVariable_67, string sDp_consVariable_b68, bit32 bconsVariable_68,
													string sDp_consVariable_b69, bit32 bconsVariable_69, string sDp_consVariable_b70, bit32 bconsVariable_70,																	
													string sDp_consVariable_b71, bit32 bconsVariable_71, string sDp_consVariable_b72, bit32 bconsVariable_72,
													string sDp_consVariable_b73, bit32 bconsVariable_73, string sDp_consVariable_b74, bit32 bconsVariable_74,
													string sDp_consVariable_b75, bit32 bconsVariable_75, string sDp_consVariable_b76, bit32 bconsVariable_76,
													string sDp_consVariable_b77, bit32 bconsVariable_77, string sDp_consVariable_b78, bit32 bconsVariable_78,
													string sDp_consVariable_b79, bit32 bconsVariable_79, string sDp_consVariable_b80, bit32 bconsVariable_80,													
													string sDp_consVariable_b81, bit32 bconsVariable_81, string sDp_consVariable_b82, bit32 bconsVariable_82,
													string sDp_consVariable_b83, bit32 bconsVariable_83, string sDp_consVariable_b84, bit32 bconsVariable_84,
													string sDp_consVariable_b85, bit32 bconsVariable_85, string sDp_consVariable_b86, bit32 bconsVariable_86,
													string sDp_consVariable_b87, bit32 bconsVariable_87, string sDp_consVariable_b88, bit32 bconsVariable_88,
													string sDp_consVariable_b89, bit32 bconsVariable_89, string sDp_consVariable_b90, bit32 bconsVariable_90,	
													string sDp_consVariable_b91, bit32 bconsVariable_91, string sDp_consVariable_b92, bit32 bconsVariable_92,
													string sDp_consVariable_b93, bit32 bconsVariable_93, string sDp_consVariable_b94, bit32 bconsVariable_94,
													string sDp_consVariable_b95, bit32 bconsVariable_95, string sDp_consVariable_b96, bit32 bconsVariable_96,
													string sDp_consVariable_b97, bit32 bconsVariable_97, string sDp_consVariable_b98, bit32 bconsVariable_98,
													string sDp_consVariable_b99, bit32 bconsVariable_99, string sDp_consVariable_b100, bit32 bconsVariable_100,	
													string sDp_consVariable_b101, bit32 bconsVariable_101, string sDp_consVariable_b102, bit32 bconsVariable_102,
													string sDp_consVariable_b103, bit32 bconsVariable_103, string sDp_consVariable_b104, bit32 bconsVariable_104,
													string sDp_consVariable_b105, bit32 bconsVariable_105, string sDp_consVariable_b106, bit32 bconsVariable_106,
													string sDp_consVariable_b107, bit32 bconsVariable_107, string sDp_consVariable_b108, bit32 bconsVariable_108,
													string sDp_consVariable_b109, bit32 bconsVariable_109, string sDp_consVariable_b110, bit32 bconsVariable_110,
													string sDp_consVariable_b111, bit32 bconsVariable_111, string sDp_consVariable_b112, bit32 bconsVariable_112,
													string sDp_consVariable_b113, bit32 bconsVariable_113, string sDp_consVariable_b114, bit32 bconsVariable_114,
													string sDp_consVariable_b115, bit32 bconsVariable_115, string sDp_consVariable_b116, bit32 bconsVariable_116,
													string sDp_consVariable_b117, bit32 bconsVariable_117, string sDp_consVariable_b118, bit32 bconsVariable_118,
													string sDp_consVariable_b119, bit32 bconsVariable_119, string sDp_consVariable_b120, bit32 bconsVariable_120,																																																																																																																																																																																																				
													string sDpSystemIntegrityAlarmValue, int iSystemIntegrityAlarmValue, 
													string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
												 /*	string sDpSystemAlarmValue, int iSystemAlarmValue,
												 	string sDpSystemAlarmEnabled, bool bSystemAlarmEnabled,*/
												 	string sDpInvalid, bool bCodeInvalid)
{


int iAlarmValue,i,iStart;
bool bDisable,bFaceplateInvalid, bReset=0, bAgError=0;
string sTemp,sDpPLCName,sBusName;
dyn_bit32 db32sensorByte ;
char temp1, temp2;
string sChannel, sCrate;

bool bIO_Error, bFIP_significant, bFIP_refreshment, bFIP_promptness,bFIP_presence;
string sHexaconsVariable;
dyn_string dsRawDatas;

// DebugN("FUNCTION: cietWFIPOddAgent_FaceplateStatusAnimationCB()");

db32sensorByte = makeDynChar(bconsVariable_1, bconsVariable_2, bconsVariable_3, bconsVariable_4, bconsVariable_5, bconsVariable_6,
									bconsVariable_7, bconsVariable_8, bconsVariable_9, bconsVariable_10, bconsVariable_11, bconsVariable_12, bconsVariable_13,
									bconsVariable_14, bconsVariable_15, bconsVariable_16, bconsVariable_17, bconsVariable_18, bconsVariable_19, bconsVariable_20,													
									bconsVariable_21, bconsVariable_22, bconsVariable_23, bconsVariable_24, bconsVariable_25, bconsVariable_26, bconsVariable_27,
									bconsVariable_28, bconsVariable_29, bconsVariable_30, bconsVariable_31, bconsVariable_32, bconsVariable_33, bconsVariable_34,
									bconsVariable_35, bconsVariable_36, bconsVariable_37, bconsVariable_38, bconsVariable_39, bconsVariable_40, bconsVariable_41,
									bconsVariable_42, bconsVariable_43, bconsVariable_44, bconsVariable_45, bconsVariable_46, bconsVariable_47, bconsVariable_48,
									bconsVariable_49, bconsVariable_50,	bconsVariable_51, bconsVariable_52, bconsVariable_53, bconsVariable_54, bconsVariable_55,
									bconsVariable_56, bconsVariable_57, bconsVariable_58, bconsVariable_59,	bconsVariable_60, bconsVariable_61, bconsVariable_62,
									bconsVariable_63, bconsVariable_64, bconsVariable_65, bconsVariable_66, bconsVariable_67, bconsVariable_68, bconsVariable_69,				
									bconsVariable_70, bconsVariable_71, bconsVariable_72, bconsVariable_73, bconsVariable_74, bconsVariable_75, bconsVariable_76,
									bconsVariable_77, bconsVariable_78, bconsVariable_79, bconsVariable_80,	bconsVariable_81, bconsVariable_82, bconsVariable_83,
									bconsVariable_84, bconsVariable_85, bconsVariable_86, bconsVariable_87, bconsVariable_88, bconsVariable_89, bconsVariable_90,	
									bconsVariable_91, bconsVariable_92, bconsVariable_93, bconsVariable_94, bconsVariable_95, bconsVariable_96, bconsVariable_97,
									bconsVariable_98, bconsVariable_99, bconsVariable_100, bconsVariable_101, bconsVariable_102, bconsVariable_103, bconsVariable_104,
									bconsVariable_105, bconsVariable_106, bconsVariable_107, bconsVariable_108, bconsVariable_109, bconsVariable_110, bconsVariable_111,
									bconsVariable_112, bconsVariable_113, bconsVariable_114, bconsVariable_115, bconsVariable_116, bconsVariable_117, bconsVariable_118,
									bconsVariable_119, bconsVariable_120);

// DebugN("Variable 4 (produced) db32sensorByte= ",db32sensorByte);

	bIO_Error = getBit (uAgentStatus, 0); 		// IO_ERROR
	bReset = getBit (uAgentStatus, 4) || getBit (uAgentStatus, 7) ; 
	bFIP_significant = getBit (uAgentStatus, 8); 
	bFIP_refreshment = getBit (uAgentStatus, 9);
	bFIP_promptness = getBit (uAgentStatus, 10);
	bFIP_presence = getBit (uAgentStatus, 11);	
	bDisable = getBit (uAgentStatus, 12);

	if ((iSystemIntegrityAlarmValue >= CRYO_SYSTEM_INGETRITY_ALARM_LIMIT)
		/*	||
			(iSystemAlarmValue < CRYO_SYSTEM_INGETRITY_ALARM_LIMIT)*/)
			iAlarmValue=c_unSystemIntegrity_alarm_value_level1;
	else
		iAlarmValue=c_unSystemIntegrity_no_alarm_value;
		
	//	iStart=strpos(sDpSystemIntegrityAlarmValue, c_cietSystemIntegrity_DS) + strlen(c_cietSystemIntegrity_DS);
//	sDpPLCName=substr(sDpSystemIntegrityAlarmValue,  iStart, strpos(sDpSystemIntegrityAlarmValue, ".alarm") - iStart);

  g_sGTWName = cietGenericFunctions_getGTWName(sDp_sgNumber);
  
	sDpPLCName=g_sGTWName;
	// EB: Modification to avoid the DATA INVALID problem due to the FESA timeStamp and no data transmitted (updated)
	// cietGenericFunctions_Header(sDpPLCName, iAlarmValue, bSystemIntegrityAlarmEnabled/* && bSystemAlarmEnabled*/, bCodeInvalid);
	cietGenericFunctions_Header(sDpPLCName, iAlarmValue, bSystemIntegrityAlarmEnabled/* && bSystemAlarmEnabled*/, 0);
	
	sCrate= cietGenericObject_getCrate ($sDpName);
	if(g_bSystemConnected)
		cietGenericObject_DisplayString(sCrate, "CRATE", "unDisplayValue_Parameter", bFaceplateInvalid);	
	
	// Warning bits
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("IO_ERROR", !bIO_Error, 1, "", bFaceplateInvalid, "cietFaceplate_Warning");
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("RESET", !bReset, 1, "", bFaceplateInvalid, "cietFaceplate_Warning");		
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("FIP_SIG", !bFIP_significant, 1, "", bFaceplateInvalid, "cietFaceplate_Info");
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("FIP_REF", !bFIP_refreshment, 1, "", bFaceplateInvalid, "cietFaceplate_Info");
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("FIP_PRE", !bFIP_presence, 1, "", bFaceplateInvalid, "cietFaceplate_Warning");
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("FIP_PRO", !bFIP_promptness, 1, "", bFaceplateInvalid, "cietFaceplate_Info");
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("DISABLE", !bDisable, 1, "", bFaceplateInvalid, "cietFaceplate_Warning");
		
	// Strings
	if(g_bSystemConnected)			
		cietGenericObject_DisplayString(g_sGTWName, "GTWNAME", "unDisplayValue_Parameter", bFaceplateInvalid);
	if(g_bSystemConnected)			
		{
			sBusName=cietGenericFuncitons_getSegmentName(sDp_sgNumber);
			// DebugN("sDp_sgNumber="+sDp_sgNumber);
			// DebugN("sBusName="+sBusName);
			cietGenericObject_DisplayString(sBusName, "BUSNAME", "unDisplayValue_Parameter", bFaceplateInvalid);	
			cietGenericObject_DisplayInt(iagNumber, "AGENTNUMBER", "unDisplayValue_Parameter", bFaceplateInvalid);	
		}
	if(g_bSystemConnected)				
		cietGenericObject_DisplayInt(isgNumber, "AGENTBUS", "unDisplayValue_Parameter", bFaceplateInvalid);	

	//	oddRawDatas
	oddRawDatas.deleteAllItems();
		
	int i=1;		
	int iChannel=0;		
	
	dsRawDatas[i++] = "        B0 B1 B2 B3 B4 B5 B6 B7";
	dsRawDatas[i++] = "        -----------------------";
	for(int iByte=0;iByte<=119;iByte++)		// i: byte
		{
			if(g_bSystemConnected)			
				{
					sHexaconsVariable = sHexaconsVariable + " " + cietGenericObject_toHexa(db32sensorByte[iByte+1]);
					if((iByte+1)%8==0)
						{
							sprintf(sChannel,"%2d",iChannel);
							dsRawDatas[i++] = "Ch "+sChannel+": "+sHexaconsVariable;
							sHexaconsVariable="";
							iChannel=iChannel+1;
						}
				}
		}	
	oddRawDatas.items = dsRawDatas;	
}		
		
		
// cietWFIPEvenAgent_FaceplateStatusAnimationCB
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in cietWFIPAgent faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietWFIPEvenAgent_FaceplateStatusAnimationCB(
													string sDp_sgNumber, int isgNumber,
													string sDp_agNumber, int iagNumber,
													string sDp_agentStatus, unsigned uAgentStatus,
													string sDp_prodVariable1_b1, bit32 bprodVariable1_1, string sDp_prodVariable1_b2, bit32 bprodVariable1_2,
													string sDp_prodVariable1_b3, bit32 bprodVariable1_3, string sDp_prodVariable1_b4, bit32 bprodVariable1_4,
													string sDp_prodVariable1_b5, bit32 bprodVariable1_5, string sDp_prodVariable1_b6, bit32 bprodVariable1_6,
													string sDp_prodVariable1_b7, bit32 bprodVariable1_7, string sDp_prodVariable1_b8, bit32 bprodVariable1_8,
													string sDp_consVariable_b1, bit32 bconsVariable_1, string sDp_consVariable_b2, bit32 bconsVariable_2,
													string sDp_consVariable_b3, bit32 bconsVariable_3, string sDp_consVariable_b4, bit32 bconsVariable_4,
													string sDp_consVariable_b5, bit32 bconsVariable_5, string sDp_consVariable_b6, bit32 bconsVariable_6,
													string sDp_consVariable_b7, bit32 bconsVariable_7, string sDp_consVariable_b8, bit32 bconsVariable_8,
													string sDp_consVariable_b9, bit32 bconsVariable_9, string sDp_consVariable_b10, bit32 bconsVariable_10,
													string sDp_consVariable_b11, bit32 bconsVariable_11, string sDp_consVariable_b12, bit32 bconsVariable_12,
													string sDp_consVariable_b13, bit32 bconsVariable_13, string sDp_consVariable_b14, bit32 bconsVariable_14,
													string sDp_consVariable_b15, bit32 bconsVariable_15, string sDp_consVariable_b16, bit32 bconsVariable_16,
													string sDp_consVariable_b17, bit32 bconsVariable_17, string sDp_consVariable_b18, bit32 bconsVariable_18,
													string sDp_consVariable_b19, bit32 bconsVariable_19, string sDp_consVariable_b20, bit32 bconsVariable_20,													
													string sDp_consVariable_b21, bit32 bconsVariable_21, string sDp_consVariable_b22, bit32 bconsVariable_22,
													string sDp_consVariable_b23, bit32 bconsVariable_23, string sDp_consVariable_b24, bit32 bconsVariable_24,
													string sDp_consVariable_b25, bit32 bconsVariable_25, string sDp_consVariable_b26, bit32 bconsVariable_26,
													string sDp_consVariable_b27, bit32 bconsVariable_27, string sDp_consVariable_b28, bit32 bconsVariable_28,
													string sDp_consVariable_b29, bit32 bconsVariable_29, string sDp_consVariable_b30, bit32 bconsVariable_30,																
													string sDp_consVariable_b31, bit32 bconsVariable_31, string sDp_consVariable_b32, bit32 bconsVariable_32,
													string sDp_consVariable_b33, bit32 bconsVariable_33, string sDp_consVariable_b34, bit32 bconsVariable_34,
													string sDp_consVariable_b35, bit32 bconsVariable_35, string sDp_consVariable_b36, bit32 bconsVariable_36,
													string sDp_consVariable_b37, bit32 bconsVariable_37, string sDp_consVariable_b38, bit32 bconsVariable_38,
													string sDp_consVariable_b39, bit32 bconsVariable_39, string sDp_consVariable_b40, bit32 bconsVariable_40,
													string sDp_prodVariable3_b1, bit32 bprodVariable3_1, string sDp_prodVariable3_b2, bit32 bprodVariable3_2,
													string sDp_prodVariable3_b3, bit32 bprodVariable3_3, string sDp_prodVariable3_b4, bit32 bprodVariable3_4,
													string sDp_prodVariable3_b5, bit32 bprodVariable3_5, string sDp_prodVariable3_b6, bit32 bprodVariable3_6,
													string sDp_prodVariable3_b7, bit32 bprodVariable3_7, string sDp_prodVariable3_b8, bit32 bprodVariable3_8,
													string sDpSystemIntegrityAlarmValue, int iSystemIntegrityAlarmValue, 
													string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
												 /*	string sDpSystemAlarmValue, int iSystemAlarmValue,
												 	string sDpSystemAlarmEnabled, bool bSystemAlarmEnabled,*/
												 	string sDpInvalid, bool bCodeInvalid)
{


int iAlarmValue,i;
bool bFaceplateInvalid, bReset=0, bAgError=0;
string sTemp,sDpPLCName, sBusName, sCrate;
dyn_bit32 db32sensorByte_v1,db32sensorByte_v2, db32sensorByte_v3 ;

bool bDisable, bIO_Error, bFIP_significant, bFIP_refreshment, bFIP_promptness, bFIP_presence;

char temp1, temp2;

// DebugN("FUNCTION: cietWFIPEvenAgent_FaceplateStatusAnimationCB()");

	bIO_Error = getBit (uAgentStatus, 0); 		// IO_ERROR
	bReset = getBit (uAgentStatus, 4) || getBit (uAgentStatus, 7) ; 	
	bFIP_significant = getBit (uAgentStatus, 8); 
	bFIP_refreshment = getBit (uAgentStatus, 9);
	bFIP_promptness = getBit (uAgentStatus, 10);
	bFIP_presence = getBit (uAgentStatus, 11);
	bDisable = getBit (uAgentStatus, 12);
	
	db32sensorByte_v1 = makeDynChar(bprodVariable1_1, bprodVariable1_2, bprodVariable1_3, bprodVariable1_4, bprodVariable1_5, bprodVariable1_6,
									bprodVariable1_7, bprodVariable1_8);
	db32sensorByte_v2 = makeDynChar(bconsVariable_1, bconsVariable_2, bconsVariable_3, bconsVariable_4, bconsVariable_5, bconsVariable_6,
									bconsVariable_7, bconsVariable_8, bconsVariable_9, bconsVariable_10, bconsVariable_11, bconsVariable_12, bconsVariable_13,
									bconsVariable_14, bconsVariable_15, bconsVariable_16, bconsVariable_17, bconsVariable_18, bconsVariable_19, bconsVariable_20,													
									bconsVariable_21, bconsVariable_22, bconsVariable_23, bconsVariable_24, bconsVariable_25, bconsVariable_26, bconsVariable_27,
									bconsVariable_28, bconsVariable_29, bconsVariable_30, bconsVariable_31, bconsVariable_32, bconsVariable_33, bconsVariable_34,
									bconsVariable_35, bconsVariable_36, bconsVariable_37, bconsVariable_38, bconsVariable_39, bconsVariable_40);
	db32sensorByte_v3 = makeDynChar(bprodVariable3_1, bprodVariable3_2, bprodVariable3_3, bprodVariable3_4, bprodVariable3_5, bprodVariable3_6,
									bprodVariable3_7, bprodVariable3_8);																		
	
	// DebugN("Variable 1 (produced) db32sensorByte_v1= ",db32sensorByte_v1);
	// DebugN("Variable 2 (consumed) db32sensorByte_v2= ",db32sensorByte_v2);
	// DebugN("Variable 3 (produced) db32sensorByte_v3= ",db32sensorByte_v3);
									
	if ((iSystemIntegrityAlarmValue >= CRYO_SYSTEM_INGETRITY_ALARM_LIMIT)	/*|| (iSystemAlarmValue < CRYO_SYSTEM_INGETRITY_ALARM_LIMIT)*/)
				iAlarmValue=c_unSystemIntegrity_alarm_value_level1;
	else
		iAlarmValue=c_unSystemIntegrity_no_alarm_value;
  
  g_sGTWName = cietGenericFunctions_getGTWName(sDp_sgNumber);
  
	sDpPLCName=g_sGTWName;
	cietGenericFunctions_Header(sDpPLCName, iAlarmValue, bSystemIntegrityAlarmEnabled/* && bSystemAlarmEnabled*/, bCodeInvalid);
	bFaceplateInvalid = false;
	
        
        
	// Warning bits        
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("IO_ERROR", !bIO_Error, 1, "", bFaceplateInvalid, "cietFaceplate_Warning");
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("RESET", !bReset, 1, "", bFaceplateInvalid, "cietFaceplate_Warning");		
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("FIP_SIG", !bFIP_significant, 1, "", bFaceplateInvalid, "cietFaceplate_Info");
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("FIP_REF", !bFIP_refreshment, 1, "", bFaceplateInvalid, "cietFaceplate_Info");
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("FIP_PRE", !bFIP_presence, 1, "", bFaceplateInvalid, "cietFaceplate_Warning");
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("FIP_PRO", !bFIP_promptness, 1, "", bFaceplateInvalid, "cietFaceplate_Info");
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("DISABLE", !bDisable, 1, "", bFaceplateInvalid, "cietFaceplate_Warning");
		
	sCrate= cietGenericObject_getCrate ($sDpName);
	if(g_bSystemConnected)
		cietGenericObject_DisplayString(sCrate, "CRATE", "unDisplayValue_Parameter", bFaceplateInvalid);	
	
	if(g_bSystemConnected)			
		cietGenericObject_DisplayString(g_sGTWName, "GTWNAME", "unDisplayValue_Parameter", bFaceplateInvalid);
	if(g_bSystemConnected)				
		{
			sBusName=cietGenericFuncitons_getSegmentName(sDp_sgNumber);
			// DebugN("sDp_sgNumber="+sDp_sgNumber);
			// DebugN("sBusName="+sBusName);
			cietGenericObject_DisplayString(sBusName, "BUSNAME", "unDisplayValue_Parameter", bFaceplateInvalid);
			cietGenericObject_DisplayInt(iagNumber, "AGENTNUMBER", "unDisplayValue_Parameter", bFaceplateInvalid);	
		}
	if(g_bSystemConnected)				
		cietGenericObject_DisplayInt(isgNumber, "AGENTBUS", "unDisplayValue_Parameter", bFaceplateInvalid);	
	
	for(i=1;i<=8;i++)
	{
		string sTemp;
		int j=i-1;
		sTemp = cietGenericObject_toHexa(db32sensorByte_v1[i]);
		if(g_bSystemConnected)		
			cietGenericObject_DisplayToCell(sTemp, "evenRawDatas1", 1 ,"B"+j, "unDisplayValue_Parameter", bFaceplateInvalid);

		sTemp = cietGenericObject_toHexa(db32sensorByte_v3[i]);		
		if(g_bSystemConnected)		
			cietGenericObject_DisplayToCell(sTemp, "evenRawDatas3", 1, "B"+j, "unDisplayValue_Parameter", bFaceplateInvalid);		
	
	}

	for(int k=1;k<=5;k++)
	{
		for(int i=0;i<8;i++)
			{
				string sTemp;
				int j=i+(k-1)*8;
				sTemp = cietGenericObject_toHexa(db32sensorByte_v2[j+1]);		
				if(g_bSystemConnected)		
					cietGenericObject_DisplayToCell(sTemp, "evenRawDatas2"+k, 1, "B"+j, "unDisplayValue_Parameter", bFaceplateInvalid);		
			}	
	}
}



//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPAgent_FaceplateStatusDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietWFIPAgent_FaceplateStatusDisconnection()
{

  //DebugN("FUNCTION:: cietWFIPAgent_FaceplateStatusDisconnection");
  
  
	//unGenericObject_ColorBoxDisconnect(makeDynString(""));

	//unGenericObject_DisplayValueDisconnect(makeDynString(""));
  
 //put in the top textbox the DATA not connected in purple
  cietGenericFunctions_HeaderDisconnect();
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPAgent_ButtonUserAccess
/**
Purpose: User access for DQAMCMB buttons

Parameters: 
	- sDpName, string, input, device name
	- sDpType, string, input, dptype
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietWFIPAgent_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
  
	bool operator, expert, admin;
	int i, length;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
        string sOperAction, sExpAction, sAdminAction;
        bool bActionDefined = false;

        unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo,CIET_ACCESS_CONTROL_DOMAIN_NAME);
        if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) ||
           (sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME))
          bActionDefined=true;
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString();
	if (operator)
	{
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
            dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_SELECT,ciet_POPUPMENU_HELP,
                                              ciet_POPUPMENU_WIDGET_SIMULATOR,ciet_POPUPMENU_OPEN_WFIPSEGMENT, 
                                              ciet_POPUPMENU_OPEN_CRATE));
	}
	if (expert)
	{
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
	    			dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_SELECT,ciet_FACEPLATE_BUTTON_CONFIG,
                                              ciet_FACEPLATE_BUTTON_RESET,ciet_FACEPLATE_BUTTON_RAWDATA, 
                                              ciet_FACEPLATE_BUTTON_DEBUG, ciet_FACEPLATE_BUTTON_DISABLE, ciet_FACEPLATE_BUTTON_ENABLECOMMWD,
                                              ciet_POPUPMENU_HELP,ciet_POPUPMENU_WIDGET_SIMULATOR,
                                              ciet_POPUPMENU_OPEN_WFIPSEGMENT, ciet_POPUPMENU_OPEN_CRATE));	
	}
        if(admin){
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
            dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_SELECT,ciet_FACEPLATE_BUTTON_CONFIG,
                                              ciet_FACEPLATE_BUTTON_RESET,ciet_FACEPLATE_BUTTON_RAWDATA, 
                                              ciet_FACEPLATE_BUTTON_DEBUG, ciet_FACEPLATE_BUTTON_DISABLE, ciet_FACEPLATE_BUTTON_ENABLECOMMWD,
                                              ciet_POPUPMENU_HELP,ciet_POPUPMENU_WIDGET_SIMULATOR,
                                              ciet_POPUPMENU_OPEN_WFIPSEGMENT, ciet_POPUPMENU_OPEN_CRATE));	
        }  
	
  //..................
  /*
  	bool operator, expert, admin;
	int i, length;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
	
//	DebugN("ButtonUserAccess");
	unGenericButtonFunctionsHMI_getUnicosPriviledges(dbPermissions, exceptionInfo);
	if (dynlen(exceptionInfo) > 0)
		{
		dynClear(dbPermissions);
		}
	while (dynlen(dbPermissions) < 4)
		{
		dynAppend(dbPermissions, false);
		}
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString();
	if (operator)
		{
		dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_SELECT,ciet_POPUPMENU_HELP,ciet_POPUPMENU_WIDGET_SIMULATOR,ciet_POPUPMENU_OPEN_WFIPSEGMENT, ciet_POPUPMENU_OPEN_CRATE));
		
		}
	if (expert)
		{
			dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_SELECT,ciet_FACEPLATE_BUTTON_CONFIG,ciet_FACEPLATE_BUTTON_RESET,ciet_FACEPLATE_BUTTON_RAWDATA, ciet_FACEPLATE_BUTTON_DEBUG, ciet_FACEPLATE_BUTTON_DISABLE,ciet_POPUPMENU_HELP,ciet_POPUPMENU_WIDGET_SIMULATOR,ciet_POPUPMENU_OPEN_WFIPSEGMENT, ciet_POPUPMENU_OPEN_CRATE));	
		}
	if (admin)
		{
			dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_SELECT,ciet_FACEPLATE_BUTTON_CONFIG,ciet_FACEPLATE_BUTTON_RESET,ciet_FACEPLATE_BUTTON_RAWDATA, ciet_FACEPLATE_BUTTON_DEBUG, ciet_FACEPLATE_BUTTON_DISABLE,ciet_POPUPMENU_HELP,ciet_POPUPMENU_WIDGET_SIMULATOR,ciet_POPUPMENU_OPEN_WFIPSEGMENT, ciet_POPUPMENU_OPEN_CRATE));	
		}	*/
  
}


//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPAgent_ButtonRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in DQAMCMB faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietWFIPAgent_ButtonRegisterCB(string sDpSystemConnected, bool bSystemConnected)
{
	string deviceName, sSystemName, sFunction,sDpType;
	int iAction, iRes, iManReg01;
	dyn_string exceptionInfo,dsUserAccess,dsFunctions;
//	DebugN("ButtonRegisterCB");
	
	deviceName = unGenericDpFunctions_getDpName($sDpName);
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	
	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
//		DebugN("ButtonRegisterCB UN_ACTION_DISCONNECT");
			cietWFIPAgent_ButtonDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
//		DebugN("ButtonRegisterCB UN_ACTION_DPCONNECT");
		sDpType = dpTypeName(deviceName);

			unGenericObject_GetFunctions(sDpType, dsFunctions, exceptionInfo);
			sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];
						 
			if (sFunction != "")
				{
				evalScript(dsUserAccess, "dyn_string main(string deviceNameTemp, string sDpTypeTemp) {" + 
							 "dyn_string dsUserAccessReturn;" +
							 "if (isFunctionDefined(\"" + sFunction  + "\"))" +
							 "    {" +
					 			sFunction  + "(deviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
							 "    }" +
							 "return dsUserAccessReturn; }", makeDynString(), deviceName, sDpType);
						
				if (dynlen(dsUserAccess) == 0)
					fwException_raise(exceptionInfo,"ERROR", "cietWFIPAgent_ButtonRegisterCB: " + getCatStr("unGeneration","UNKNOWNFUNCTION"), sFunction);
        				else{
					g_dsUserAccess=dsUserAccess;	
        }
				}
			else
				fwException_raise(exceptionInfo,"ERROR", "cietWFIPAgent_ButtonRegisterCB: " + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");

			iRes = dpConnect("cietWFIPAgent_ButtonAnimationCB",
							 				 deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							 				 deviceName + ".statusInformation.selectedManager",
							 				 deviceName + ".ProcessInput.Data.agentStatus"
                    );
			g_bCallbackConnected = (iRes >= 0);
			
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
//			DebugN("ButtonRegisterCB UN_ACTION_DPDISCONNECT_DISCONNECT");
			iRes = dpDisconnect("cietWFIPAgent_ButtonAnimationCB",
							 						deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							 						deviceName + ".statusInformation.selectedManager",
							 						deviceName + ".ProcessInput.Data.agentStatus"
                       );
			g_bCallbackConnected = !(iRes >= 0);
			cietWFIPAgent_ButtonDisconnection();
			break;
		case UN_ACTION_NOTHING:
//				DebugN("ButtonRegisterCB UN_ACTION_NOTHING");
		default:
			break;
		}
}


//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPAgent_ButtonAnimationCB
/**
Purpose: animate buttons

Parameters: 
	- dbPermissions, dyn_bool, input, list of permissions, length = 4, order : 1=monitor, 2=operator, 3=expert, 4=admin
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietWFIPAgent_ButtonAnimationCB(string sDpLock, bool bLocked, 
														 string sDpSelect, string sSelectedManager, 
														 string sDpAgentStatus, unsigned uAgentStatus)
{
string deviceName, deviceType;
bool bStatusRawData, bDisable, bEnableWD;
string sDisableText, sDisableWDText,sDpType;

	bDisable = getBit (uAgentStatus, 12);
	
	if (bDisable ==1)
			sDisableText=ciet_FACEPLATE_BUTTON_ENABLE;
	else if (bDisable ==0)
			sDisableText=ciet_FACEPLATE_BUTTON_DISABLE;			
  
  	bEnableWD = getBit (uAgentStatus, 13);
	
    //if the device is a WFIPOddAgent put nothing in the text of the EnableWD button
  deviceName = unGenericDpFunctions_getDpName($sDpName);
  sDpType = dpTypeName(deviceName);
  if (sDpType == "WFIPOddAgent"){
    sDisableWDText="";
  }
  else{
     	if (bEnableWD ==1)
  			sDisableWDText=ciet_FACEPLATE_BUTTON_DISABLEWD;
    	else if (bEnableWD ==0)
  			sDisableWDText=ciet_FACEPLATE_BUTTON_ENABLEWD;			
	}
	setMultiValue("DISABLE_BUTTON.buttonDisable", "text", sDisableText);
  	
  setMultiValue("ENABLEWD_BUTTON.buttonEnableWD", "text", sDisableWDText);
 
 
	if(StatusRawDataText.text=="true")
				bStatusRawData = 1;
	else if(StatusRawDataText.text=="false")				
				bStatusRawData = 0;

  	deviceType = dpTypeName($sDpName);
	if(deviceType == "WFIPEvenAgent")
		cietWFIPAgent_ButtonSetState(unGenericDpFunctions_getDpName(sDpLock), "WFIPEvenAgent", g_dsUserAccess, makeDynString(bLocked, sSelectedManager, bStatusRawData));
	else if (deviceType == "WFIPOddAgent")
		cietWFIPAgent_ButtonSetState(unGenericDpFunctions_getDpName(sDpLock), "WFIPOddAgent", g_dsUserAccess, makeDynString(bLocked, sSelectedManager,	bStatusRawData));	

}



//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPAgent_ButtonSetState
/**
Purpose: set button state

Parameters: 
	- deviceName, string, input, device
	- sDpType, string, input, dptype
	- dsUserAccess, dyn_string, input, user access
	- dsData, dyn_string, parameter
		
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietWFIPAgent_ButtonSetState(string deviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
	bool bSelected, buttonEnabled, bit1, bit2;
	dyn_string dsButtons;
	int i, length, alertType;
	string localManager;
	bool bLocked, ListEnabled, bStatusRawData;
	string sSelectedManager;
  
//	DebugN("ButtonSetState");

	bLocked = dsData[1];
	sSelectedManager = dsData[2];
  bStatusRawData = dsData[3];
	
// 1. Selection state
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = (localManager == "S");
  
  string	sDeviceType = dpTypeName(deviceName);
	
	if (sDeviceType == "WFIPEvenAgent")
  dsButtons = makeDynString(ciet_FACEPLATE_BUTTON_RESET,ciet_FACEPLATE_BUTTON_RAWDATA,ciet_FACEPLATE_BUTTON_CONFIG, ciet_FACEPLATE_BUTTON_DEBUG, ciet_FACEPLATE_BUTTON_DISABLE, ciet_FACEPLATE_BUTTON_ENABLECOMMWD); // Each button except "Select"
	else if (sDeviceType == "WFIPOddAgent")
  	dsButtons = makeDynString(ciet_FACEPLATE_BUTTON_RESET,ciet_FACEPLATE_BUTTON_RAWDATA,ciet_FACEPLATE_BUTTON_CONFIG, ciet_FACEPLATE_BUTTON_DEBUG, ciet_FACEPLATE_BUTTON_DISABLE); // Each button except "Select"
	length = dynlen(dsButtons);
// 1. Selection state
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = (localManager == "S");
// 2. Button state
	for(i=1;i<=length;i++)
		{
		buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0);	// User access
		switch (dsButtons[i])
			{
			case ciet_FACEPLATE_BUTTON_RESET:
				buttonEnabled = buttonEnabled && bSelected;				
				break;
			case ciet_FACEPLATE_BUTTON_RAWDATA:
				buttonEnabled = buttonEnabled && bSelected  && !bStatusRawData;		
				// DebugN ("Button State = " + bStatusRawData );		
				break;
			case ciet_FACEPLATE_BUTTON_CONFIG:
				buttonEnabled = buttonEnabled && bSelected;				
				break;	
			case ciet_FACEPLATE_BUTTON_DEBUG:
				buttonEnabled = buttonEnabled && bSelected;
				break;				

			case ciet_FACEPLATE_BUTTON_DISABLE:
				buttonEnabled = buttonEnabled && bSelected;
				break;						
     
     case ciet_FACEPLATE_BUTTON_ENABLECOMMWD:
				buttonEnabled = buttonEnabled && bSelected;
				break;		
				
			default:
				break;
			}
		setValue(ciet_FACEPLATE_BUTTON_PREFIX + dsButtons[i], "enabled", buttonEnabled);
		}
// 3. Button Select state
	unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPAgent_UserLoginGetButtonState
/**
Purpose: function called before the button set state when a user logs in. 

Parameters:
	deviceName: string, input, the device name
	dsReturnData: dyn_string, output, the data are returned here

Usage: Internal function

PVSS manager usage:  Ctrl, NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
		-> $sDpName, string, the device DP name
		-> g_sFaceplateButtonType, string, faceplate type
	. PVSS version: 3.0 
	. operating system: NT, W2000 and Linux, but tested only under W2000.
	. distributed system: yes but not tested.
*/

cietWFIPAgent_UserLoginGetButtonState(string deviceName, string sType, dyn_string &dsReturnData)
{
	
		int iRes;
	string sDpLock, sDpSelectedManager, sSelectedManager, sInvalidDp, sSystemName;
	bool bLocked;
//	DebugN("UserLoginGetButtonState");
// 2. Get parameters to call the set button function
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);

	sDpLock = deviceName + ".statusInformation.selectedManager:_lock._original._locked";
	sDpSelectedManager = deviceName + ".statusInformation.selectedManager";
	
	if (dpExists(deviceName))
	{
		iRes = dpGet(sDpLock, bLocked, sDpSelectedManager, sSelectedManager);
	}
	else
	{
		iRes = -1;
	}
	if (iRes < 0) // Errors during dpget
	{
		sDpLock = "";
		bLocked = false;
		sDpSelectedManager = "";
		sSelectedManager = "";
	}
	dsReturnData=makeDynString(bLocked, sSelectedManager, "", "", "");
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPAgent_ButtonDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietWFIPAgent_ButtonDisconnection()
{
			   
//	DebugN("ButtonDisconnection");
		unGenericObject_ButtonDisconnect(makeDynString(ciet_FACEPLATE_BUTTON_RESET,ciet_FACEPLATE_BUTTON_RAWDATA, ciet_FACEPLATE_BUTTON_CONFIG ,UN_FACEPLATE_BUTTON_SELECT, ciet_FACEPLATE_BUTTON_DEBUG, ciet_FACEPLATE_BUTTON_DISABLE, ciet_FACEPLATE_BUTTON_ENABLECOMMWD));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPAgent_WidgetRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in WFIPEvenAgentwidget
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietWFIPAgent_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, deviceType;
	int iAction,iRes;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName;
	bool bRemote;
	
//DebugN("WidgetRegisterCB");

	deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);

	if (deviceName == "")	// In case of disconnection
		{
		deviceName = g_sDpName;
		}
		// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	
	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;

	if(sPlcName != "") {

		if (dpExists(sSystemName+c_CRYO_FRONTEND_UNSYSTEMALARM+c_cietSystemIntegrity_DS+sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;

	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			cietWFIPAgent_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:
			g_sDpName = deviceName;
			g_bDisconnectCB = false;

			if(g_bUnSystemAlarmPlc) 
				{
						deviceType = dpTypeName(deviceName);
						if((deviceType == "WFIPEvenAgent")||(deviceType == "WFIPOddAgent"))
						iRes = dpConnect("cietWFIPAgent_WidgetLockCallBack", deviceName + ".ProcessInput.Data.agentStatus:_lock._alert_hdl._locked");
						else{
							g_bUnSystemAlarmPlc = false;
							cietWFIPEvenAgent_WidgetDisconnection(g_sWidgetType);
						}
						g_bCallbackConnected = (iRes >= 0);
				}
			else
						cietWFIPAgent_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:

			if(g_bUnSystemAlarmPlc) 
				iRes = dpDisconnect("cietWFIPAgent_WidgetLockCallBack", deviceName + ".ProcessInput.Data.agentStatus:_lock._alert_hdl._locked");

			if(g_bDisconnectCB) {
						cietWFIPAgent_WidgetDisconnect(g_sDpName, sPlcName);
				}
			g_bCallbackConnected = !(iRes >= 0);
			g_bDisconnectCB = false;

			cietWFIPAgent_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}
//---------------------------------------------------------------------------------------------------------------------------------------
// cietWFIPAgent_WidgetLockCallBack
/**
Purpose: set up the lock callback function for the widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in WFIPEvenAgentwidget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietWFIPAgent_WidgetLockCallBack(string sdpe, bool bLockState)
{
	int iRes;
	string sPlcName;

//DebugN("cietWFIPAgent_WidgetLockCallBack");	

	if(bLockState) 
		{ // lock set so disconnect if not already disconnected
		if(g_bDisconnectCB) 
			{
				cietWFIPAgent_WidgetDisconnect(g_sDpName, sPlcName);
			}
		g_bDisconnectCB = false;
		cietWFIPAgent_WidgetDisconnection(g_sWidgetType);
		}
	else 
		{ // lock not set so connect if not already connected
		g_bDisconnectCB = true;
		sPlcName = unGenericObject_GetPLCNameFromDpName(sdpe);
		cietWFIPAgent_WidgetConnect(g_sDpName,sPlcName);
		}
}
//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPAgent_WidgetConnect
/**
Purpose: Connect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Analog faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietWFIPAgent_WidgetConnect(string deviceName, string sPlcName)
{
	int iRes;
	string sSystemName, sAlias, sDeviceType;
	dyn_string dsDeviceNameSplit;
	


//  check if the device name is of type "cietWFIPAgent" if not like of no PLC. -> not available.
	sDeviceType = dpTypeName(deviceName);

//DebugN("WidgetConnect",sDeviceType, g_bUnSystemAlarmPlc);
			
	if ((sDeviceType != "WFIPEvenAgent")&(sDeviceType != "WFIPOddAgent"))
			g_bUnSystemAlarmPlc = false;
	
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	if(g_bUnSystemAlarmPlc)
	{
		
		sAlias = unGenericDpFunctions_getAlias(deviceName);
		dsDeviceNameSplit=strsplit(sAlias, ".");
		iRes = dpConnect("cietWFIPAgent_WidgetAnimationCB", 
						 					deviceName + ".ProcessInput.Data.agentStatus",
				 							sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".enabled",
											deviceName + ".ST_GTWDEV",
/*											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".enabled",*/
											deviceName + ".ProcessInput.Data.agentStatus:_online.._invalid", 
									 		deviceName + ".statusInformation.selectedManager:_lock._original._locked",
						 					deviceName + ".statusInformation.selectedManager");
						 					
		g_bCallbackConnected = (iRes >= 0);		
	}else{
		cietWFIPAgent_WidgetDisconnection(g_sWidgetType);
	}
}
//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPAgent_WidgetDisconnect
/**
Purpose: Disconnect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Analog faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietWFIPAgent_WidgetDisconnect(string deviceName, string sPlcName)
{
	int iRes;
	string sSystemName, sAlias;
	dyn_string dsDeviceNameSplit;

//DebugN("WidgetDisconnect");

	sSystemName = unGenericDpFunctions_getSystemName(deviceName);

	sAlias = unGenericDpFunctions_getAlias(deviceName);
	dsDeviceNameSplit=strsplit(sAlias, ".");
		
	if(g_bUnSystemAlarmPlc)
	{
		iRes = dpDisconnect("cietWFIPAgent_WidgetAnimationCB", 
						 					deviceName + ".ProcessInput.Data.agentStatus",
			 							  sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".enabled",
											deviceName + ".ST_GTWDEV",
/*										sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".enabled",*/
											deviceName + ".ProcessInput.Data.agentStatus:_online.._invalid", 
									 		deviceName + ".statusInformation.selectedManager:_lock._original._locked",
						 					deviceName + ".statusInformation.selectedManager");
		
		g_bCallbackConnected = (iRes >= 0);
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPAgent_WidgetAnimationCB
/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Analog faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietWFIPAgent_WidgetAnimationCB(string sDp_agentStatus, unsigned uAgentStatus,
											 string sDpSystemIntegrityAlarmValue, int iSystemIntegrityAlarmValue, 
											 string sDpSystemIntegrityAlarmEnabled, bool bSystemIntegrityAlarmEnabled,
											 string sDp_ST_GTWDEV, bool bST_GTWDEV,
										/*	 string sDpSystemAlarmValue, int iSystemAlarmValue,
											 string sDpSystemAlarmEnabled, bool bSystemAlarmEnabled,*/
											 string sDpCodeInvalid, bool bCodeInvalid,
											 string sDpLocked, bool bLocked,
											 string sDpSelectedManager, string sSelectedManager)
{



//DebugN("WidgetAnimationCB - g_sWidgetType = " + g_sWidgetType);
//	WFIPAgent  -   WFIPAgentLed

if (g_sWidgetType=="WFIPAgent")
	{
				string formattedValue;
				string warningLetter, warningColor, warningVisible, alarmLetter, alarmColor, controlLetter, controlColor, body1Text, body1Color, selectColor;
				bool lockVisible;
			
				bool bFIP_significant, bFIP_refreshment, bFIP_promptness, bIO_Error;
				
				bIO_Error=getBit(uAgentStatus, 0);
				bFIP_significant=getBit(uAgentStatus, 8);
				bFIP_refreshment=getBit(uAgentStatus, 9);
				bFIP_promptness=getBit(uAgentStatus, 10);
			
				// WARNINGS
				
				if (getBit(uAgentStatus, 12)==1)			// Disable
					{
						body1Color = "cietWidget_Disabled";
						body1Text = "DIS";		
						
						warningLetter = ciet_WIDGET_TEXT_WARNING_DISABLE;
						warningColor = "cietWidget_Warning";	
						warningVisible=true;

						if (bIO_Error==1)																// I/O Error
					  	{
					  	  warningLetter = UN_WIDGET_TEXT_WARNING_ERROR + ciet_WIDGET_TEXT_WARNING_DISABLE;
								warningColor = "cietWidget_Warning";	
								warningVisible=true;
															  	
								/*body1Text = "ER:";
								if (bFIP_significant || bFIP_refreshment || bFIP_promptness)
									{
										body1Text=body1Text + ":";
										if (bFIP_significant)
											body1Text=body1Text + "S";
										if (bFIP_refreshment)
											body1Text=body1Text + "R";
										if (bFIP_promptness)
											body1Text=body1Text + "P";					
									}*/
							}						
						
					}
				else
					{
						 if (bCodeInvalid)
						 	{
								 warningLetter = UN_WIDGET_TEXT_INVALID;					// Invalid
								 warningColor = "unDataNotValid";
					  		 warningVisible=true; 
						 	}	
						 else
						 	{
								if (bIO_Error==1)																// I/O Error
							  	{
							  	  warningLetter = UN_WIDGET_TEXT_WARNING_ERROR;
										warningColor = "cietWidget_Warning";	
										warningVisible=true;
										
										body1Text = "ER";
										body1Color="unAlarm_AnalogH";
										if (bFIP_significant || bFIP_refreshment || bFIP_promptness)
											{
												body1Text=body1Text + ":";
												if (bFIP_significant)
													body1Text=body1Text + "S";
												if (bFIP_refreshment)
													body1Text=body1Text + "R";
												if (bFIP_promptness)
													body1Text=body1Text + "P";					
											}
									}
								else
									{
										body1Text = "OK";														// OK
										body1Color="unDisplayValue_Status";
								  	warningVisible=false;
									}
					
								if ((!bSystemIntegrityAlarmEnabled) || (iSystemIntegrityAlarmValue >= CRYO_SYSTEM_INGETRITY_ALARM_LIMIT))
									{
										warningLetter = UN_WIDGET_TEXT_OLD_DATA;		// Old Data
										warningColor = "unDataNotValid";
							 	  	warningVisible=true; 
									}
									
								if (bST_GTWDEV==0)															// Status Gateway Device 
									{
								   warningLetter= ciet_WIDGET_TEXT_STATUS_GATEWAY_DEVICE;  	   	 
							  	 warningColor="cietWidget_Warning";
							  	 warningVisible=true; 
									}
									
							 }	
					}					 
			
			
				// Set the widget.
				unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, selectColor, lockVisible);
			
				// Animate widget
				if(g_bSystemConnected)			
					setMultiValue("Body1", "text", body1Text,
												"Body1", "foreCol", body1Color, 
							  				"WarningText", "text", warningLetter,
							  				"WarningText", "foreCol", warningColor,
							  				"WarningText", "visible", warningVisible,
							  				"SelectArea", "foreCol", selectColor,
							   				"LockBmp", "visible", lockVisible, 
							  				"WidgetArea", "visible", true);
	}
	
else if(g_sWidgetType=="WFIPAgentLed")
	{
				string formattedValue, selectColor;
				string sLedColor;
				bool lockVisible;
			
				bool bFIP_significant, bFIP_refreshment, bFIP_promptness, bIO_Error;
				
				bIO_Error=getBit(uAgentStatus, 0);
				bFIP_significant=getBit(uAgentStatus, 8);
				bFIP_refreshment=getBit(uAgentStatus, 9);
				bFIP_promptness=getBit(uAgentStatus, 10);
			
				// WARNINGS
				
				if (getBit(uAgentStatus, 12)==1)												// Disable
					{
						sLedColor = "cietWidget_Disabled";
					}
				else
					{
						 if (bCodeInvalid)
						 	{
								 sLedColor = "unDataNotValid"; 									// Invalid
						 	}	
						 else
						 	{
								if (bIO_Error==1)																// I/O Error
										sLedColor="cietWidget_Error";
								else
										sLedColor="unDisplayValue_Status";					// OK
										
								if ((!bSystemIntegrityAlarmEnabled) || (iSystemIntegrityAlarmValue >= CRYO_SYSTEM_INGETRITY_ALARM_LIMIT))
										sLedColor = "unDataNotValid";								// Old Data								

								if (bST_GTWDEV==0)															// Status Gateway Device 
							  	 sLedColor="cietWidget_Warning";
							 }	
					}					 
			
			
				// 3. set the widget.
				unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, selectColor, lockVisible);
			
			//	DebugN("bLocked, sSelectedManager, selectColor,lockVisible", bLocked, sSelectedManager, selectColor,lockVisible);
				// 8. Animate widget
				if(g_bSystemConnected)			
					setMultiValue("Led", "backCol", sLedColor, "Led", "foreCol", sLedColor);	
	}


}
			
//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPAgent_WidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
	- sWidgetType
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietWFIPAgent_WidgetDisconnection(string sWidgetType)
{
//DebugN("FUNCTION: cietWFIPAgent_WidgetDisconnection - sWidgetType = " + sWidgetType);
if(sWidgetType=="WFIPAgent")
	{	
    setMultiValue(	"Body1", "foreCol", "unDataNoAccess",
					"Body1", "backCol", "unWidget_Background",
					"WarningText", "text", "",
					"LockBmp", "visible", false,
					"WidgetArea", "visible", false);	
	}
	
else if(sWidgetType=="WFIPAgentLed")
	{
    setMultiValue(	"Led", "foreCol", "unDataNoAccess");	
	}												
}								


//@}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPAgent_PanelWidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
	- sWidgetType
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietWFIPAgent_PanelWidgetDisconnection(string sWidgetType)
{
	string warningText;

//DebugN("PanelWidgetDisconnection - sWidgetType = " + sWidgetType);

if(sWidgetType=="WFIPAgent")
	{	
	warningText = UN_WIDGET_TEXT_INVALID;

	setMultiValue("Body1", "text", "",
								"WarningText", "text", warningText,
								"LockBmp", "visible", false);
	}



}								

//------------------------------------------------------------------------------------------------------------------------
//@}

// cietWFIPAgent_MenuConfiguration
/**
Purpose: configuration of popup menu

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsAccessOk, dyn_string, input, authorized action
	- menuList, dyn_string, output, menu to pop-up
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 3.0 
	. distributed system: yes.
*/

cietWFIPAgent_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{

dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, 
								 ciet_POPUPMENU_RESET, ciet_POPUPMENU_CONFIG, UN_POPUPMENU_FACEPLATE_TEXT, UN_POPUPMENU_TREND_TEXT, ciet_FACEPLATE_BUTTON_DEBUG, ciet_FACEPLATE_BUTTON_DISABLE, ciet_FACEPLATE_BUTTON_ENABLECOMMWD,ciet_POPUPMENU_HELP,ciet_POPUPMENU_WIDGET_SIMULATOR,ciet_POPUPMENU_OPEN_WFIPSEGMENT, ciet_POPUPMENU_OPEN_CRATE);
int granted;

string localManager, sSelectedManager;
bool bSelected, bLocked;

int iUnicosPopupMenuMin = UN_POPUPMENU_MIN;	

//  Selection state

	dpGet(sDpName + ".statusInformation.selectedManager:_lock._original._locked", bLocked);
	dpGet(sDpName + ".statusInformation.selectedManager",sSelectedManager);
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = (localManager == "S");
								 
// Select
	unGenericObject_addSelectToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	dynAppend(menuList, "SEPARATOR");

// 1. Reset
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_RESET) > 0)
	{
		granted = 0;
		if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_RESET) > 0) && bSelected)
		{
			granted = 1;
		}
		dynAppend(menuList,"PUSH_BUTTON, Reset, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_RESET)+", " + granted);
	}

// 2. WFip Config
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_CONFIG) > 0)
	{
		granted = 0;
		if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_CONFIG) > 0) && bSelected)
		{
			granted = 1;
		}
		dynAppend(menuList,"PUSH_BUTTON, WFip Config, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_CONFIG)+", " + granted);
	}

	// 3. Disable
	
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_DISABLE) > 0)
	{
		granted = 0;
		if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_DISABLE) > 0 ) && bSelected)
		{
			granted = 1;
		}
		dynAppend(menuList, "PUSH_BUTTON, Enable / Disable, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_DISABLE)+", " + granted);
	}	
	
	// 4. Debug
	
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_DEBUG) > 0)
	{
		granted = 0;
		if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_DEBUG) > 0 ) && bSelected)
		{
			granted = 1;
		}
		dynAppend(menuList, "PUSH_BUTTON, Debug, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_DEBUG)+", " + granted);
	}		
	

	if (dynlen(menuList) > 0)
	{
		if (menuList[dynlen(menuList)] != "SEPARATOR")
		{
			dynAppend(menuList, "SEPARATOR");
		}
	}

// Open Segment	
if (dynContains(dsMenuConfig, ciet_POPUPMENU_OPEN_WFIPSEGMENT) > 0)
{
	granted = 0;
	if (dynContains(dsAccessOk, ciet_POPUPMENU_OPEN_WFIPSEGMENT) > 0)
		{
			granted = 1;
		}
	dynAppend(menuList,"PUSH_BUTTON, WFIPSegment, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_OPEN_WFIPSEGMENT)+", " + granted);
}
	
// Open Crate	
if (dynContains(dsMenuConfig, ciet_POPUPMENU_OPEN_CRATE) > 0)
{
	granted = 0;
	if (dynContains(dsAccessOk, ciet_POPUPMENU_OPEN_CRATE) > 0)
		{
			granted = 1;
		}
	dynAppend(menuList,"PUSH_BUTTON, Crate, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_OPEN_CRATE)+", " + granted);
}
	
// Default Unicos action
	unGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	if (dynlen(menuList) > 0)
	{
		if (menuList[dynlen(menuList)] != "SEPARATOR")
		{
			dynAppend(menuList, "SEPARATOR");
		}
	}
	
// 13. Help

	if (dynContains(dsMenuConfig, ciet_POPUPMENU_HELP) > 0)
		{
			granted = 0;
			if (dynContains(dsAccessOk, ciet_POPUPMENU_HELP) > 0)
				{
					granted = 1;
				}
			dynAppend(menuList,"PUSH_BUTTON, Help, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_HELP)+", " + granted);
		}	

	if (dynContains(dsMenuConfig, ciet_POPUPMENU_WIDGET_SIMULATOR) > 0)
		{
			granted = 0;
			if (dynContains(dsAccessOk, ciet_POPUPMENU_WIDGET_SIMULATOR) > 0)
				{
					granted = 1;
				}
			dynAppend(menuList,"PUSH_BUTTON, Widget Simulator, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_WIDGET_SIMULATOR)+", " + granted);
		}
	
// Trend Action
	unGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);

}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPAgent_HandleMenu
/**
Purpose: handle the answer of the popup menu

Parameters:
	- deviceName, string, input, the device dp name
	- sDpType, string, input, the device dp type
	- menuList, dyn_string, input, list of requested action
	- menuAnswer, int, input, selected action from the menuList
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 3.0 
	. distributed system: yes.
*/
cietWFIPAgent_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
{
dyn_string exceptionInfo,dsTemp, dsFIPInfos;
string descripName,sDpGTWName,sDpAgentName;
string sBusName;
bool bRes;
int iAgentNumber, iRes;
string sPanel,sType;
int iUnicosPopupMenuMin = UN_POPUPMENU_MIN;

//DebugN("HandleMenu", deviceName, sDpType, menuList, menuAnswer);
//	unGenericObject_HandleUnicosMenu(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);

// handle the select
	unGenericObject_handleSelect(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);
// handle the UNICOS default (open faceplate, diagnostic and info)
	unGenericObject_handleDefaultUnicos(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);
// handle the ciet device actions
	switch (menuAnswer-iUnicosPopupMenuMin)
	{
		case ciet_POPUPMENU_ORDER_RESET:					// Reset
			cietGenericObject_resetDevice(deviceName, exceptionInfo);
			break;

		case ciet_POPUPMENU_ORDER_CONFIG:		// Config
			// DebugN("Right click: WFip COnfig");
			ChildPanelOnModal("vision/cietDevices/General/cietSetConfigAgent.pnl",
							  "WFip Conf: " + unGenericDpFunctions_getAlias(deviceName),
							  makeDynString(/*"$sDpGTWName:"+sDpGTWName,
							  							"$sDpAgentName:"+sDpAgentName,*/
							  							"$sDpName:"+deviceName/*,
							  							"$iagNumber:" +iAgentNumber*/),
							  0, 0);
		break;

		case ciet_POPUPMENU_ORDER_DISABLE:					// Disable
			//DebugN("Right -  Disable");
   		 /// open  Disable Panel
				ChildPanelOnModal("vision/cietDevices/General/cietDisableAgent.pnl",
								  "Disable" + unGenericDpFunctions_getAlias(deviceName),
								  makeDynString("$sDpName:"+deviceName),0,0);	
		break; 
		
		case ciet_POPUPMENU_ORDER_DEBUG:					// Debug
					cietGenericObject_OpenDebug(deviceName);
		break;   		 

		case ciet_POPUPMENU_ORDER_OPEN_WFIPSEGMENT:					// Open Segment
					sBusName=unGenericDpFunctions_getSystemName(deviceName) + cietGenericFuncitons_getSegmentName(deviceName);
					unGenericObject_OpenFaceplate(unGenericDpFunctions_getWidgetDpName(sBusName), exceptionInfo);
		break;

		case ciet_POPUPMENU_ORDER_OPEN_CRATE:										// Open Crate
					cietGenericObject_openCratePanel (deviceName);
		break;	

		case ciet_POPUPMENU_ORDER_HELP:					// Help
					cietGenericObject_OpenHtmlFile("Help.html");
		break;				
		
		case ciet_POPUPMENU_ORDER_WIDGET_SIMULATOR:					// Widget Simulator
			ChildPanelOnRelativ("vision/cietDevices/General/cietWidgetSimulator.pnl",
			"Widget Simulator","",0, 0);			  
		break;	
		
		default:
		break;		

	}
// handle the trend action
	unGenericObject_handleTrendAction(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);

	if (dynlen(exceptionInfo) > 0)
		{
			unSendMessage_toExpertException("Right Click",exceptionInfo);
		}
}




