/**@name LIBRARY: cietWFIPSegment.ctl

@author: Celine VUILLERMOZ (AB/CO)

Creation Date: 20-07-2005

Modification History: 
 04/11/11  achiron: call only cietGenericFunctions_HeaderDisconnect(); in the ciet-DPT-_FaceplateStatusDisconnection()
 02/11/11  achiron: Corrected the call to the function ciet-DTD-_WidgetDisconnect iRes was argument instead of a PlcName
 07/10/11  achiron: Included the stuff for the new square which indicates if a PLC is connected or note to the segment
 06/10/11  achiron: Inluded some stuff for taking in account the new button PLC Connected
 22/07/08  Quique: Identification of the segment in the widget.
 	09/05/08 	Pablo: 	Button UserAccess adapted to QT.	
 	05/11/07 	Pablo: 	Open plc from popup menu.	
 										debug application included. 	
 	23/08/07 	Pablo: 	Help on handle menu.
	22/08/07	Pablo:	Added Agent Status animation on Table Agents Connected.
	27/06/07	Pablo:	Added Segment Status animation on faceplate and widget.
	21/06/07	Pablo:	Added compute time.
	18/06/07	Pablo:	Adapted to distributed system.
	06/06/07	Pablo:	Added PLC-FEC Counters.
	19/04/07	Pablo:	Updated buttons animation.
	18/04/07	Pablo:	Added FIPInfo (Button and Right-click).
	19/12/06	Pablo:	Added FIPDiag References on Faceplate.
	18/12/06	Pablo:	Reordered Warnings on Widget Animation.
										Added PLC Errors boxes on Faceplate.
										Added type of error on widget (PLC or FIP)
	17/11/06	Quique: New function cietWFIPSegment_ObjectListGetValueTime
	10/10/06	Pablo:	Modified widgetDisconnection function to visualize right behaviour.
	03/10/06	Quique:	Correct formats for FIP,PLC status and add new LEDS to visualize in the faceplate
	26/09/06	Pablo:  Added ST_GTWDEV indicator on widget animation.
  25/09/06	Pablo:	Changed statusFIP and StatusPLC on Faceplate from colorbox to valuebox
  25/09/06	Quique:	Correct widget alarm value. c_unSystemIntegrity_no_alarm_value is used.
  16/09/06	Quique: Get only values in faceplate animation when the agent value es bigger than 0. If it's not conected 
  									it can be 0 and then an error appears.
  06/09/06	Quique:	Added in the faceplate the name of the segment
	16/08/06	Quique: corrected format for config numbers

version 1.0

External Function : 
	. cietWFIPSegment_FaceplateStatusDisconnection : animate faceplate disconnection
	. cietWFIPSegment_ButtonUserAccess : User access for Analog buttons
	. cietWFIPSegment_ButtonSetState : set button state
	. cietWFIPSegment_WidgetDisconnection : animate widget disconnection
	. cietWFIPSegment_MenuConfiguration : configuration of popup menu

Internal Functions :
	. cietWFIPSegment_FaceplateStatusRegisterCB : register callback function
	. cietWFIPSegment_ButtonRegisterCB : register callback function
	. cietWFIPSegment_ButtonAnimationCB : animate buttons
	. cietWFIPSegment_ButtonDisconnection : animate buttons disconnection
	. cietWFIPSegment_WidgetRegisterCB : register callback function
	. cietWFIPSegment_WidgetAnimationCB : animate widget
		
Purpose: This library contains Analog object functions.

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. cietWFIPSegment object (DPType etc.)
	. Global variables and $parameters defined in Analog faceplate and widget
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

// constant declaration

//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPSegment_ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 3.1
	. operating system: WinXP
	. distributed system: yes.
*/
cietWFIPSegment_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
	float value;
	string sTime;
	  
	dpGet(sDeviceName+".ProcessInput.Data.statusFIP", value, sDeviceName+".ProcessInput.Data.statusFIP:_online.._stime", sTime);
	dsReturnData[1] = sTime;
	dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(sDeviceName+".ProcessInput.Data.statusFIP"), value) + " " 
							+ dpGetUnit(sDeviceName+".ProcessInput.Data.statusFIP");
}

//---------------------------------------------------------------------------------------------------------------------------------------
//@{

// cietWFIPSegment_FaceplateStatusRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in cietWFIPSegment faceplate
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietWFIPSegment_FaceplateStatusRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, sSystemName, sPlcName;
	int iAction, iRes, iAlertType1;
	dyn_string exceptionInfo;
	bool bRemote;
	
	deviceName = unGenericDpFunctions_getDpName($sDpName);
// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	if(sPlcName != "") {
		if (dpExists(sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;
		unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;																			 
	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	
//	DebugN("FaceplateStatusRegisterCB", deviceName,iAction);
	
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
//DebugN("cietWFIPSegment_FaceplateStatusDisconnection();");
			cietWFIPSegment_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
		
			g_bDisconnectCB = false;
			if(g_bUnSystemAlarmPlc) {
				iRes = dpConnect("cietWFIPSegment_FaceplateLockCallBack", deviceName + ".ProcessInput.Data.statusFIP:_lock._alert_hdl._locked");

				g_bCallbackConnected = (iRes >= 0);
			}
			else
				cietWFIPSegment_FaceplateStatusDisconnection();
			
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
		
			if(g_bUnSystemAlarmPlc) 
				iRes = dpDisconnect("cietWFIPSegment_FaceplateLockCallBack", deviceName + ".ProcessInput.Data.statusFIP:_lock._alert_hdl._locked");

			if(g_bDisconnectCB)
				cietWFIPSegment_FaceplateDisconnect(deviceName, iRes);

			g_bCallbackConnected = !(iRes >= 0);
			g_bDisconnectCB = false;
				cietWFIPSegment_FaceplateStatusDisconnection();
			
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
	}
//---------------------------------------------------------------------------------------------------------------------------------------
// cietWFIPSegment_FaceplateLockCallBack
/**
Purpose: set up the lock callback function for the faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in PT widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietWFIPSegment_FaceplateLockCallBack(string sdpe, bool bLockState)
{
	int iRes;
	string deviceName;

	deviceName = unGenericDpFunctions_getDpName(sdpe);
	if(bLockState) { 
		if(g_bDisconnectCB) {
			cietWFIPSegment_FaceplateDisconnect(deviceName, iRes);
			g_bDisconnectCB = false;
		}
		cietWFIPSegment_FaceplateStatusDisconnection();
	}
	else { 
		g_bDisconnectCB = true;
		cietWFIPSegment_FaceplateConnect(deviceName);
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPSegment_FaceplateConnect
/**
Purpose: connect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in cietWFIPSegment faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietWFIPSegment_FaceplateConnect(string deviceName)
{
int iAction, iRes, iAlertType1, iAlertType2, iRangeType;
	dyn_string exceptionInfo, dsDeviceNameSplit, dsTemp,dsFIPInfos;
	string sPlcName, sSystemName, sAlias, sAgentName, sNature,sDpAgentName,	sDescrip, sPattern ;
	dyn_string dsAliases, dsDpes, dsAgNumbers, dsAliasesOrd, dsDpesOrd; 
	int iAgNumber;
	
	// DebugN("cietWFIPSegment_FaceplateConnect");

	// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	
	// System Alarm
	if(sPlcName != "") {
		if (dpExists(sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;
	
	// Infos
	dsTemp=strsplit(deviceName,"-");
	sPattern=dsTemp[2];
	sAlias=	unGenericDpFunctions_getAlias(sSystemName+deviceName);
	dsDpes = dpNames(sSystemName+"*"+sPattern+"*");

//DebugN("dsDpes = " + dsDpes);					

	for (int i=1;i<=dynlen(dsDpes);i++)
		{	
			if (dpTypeName(dsDpes[i])=="WFIPEvenAgent" || dpTypeName(dsDpes[i])=="WFIPOddAgent")		// Only WFIP agents
				{
					iAgNumber= cietGenericFunctions_getAgentNumber(dsDpes[i]);
					dsAliases[i]=unGenericDpFunctions_getAlias(dsDpes[i]);
					if (iAgNumber >0)
						{
							dsAgNumbers[iAgNumber]=iAgNumber;
							dsAliasesOrd[iAgNumber]=dsAliases[i];
						}
				}
		}	
		
	for (int i=1;i<=dynlen(dsAliasesOrd);i++)
		{
			//DebugN("Candidate: "+dsAliasesOrd[i]);
			if (dsAgNumbers[i]!="")
				{
				//DebugN("Succeeded: "+dsAliasesOrd[i]);
				Table1.appendLine("Agent",dsAgNumbers[i],"Name",dsAliasesOrd[i]);
				}
		} 
				
	if(g_bUnSystemAlarmPlc) 
		{
	 		startThread("cietWFIPSegment_animateTableStatusAgent", sSystemName,sPlcName);

		
			iRes = dpConnect("cietWFIPSegment_FaceplateStatusAnimationCB",
							 				deviceName + ".ProcessInput.Data.segmentStatus",
							 				deviceName + ".ProcessInput.Data.statusFIP", 
							 				deviceName + ".ProcessInput.Data.statusPLC",
						 					deviceName + ".ProcessInput.Data.aliveCounter",
						 					deviceName + ".ProcessInput.Diag.fipCycleTime",
						 					deviceName + ".ProcessInput.Diag.fipCycleCounter",
						 					deviceName + ".ProcessInput.Diag.fipDiagModel",
						 					deviceName + ".ProcessInput.Diag.fipDiagConstructor",
						 					sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".enabled",
										/*	sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".enabled",	*/
						 					deviceName + ".ProcessInput.Data.wdPlc2Fec",
						 					deviceName + ".ProcessInput.Data.wdFec2Plc",
						 					deviceName + ".ProcessInput.Data.computeTime",						 													
											deviceName + ".ProcessInput.Data.statusFIP:_online.._invalid");
		}
		else
			cietWFIPSegment_FaceplateStatusDisconnection();
	
	g_bCallbackConnected = (iRes >= 0);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPSegment_FaceplateDisconnect
/**
Purpose: disconnect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in cietWFIPSegment faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietWFIPSegment_FaceplateDisconnect(string deviceName)
{
	int iRes;
	string sPlcName, sSystemName, sAlias,sAgentName;
	dyn_string dsDeviceNameSplit;
	
//	DebugN("FaceplateDisconnect");
	if(g_bUnSystemAlarmPlc) {
// get the PLC name 
		sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
		sSystemName = unGenericDpFunctions_getSystemName(deviceName);
		

		sAgentName = cietGenericFunctions_getAgentName(deviceName);
		sAlias = unGenericDpFunctions_getAlias(sAgentName);
	
	  stopThread (g_iThreadId);
 

		iRes = dpDisconnect("cietWFIPSegment_FaceplateStatusAnimationCB",
							 				deviceName + ".ProcessInput.Data.segmentStatus",		
							 				deviceName + ".ProcessInput.Data.statusFIP", 
							 				deviceName + ".ProcessInput.Data.statusPLC",
						 					deviceName + ".ProcessInput.Data.aliveCounter",
						 					deviceName + ".ProcessInput.Diag.fipCycleTime",
						 					deviceName + ".ProcessInput.Diag.fipCycleCounter",
						 					deviceName + ".ProcessInput.Diag.fipDiagModel",
						 					deviceName + ".ProcessInput.Diag.fipDiagConstructor",
						 					sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".enabled",
									/*		sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".enabled",	*/
						 					deviceName + ".ProcessInput.Data.wdPlc2Fec",
						 					deviceName + ".ProcessInput.Data.wdFec2Plc",
						 					deviceName + ".ProcessInput.Data.computeTime",						 					
											deviceName + ".ProcessInput.Data.statusFIP:_online.._invalid");
	}
	g_bCallbackConnected = !(iRes >= 0);
}



//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPSegment_animateTableStatusAgent
/*
Purpose: Animate Agent Status in Table.

Parameters:

Usage: Internal - Thread

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in cietWFIPSegment faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietWFIPSegment_animateTableStatusAgent(string sSystemName, string sPlcName)	
{
		string sAgentdeviceName, sAgentAlias, sColor;
		int j;
		
		unsigned uAgentStatus;
		bool bCodeInvalid, bIO_Error,bST_GTWDEV,bSystemIntegrityAlarmEnabled;
		int iSystemIntegrityAlarmValue;		
		
		
		g_iThreadId = getThreadId ();
		
		
		for (;;)
		{

					for (j=0;j<Table1.lineCount();j++)
						{
							getValue("Table1", "cellValueRC", j, "Name", sAgentAlias);	
							sAgentdeviceName = unGenericDpFunctions_dpAliasToName(sSystemName+sAgentAlias);
					
							dpGet(sAgentdeviceName + "ProcessInput.Data.agentStatus",uAgentStatus);
							bIO_Error = getBit (uAgentStatus, 0);
			  			dpGet(sAgentdeviceName + "ProcessInput.Data.agentStatus:_online.._invalid",bCodeInvalid);
			  			dpGet(sAgentdeviceName + "ST_GTWDEV",bST_GTWDEV);
			  			dpGet(sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".enabled",bSystemIntegrityAlarmEnabled);
							dpGet(sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".alarm",iSystemIntegrityAlarmValue);
			
							
							if (getBit(uAgentStatus, 12)==1)												// Disable
								{
									sColor = "cietWidget_Disabled";
								}
							else
								{
									 if (bCodeInvalid)
									 	{
											 sColor = "unDataNotValid"; 									// Invalid
									 	}	
									 else
									 	{
											if (bIO_Error==1)															// I/O Error
													sColor="cietWidget_Error";
											else
													sColor="unDisplayValue_Status";						// OK
													
											if ((!bSystemIntegrityAlarmEnabled) || (iSystemIntegrityAlarmValue >= CRYO_SYSTEM_INGETRITY_ALARM_LIMIT))
													sColor = "unDataNotValid";								// Old Data								
			
											if (bST_GTWDEV==0)														// Status Gateway Device 
										  	 sColor="cietWidget_Warning";
										 }	
								}					 
				
							setValue ("Table1", "cellBackColRC", j, "Status", sColor);		
						}

						delay(5);
		}		
}



//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPSegment_FaceplateStatusAnimationCB
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in cietWFIPSegment faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietWFIPSegment_FaceplateStatusAnimationCB(string sDp_segmentStatus, unsigned usegmentStatus,
																				string sDp_statusFIP, int istatusFIP,
																				string sDp_statusPLC, int istatusPLC,
																				string sDp_aliveCounter, int ialiveCounter,
																				string sDp_fipCycleTime, int ifipCycleTime,
																				string sDp_fipCycleCounter, int ifipCycleCounter,
																				string sDp_fipDiagModel, string sfipDiagModel,
																				string sDp_fipDiagConstructor, string sfipDiagConstructor,
																				string sDpSystemIntegrityAlarmValue, int iSystemIntegrityAlarmValue, 
																			 	string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
																			 	string sDp_PlcToFecCounter, int iPlcToFecCounter,
																			 	string sDp_FecToPlcCounter, int iFecToPlcCounter,
																			 	string sDp_ComputeTime, float fComputeTime,
																			/* 	string sDpSystemAlarmValue, int iSystemAlarmValue,
																			 	string sDpSystemAlarmEnabled, bool bSystemAlarmEnabled,*/
																			 	string sDpInvalid, bool bCodeInvalid)
{

int iAlarmValue,i,iStart;
bool bFaceplateInvalid, bReset, bAgError, bWatchdog, bIoError, bPLCError, bFIPError, bAgentsError, bFIPDiagError, bHasAPlcConnected;
string sTemp,sDpPLCName, sfipDiagLocation, sfipDiagSystem, sfipDiagSegment, sfipCycleTime, sGTWName;
dyn_bit32 db32sensorByte,dsTemp ;
char temp1, temp2;
bool bStatFIP, bPLCSystemError, bPLCMissedConfiguration, bPLCConnectionFault;
bit32 BStatPLC;
time tfipCycleTime;

	if ((iSystemIntegrityAlarmValue >= CRYO_SYSTEM_INGETRITY_ALARM_LIMIT)
			/*||
			(iSystemAlarmValue < CRYO_SYSTEM_INGETRITY_ALARM_LIMIT)*/)
//	if (iSystemIntegrityAlarmValue >= CRYO_SYSTEM_INGETRITY_ALARM_LIMIT)
			iAlarmValue=c_unSystemIntegrity_alarm_value_level1;
	else
		iAlarmValue=c_unSystemIntegrity_no_alarm_value;
		
	iStart=strpos(sDpSystemIntegrityAlarmValue, c_cietSystemIntegrity_DS) + strlen(c_cietSystemIntegrity_DS);
	sDpPLCName=substr(sDpSystemIntegrityAlarmValue,  iStart, strpos(sDpSystemIntegrityAlarmValue, ".alarm") - iStart);
	
	cietGenericFunctions_Header(sDpPLCName, iAlarmValue, bSystemIntegrityAlarmEnabled/* && bSystemAlarmEnabled*/, bCodeInvalid);
	
	bFaceplateInvalid = false;
	
	// Animation LEDs
	if (istatusFIP == c_unSystemIntegrity_no_alarm_value)
			bStatFIP=false;
	else 
			bStatFIP=true;
	
  if (istatusPLC == c_unSystemIntegrity_no_alarm_value)
  	{
		  bPLCSystemError=false;
		  bPLCMissedConfiguration=false;
		  bPLCConnectionFault=false;	  	  
	  }	  
		BStatPLC=istatusPLC;
		bPLCSystemError=getBit(BStatPLC,4);
		bPLCMissedConfiguration=getBit(BStatPLC,5);
		bPLCConnectionFault=getBit(BStatPLC,6);		
		bWatchdog=getBit(BStatPLC,9);		
		
		bIoError=getBit(usegmentStatus,0);	
		bPLCError=getBit(usegmentStatus,1);	
		bFIPError=getBit(usegmentStatus,2);	
		bAgentsError=getBit(usegmentStatus,3);	
		bFIPDiagError=getBit(usegmentStatus,4);	
		bHasAPlcConnected=getBit(usegmentStatus,5);	
    
	if(g_bSystemConnected)			
				unGenericObject_ColorBoxAnimate ("IO_ERROR", !bIoError, 1, "", bFaceplateInvalid, "cietFaceplate_Warning");
	if(g_bSystemConnected)			
				unGenericObject_ColorBoxAnimate ("ERROR_PLC", !bPLCError, 1, "", bFaceplateInvalid, "cietFaceplate_Warning");
	if(g_bSystemConnected)			
				unGenericObject_ColorBoxAnimate ("ERROR_FIP", !bFIPError, 1, "", bFaceplateInvalid, "cietFaceplate_Warning");
	if(g_bSystemConnected)			
				unGenericObject_ColorBoxAnimate ("ERROR_AGENTS", !bAgentsError, 1, "", bFaceplateInvalid, "cietFaceplate_Warning");
	if(g_bSystemConnected)			
				unGenericObject_ColorBoxAnimate ("ERROR_FIPDIAG", !bFIPDiagError, 1, "", bFaceplateInvalid, "cietFaceplate_Warning");		
  if(g_bSystemConnected)			
				unGenericObject_ColorBoxAnimate ("HASAPLCCONNECTED", !bHasAPlcConnected, 1, "", bFaceplateInvalid, "cietFaceplate_ModeActive");																
		
		
		
	if(g_bSystemConnected)			
				unGenericObject_ColorBoxAnimate ("FIP_LED", !bStatFIP, 1, "", bFaceplateInvalid, "cietFaceplate_Warning");
	if(g_bSystemConnected)			
				unGenericObject_ColorBoxAnimate ("PLC_SYSTEM_ERROR", !bPLCSystemError, 1, "", bFaceplateInvalid, "cietFaceplate_Warning"); 
	if(g_bSystemConnected)			
				unGenericObject_ColorBoxAnimate ("PLC_MISSED_CONFIGURATION", !bPLCMissedConfiguration, 1, "", bFaceplateInvalid, "cietFaceplate_Warning");
	if(g_bSystemConnected)			
				unGenericObject_ColorBoxAnimate ("PLC_CONNECTION_FAULT", !bPLCConnectionFault, 1, "", bFaceplateInvalid, "cietFaceplate_Warning");
	if(g_bSystemConnected)			
				unGenericObject_ColorBoxAnimate ("WATCHDOG", !bWatchdog, 1, "", bFaceplateInvalid, "cietFaceplate_Warning");				
												

		sGTWName=unGenericDpFunctions_getAlias(sDpPLCName);
		sGTWName=substr(sGTWName, strlen(CRYO_CRYOGTW_DPTYPE)+1);
		
	if(g_bSystemConnected)
		cietGenericObject_DisplayString(sGTWName, "GTWNAME", "unDisplayValue_Parameter", bFaceplateInvalid);
		
	if(g_bSystemConnected)
		cietGenericObject_DisplayString(cietGenericFuncitons_getSegmentSlotRef(sDp_statusFIP), "SLOTNUM", "unDisplayValue_Parameter", bFaceplateInvalid);
		
	if(g_bSystemConnected)	
			cietGenericObject_DisplayInt(istatusFIP, "STATUSFIP", "unDisplayValue_Parameter", bFaceplateInvalid);	
	if(g_bSystemConnected)		
		cietGenericObject_DisplayInt(istatusPLC, "STATUSPLC", "unDisplayValue_Parameter", bFaceplateInvalid);	

	tfipCycleTime=ifipCycleTime;	
	sfipCycleTime = formatTime ( "%d %b %y %H:%M:%S", tfipCycleTime );
	if(g_bSystemConnected)			
		cietGenericObject_DisplayString(sfipCycleTime, "FIPCYCLETIME", "unDisplayValue_Parameter", bFaceplateInvalid);

	if(g_bSystemConnected)			
		cietGenericObject_DisplayInt(ialiveCounter, "ALIVECOUNTER", "unDisplayValue_Parameter", bFaceplateInvalid);
	if(g_bSystemConnected)			
		cietGenericObject_DisplayInt(ifipCycleCounter, "FIPCYCLECOUNTER", "unDisplayValue_Parameter", bFaceplateInvalid);

	if(g_bSystemConnected)			
		cietGenericObject_DisplayInt(iPlcToFecCounter, "PLC2FEC", "unDisplayValue_Parameter", bFaceplateInvalid);
	if(g_bSystemConnected)			
		cietGenericObject_DisplayInt(iFecToPlcCounter, "FEC2PLC", "unDisplayValue_Parameter", bFaceplateInvalid);		
	if(g_bSystemConnected)			
		unGenericObject_DisplayValue("", "", fComputeTime, "COMPUTETIME", "unDisplayValue_Parameter", bFaceplateInvalid);				
	
	if(g_bSystemConnected)			
		cietGenericObject_DisplayString(sfipDiagConstructor, "FIPDIAGCONSTRUCTOR", "unDisplayValue_Parameter", bFaceplateInvalid);
	if(g_bSystemConnected)			
		cietGenericObject_DisplayString(sfipDiagModel, "FIPDIAGMODEL", "unDisplayValue_Parameter", bFaceplateInvalid);	

	// Diag References

	cietGenericFunctions_DecodeFIPDiagReferences(sfipDiagConstructor, sfipDiagModel, sfipDiagLocation, sfipDiagSystem, sfipDiagSegment);

	if(g_bSystemConnected)			
		cietGenericObject_DisplayString(sfipDiagLocation, "DIAG_LOCATION", "unDisplayValue_Parameter", bFaceplateInvalid);	
	if(g_bSystemConnected)			
		cietGenericObject_DisplayString(sfipDiagSystem, "DIAG_SYSTEM", "unDisplayValue_Parameter", bFaceplateInvalid);	
	if(g_bSystemConnected)			
		cietGenericObject_DisplayString(sfipDiagSegment, "DIAG_SEGMENT", "unDisplayValue_Parameter", bFaceplateInvalid);							
		
	
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPSegment_FaceplateStatusDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietWFIPSegment_FaceplateStatusDisconnection()
{

//DebugN("FaceplateStatusDisconnection");
//	unGenericObject_ColorBoxDisconnect(makeDynString("IO_ERROR","RESET", "AG_ERROR" ));

/*	unGenericObject_DisplayValueDisconnect(makeDynString("FIPDIAGMODEL", "FIPDIAGCONSTRUCTOR", "FIPCYCLECOUNTER", "ALIVECOUNTER", 
	"FIPCYCLETIME", "STATUSPLC", "STATUSFIP"));*/
  //put in the top textbox the DATA not connected in purple
  cietGenericFunctions_HeaderDisconnect();
	
}



//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPSegment_ButtonUserAccess
/**
Purpose: User access for DQAMCMB buttons

Parameters: 
	- sDpName, string, input, device name
	- sDpType, string, input, dptype
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietWFIPSegment_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{

  bool operator, expert, admin;
	int i, length;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
        string sOperAction, sExpAction, sAdminAction;
        bool bActionDefined = false;

        unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo,CIET_ACCESS_CONTROL_DOMAIN_NAME);
        if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) ||
           (sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME))
          bActionDefined=true;
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString();
	if (operator)
	{
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
            dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_SELECT,ciet_POPUPMENU_HELP,
                                              ciet_POPUPMENU_WIDGET_SIMULATOR,ciet_POPUPMENU_OPEN_PLC));
	}
	if (expert)
	{
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
	    			dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_SELECT,ciet_FACEPLATE_BUTTON_DIAG, 
                                              ciet_FACEPLATE_BUTTON_DEBUG, ciet_FACEPLATE_BUTTON_PLCCONFIG, ciet_FACEPLATE_BUTTON_FIPINFOS,
                                              ciet_POPUPMENU_HELP,ciet_POPUPMENU_WIDGET_SIMULATOR,ciet_POPUPMENU_OPEN_PLC));	
	}
        if(admin){
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
            dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_SELECT,ciet_FACEPLATE_BUTTON_DIAG, 
                                              ciet_FACEPLATE_BUTTON_DEBUG, ciet_FACEPLATE_BUTTON_PLCCONFIG,ciet_FACEPLATE_BUTTON_FIPINFOS,
                                              ciet_POPUPMENU_HELP,ciet_POPUPMENU_WIDGET_SIMULATOR,ciet_POPUPMENU_OPEN_PLC));	
        }
//.............................  
  /*
  bool operator, expert, admin;
	int i, length;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
	
//	DebugN("ButtonUserAccess");
	unGenericButtonFunctionsHMI_getUnicosPriviledges(dbPermissions, exceptionInfo);
	if (dynlen(exceptionInfo) > 0)
		{
		dynClear(dbPermissions);
		}
	while (dynlen(dbPermissions) < 4)
		{
		dynAppend(dbPermissions, false);
		}
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString();
	if (operator)
		{
		dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_SELECT,ciet_POPUPMENU_HELP,ciet_POPUPMENU_WIDGET_SIMULATOR,ciet_POPUPMENU_OPEN_PLC));
		
		}
	if (expert)
		{
			dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_SELECT,ciet_FACEPLATE_BUTTON_DIAG, ciet_FACEPLATE_BUTTON_DEBUG, ciet_FACEPLATE_BUTTON_FIPINFOS,ciet_POPUPMENU_HELP,ciet_POPUPMENU_WIDGET_SIMULATOR,ciet_POPUPMENU_OPEN_PLC));	
		}
	if (admin)
		{
			dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_SELECT,ciet_FACEPLATE_BUTTON_DIAG, ciet_FACEPLATE_BUTTON_DEBUG, ciet_FACEPLATE_BUTTON_FIPINFOS,ciet_POPUPMENU_HELP,ciet_POPUPMENU_WIDGET_SIMULATOR,ciet_POPUPMENU_OPEN_PLC));	
		}	*/
  
}


//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPSegment_ButtonRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in  faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietWFIPSegment_ButtonRegisterCB(string sDpSystemConnected, bool bSystemConnected)
{
	string deviceName, sSystemName, sDpType, sFunction;
	int iAction, iRes, iManReg01;
	dyn_string exceptionInfo, dsFunctions, dsUserAccess;
//	DebugN("ButtonRegisterCB");
	
	deviceName = unGenericDpFunctions_getDpName($sDpName);
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	
	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
//		DebugN("ButtonRegisterCB UN_ACTION_DISCONNECT");
			cietWFIPSegment_ButtonDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
//		DebugN("ButtonRegisterCB UN_ACTION_DPCONNECT");

sDpType="WFIPSegment";	
			unGenericObject_GetFunctions(sDpType, dsFunctions, exceptionInfo);
			sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];
						 
			if (sFunction != "")
				{
				evalScript(dsUserAccess, "dyn_string main(string deviceNameTemp, string sDpTypeTemp) {" + 
							 "dyn_string dsUserAccessReturn;" +
							 "if (isFunctionDefined(\"" + sFunction  + "\"))" +
							 "    {" +
					 			sFunction  + "(deviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
							 "    }" +
							 "return dsUserAccessReturn; }", makeDynString(), deviceName, sDpType);
						
				if (dynlen(dsUserAccess) == 0)
					fwException_raise(exceptionInfo,"ERROR", "cietWFIPSegment_ButtonRegisterCB: " + getCatStr("unGeneration","UNKNOWNFUNCTION"), sFunction);
				else
					g_dsUserAccess=dsUserAccess;	
				}
			else
				fwException_raise(exceptionInfo,"ERROR", "cietWFIPSegment_ButtonRegisterCB: " + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");


			iRes = dpConnect("cietWFIPSegment_ButtonAnimationCB",
							 				 deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							 				 deviceName + ".statusInformation.selectedManager");
			g_bCallbackConnected = (iRes >= 0);
			
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
//			DebugN("ButtonRegisterCB UN_ACTION_DPDISCONNECT_DISCONNECT");
			iRes = dpDisconnect("cietWFIPSegment_ButtonAnimationCB",
							 						deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							 						deviceName + ".statusInformation.selectedManager");
			g_bCallbackConnected = !(iRes >= 0);
			cietWFIPSegment_ButtonDisconnection();
			break;
		case UN_ACTION_NOTHING:
//				DebugN("ButtonRegisterCB UN_ACTION_NOTHING");
		default:
			break;
		}
}


//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPSegment_ButtonAnimationCB
/**
Purpose: animate buttons

Parameters: 
	- dbPermissions, dyn_bool, input, list of permissions, length = 4, order : 1=monitor, 2=operator, 3=expert, 4=admin
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietWFIPSegment_ButtonAnimationCB(string sDpLock, bool bLocked, 
														 string sDpSelect, string sSelectedManager)
{

bool bStatusDiag="true", bStatusFIPInfos="true";
cietWFIPSegment_ButtonSetState(unGenericDpFunctions_getDpName(sDpLock), "WFIPSegment", g_dsUserAccess, makeDynString(bLocked, sSelectedManager, bStatusFIPInfos));
}



//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPSegment_ButtonSetState
/**
Purpose: set button state

Parameters: 
	- deviceName, string, input, device
	- sDpType, string, input, dptype
	- dsUserAccess, dyn_string, input, user access
	- dsData, dyn_string, parameter
		
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietWFIPSegment_ButtonSetState(string deviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
	bool bSelected, buttonEnabled, bit1, bit2;
	dyn_string dsButtons;
	int i, length, alertType;
	string localManager;
	bool bLocked, ListEnabled;
	string sSelectedManager;
//	DebugN("ButtonSetState");

	bLocked = dsData[1];
	sSelectedManager = dsData[2];
	
// 1. Selection state
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = (localManager == "S");
	
	dsButtons = makeDynString(ciet_FACEPLATE_BUTTON_DIAG, ciet_FACEPLATE_BUTTON_FIPINFOS,ciet_FACEPLATE_BUTTON_DEBUG, ciet_FACEPLATE_BUTTON_PLCCONFIG); // Each button except "Select"
	length = dynlen(dsButtons);
// 1. Selection state
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = (localManager == "S");

// 2. Button state
	for(i=1;i<=length;i++)
		{
		buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0);	// User access
		switch (dsButtons[i])
			{
			case ciet_FACEPLATE_BUTTON_DIAG:
				buttonEnabled = buttonEnabled && bSelected;
				break;
			case ciet_FACEPLATE_BUTTON_FIPINFOS:
				buttonEnabled = buttonEnabled && bSelected;
				break;
			case ciet_FACEPLATE_BUTTON_DEBUG:
				buttonEnabled = buttonEnabled && bSelected;
				break;				
    case ciet_FACEPLATE_BUTTON_PLCCONFIG:
				buttonEnabled = buttonEnabled && bSelected;
				break;				
			default:
				break;
			}
		setValue(ciet_FACEPLATE_BUTTON_PREFIX + dsButtons[i], "enabled", buttonEnabled);
		}
// 3. Button Select state
	unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
}


//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPSegment_UserLoginGetButtonState
/**
Purpose: function called before the button set state when a user logs in. 

Parameters:
	deviceName: string, input, the device name
	dsReturnData: dyn_string, output, the data are returned here

Usage: Internal function

PVSS manager usage:  Ctrl, NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
		-> $sDpName, string, the device DP name
		-> g_sFaceplateButtonType, string, faceplate type
	. PVSS version: 3.0 
	. operating system: NT, W2000 and Linux, but tested only under W2000.
	. distributed system: yes but not tested.
*/

cietWFIPSegment_UserLoginGetButtonState(string deviceName, string sType, dyn_string &dsReturnData)
{
	
		int iRes;
	string sDpLock, sDpSelectedManager, sSelectedManager, sInvalidDp, sSystemName;
	bool bLocked;
//	DebugN("UserLoginGetButtonState");
// 2. Get parameters to call the set button function
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);

	sDpLock = deviceName + ".statusInformation.selectedManager:_lock._original._locked";
	sDpSelectedManager = deviceName + ".statusInformation.selectedManager";
	
	if (dpExists(deviceName))
	{
		iRes = dpGet(sDpLock, bLocked, sDpSelectedManager, sSelectedManager);
	}
	else
	{
		iRes = -1;
	}
	if (iRes < 0) // Errors during dpget
	{
		sDpLock = "";
		bLocked = false;
		sDpSelectedManager = "";
		sSelectedManager = "";
	}
	dsReturnData=makeDynString(bLocked, sSelectedManager, "", "", "");
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPSegment_ButtonDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietWFIPSegment_ButtonDisconnection()
{
			   
//	DebugN("ButtonDisconnection");
		unGenericObject_ButtonDisconnect(makeDynString(UN_FACEPLATE_BUTTON_SELECT,ciet_FACEPLATE_BUTTON_DIAG, ciet_FACEPLATE_BUTTON_DEBUG, ciet_FACEPLATE_BUTTON_FIPINFOS, ciet_FACEPLATE_BUTTON_PLCCONFIG));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPSegment_WidgetRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in PT widget
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietWFIPSegment_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, deviceType;
	int iAction,iRes;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName;
	bool bRemote;
	
//		DebugN("WidgetRegisterCB");
	deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);

	if (deviceName == "")	// In case of disconnection
		{
		deviceName = g_sDpName;
		}
		// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	
	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;

	if(sPlcName != "") {
		if (dpExists(sSystemName+c_CRYO_FRONTEND_UNSYSTEMALARM+c_cietSystemIntegrity_DS+sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;

	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			cietWFIPSegment_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:
			g_sDpName = deviceName;
			g_bDisconnectCB = false;
		//	cietWFIPSegment_WidgetConnect(deviceName, sPlcName);
	
		if(g_bUnSystemAlarmPlc) {
					deviceType = dpTypeName(deviceName);
				if(deviceType == "WFIPSegment")
					iRes = dpConnect("cietWFIPSegment_WidgetLockCallBack", deviceName + ".ProcessInput.Data.statusFIP:_lock._alert_hdl._locked");
				else{
					g_bUnSystemAlarmPlc = false;
					cietWFIPSegment_WidgetDisconnection(g_sWidgetType);
				}
	
				g_bCallbackConnected = (iRes >= 0);
			}
			else
					cietWFIPSegment_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
		/*
			cietWFIPSegment_WidgetDisconnect(deviceName, sPlcName);
			cietWFIPSegment_WidgetDisconnection(g_sWidgetType);*/
			if(g_bUnSystemAlarmPlc) 
				iRes = dpDisconnect("cietWFIPSegment_WidgetLockCallBack", deviceName + ".ProcessInput.Data.statusFIP:_lock._alert_hdl._locked");

			if(g_bDisconnectCB) {
						cietWFIPSegment_WidgetDisconnect(g_sDpName, sPlcName);
				}
			g_bCallbackConnected = !(iRes >= 0);
			g_bDisconnectCB = false;

			cietWFIPSegment_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}
//---------------------------------------------------------------------------------------------------------------------------------------
// cietWFIPSegment_WidgetLockCallBack
/**
Purpose: set up the lock callback function for the widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in PT widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietWFIPSegment_WidgetLockCallBack(string sdpe, bool bLockState)
{
	int iRes;
	string sPlcName;

	if(bLockState) 
		{ // lock set so disconnect if not already disconnected
		if(g_bDisconnectCB) 
			{
				cietWFIPSegment_WidgetDisconnect(g_sDpName, sPlcName);
			}
		g_bDisconnectCB = false;
		cietWFIPSegment_WidgetDisconnection(g_sWidgetType);
		}
	else 
		{ // lock not set so connect if not already connected
		g_bDisconnectCB = true;
		sPlcName = unGenericObject_GetPLCNameFromDpName(sdpe);
		cietWFIPSegment_WidgetConnect(g_sDpName,sPlcName);
		}
}
//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPSegment_WidgetConnect
/**
Purpose: Connect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Analog faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietWFIPSegment_WidgetConnect(string deviceName, string sPlcName)
{
	int iRes;
	string sSystemName, sAlias,sAgentName;
	dyn_string dsDeviceNameSplit;
//			DebugN("WidgetConnect");
//  check if the device name is of type "WFIPSegment" if not like of no PLC. -> not available.
	if(dpTypeName(deviceName) != "WFIPSegment")
		g_bUnSystemAlarmPlc = false;
	
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	
	if(g_bUnSystemAlarmPlc)
	{
		
	/*	sAgentName = cietGenericFunctions_getAgentName(deviceName);
		sAlias = unGenericDpFunctions_getAlias(sAgentName);
		dsDeviceNameSplit=strsplit(sAlias, ".");*/

		iRes = dpConnect("cietWFIPSegment_WidgetAnimationCB", 
						 					deviceName + ".ProcessInput.Data.segmentStatus",		
						 					deviceName + ".ProcessInput.Data.statusFIP",
						 					deviceName + ".ProcessInput.Data.statusPLC",
				 							sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".enabled",
											deviceName + ".ST_GTWDEV",
									/*		sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".enabled",*/
											deviceName + ".ProcessInput.Data.statusFIP:_online.._invalid", 
									 		deviceName + ".statusInformation.selectedManager:_lock._original._locked",
						 					deviceName + ".statusInformation.selectedManager");
		g_bCallbackConnected = (iRes >= 0);		
	}else{
		cietWFIPSegment_WidgetDisconnection(g_sWidgetType);
	}
}
//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPSegment_WidgetDisconnect
/**
Purpose: Disconnect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Analog faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
cietWFIPSegment_WidgetDisconnect(string deviceName, string sPlcName)
{
	int iRes;
	string sSystemName, sAlias,sAgentName;
	dyn_string dsDeviceNameSplit;
//				DebugN("WidgetDisconnect");
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);

	/*	sAgentName = cietGenericFunctions_getAgentName(deviceName);
		sAlias = unGenericDpFunctions_getAlias(sAgentName);
		dsDeviceNameSplit=strsplit(sAlias, ".");*/
		
	if(g_bUnSystemAlarmPlc)
	{
		iRes = dpDisconnect("cietWFIPSegment_WidgetAnimationCB", 
						 					deviceName + ".ProcessInput.Data.segmentStatus",
						 					deviceName + ".ProcessInput.Data.statusFIP",
						 					deviceName + ".ProcessInput.Data.statusPLC",
			 							sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM + c_cietSystemIntegrity_DS + sPlcName+".enabled",
											deviceName + ".ST_GTWDEV",
								/*			sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".alarm",
											sSystemName + c_CRYO_FRONTEND_UNSYSTEMALARM+"_FESystemAlarm_"+sPlcName+"_"+dsDeviceNameSplit[1]+".enabled",*/
											deviceName + ".ProcessInput.Data.statusFIP:_online.._invalid", 
									 		deviceName + ".statusInformation.selectedManager:_lock._original._locked",
						 					deviceName + ".statusInformation.selectedManager");
		
		g_bCallbackConnected = (iRes >= 0);
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPSegment_WidgetAnimationCB
/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Analog faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietWFIPSegment_WidgetAnimationCB(string sDp_segmentStatus, unsigned usegmentStatus,
																	string sDp_statusFIP, int istatusFIP,
																	string sDp_statusPLC, int istatusPLC,
											 string sDpSystemIntegrityAlarmValue, int iSystemIntegrityAlarmValue, 
											 string sDpSystemIntegrityAlarmEnabled, bool bSystemIntegrityAlarmEnabled,
											 string sDp_ST_GTWDEV, bool bST_GTWDEV,
										/*	 string sDpSystemAlarmValue, int iSystemAlarmValue,
											 string sDpSystemAlarmEnabled, bool bSystemAlarmEnabled,*/
											 string sDpCodeInvalid, bool bCodeInvalid,
											 string sDpLocked, bool bLocked,
											 string sDpSelectedManager, string sSelectedManager)
{



if (g_sWidgetType=="WFIPSegment")
				{
						string formattedValue;
						string warningLetter, warningColor, lineColor, warningVisible, alarmLetter, alarmColor, controlLetter, controlColor, body1Text, body1Color, selectColor;
                                                string segmentName,deviceName;
                                                
						bool lockVisible;
					
						// Warnings
												
						if (bCodeInvalid) 
							{
									warningLetter = UN_WIDGET_TEXT_INVALID; 								// Invalid
									warningColor = "unDataNotValid";
									warningVisible=true; 
									lineColor = "unDataNotValid";
							}	
						else
							{
									if (getBit(usegmentStatus, 0)==1)												// I/O Error
										{
											warningLetter = UN_WIDGET_TEXT_WARNING_ERROR;
											warningColor = "cietWidget_Warning";	
											warningVisible=true;
											lineColor = "cietWidget_Error";
											
											body1Color="unAlarm_AnalogH";																	
											body1Text = "ER";
											
											body1Text=body1Text + ":";
											if (getBit(usegmentStatus, 1)==1)
													body1Text=body1Text + "P";
											if (getBit(usegmentStatus, 2)==1)
													body1Text=body1Text + "F";
											if (getBit(usegmentStatus, 3)==1)
													body1Text=body1Text + "A";
											if (getBit(usegmentStatus, 4)==1)
													body1Text=body1Text + "D";																								
										}		
									else
										{
								 			body1Text = "OK";																			// OK
											body1Color="unDisplayValue_Status";
										 	warningVisible=false; 
	 										lineColor = "unDisplayValue_Status";
										}													
											
									if (!bSystemIntegrityAlarmEnabled||(iSystemIntegrityAlarmValue >= CRYO_SYSTEM_INGETRITY_ALARM_LIMIT))		
										{
											warningLetter = UN_WIDGET_TEXT_OLD_DATA; 						// Old Data			
											warningColor = "unDataNotValid";
	 										lineColor = "unDataNotValid";											
											warningVisible=true; 
										}	

									if (bST_GTWDEV==0)																				// Status Gateway Device 	
										{
										 	warningLetter= ciet_WIDGET_TEXT_STATUS_GATEWAY_DEVICE;  	   	 
									   	warningColor="cietWidget_Warning";
	 										lineColor = "cietWidget_Warning";										   	
									   	warningVisible=true; 
										}																																								
							}							
						
						// Set the widget
						unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, selectColor, lockVisible);
					
						// Animate widget
						if(g_bSystemConnected)			
							setMultiValue("Body1", "text", body1Text,
														"Body1", "foreCol", body1Color, 
														"Line", "foreCol", lineColor, 														
									  				"WarningText", "text", warningLetter,
									   				"WarningText", "foreCol", warningColor,
									   				"WarningText", "visible", warningVisible,
									  				"SelectArea", "foreCol", selectColor,
									   				"LockBmp", "visible", lockVisible, 
									  				"WidgetArea", "visible", true);
                                                // Identificate widget
						if(g_bSystemConnected)
                                                {
                                                 dyn_string dsResult;
                                                 deviceName = $sIdentifier; 
                                                 dsResult=strsplit(deviceName,"_");
                                                 if (dynlen(dsResult) > 0)
                                                   segmentName=dsResult[dynlen(dsResult)];    
                                                 else
                                                   segmentName="WFip";   
                                                 setValue("idetificatorText","text",segmentName);
                                                         
                                                }
				}
		
else if (g_sWidgetType=="WFIPSegmentLed")
				{	
					string formattedValue;
					string LedColor, selectColor;
					bool lockVisible;
				
					// Warnings
		
					if (bCodeInvalid) 							
						{
							LedColor = "unDataNotValid";						// Invalid
						}	
					else
						{
							if (getBit(usegmentStatus, 0)==1)
									LedColor="cietWidget_Error";					// I/O Error		
							else
									LedColor="unDisplayValue_Status";		// OK										
		
							if (!bSystemIntegrityAlarmEnabled||(iSystemIntegrityAlarmValue >= CRYO_SYSTEM_INGETRITY_ALARM_LIMIT))		
									LedColor = "unDataNotValid";				// Old Data	

							if (bST_GTWDEV==0)									
							   	LedColor="cietWidget_Warning";			// Status Gateway Device 
						}
					
					// Set the widget
					
					unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, selectColor, lockVisible);
				
					// Animate widget
					
					if (g_bSystemConnected)			
							setMultiValue("Led", "foreCol",LedColor);		
				}


}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPSegment_WidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
	- sWidgetType
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietWFIPSegment_WidgetDisconnection(string sWidgetType)
{

	//DebugN("WidgetDisconnection");
	if(sWidgetType=="WFIPSegment")
		{		
			setMultiValue("Body1", "text", "???",
										"Body1", "foreCol", "unDataNoAccess", 
										"Body1", "visible", true,
					  				"WarningText", "text", "",
					   				"WarningText", "visible", false,
					  				"SelectArea", "visible", false,
					   				"LockBmp", "visible", false, 
					  				"WidgetArea", "visible", false);	
		}	
		
	if(sWidgetType=="WFIPSegmentLed")
		{		
			setMultiValue("Led", "foreCol", "unDataNoAccess");	
		}			
								  											
}								


//@}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPSegment_PanelWidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
	- sWidgetType
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

cietWFIPSegment_PanelWidgetDisconnection(string sWidgetType)
{

	if(sWidgetType=="WFIPSegment")
		{
			string warningText;
			//DebugN("PanelWidgetDisconnection");
			warningText = UN_WIDGET_TEXT_INVALID;
		
			setMultiValue("Body1", "text", "",
										"WarningText", "text", warningText,
										"LockBmp", "visible", false);
		}
		
	else if(sWidgetType=="WFIPSegmentLed")
		{
		     setMultiValue("Led", "foreCol", "unDataNoAccess");	
			//DebugN("PanelWidgetDisconnection");
		}

		
}								

//------------------------------------------------------------------------------------------------------------------------
//@}

// cietWFIPSegment_MenuConfiguration
/**
Purpose: configuration of popup menu

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsAccessOk, dyn_string, input, authorized action
	- menuList, dyn_string, output, menu to pop-up
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 3.0 
	. distributed system: yes.
*/

cietWFIPSegment_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{

dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT,UN_POPUPMENU_FACEPLATE_TEXT, UN_POPUPMENU_TREND_TEXT, ciet_POPUPMENU_FIPINFOS, 
																				ciet_FACEPLATE_BUTTON_DEBUG, ciet_FACEPLATE_BUTTON_PLCCONFIG, ciet_POPUPMENU_HELP,ciet_POPUPMENU_WIDGET_SIMULATOR,ciet_POPUPMENU_OPEN_PLC);
int granted;

string localManager, sSelectedManager;
bool bSelected, bLocked;

int iUnicosPopupMenuMin = UN_POPUPMENU_MIN;	

//  Selection state

	dpGet(sDpName + ".statusInformation.selectedManager:_lock._original._locked", bLocked);
	dpGet(sDpName + ".statusInformation.selectedManager",sSelectedManager);
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = (localManager == "S");
								 
// Select
	unGenericObject_addSelectToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	dynAppend(menuList, "SEPARATOR");
	
// FIPInfos
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_FIPINFOS) > 0)
	{
		granted = 0;
		if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_FIPINFOS) > 0) && bSelected)
		{
			granted = 1;
		}
		dynAppend(menuList,"PUSH_BUTTON, FIPInfos, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_FIPINFOS)+", " + granted);
	}
		
	if (dynlen(menuList) > 0)
	{
		if (menuList[dynlen(menuList)] != "SEPARATOR")
		{
			dynAppend(menuList, "SEPARATOR");
		}
	}
	
// 4. Debug
	
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_DEBUG) > 0)
	{
		granted = 0;
		if ((dynContains(dsAccessOk, ciet_FACEPLATE_BUTTON_DEBUG) > 0 ) && bSelected)
		{
			granted = 1;
		}
		dynAppend(menuList, "PUSH_BUTTON, Debug, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_DEBUG)+", " + granted);
	}		
	
// Open PLC
if (dynContains(dsMenuConfig, ciet_POPUPMENU_OPEN_PLC) > 0)
{
	granted = 0;
	if (dynContains(dsAccessOk, ciet_POPUPMENU_OPEN_PLC) > 0)
		{
			granted = 1;
		}
	dynAppend(menuList,"PUSH_BUTTON, PLC, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_OPEN_PLC)+", " + granted);
}
	
//5. Default Unicos action
	unGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	if (dynlen(menuList) > 0)
	{
		if (menuList[dynlen(menuList)] != "SEPARATOR")
		{
			dynAppend(menuList, "SEPARATOR");
		}
	}

// 13. Help

	if (dynlen(menuList) > 0)
		{
			if (menuList[dynlen(menuList)] != "SEPARATOR")
			{
				dynAppend(menuList, "SEPARATOR");
			}
		}

	if (dynContains(dsMenuConfig, ciet_POPUPMENU_HELP) > 0)
		{
			granted = 0;
			if (dynContains(dsAccessOk, ciet_POPUPMENU_HELP) > 0)
				{
					granted = 1;
				}
			dynAppend(menuList,"PUSH_BUTTON, Help, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_HELP)+", " + granted);
		}	
		
	if (dynContains(dsMenuConfig, ciet_POPUPMENU_WIDGET_SIMULATOR) > 0)
		{
			granted = 0;
			if (dynContains(dsAccessOk, ciet_POPUPMENU_WIDGET_SIMULATOR) > 0)
				{
					granted = 1;
				}
			dynAppend(menuList,"PUSH_BUTTON, Widget Simulator, "+(iUnicosPopupMenuMin+ciet_POPUPMENU_ORDER_WIDGET_SIMULATOR)+", " + granted);
		}		
	
//6. Trend Action
	unGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);

}

//---------------------------------------------------------------------------------------------------------------------------------------

// cietWFIPSegment_HandleMenu
/**
Purpose: handle the answer of the popup menu

Parameters:
	- deviceName, string, input, the device dp name
	- sDpType, string, input, the device dp type
	- menuList, dyn_string, input, list of requested action
	- menuAnswer, int, input, selected action from the menuList
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 3.0 
	. distributed system: yes.
*/
cietWFIPSegment_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
{

dyn_string exceptionInfo,dsTemp;
string descripName,sDpGTWName,sDpAgentName;
string sPlcName;
bool bRes;
int iUnicosPopupMenuMin = UN_POPUPMENU_MIN, iRes;

// handle the select
	unGenericObject_handleSelect(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);
// handle the UNICOS default (open faceplate, diagnostic and info)
	unGenericObject_handleDefaultUnicos(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);
// handle the ciet device actions
	switch (menuAnswer-iUnicosPopupMenuMin)
	{
	
		case ciet_POPUPMENU_ORDER_FIPINFOS:					// FIP INFOS
		  cietGenericFunctions_opendeviceCIETConsole (deviceName);
			//DebugN("Right - Click FIP INFOS");
		break;	
		
		case ciet_POPUPMENU_ORDER_DEBUG:					// Debug
					cietGenericObject_OpenDebug(deviceName);
		break; 
		
		case ciet_POPUPMENU_ORDER_OPEN_PLC:
					sPlcName=unGenericDpFunctions_getSystemName(deviceName) + cietGenericFuncitons_getPlcName(deviceName);
					unGenericObject_OpenFaceplate(unGenericDpFunctions_getWidgetDpName(sPlcName), exceptionInfo);
		break;			
		
		case ciet_POPUPMENU_ORDER_HELP:					// Help
					cietGenericObject_OpenHtmlFile("Help.html");
		break;	
		
		case ciet_POPUPMENU_ORDER_WIDGET_SIMULATOR:					// Widget Simulator
			ChildPanelOnRelativ("vision/cietDevices/General/cietWidgetSimulator.pnl",
			"Widget Simulator","",0, 0);			  
		break;			
	
		default:
		break;
	}
// handle the trend action
	unGenericObject_handleTrendAction(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);

	if (dynlen(exceptionInfo) > 0)
	{
		unSendMessage_toExpertException("Right Click",exceptionInfo);
	}
}







